/***************************************************************\
*																*
* General routine to read the timer                             *
*																*
* By:  Stephen H. Landrum										*
*																*
* Last update:  27-Jul-93										*
*																*
* Copyright (c) 1993, The 3DO Company                           *
*																*
* This program is proprietary and confidential					*
*																*
* Modifications for M2 system:  Tom Watson						*
*																*
* Updates M2 on Sept 12, 1995									*
*																*
\***************************************************************/



#define DBUG(x)	{ printf x ; }
/* #define FULLDBUG(x) { printf x ; } */
#define FULLDBUG(x) 

/* Kernel includes, only some of which we need... */

/* #include <kernel/folio.h> */
#include <kernel/io.h>
/* #include <kernel/kernel.h> */
#include <kernel/kernelnodes.h>
#include <kernel/list.h>
#include <kernel/mem.h>
#include <kernel/nodes.h>
#include <kernel/operror.h>
#include <kernel/semaphore.h>
#include <kernel/task.h>
#include <kernel/types.h>

/* ANSI Style includes */

#include <strings.h>
#include <stdlib.h>
#include <stdio.h>


/* Local Includes */

#include "gettime.h"

/* Macro to clear out a structure */
#define CLEAR(x)  memset (&x, 0, sizeof(x))

static Item timerior = 0;
static IOReq *timerioreq;
static IOInfo ioi;
struct timeval tv;

uint32 overhead[2];

extern void terminate (int32);




void opentimer (void)
	{
	TagArg targs[2];
	static Item TimerDevice = 0;
	List *list;
	Err err;				/* for error returns */
	
	FULLDBUG (("Opening Timer device\n"));
	/* Some code copied from the other newer CD test */
	if (TimerDevice == 0)
		{
#ifdef ONE_DOT_ONE
		TimerDevice = OpenDevice ("timer", 0);
#else
		err = CreateDeviceStackListVA (&list, "cmds",
					DDF_EQ, DDF_INT, 1, TIMERCMD_GETTIME_USEC, NULL);
		if (err < 0 || GetNodeCount (list) < 1)
			{
			DBUG (("Getting device list (timer): %d, err: 0x%08X\n", 
					GetNodeCount (list), err));
			terminate (2);
			}
		TimerDevice = OpenDeviceStack ((DeviceStack *)FirstNode (list));
		DeleteDeviceStackList (list);
#endif
		if (TimerDevice < 0)
			{
			DBUG (("Error opening timer device (%ld)\n", TimerDevice));
			terminate (1);
			}
		}
	
	
	FULLDBUG (("Creating timer IO request\n"));
	timerior = CreateIOReq (0, 0, TimerDevice, 0);
	if (timerior < 0)
		{
		DBUG (("Error in creation of Timer IO Request (0x%08X)\n", timerior));
		terminate (1);
		}
	FULLDBUG (("timerior = %lx\n", timerior));
	
	memset (&ioi, 0, sizeof(ioi));
	
	ioi.ioi_Command = TIMERCMD_GETTIME_USEC;
	ioi.ioi_Recv.iob_Buffer = &tv;		/* In case it isn't filled in */
	ioi.ioi_Recv.iob_Len = sizeof(tv);
	}




void gettime (uint32 timebuffer[2])
	{
	int32 error;
	
	if (timerior == 0)
		{
		DBUG (("Timer not properly initialized\n"));
		terminate (1);
		}
	
	ioi.ioi_Recv.iob_Buffer = (void*)timebuffer;
	
	error = DoIO (timerior, &ioi);
	if (error<0)
		{
		DBUG (("Error getting timer status (%lx)\n", error));
		PrintfSysErr (error);
		FULLDBUG (("timerior = %lx\n", timerior));
		}
	}




void starttime (uint32 *time)
	{
	int32 error;

	if (timerior == 0)
		{
		DBUG (("Timer not properly initialized\n"));
		terminate (1);
		}
	
	ioi.ioi_Recv.iob_Buffer = (void*)time; /* time[0] & time [1] */
	
	error = DoIO (timerior, &ioi);
	if (error<0)
		{
		DBUG (("Error getting timer status (%lx)\n", error));
		PrintfSysErr (error);
		FULLDBUG (("timerior = %lx\n", timerior));
		}
	}



uint32 endtime (uint32 *time)
	{
	int32 error;
	uint32 usecs;
	
	if (timerior == 0)
		{
		DBUG (("Timer not properly initialized\n"));
		terminate (1);
		}

	ioi.ioi_Recv.iob_Buffer = (void*)(time+2); /* time[2] & time [3] */
	
	error = DoIO (timerior, &ioi);
	if (error<0)
		{
		DBUG (("Error getting timer status (%lx)\n", error));
		PrintfSysErr (error);
		FULLDBUG (("timerior = %lx\n", timerior));
		}
	time[3] -= time[1]+overhead[1];
	time[2] -= time[0];
	/* Calculate the difference */
	/* Note:  time[3] can be negative from the difference calculation.  This
	   is compensated by adding in the seconds difference (time[2]). */
	usecs = (time[2]*1000000)+time[3];
	return(usecs);
	}



void  converthms(uint32 *time, uint32 *hms)
	{
	uint32 hrs, min, sec;
	
	sec = time[2];
	hrs = sec / (60*60);
	hms[0] = hrs;
	sec -= hrs*60*60;
	min = sec / (60);
	hms[1] = min;
	sec -= min * 60;
	hms[2] = sec;
	return;
	}
