/***************************************************************\
*																*
* General routine to load a file 				       			*
*																*
* By:  Stephen H. Landrum										*
*																*
* Last update:  27-Jul-93										*
*																*
* Copyright (c) 1993, The 3DO Company, Inc.                     *
*																*
* This program is proprietary and confidential					*
*																*
\***************************************************************/

/* Formatting: Tabs at every 4th */


#define HAVEECC					/* This may work now */

#define DBUG(x)	{ printf x ; }
#define FULLDBUG(x) /* { printf x ; } */
#define FSDBUG(x) /* printf x */
/* #define FSDBUG(x)  printf x  /* */

/*

#include "filestream.h"
#include "filestreamfunctions.h"

#include "Portfolio.h"
#include "Init3DO.h"
#include "Parse3DO.h"
#include "Utils3DO.h"
#include "BlockFile.h"
#include "UMemory.h"

#include "Debug3DO.h"
*/

#include <device/cdrom.h>
#include <file/fileio.h>
#include <file/filesystem.h>
#include <file/filefunctions.h>
#include <file/directory.h>
#include <file/directoryfunctions.h>
#include <kernel/driver.h>
#include <kernel/device.h>
#include <kernel/types.h>
#include <kernel/nodes.h>
#include <kernel/kernelnodes.h>
#include <kernel/list.h>
/* #include <kernel/folio.h> */
#include <kernel/task.h>
/* #include <kernel/kernel.h> */
#include <kernel/mem.h>
#include <kernel/semaphore.h>
#include <kernel/io.h>


#include <string.h>
#include <stdlib.h>
#include <stdio.h>


/* Local includes */

#include "loadfile.h"


static IOInfo 		ioInfo;
static Item 		IOReqItem;
static IOReq 		*ior;

static	IOInfo 		romioInfo;
static	Item 		romIOReqItem;
static	IOReq 		*romior = NULL;
static	Item 		romfitem;
 
		uint32 		romBlockSize, logRomBlockSize;		/* Silly global!! */

  
int32 OpenRomFile (char *name)
	{
	DeviceStatus devStatus;
	IOInfo devStatInfo;
	
	romfitem = OpenDiskFile (name);
	romIOReqItem = CreateIOReq (0, 0, romfitem, 0);
	romior = (IOReq *)LookupItem (romIOReqItem);
	
	memset (&devStatInfo, 0, sizeof(devStatInfo));
	devStatInfo.ioi_Command = CMD_STATUS;
	devStatInfo.ioi_Recv.iob_Buffer = &devStatus;
	devStatInfo.ioi_Recv.iob_Len = sizeof(devStatus);
	if ((DoIO(romIOReqItem,&devStatInfo) < 0) || (romior->io_Error!=0)) 
		{
		DBUG (("OpenRomFil:  Error in attempt to get device status\n"));
		return -1;
		}
	romBlockSize = (devStatus.ds_DeviceBlockSize);

	logRomBlockSize = 0;
	while ( ((uint32) 1 << logRomBlockSize) < romBlockSize) 
		logRomBlockSize++;
		
	/* Setup the command to read the block */
	romior = (IOReq *)LookupItem(romIOReqItem);
	memset (&romioInfo, 0, sizeof(romioInfo));
	romioInfo.ioi_Command = CMD_READ;
	romioInfo.ioi_Send.iob_Buffer = NULL;
	romioInfo.ioi_Send.iob_Len = 0;
	romioInfo.ioi_Offset = 0;
	return (0);
	}

uint32 OpenDeviceQ (char *name)
	{
	uint32 disksize;
	int32 err;
	IOInfo devStatInfo;
	List *list;
	
	DeviceStatus devStatus;
	TagArg targs[3];
									/* This all changes soon with 'OpenDeviceStack' */
#ifdef ONE_DOT_ONE
	romfitem = OpenDevice (name);	/* Was 'OpenNamedDevice' */
#else
	err = CreateDeviceStackListVA (&list, "cmds", 
				DDF_EQ, DDF_INT, 1, CDROMCMD_READ, NULL);
	if (err < 0)
		{
		DBUG (("Device stack list: Obtaining devices (0x%08X)\n", err));
		PrintfSysErr (err);
		return -1;		/* Need more error */
		}
	if (GetNodeCount (list) != 1)
		{
		DBUG (("Wrong number of nodes obtained (%d)\n", GetNodeCount (list)));
		return -1;
		}
	romfitem = OpenDeviceStack ((DeviceStack *)FirstNode (list));
	/* Remove list stuff */
	DeleteDeviceStackList (list);
#endif	/* 1.1 / 1.2 code */
	if (romfitem < 0)
		return 0;
	
	romIOReqItem = CreateIOReq (0, 0, romfitem, 0);
	romior = (IOReq *)LookupItem (romIOReqItem);

	memset (&devStatInfo, 0, sizeof(devStatInfo));
	devStatInfo.ioi_Command = CMD_STATUS;
	devStatInfo.ioi_Recv.iob_Buffer = &devStatus;
	devStatInfo.ioi_Recv.iob_Len = sizeof(devStatus);
	err = DoIO (romIOReqItem, &devStatInfo);
	if ((err < 0) /* || (romior->io_Error != 0) */ ) 
		{
		DBUG (("OpenRomFil:  Error in attempt to get device status (0x%08X)\n", err));
		PrintfSysErr (err);		/* What happened!!? ? */
		return -1;
		}
	romBlockSize = (devStatus.ds_DeviceBlockSize);

	logRomBlockSize = 0;
	while ( ((uint32) 1 << logRomBlockSize) < romBlockSize) 
		logRomBlockSize++;
			
	romior = (IOReq *)LookupItem(romIOReqItem);

	memset (&romioInfo, 0, sizeof(romioInfo));
	romioInfo.ioi_Command = CMD_STATUS;
	romioInfo.ioi_Recv.iob_Buffer = &devStatus;
	romioInfo.ioi_Recv.iob_Len = sizeof(devStatus);
	romioInfo.ioi_Send.iob_Buffer = NULL;
	romioInfo.ioi_Send.iob_Len = 0;
	romioInfo.ioi_Offset = 0;
	if (((err = DoIO (romIOReqItem, &romioInfo)) < 0) || (romior->io_Error != 0))
		{
		DeleteItem (romIOReqItem);
		romIOReqItem = 0;			/* Remove it! */
		DBUG (("GETDEVICESTATUS:  Error (0x%08X) in attempt to get device status\n", err));
		return 0;
		}
	romBlockSize = (devStatus.ds_DeviceBlockSize);
	disksize = (devStatus.ds_DeviceBlockCount - 1) * romBlockSize;
	logRomBlockSize = 0;
	while ( ((uint32) 1 << logRomBlockSize) < romBlockSize)
		logRomBlockSize++;

	/* Setup the command to read the block */
	memset (&romioInfo, 0, sizeof(romioInfo));
	romioInfo.ioi_Command = CMD_READ;
	romioInfo.ioi_Send.iob_Buffer = NULL;
	romioInfo.ioi_Send.iob_Len = 0;
	romioInfo.ioi_Offset = 0;
	return (disksize);
	}
		
int32 ReadRom(uint32 offset, void *buf, int32 len)
	{
	int32 err;
	uint32 *data;
	int32 retries;
	retries = 0;
	if (romIOReqItem == (Item) 0)
		return -1;
	len = ((len + (( (uint32)1<<logRomBlockSize) - (uint32)1) )  >> logRomBlockSize) << logRomBlockSize;
	totalblocks += len>>logRomBlockSize;
	if (len == 0)
		{
		printf ("We have problems here.  Len == 0, offset = %d\n", offset);
		exit (9);
		}
Retry:
	romioInfo.ioi_Offset = offset>>logRomBlockSize;
	romioInfo.ioi_Recv.iob_Buffer = buf;
    /* We have to round UP length to the nearest integral Block Size */
	romioInfo.ioi_Recv.iob_Len = len;

	if (((err = DoIO(romIOReqItem,&romioInfo)) < 0) || (romior->io_Error!=0))
		{
		retries++;
		rawioerrs++;
		PrintfSysErr(err);
		data = (uint32 *)buf;
		FSDBUG (("ReadRom:  Error in DoIO offset = %ld (%lx)\n",
				offset, offset));
		FSDBUG (("First Data byte returned was %lx \n", *data));
		FSDBUG (("Offset %d blk %d, @buf %x len %d\n",  offset, 
					offset>>logRomBlockSize, buf, len));
		if(retries < 10)
			goto Retry;
		DeleteItem (romIOReqItem);
		romIOReqItem = (Item) 0;
		return -1;
		}
	return (len);
	}
	
	
int32 ReadLong (uint32 offset, void *buf, int32 len)
	{
	static void *old_buffer = NULL;
	static int blockno[2];
	int err;
	CDROMCommandOptions cd_opts;
	static int call_count[2] = 0;
	
	/* Read the whole sector, including the ecc info */
	/* This reads only one sector, no more, no less */
	
	if (old_buffer != buf)
		{
		FSDBUG (("Buffer is now: %p, block number at: %p, call count at: %p\n", 
				buf, blockno, call_count));
		old_buffer = buf;
		}
	
	blockno[1] = blockno[0];				/* Save previous */
	blockno[0] = offset >> logRomBlockSize;
	/* Setup the command, but first see that we have some correctness */
	if (romIOReqItem == (Item) 0)
		return -1;
	if (len != 2340)		/* 4 header, 2048 data, 4 CRC, 8 zero, 276 ECC */
		return -1;

	romioInfo.ioi_Command = CDROMCMD_READ;
	romioInfo.ioi_Offset = offset >> logRomBlockSize;	/* Faked for 2k sectors */
	romioInfo.ioi_Recv.iob_Buffer = buf;
	romioInfo.ioi_Recv.iob_Len = len;
	cd_opts.asLongword = 0;							/* Clear it out?! */
	cd_opts.asFields.densityCode = 0;
	cd_opts.asFields.errorRecovery = CDROM_CIRC_RETRIES_ONLY;
	/* cd_opts.asFields.errorRecovery = CDROM_BEST_ATTEMPT_RECOVERY;	don't do it! */
	cd_opts.asFields.retryShift = 0;				/* I think 0 mucks things up */
	cd_opts.asFields.speed = 0;						/* Default value */
	cd_opts.asFields.pitch = 0;						/* Default value */
	cd_opts.asFields.blockLength = 2340;			/* To get stuff */

	romioInfo.ioi_CmdOptions = cd_opts.asLongword;	/* To get stuff */
	
	call_count[0]++;
	err = DoIO (romIOReqItem, &romioInfo);
	call_count[1]++;
	/* May want to log errors?? */
	if (err < 0)
		rawioerrs++;			/* Increment these.  We may later correct it */
	/* Clear out command options */
	romioInfo.ioi_CmdOptions = 0;					/* Reset these things */
	return err;										/* What we should get */
	}
	
int CorrectIt (void *inbuf, void *outbuf)
	{
#ifdef HAVEECC
	extern uint32 SectorECC (uint8 *);					/* In libecc.a */
	uint32 rval;
	char *in;
	char copybuf[2340];
	/* The input buffer has the raw data (directly from the read operation), and
	   we return by copying back the corrected data */
	/* This does an error correct on the buffer.  Returns -1 if uncorrectable,
	   or zero if it works */
	/* Copy the input into the temp buffer */
	memmove (copybuf, inbuf, sizeof(copybuf));
	FSDBUG (("Before SectorECC\n"));
	rval = SectorECC (copybuf);
	FSDBUG (("After SectorECC\n"));
	/* Now copy the data back out to the 2k sector buffer */
	in = copybuf;
	memmove (outbuf, in + 4, 2048);		/* Skip over the header m:s:f */
	/* Need code to do this, along with the data input/output description */
	return rval;
#else	/* HAVEECC */
	return -1;
#endif	/* HAVEECC */
	}
	
	
	
int32 openmacdevice (void)
	{
	return 0;
	}


int32 getfilesize (char *name)
	{
	Item fitem;
	
	int32 filesize;
	int32 j;
	int32 tries=0;
	FileStatus statbuffer;

	fitem = OpenDiskFile (name);
	IOReqItem = CreateIOReq (0, 0, fitem, 0);
	
	if (IOReqItem < 0)
		{
		DBUG (("GETFILESIZE:  Unable to create IOReq node (%ld)\n", IOReqItem));
		return -1;
		}

	ior = (IOReq *)LookupItem(IOReqItem);
	

tryAgain:
	memset (&ioInfo, 0, sizeof(ioInfo));
	
	ioInfo.ioi_Command = CMD_STATUS;
	ioInfo.ioi_Recv.iob_Buffer = &statbuffer;
	ioInfo.ioi_Recv.iob_Len = sizeof(statbuffer);
	ioInfo.ioi_Send.iob_Buffer = NULL;
	ioInfo.ioi_Send.iob_Len = 0;
	ioInfo.ioi_Offset = 0;
	if (((j=DoIO(IOReqItem,&ioInfo)) < 0) || (ior->io_Error!=0))
		{
		DeleteItem (IOReqItem);
		DBUG (("GETFILESIZE:  Error (0x%08X) in attempt to get file status\n", j));
		return -1;
		}

	filesize = statbuffer.fs_ByteCount;
	if (filesize == 0)   /* Operator Bug !!   try again  */
		{
		printf( "Filesize is Zero! ior->io_Error is %ld \n", ior->io_Error);
		if(tries++ < 10) goto tryAgain;
		}

	DeleteItem (IOReqItem);
	FSDBUG (("fitem = %lx\n", fitem));
	CloseDiskFile (fitem);

	return filesize;
	}



void *loadfile (char *name, void *buffer, uint32 buffersize, uint32 memtype)
	{
	int32 filesize;
	RawFile *fstream;
	int err;
	
	filesize = getfilesize (name);
	
	if (filesize<0)
		{
		return NULL;
		}
	
	if (filesize==0)
		{
		DBUG (("Empty file: %s\n", name));
		return NULL;
		}
	
	if (buffer != NULL && filesize > buffersize)
		{
		DBUG (("File size exceeds buffer size (%ld > %ld): %s\n",
				filesize, buffersize, name));
		/* return NULL; */
		filesize = buffersize;
		}
	if (buffer == NULL)
		{
		if (memtype)
			{
			buffer = AllocMem ((int)(filesize), memtype);
			}
		else
			{
			buffer = malloc((int)(filesize));
			}
		if (!buffer)
			{
			DBUG (("Unable to allocate memory for file (%ld): %s\n",
					filesize, name));
			return NULL;
			}
		}

/* Converted this to raw file things */
	err = OpenRawFile (&fstream, name, FILEOPEN_READ);
	if (err)
		{
		DBUG (("Error opening file %s\n", name));
		return NULL;
		}
		
	if (ReadRawFile(fstream,(char *)buffer,filesize) != filesize)
		{
		DBUG (("Error reading file %s\n", name));
		CloseRawFile (fstream);
		return NULL;
		}

	CloseRawFile (fstream);
	return buffer;
	}


