/***************************************************************\
*																*
* File MiniUI.c													*
*																*
* By:	David S. Maynard										*
*		Tom Watson												*
*																*
* Last update:  28-Jan-94										*
* Modifications for M2 system Oct 4, 1995.						*
*																*
* Copyright (c) 1992, 1993, The 3DO Company                     *
*				1994, 1995, 1996.								*
*																*
* This program is proprietary and confidential					*
*																*
* Edits and updates for M2 system Tom Watson Sep 12,'95			*
*																*
\***************************************************************/


/* Added Oct 17,'95:	Control Left/Right calls procedure as well (tsw) */
/* Added Oct 20,'95:	The selectability of an menu item is dynamic, this
						allows the enabling/disabling of selectable items
						on the fly.  It also shortens the selector structure
						a bit, and an allocation it used to point to.  */
/* Added Oct 20,'95:	The menu choice items can execute a procedure if it
						is specified. */
/* Added Oct 23,'95:	Made the buffer used for print... big enough for the
						whole screen.  It was 132 before, which is too small */
/* New Mar 07,'96:		Modifications for the new font folio.  Basically some
						changes from DrawTextString to DrawString */
/* New Mar 14,'96:		Made menu choice text based at 'low' value */
/* New May 08,'96:		Negative field widths display variables in hex 
						with (leading 0's) */

/* 
How to call:
1)  Setup menu array (statically/dynamically)
2)	Have a pointer to a 'Selector' element
3)  Call DOUI with menu array and pointer to 'Selector' holder (handle)
4)  It returns.  The 'joyold' variable will indicate what caused the routine
	to exit (ControlStart or ControlX)
	
Generally the button do:
ControlUp		Previous menu item
ControlDown		Next menu item
ControlLeft		Increment/Toggle/Procedure
ControlRight	Decrement/Toggle/Procedure
ControlA		Toggle/Exit/Procedure
ControlB		.. ..
ControlC		.. ..
ControlStart	return from routine
ControlX		return from routine


Types of variables in the menu array...
1)  true/false (prints text, then YES/NO)
2)  multiple choice (like a variable, but prints with index to text array)
3)  variable (prints text, then variable)
4)  print[one|two|three] uses printf style things to print 1, 2, 3 things.
5)  text - just print text
6)  exit - leave the routine - print text
7)  procedure - execute procedure - print text
For the definition of the menu array see miniui.h
*/

/* Formatted with tabs every 4 spaces */

#define FULLDBUG(x) { printf x ; }
/* #define DBUG(x) {printf x;}	/* */
#define DBUG(x)					/* */
#define CKERR(x,y) {int32 err=(x);if(err<0){printf("Error - %s\n",y);PrintfSysErr(x); exit(1);}}


/* M2 Headers */

#include <kernel/types.h>

#include <kernel/nodes.h>
#include <kernel/kernelnodes.h>
#include <kernel/list.h>
/* #include <kernel/folio.h> */
#include <kernel/task.h>
#include <kernel/types.h>
/* #include <kernel/kernel.h> */
#include <kernel/mem.h>
#include <kernel/semaphore.h>
#include <kernel/io.h>
#include <graphics/clt/gstate.h>
#include <graphics/view.h>
#include <graphics/font.h>
#include <misc/event.h>

/* ANSI Headers */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>


/* Definitions, here and abroad */

#include "miniui.h"


#define TRUESTR			"YES"
#define FALSESTR		"NO "
#define DEFAULT_FONT	"default_menu"	/* This ought to work */
#define FBWIDTH			320
#define FBHEIGHT		240
#define DEPTH			16
#define viewType		VIEWTYPE_16





/* Some or all of the externs should be moved into this file */

static ControlPadEventData cped;
static Item vblioreq;

static GState	*gs = NULL;		/* Graphics state pointer (generally) */
static Item		viewItem;

/* Prototypes, this file */
static void do_display_swap (void);
static void init_display_data (void);
static void text_line(int32 x, int32 y, char *text, uint32 fg, uint32 bg);
static void DrawButton(int32 x, int32 y, char *text, int32 selected);
static void PaintItem(ustring *u, Boolean selected);
static void DrawUI(ustring menu[], int32 nitems, int32 sel);
static void selectPrev (ustring menu[], Selector *select);
static void selectNext (ustring menu[], Selector *select);
static void UIIncVar(ustring menu[], int32 is, int32 val);
static void UIDecVar(ustring menu[], int32 is, int32 val);
static void UIToggleVar(ustring menu[], int32 is);




static void do_display_swap (void)
{
	if (gs == NULL)
		{
		printf ("Incorrect init of graphic state!!\n");
		exit (1);
		}
	GS_SendList(gs);
	GS_WaitIO(gs);
	ModifyGraphicsItemVA(viewItem,
			   VIEWTAG_BITMAP, GS_GetDestBuffer(gs),
			   TAG_END );
	CLT_ClearFrameBuffer (gs, .0, .0, .3, 0., TRUE, TRUE);	
						 /* clear the new frame buffer */
						 /*    R   G   B  	*/
	CLT_SetSrcToCurrentDest (gs);
	
}



static void init_display_data (void)
{
/*
 * Initialize colors and GP state
 */

static int buffSignal;
int			err;
Item		bitmaps[3];

if (gs != NULL)
	return;						/* Already here */
/* Initialization */
err = OpenGraphicsFolio ();
if (err < 0)
	{
	printf ("Open of graphics folio failed (%d)\n", err);
	PrintfSysErr (err);
	exit (1);
	}

err = OpenFontFolio ();
if (err < 0)
	{
	printf ("Open of font folio failed (%d)\n", err);
	PrintfSysErr (err);
	exit (1);
	}

err = InitEventUtility (1, 0, LC_ISFOCUSED);
if (err < 0)
	{
	printf ("Open of event utility failed (%d)\n", err);
	PrintfSysErr (err);
	exit (1);
	}


buffSignal = AllocSignal (0);
gs = GS_Create ();
if (gs == NULL) 
	{
	printf ("Error creating GState\n");
	exit (1);
	}
vblioreq = CreateTimerIOReq();	/* So we can do timer things */

GS_AllocLists(gs, 2, 2048);				/* Larger buffer */
GS_AllocBitmaps(bitmaps, FBWIDTH, FBHEIGHT, viewType, 2, 1);
GS_SetDestBuffer (gs, bitmaps[0]);
GS_SetZBuffer (gs, bitmaps[2]);
viewItem = CreateItemVA(MKNODEID(NST_GRAPHICS, GFX_VIEW_NODE),
			  VIEWTAG_VIEWTYPE, viewType,
			  VIEWTAG_DISPLAYSIGNAL, buffSignal,
			  VIEWTAG_BITMAP, bitmaps[0],
			  TAG_END );
CKERR(viewItem,"can't create view item");
AddViewToViewList( viewItem, 0 );

/* Initially clear the frame buffer to black and make it visible */
CLT_ClearFrameBuffer (gs, 0., 0., 0., 0., TRUE, TRUE);
}


static void DrawButton(int32 x, int32 y, char *text, int32 selected)
	{
	uint32 fg;			/* Foreground color */
	uint32 bg;			/* Background color */
	bg = 0x00000000;	/* Transparent */
	fg = 0x00ffffff;	/* White */
	
	if (selected)			/* Reverse as needed */
		{
		bg = 0x00ffffff;	/* White background */
		fg = 0x00000000;	/* Black letters */
		}
	text_line (x, y, text, fg, bg);
	}
	
	
static void text_line(int32 x, int32 y, char *the_text, uint32 fg, uint32 bg)
	{
	PenInfo pen;
	static font = 0;			/* Fill in later */
	char *nl_loc;				/* Location of the new line when drawing! */

	memset (&pen, 0, sizeof(pen));
	if (font == 0)
		{
		if (gs == NULL)			/* Need in case called first!!! */
			init_display_data();
		if ((font = OpenFont (DEFAULT_FONT)) < 0)
			{
			printf ("Couldn't open font ('%s'): err = %d\n", DEFAULT_FONT, font);
			font = 0;			/* Might try again */
			return;				/* Won't work!! */
			}
		}
	
	pen.pen_X = x;				/* Int32 to gfloat conversion */
	pen.pen_Y = y + 10;			/* Setup the coordinates */
				/* The '+ 10' part is because we formerly refered to the TOP not
				   the baseline of the text string.  I may adjust this!! */
	pen.pen_BgColor = bg;
	pen.pen_FgColor = fg;
	pen.pen_XScale = 1.0;		/* Added with the new font folio!! */
	pen.pen_YScale = 1.0;
	/* This used to just strings things end-to-end, not one under another! */
	/* We now leave the string intact!! */
	while ((nl_loc = strchr (the_text, '\n')) != NULL)
		{
		char partial_text[100];		/* Original text could be read only! */
									/* Now copy up to and including the '\n' */
		strncpy (partial_text, the_text, nl_loc - the_text + 1);
									/* Add terminating null character.  */
		partial_text [nl_loc - the_text + 1] = '\0';
		DrawString (gs, font, &pen, partial_text, strlen (partial_text));
		the_text = nl_loc + 1;
		}
	/* Do remainder! */
	DrawString (gs, font, &pen, the_text, strlen (the_text));
	}
	

/* These routines are for the benefit of others that want to put text on the screen.
   Please use them.  The color argument is a 24 bit color number (alpha and R G B) */

void PaintText (char *s, int32 x, int32 y, uint32 color)
	{
	text_line (x, y, s, color, 0x00000000);
	}
	

void PaintLine (char *s, int32 line,  uint32 color)
	{
	PaintText (s, LEFTMARGIN, line * LINEHEIGHT, color);
	}


static void PaintItem(ustring *u, Boolean selected)
	{
	char uiwork[17*41];		/* Local to here big enough for whole screen! */
	if (u->text == NULL)
		u->text = "";		/* Empty string */
	switch (u->type)
		{
	case menuproc:
	case menuexit: 
	case text:
		/* Direct selection */
		DrawButton (u->x, u->y, u->text, selected);
		break;
		
	case indrtext:
		/* Indirect selection */
		DrawButton (u->x, u->y, *u->mtext, selected);
		break;
		
		
	/* The print[one|two|three] are here so that the variable pointers need not
	   be evaluated unless it is necessary */
	case printone: 
		sprintf (uiwork, u->text, *u->varptr);
		DrawButton (u->x, u->y, uiwork, selected);
		break;
	case printtwo: 
		sprintf (uiwork, u->text, *u->varptr, *u->varmin);
		DrawButton (u->x, u->y, uiwork, selected);
		break;
	case printthree: 
		sprintf (uiwork, u->text, *u->varptr, *u->varmin, *u->varmax);
		DrawButton (u->x, u->y, uiwork, selected);
		break;
	case truefalse:
		sprintf (uiwork, "%s%s", u->text, *u->varptr ? TRUESTR : FALSESTR);
		DrawButton (u->x, u->y, uiwork, selected);
		break;
	case variable:
		/* Use width if less than 10.  This was a switch statement */
		/* Do it in hex if number negative!! */
		if (u->q.varwidth < 0)
			{
			sprintf (uiwork, "%s%0*X", u->text, 
					u->q.varwidth < -10 ? 8 : -u->q.varwidth, *u->varptr);
			}
		else
			{
			sprintf (uiwork, "%s%*d", u->text, 
					u->q.varwidth > 10 ? 8 : u->q.varwidth, *u->varptr);
			}
		DrawButton (u->x, u->y, uiwork, selected);
		break;
	case mchoice:			/* Is alterable */
	case mchoicec:			/* Is alterable */
	case indextext:			/* Is NOT alterable */
					/* The text is offset by the minimum value (if there) */
		DrawButton (u->x, u->y,
		            u->mtext[*u->varptr - (u->varmin ? *u->varmin : 0)], 
					selected);
		break;
		}
	}




void DrawScreen (void)
	{
	if (gs == NULL)
		init_display_data();
	do_display_swap();
	}

static void DrawUI(ustring menu[], int32 nitems, int32 sel)
	{
	int32 i;
	for (i = 0; i < nitems; i++)
		{
		PaintItem (&menu[i], i == sel);		/* Second arg is '1' when selected */
		}
	DrawScreen ();
	}



/* See if the menu item is of the selectable type */
static int is_selectable (ustring menu[], int32 is)
	{
	if (is < 0)
		return 0;								/* If index goes south */
	if ((menu[is].type == truefalse) || 		/* If we can select it!! */
		(menu[is].type == variable) || 
		(menu[is].type == mchoice) || 
		(menu[is].type == mchoicec) || 
		(menu[is].type == menuexit) || 
		(menu[is].type == menuproc)) 
		return 1;								/* return TRUE, selectable */
	return 0;									/* Not selectable */
	}

/* Alter selection to previous */
static void selectPrev (ustring menu[], Selector *select)
	{
	int loop = 0;								/* Indicate no loop yet */
	while (!is_selectable (menu, --select->item))
		{
		if (select->item < 0)
			{
			select->item = select->nitems;		/* Point to last */
			if (loop)
				{
				/* We looped thru the thing once!! */
				return;							/* pointer to 'last' */
												/* Nothing is selectable */
				}
			loop++;								/* Indicate we have cycled */
			}
		/* Cycle back to select another one back a ways */
		}
	return;
	}

/* Alter selection to next */
static void selectNext (ustring menu[], Selector *select)
	{
	int loop = 0;								/* No loop yet */
	while (!is_selectable (menu, ++select->item))
		{
		if (select->item >= select->nitems)
			{
			if (loop)
				{
				/* We've been around here, but didn't find anything! */
				select->item = select->nitems;	/* The 'last' item */
				return;
				}
			select->item = -1;					/* Incremented to zero next time */
			loop++;								/* Indicate we've been around! */
			}
		}
	return;
	}

/* Increment menu value */
static void UIIncVar(ustring menu[], int32 is, int32 val)
	{
	if (menu[is].type == variable || menu[is].type == mchoice) 
		{
		if (*menu[is].varptr < *menu[is].varmax) 
			*menu[is].varptr += val;
		if (*menu[is].varptr > *menu[is].varmax) 
			*menu[is].varptr = *menu[is].varmax;
		}
	if (menu[is].type == mchoicec)
		{
		*menu[is].varptr += val;
		if (*menu[is].varptr > *menu[is].varmax)
			*menu[is].varptr = *menu[is].varmin;
		if (*menu[is].varptr < *menu[is].varmin)
			*menu[is].varptr = *menu[is].varmax;
		}
	}

/* Decrement menu value */
static void UIDecVar(ustring menu[], int32 is, int32 val)
	{
	if (menu[is].type == variable || menu[is].type == mchoice) 
		{
		if (*menu[is].varptr > *menu[is].varmin) 
			*menu[is].varptr -= val;
		if (*menu[is].varptr < *menu[is].varmin) 
			*menu[is].varptr = *menu[is].varmin;
		}
	if (menu[is].type == mchoicec)	/* Circular */
		{
		*menu[is].varptr -= val;
		if (*menu[is].varptr > *menu[is].varmax)
			*menu[is].varptr = *menu[is].varmin;
		if (*menu[is].varptr < *menu[is].varmin)
			*menu[is].varptr = *menu[is].varmax;
		}
	}

/* Toggle menu variable */
static void UIToggleVar(ustring menu[], int32 is)
	{
	if (menu[is].type == truefalse)
		*menu[is].varptr = *menu[is].varptr ? false : true;
	}



/* still to do.  Eliminate gloabals to enable recursive menu processing */
/* Most of the global elimination is done.  I'm keeping at it */
void DOUI (ustring menu[], Selector **select_handle)
	{
	int32 i;
	int32 joycur;
	int32 incval;
	callProc vfp;
	Selector *select;


	if (*select_handle == NULL)
		{
		init_display_data();			/* Start things out */
		/* First time thru, build up structure */
		*select_handle = malloc (sizeof (*select));
		(*select_handle)->item = -1;
		(*select_handle)->nitems = -1;
		}
	
	select = *select_handle;			/* Resolve handle goodie */

	incval = 1;
	if (select->nitems == -1)
		{
		/* Number of items haven't been counted... */
		while (menu[++select->nitems].type != last)
			/* That's it */ ;
		}

	/* Find the first one to select */		
	if (select->item == -1) 
		{
		selectNext (menu, select);
		}
	DrawScreen();
	DrawUI(menu,  select->nitems, select->item);
	GetControlPad (1, FALSE, &cped);		/* Initial state right now */
	select->joyold = cped.cped_ButtonBits;
	while (true)
		{
		GetControlPad (1, TRUE, &cped);		/* Wait for something!! */
		select->joynew = cped.cped_ButtonBits;
		select->joyedge = select->joynew & ~select->joyold;
		if (select->joyedge != 0)
			{
			DrawUI(menu,  select->nitems, select->item);
			if (select->joynew & ControlUp) 
				{
				selectPrev(menu, select);
				}
			if (select->joynew & ControlDown) 
				{
				selectNext(menu, select);
				}
			if (select->joynew & ControlLeft) 
				{
				/* True/false: toggle
				   Procedure:  execute
				   Variable:  Step value - accelerate
				   Choice:  Step value only, bounding
				   CirChoice:  Step value, around the horn.
				*/
				if (menu[select->item].type == truefalse)
					UIToggleVar(menu, select->item);
				if (menu[select->item].type == menuproc)
					{
					vfp = menu[select->item].q.varproc;
					/* Wait for all the buttons to come up, the execute! */
					while ((select->joynew = cped.cped_ButtonBits) != 0) 
						GetControlPad (1, TRUE, &cped);
					vfp(menu, select);
					}
				if (menu[select->item].type == variable || 
					menu[select->item].type == mchoicec ||
					menu[select->item].type == mchoice)
					{
					do 
						{
						UIDecVar(menu, select->item, incval);
						if (menu[select->item].type != variable)
							break;
						DrawUI(menu,  select->nitems, select->item);
						WaitTimeVBL(vblioreq, (incval == 1) ? 15: 5);
						GetControlPad (1, FALSE, &cped);
						joycur = cped.cped_ButtonBits;
						if (incval < 10) 
							incval++;
						} 
						while (joycur&ControlLeft);
						
					}
				}
			if (select->joynew & ControlRight) 
				{
				/* Functions as in 'Left' */
				if (menu[select->item].type == truefalse)
					UIToggleVar(menu, select->item);
				if (menu[select->item].type == menuproc)
					{
					vfp = menu[select->item].q.varproc;
					/* Wait for all the buttons to come up, the execute! */
					while ((select->joynew = cped.cped_ButtonBits) != 0) 
						GetControlPad (1, TRUE, &cped);
					vfp(menu, select);
					}
				if (menu[select->item].type == variable || 
					menu[select->item].type == mchoicec ||
					menu[select->item].type == mchoice)
					{
					do 
						{
						UIIncVar(menu, select->item, incval);
						if (menu[select->item].type != variable)
							break;
						DrawUI(menu,  select->nitems, select->item);
						WaitTimeVBL(vblioreq, (incval == 1) ? 15 : 5);
						GetControlPad (1, FALSE, &cped);
						joycur = cped.cped_ButtonBits;
						if (incval < 10) 
							incval++;
						} 
						while (joycur&ControlRight);
						
					}
				}
			if (select->joynew & (ControlA | ControlB | ControlC))
				{
				/* True/False: Toggle
				   Exit:  Terminate the menu
				   Procedure:  execute.
				*/
				if (menu[select->item].type == truefalse)
					UIToggleVar(menu,select->item);
				if (menu[select->item].type == menuexit)
					break;
				if (menu[select->item].type == menuproc ||
					menu[select->item].type == mchoice ||
					menu[select->item].type == mchoicec)
					{
					vfp = menu[select->item].q.varproc;
					if (vfp != NULL)	/* If we have a proc!! */
						{
						/* Wait for all the buttons to come up, the execute! */
						while ((select->joynew = cped.cped_ButtonBits) != 0) 
							GetControlPad (1, TRUE, &cped);
						vfp(menu, select);
						}
					}
				}
			if (select->joynew & ControlStart) 
				{
				break;
				}
			if (select->joynew & ControlX) 
				{
				break;
				}
			/* wait for button up */
			if (incval == 1)
				DrawUI(menu,  select->nitems, select->item);
			incval = 1;
			while ((select->joynew = cped.cped_ButtonBits) != 0) 
				GetControlPad (1, TRUE, &cped);
			}
		select->joyold = select->joynew;
		}
	}
	

