/* @(#) mytime.c 95/09/18 1.1 */

#include <kernel/types.h>
#include <kernel/io.h>
#include <kernel/device.h>
#include <kernel/time.h>
#include <kernel/debug.h>

#include <stdlib.h>
#include <string.h>			/* Has memset in it */
#include <stdio.h>


#define CHK4ERR(err, str)	\
		if ((err) < 0) { printf (str); PrintfSysErr(err); exit(9); }

#define DBUG(x)	{ printf x ; }


uint32 MyTime(void)
{

	/* I added a bias to counter the fact that after 4000+ seconds, this counter
	   will overflow and start over.  This can happen if the system is booted and
	   is left running for a little over 1 hour (getting out calculator: 1.19 hours).
	   The fix is to bias everything from the first call to this routine, and take the
	   difference from there.  I don't do it with the microsecond portion, as that
	   would be quite a bit of work.  Really what should be done is return a 'double'
	   value which has more accuracy (about 52 bits, or 1 million times what we have
	   here).  Tsw 9-28-95 */
	/* I changed this to keep the timer item open.  My assumption is that it should
	   be quicker to run thru the routine indtead of open/close each time we go thru
	   here.  */
	/* Added mods for 1.2 release of the system (1/29/96 tsw) */
	   
	static Item	devItem = 0;
	Item	IOReqItem;
	IOInfo	ioInfo;
	int32	ret;
	IOReq	*ior;
	uint32	mytime;
	struct timeval tv;
	static	uint32 sec_bias = 0;	/* Seconds bias in case we loop around on this */
	List *list;
	int32 err;
	
	/* try to open device.  We keep this open for the duration. */
	if (devItem == 0)
		{
#ifdef ONE_DOT_ONE
		devItem = OpenDevice("timer");
#else
		err = CreateDeviceStackListVA (&list, "cmds", 
					DDF_EQ, DDF_INT, 1, TIMERCMD_GETTIME_USEC, NULL);
		if (err < 0)
			{
			DBUG (("Device stack list: Obtaining devices (0x%08X)\n", err));
			PrintfSysErr (err);
			return -1;		/* Need more error */
			}
		if (GetNodeCount (list) != 1)
			{
			DBUG (("Wrong number of nodes obtained (%d)\n", GetNodeCount (list)));
			return -1;
			}
		devItem = OpenDeviceStack ((DeviceStack *)FirstNode (list));
		/* Remove list stuff */
		DeleteDeviceStackList (list);
#endif	/* 1.1 / 1.2 code */
		CHK4ERR(devItem, "Failed to open device\n");
		}

	IOReqItem = CreateIOReq(0, 0, devItem, 0);
	CHK4ERR(IOReqItem, "Could not create ioReq\n");

	/* not used	
	ior = (IOReq *)LookupItem(IOReqItem);
	*/
	
	memset(&ioInfo, 0, sizeof(ioInfo));
	
	ioInfo.ioi_Command = TIMERCMD_GETTIME_USEC;
	ioInfo.ioi_Recv.iob_Buffer = &tv;
	ioInfo.ioi_Recv.iob_Len = sizeof(tv);

	ret = DoIO(IOReqItem, &ioInfo);
	CHK4ERR(ret, "Error in DoIO()\n");
	if (sec_bias == 0)
		{
		/* Bias is not setup */
		sec_bias = tv.tv_sec;		/* The first time, we bias back to zero */
		}

	tv.tv_sec -= sec_bias;

	mytime = (tv.tv_sec * 1000000) + tv.tv_usec;

	DeleteIOReq(IOReqItem);

	return (mytime);
}
