/* @(#) testcard.c May 20, 1996 */
/* Updated for a user interface on the storage card April 29, 1996 tsw.  */
/* Progress towards the goal, UI functional, adding functionality May 8, 1996 tsw.  */

#define DEBUG /* */

#include <kernel/device.h>
#include <kernel/operror.h>
#include <kernel/io.h>
#include <misc/event.h>
#include <misc/script.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "miniui.h"				/* THE small user interface */
#include "dump_record.h"		/* User interface to display records, etc.  */



#ifdef DEBUG
#define DBUG(x) printf x
#else
#define DBUG(x)
#endif

#define BUFSIZE	2048		/* Size of the buffer */
#define BIG_BLOCK (128*600)	/* Most the whole card!!  Over 64k bytes!! */

/* Globals for us to use */
		Item stcrdItem;		/* Storage card item */
static	Item iorItem;		/* I/O item */
static	DeviceStatus ds;
static	int32 max_block_num;
		int32 block_num = 0;
static	int32 zero = 0;
static	int32 one = 1;
static	int32 two = 2;
static	int32 two55 = 0xFF;
		char *text_line = "No Data Presently";
static	int32 io_type = 0;
static	int32 clear_type = 0;
static	int32 random_type = 0;
static	int32 mount_type = 0;
static	int32 random_seed = 0x01;
static	int32 clear_value = 0;
static	int32 pass_count = 1;		/* Number of times we go thru comprehensive test */
		Err wait_error = 0;
		Err send_error = 0;
		int bytes_returned = 0;
static	char *io_type_text[] = {"Read", "Read/Write", "Display"};
static	char *clear_type_text[] = {"Clear I/O block", "Clear entire card"};
static	char *random_type_text[] = {"Random fill block", "Random fill card"};
static	char *mount_type_text[] = {"Dismount storage card", "Mount storage card"};


/* Prototypes */
static void random_fill (uint8 *buf, int buffer_length);
void do_status (ustring *, Selector *);
void do_io (ustring *, Selector *);
void do_zero (ustring *, Selector *);
void do_random (ustring *, Selector *);
void do_comp_test (ustring *, Selector *);
void do_card_format (ustring *, Selector *);
void do_mounts (ustring *, Selector *);
static void exec_write (int block, int type);
static Err w_block (void *buffer, int block, int nblocks);
static Err r_block (void *buffer, int block, int nblocks);
static void complement (uint8 *reversed, int size);




/* Now declare some user interface structures to work with */
ustring main_menu[] =					/* Parts stolen from the CD-Rom tests */
{
PRINT2 (20, 28, stcrdItem, iorItem, "Storage Card test (I's: %d, %d)"),

MPROC (20, 40, do_status, "Obtain status word"),
VARIABLE (20, 52, block_num, zero, max_block_num, 5, "I/O Block number: "),
/* MTEXT (200, 52, "<--Block"), */
MCHOICECX (20, 64, io_type, zero, two, io_type_text, do_io),
MTEXT (120, 64, "A = Execute I/O routine"),
MCHOICECX (20, 76, clear_type, zero, one, clear_type_text, do_zero),
VARIABLE (150, 76, clear_value, zero, two55, -2 /* Hex */, "Clear value: 0x"),
MCHOICECX (20, 88, random_type, zero, one, random_type_text, do_random),
VARIABLE (150, 88, random_seed, one, two55, -2 /* Hex */, "Random seed: 0x"),
MPROC (20, 100, do_comp_test, "Comprehensive test(s)"),
VARIABLE (174, 100, pass_count, one, two55, 4, "Pass Count: "),
MPROC (20, 112, do_card_format, "Format for a file system"),
MCHOICECX (20, 124, mount_type, zero, one, mount_type_text, do_mounts),

PRINT3 (20, 136, send_error, wait_error, bytes_returned, 
			"ESend=%X, EWait=%X, bytes=%d"),
	
PRINT1 (20, 148, text_line, "%s"),

MLAST
};




/* Missing prototype */
Item OpenNamedDeviceStack(const char *stackName);


static char* prog;
static bool verify;
static uint8 pattern = 0xff;
static char* devName = "storagecard";
static bool random;
static uint8 *buffer = NULL;
static uint8 *ver_block = NULL;	/* Where we verify things */





static void random_fill (uint8 *buf, int buffer_length)
	{
	int i;
	static r_start = 0x01;			/* Where we start in the sequence */
	
	if (buf == NULL)
		{
		r_start = buffer_length & 0xFF;
		if (r_start == 0)
			r_start = 0x01;			/* If we get to zero, it latches!! */
		return;						/* Just init the thing */
		}
	for (i = 0; i < buffer_length; i++)
		{
		buf[i] = r_start;
		r_start <<= 1;
		if (r_start & 0x100)
			r_start ^= 0x11d;		/* Makes 255 sequence of bytes */
		}
	}

int main(int argc, char *argv[])
	{
    int i;
	Selector *the_selector = NULL;			/* For the user interface goodies */
	prog = argv[0];							/* Sets up the program name */


    DBUG(("Opening...\n"));
    stcrdItem = OpenNamedDeviceStack (devName);
    if(stcrdItem < 0)
		{
		printf ("%s: cannot open the %s device\n", prog, devName);
		PrintfSysErr (stcrdItem);
		return stcrdItem /* error */;
		}

    DBUG (("Creating I/O...\n"));
    iorItem = CreateIOReq (NULL, 0, stcrdItem, 0);
    if(iorItem < 0)
		{
		printf ("%s: cannot create %s IOReq\n", prog, devName);
		PrintfSysErr (iorItem);
		CloseDeviceStack (stcrdItem);
		return iorItem /* error */;
		}

	/* Just get the status information and go for it... */
	do_status (NULL, NULL);				/* We don't use the arguments anyway */
	
	

/* ##### Now possibly vector into the menu routine */

	DBUG (("Entering the menu operation\n"));
	
	do
		{
		DOUI (main_menu, &the_selector);	/* Goes into the menu, and does things */
		}
		while ((the_selector->joynew & ControlX) == 0);	/* Wait for stop! */

    DeleteItem (iorItem);
    CloseDeviceStack (stcrdItem);

    DBUG (("Done with the storage card tests\n"));

    return 0;
	}





/* Now, on seperate pages, we do all the 'particulars' of things */

void do_status (ustring *the_menu, Selector *select)
	{
    IOInfo ioi;
	Err		err;
	static char return_text[100];

	return_text[0] = '\0';				/* Initial conditions */
	text_line = return_text;			/* point to the return things */
	
	DBUG (("Getting status\n"));
	
    memset (&ioi, 0, sizeof(ioi));
    ioi.ioi_Command = CMD_STATUS;
    ioi.ioi_Recv.iob_Buffer = &ds;
    ioi.ioi_Recv.iob_Len = sizeof(ds);
	
    err = DoIO (iorItem, &ioi);
	send_error = err;
	bytes_returned = ioi.ioi_Recv.iob_Len;
    if (err < 0)
		{
		sprintf (return_text + strlen(return_text),
				"%s: error getting %s status\n", prog, devName);
		GetSysErr (return_text + strlen(return_text), 
					sizeof(return_text) - strlen(return_text) - 1,
					err);
		return;
		}

	/* Print out status here?? */
	sprintf (return_text + strlen(return_text),
		"Block start = %d\n", ds.ds_DeviceBlockStart);
	sprintf (return_text + strlen(return_text),
		"Block count = %d\n", ds.ds_DeviceBlockCount);
	max_block_num = ds.ds_DeviceBlockCount - 1;
	sprintf (return_text + strlen(return_text),
		"Block size  = %d\n", ds.ds_DeviceBlockSize);
		
	/* Allocate for some room, at least 'BUFSIZE' large */
	ver_block = realloc (ver_block,
			ds.ds_DeviceBlockSize > BUFSIZE ? ds.ds_DeviceBlockSize : BUFSIZE);
	buffer = realloc (buffer,
			ds.ds_DeviceBlockSize > BUFSIZE ? ds.ds_DeviceBlockSize : BUFSIZE);
	/* We don't check status on these very well, and let it pass a bit... */
	if (ver_block || buffer)
		{
		}
	else
		{
		strcat (return_text, "Bad allocation of buffers!");
		}
	}
	
	

static void complement (uint8 *reversed, int size)
	{
	/* Complements a buffer (first arg) of length (second arg) */
	while (size-- > 0)
		{
		*reversed++ ^= 0xff;
		}
	}
	
void do_io (ustring *the_menu, Selector *select)
	{
	Err		err[8];			/* Subscripts 0-7, 0 not used */
	int		block_size = ds.ds_DeviceBlockSize;
	int		i;				/* Simple counter */
	static	char return_text[150];
	
	int		incr = !!(select->joyedge & ControlB);	/* Set if we want to increment */
	
	switch (*the_menu[select->item].varptr)
		{
		case 0:
		/* Read entire storage card, stopping on error */
		/* Read, then read again, then compare */
		err[1] = r_block (buffer, block_num, 1);		/* Initial read */
		err[2] = r_block (ver_block, block_num, 1);	/* Verify read */
		err[3] = 0;
		for (i = 0; i < block_size; i++)
			{
			if (buffer[i] != ver_block[i])
				{
				err[3]++;
				}
			}
		/* Need count of bad comparisons, so we use above loop! (not memcmp). */
		sprintf (return_text, "Error(s): 0x%08X, 0x%08X.\nBytes miscompared: %d.\n",
				err[1], err[2], err[3]);
		sprintf (strchr (return_text, '\0'), "Read test done, block %d.", block_num);
		text_line = return_text;
		if (incr)
			{
			block_num++;
			if (block_num >= ds.ds_DeviceBlockCount)
				block_num = 0;			/* Back to the beginning */
			}
		break;
		
		case 1:
		/* Read block, write same block, verify block, stopping on error */
		/* Read block, complement, write block, read block, compare, complement, write,
		   read, compare.  This restores the block to previous state if sucessful. */
		err[1] = r_block (buffer, block_num, 1);
		complement (buffer, block_size);
		err[2] = w_block (buffer, block_num, 1);
		err[3] = r_block (ver_block, block_num, 1);
		err[4] = 0;								/* Miscompare count */
		for (i = 0; i < block_size; i++)
			{
			if (buffer[i] != ver_block[i])
				{
				err[4]++;
				}
			}
		complement (buffer, block_size);
		err[5] = w_block (buffer, block_num, 1);
		err[6] = r_block (ver_block, block_num, 1);
		err[7] = 0;								/* Second miscompare count */
		for (i = 0; i < block_size; i++)
			{
			if (buffer[i] != ver_block[i])
				{
				err[7]++;
				}
			}
		
		sprintf (return_text, "RErr: 0x%08X, 0x%08X, 0x%08X.\n", err[1], err[2], err[3]);
		sprintf (strchr (return_text, '\0'), "Miscompare(s): %d\n", err[4]);
		sprintf (strchr (return_text, '\0'), "WErr: 0x%08X, 0x%08X.\n", err[5], err[6]);
		sprintf (strchr (return_text, '\0'), "Miscompare(s): %d\n", err[7]);
		sprintf (strchr (return_text, '\0'), "Read & Write test done, block %d.", block_num);
		
		text_line = return_text;
		if (incr)
			{
			block_num++;
			if (block_num >= ds.ds_DeviceBlockCount)
				block_num = 0;			/* Back to the beginning */
			}
		break;
		
		case 2:
		do_read (the_menu, select);				/* Calls other routine */
		/* text_line = "do_io-display executed"; */
		break;
		}
	}
	
	

void do_zero (ustring *the_menu, Selector *select)
	{
	int	i;		/* Simple counter variable */
	
	switch (*the_menu[select->item].varptr)
		{
		case 0:
		exec_write (block_num, 0);
		text_line = "do_clear partial";
		break;
		
		case 1:
		for (i = 0; i < ds.ds_DeviceBlockCount; i++)
			exec_write (i, 0);
		text_line = "do_clear whole";
		break;
		}
	}
	
	

void do_random (ustring *the_menu, Selector *select)
	{
	int	i;		/* Simple counter variable */
	random_fill (NULL, random_seed);
	
	switch (*the_menu[select->item].varptr)
		{
		case 0:
		exec_write (block_num, 1);		/* Write with random fill */
		text_line = "do_random partial";
		break;
		
		case 1:
		for (i = 0; i < ds.ds_DeviceBlockCount; i++)
			exec_write (i, 1);			/* The whole device fill */
		text_line = "do_random whole";
		break;
		}
	}
	
	

void do_comp_test (ustring *the_menu, Selector *select)
	{
	int		i;					/* Simple counter, pass in progress. */
	int		j;					/* Compare position counter */
	int		num_err = 0;		/* Number of errors in compare pass.  
								   We stop when we encounter the first! */
	static char line[100];		/* Results message */
	ControlPadEventData cped;	/* Data from the control pad */
	/* Number of passes to execute?? (use: 'pass_count') */
	/* Allocate the BIG BUFFERS */
	buffer = realloc (buffer, BIG_BLOCK);
	ver_block = realloc (ver_block, BIG_BLOCK);
	/* See if we are starting on a block that we can't do, or something... */
	if (block_num + (BIG_BLOCK / ds.ds_DeviceBlockSize) > 
					ds.ds_DeviceBlockCount)
		{
		block_num = 0;				/* back to the beginning */
		}
	
	for (i = 0; i < pass_count; i++)
		{
		sprintf (line, "Comprehensive test\nPasses Completed: %d\n", i);
		PaintLine (line, 10, 0x00ffffff);
		DrawScreen ();
		GetControlPad (1, FALSE, &cped);		/* Get the control pad things */
		select->joyold = select->joynew;
		select->joynew = cped.cped_ButtonBits;
		select->joyedge = select->joynew & ~select->joyold;
		if (select->joyedge & ControlX)
			{
			do		/* Loop until we release the button!! */
				{
				GetControlPad (1, TRUE, &cped);
				select->joyold = select->joynew;
				select->joynew = cped.cped_ButtonBits;
				select->joyedge = select->joynew & ~select->joyold;
				} while (select->joyedge & ControlX);
			break;								/* If anything, "STOP THE TAPE!!" */
			}
		/* Need to do long reads & writes (in excess of 64k) */
		
		/* Now for the tests */
		random_fill (NULL, block_num + 1);		/* Seed the generator */
		random_fill (buffer, BIG_BLOCK);		/* Fill up */
		strcat (line, "Write ");
		PaintLine (line, 10, 0x00ffffff);
		DrawScreen ();		
		w_block (buffer, block_num, BIG_BLOCK / ds.ds_DeviceBlockSize);
		strcat (line, "Read ");
		PaintLine (line, 10, 0x00ffffff);
		DrawScreen ();		
		r_block (ver_block, block_num, BIG_BLOCK / ds.ds_DeviceBlockSize);
		/* Now compare the results!! */
		for (j = 0; j < BIG_BLOCK; j++)
			{
			if (buffer[j] != ver_block[j])
				{
				num_err++;
				}
			}
		if (num_err != 0)
			break;								/* Quit, we found an error */
		/* Now complement the data and do it again!! */
		complement (buffer, BIG_BLOCK);
		strcat (line, "Write/Comp ");
		PaintLine (line, 10, 0x00ffffff);
		DrawScreen ();		
		w_block (buffer, block_num, BIG_BLOCK / ds.ds_DeviceBlockSize);
		strcat (line, "Read/Comp ");
		PaintLine (line, 10, 0x00ffffff);
		DrawScreen ();		
		r_block (ver_block, block_num, BIG_BLOCK / ds.ds_DeviceBlockSize);
		/* Now compare the results!! */
		for (j = 0; j < BIG_BLOCK; j++)
			{
			if (buffer[j] != ver_block[j])
				{
				num_err++;
				}
			}
		if (num_err != 0)
			break;								/* Quit, we found an error */
		if (++block_num + (BIG_BLOCK / ds.ds_DeviceBlockSize) > 
						ds.ds_DeviceBlockCount)
				block_num = 0;				/* back to the beginning */
		}
		
	sprintf (line, "Comprehensive:  %d passes\n", i);
	if (num_err != 0)
		sprintf (strchr (line, '\0'), "Errors encountered:  %d\n", num_err);
	else
		sprintf (strchr (line, '\0'), "No errors\n");
	/* Allocate back to 'normal' */
	ver_block = realloc (ver_block,
			ds.ds_DeviceBlockSize > BUFSIZE ? ds.ds_DeviceBlockSize : BUFSIZE);
	buffer = realloc (buffer,
			ds.ds_DeviceBlockSize > BUFSIZE ? ds.ds_DeviceBlockSize : BUFSIZE);
	text_line = line;
	}



void do_card_format (ustring *the_menu, Selector *select)
	{
	int32	pStatus;
	static char	line[200];		/* Eventual command line */
	
	sprintf (line, "acroformat %s 0 stcard", devName);
	
	ExecuteCmdLineVA (line, &pStatus, TAG_END);
	
	sprintf (line, "Formatted %s as 'stcard'\nStatus = 0x%08X\n", devName, pStatus);
	if (pStatus < 0)
		{
		GetSysErr (strchr (line, '\0'), 100, pStatus);
		}
	else if (pStatus == 0)
		{
		strcat (line, "Completed sucessfully");
		}
	else if (pStatus == 1)
		{
		strcat (line, "Storage card already mounted");
		}
	else
		{
		strcat (line, "Funny completion?!?!");
		}
	text_line = line;
	}


	
void do_mounts (ustring *the_menu, Selector *select)
	{
	int32	pStatus;
	Err		err;
	static char line[200];
	
	switch (*the_menu[select->item].varptr)
		{
		case 0:						/* Dismount storage card */
		
		/* Read of block zero */
		err = r_block (buffer, 0, 1);
		if (err < 0)
			{
			text_line = "Dismount: can't read block zero.";
			return;
			}
		/* Check to see if identifiers are in proper places */
		/* The magic stuff looks like it should be there, but I don't know!! */
		if (memcmp (&buffer[1], "ZZZZZ", 5) != 0)
			{
			text_line = "Dismount: storage card bad format.";
			}
		/* Point to name of storage card */
		sprintf (line, "dismount %s", &buffer[40]);	/* Has mounted name!?!? */
		/* Dismount same */
		ExecuteCmdLineVA (line, &pStatus, TAG_END);
		sprintf (line, "Dismounted storage card\nStatus = 0x%08X\n", devName, pStatus);
		GetSysErr (strchr (line, '\0'), 100, pStatus);
		text_line = line;
		break;
		
		case 1:
		
		/* Read of block zero */
		/* See if things are there */
		/* Mount the thing */
		sprintf (line, "mount %s", devName);
		ExecuteCmdLineVA (line, &pStatus, TAG_END);
		sprintf (line, "Mounted storage card\nStatus = 0x%08X\n", devName, pStatus);
		GetSysErr (strchr (line, '\0'), 100, pStatus);
		text_line = line;
		
		break;
		}	
	}
	


static void exec_write (int block, int type)
	{
	Err		err;

	/* Block is the block number, type is 0->clear, 1->random. */
	switch (type)
		{
		case 0:		/* Clear case */
		memset (buffer, clear_value, ds.ds_DeviceBlockSize);
		break;
		
		case 1:		/* Random case */
		random_fill (buffer, ds.ds_DeviceBlockSize);
		break;
		
		default:	/* We don't have any defaults to do */
		break;
		}

	err = w_block (buffer, block, 1);
	if (err < 0)
		{
			printf ("%s: %s failed at offset %d\n", prog, devName, block);
			PrintfSysErr (err);
		}
	}


/* This is a simple write block routine */
static Err w_block (void *buffer, int block, int nblocks)
	{
	IOInfo 	ioi;
	Err		err;

	memset (&ioi, 0, sizeof(ioi));

	ioi.ioi_Command = CMD_BLOCKWRITE;
	ioi.ioi_Recv.iob_Len = 0;
	ioi.ioi_Send.iob_Buffer = buffer;
	ioi.ioi_Send.iob_Len = ds.ds_DeviceBlockSize * nblocks;
	ioi.ioi_Offset = block;
	err = DoIO (iorItem, &ioi);
	
	return err;
	}

/* This is a simple read block routine */
static Err r_block (void *buffer, int block, int nblocks)
	{
	IOInfo 	ioi;
	Err		err;

	memset (&ioi, 0, sizeof(ioi));

	ioi.ioi_Command = CMD_BLOCKREAD;
	ioi.ioi_Send.iob_Len = 0;
	ioi.ioi_Recv.iob_Buffer = buffer;
	ioi.ioi_Recv.iob_Len = ds.ds_DeviceBlockSize * nblocks;
	ioi.ioi_Offset = block;
	err = DoIO (iorItem, &ioi);
	
	return err;
	}
	
