/*	
 *	File:				CreateEnvelopeEST.c
 *	Author:				Eric Matsuno
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the CreateEnvelope error-checking test.  This program forms a buffer for holding the
 *	envelope data.  A valid number of points is assigned to the appropriate member of the numpoints argument
 *	structure. A bogus tag argument list is created and set to the
 *	bogus tag argument in the tag argument structure.  DepthTest_TopPortion.h is then
 *	included.  This sets up the loops which allows the test program to call CreateEnvelope for all possible
 *	combinations of the arguments found in the argument structures defined in CreateEnvelopeEST_data.h. The test
 *	program indicates if the result returned by CreateEnvelope with that certain configuration of arguments
 *	is correct.  (The first argument in each argument structure is valid.  The first call to CreateEnvelope
 *	should return no error.  All other calls should return an error.)  After the tests, DepthTest_BottomPortion.h
 *	is included to close all of the loops.
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */

#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include <TestMacros.h>
#include <tests/3do_scaf.h>


/**************************************Test Environment Globals****************/
/******************************************************************************/
int main(int argc, char ** argv)
{
	#include "CreateEnvelopeEST_data.h"
	
	char *				TagString;
	EnvelopeSegment 	theEnvelope[] = {0, 0,    500,32000,    1000, 15000,    1400, 80000,    1900, 0 };
	int32				p[10];	
	int32				numOfPoints	= 5;
	int32 				theError;
	Item 				Result;
	TagArg				BogusTAG[4];
	

	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(EST, theError,"OpenAudioFolio()",cleanup);
		
	PARAM_01[0]   = numOfPoints;
	
	BogusTAG[0].ta_Tag = AF_TAG_CHANNELS ;
	BogusTAG[0].ta_Arg = (int32 *) 5;
	BogusTAG[1].ta_Tag = TAG_END;
	
	PARAM_02[0] = NULL;
	PARAM_02[1] = BogusTAG;

	#define		expected   0
	#include   <DepthTest_TopPortion.h>

	if( (p[1] > 0) || (p[2] > 0) )
	{
		switch( p[2] )
		{
			case	0 :
							TagString = "NULL";
							break;
			case	1 :
							TagString = "AF_TAG_CHANNELS";
							break;
			default	:
							TagString = "NULL";
							break;
		}
			
		BeginTest(EST, "CreateEnvelope()");
		Result = CreateEnvelope( theEnvelope, ARGUMENT_01, ARGUMENT_02);
		if (Result >= expected)
			LogResult(FAILED, "Expected negative return, but got %d from CreateEnvelope(theEnvelope,%d,(tag = %s))", 
							Result,ARGUMENT_01,TagString);
		else
			LogResult(PASSED, "");
							
		DeleteEnvelope(Result);
	}
	else
	{
		switch( p[2] )
		{
			case	0 :
							TagString = "NULL";
							break;
			case	1 :
							TagString = "AF_TAG_CHANNELS";
							break;
			default	:
							TagString = "NULL";
							break;
		}
			
		BeginTest(EST, "CreateEnvelope()");
		Result = CreateEnvelope( theEnvelope, ARGUMENT_01, ARGUMENT_02);
		if (Result < expected)
		{
			LogResult(FAILED, "Expected positive return, but got (hex)%x from CreateEnvelope(theEnvelope,%d,(tag = %s))", 
							Result,ARGUMENT_01,TagString);
			PrintfSysErr(Result);
		}
		else
			LogResult(PASSED, "");
							
		DeleteEnvelope(Result);
	}

	#include <DepthTest_BottomPortion.h>

	cleanup:
	theError = CloseAudioFolio();
	CHECKRESULT_SETUP(EST, theError,"CloseAudioFolio()",cleanup);
	

	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

void fu (void){
	LoadInstrument("square.dsp", 1, 100);
}

