/* @(#) CreateInstrumentEST.c 96/07/16 1.1 */
/*	
 *	File:				CreateInstrumentEST.c
 *	Author:				Eric Matsuno
 *	Creation Date:		
 *	
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the CreateInstrument error-checking test.  This program loads an instrument template.
 *	The Item numbes returned by the LoadInsTemplate call is set to the valid insTemplate
 *	Item numbers in the insTemplate argument structure.  A bogus tag argument list is created and set to the
 *	bogus tag argument in the tag argument structure.  DepthTest_TopPortion.h is then
 *	included.  This sets up the loops which allows the test program to call CreateInstrument for all possible
 *	combinations of the arguments found in the argument structures defined in CreateInstrumentEST_data.h. The test
 *	program indicates if the result returned by CreateInstrument with that certain configuration of arguments
 *	is correct.  (The first argument in each argument structure is valid.  The first call to CreateInstrument
 *	should return no error.  All other calls should return an error.)  After the tests, DepthTest_BottomPortion.h
 *	is included to close all of the loops.
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */

#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/

int main (int argc, char ** argv)
{
	#include "CreateInstrumentEST_data.h"
	
	char *	TagString;
	Err		theError;
	int32	p[10];
	Item	Result;
	TagArg	BogusTAG[4];


	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(EST,theError,"OpenAudioFolio",cleanup);

	PARAM_01[0] = LoadInsTemplate("square.dsp",0);
	CHECKRESULT_SETUP(EST, PARAM_01[0],"LoadInsTemplate",cleanup);

	BogusTAG[0].ta_Tag = AF_TAG_ADDRESS ;
	BogusTAG[0].ta_Arg = (int32 *) 5;
	BogusTAG[1].ta_Tag = TAG_END;
	
	PARAM_02[0] = NULL;
	PARAM_02[1] = BogusTAG;
	
	#define expected 0
	#include "DepthTest_TopPortion.h"

	if( (p[1] > 0) || (p[2] > 0) )
	{
		switch( p[2] )
		{
			case	0 :
							TagString = "NULL";
							break;
			case	1 :
							TagString = "AF_TAG_ADDRESS";
							break;
			default	:
							TagString = "NULL";
							break;
		}
			
		BeginTest(EST, "CreateInstrument");
		Result = CreateInstrument(ARGUMENT_01,ARGUMENT_02);
		
		if (Result >= expected)
			LogResult(FAILED, "Expected: Negative return, but got %d from CreateInstrument(%d,(tag = %s))", 
							Result,ARGUMENT_01,TagString);
		else
			LogResult(PASSED, "");

		DeleteInstrument(Result);
	}
	else
	{
		switch( p[2] )
		{
			case	0 :
							TagString = "NULL";
							break;
			case	1 :
							TagString = "AF_TAG_ADDRESS";
							break;
			default	:
							TagString = "NULL";
							break;
		}
			
		BeginTest(EST, "CreateInstrument");
		Result = CreateInstrument(ARGUMENT_01,ARGUMENT_02);
		
		if (Result < expected)
		{
			LogResult(FAILED, "Expected: Positive return, but got (hex)%x from CreateInstrument(%d,(tag = %s))", 
							Result,ARGUMENT_01,TagString);
			PrintfSysErr(Result);
		}
		else
			LogResult(PASSED, "");
	
		DeleteInstrument(Result);
	}

	
	#include "DepthTest_BottomPortion.h"

	cleanup:
	UnloadInsTemplate(PARAM_01[0]);

	theError	= CloseAudioFolio();
	CHECKRESULT(EST,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

