/* @(#) DisconnectInsPartsEST.c 96/07/16 1.1 */
/*	
 *	File:				DisconnectInsPartsEST.c
 *	Author:				Eric Matsuno
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the DisconnectInstrumentParts error-checking test.  This program loads two instruments.
 *	The Item number returned by the second LoadInstrument call is set to the valid dstInstrument
 *	Item numbers in the dstInstrument argument structures.  DepthTest_TopPortion.h is then
 *	included.  This sets up the loops which allows the test program to call DisconnectInstrumentParts for all possible
 *	combinations of the arguments found in the argument structures defined in DisconnectInsPartsEST_data.h. The test
 *	program indicates if the result returned by DisconnectInstrumentParts with that certain configuration of arguments
 *	is correct.  (The first argument in each argument structure is valid.  The first call to DisconnectInstrumentParts
 *	should return no error.  All other calls should return an error.)  After the tests, DepthTest_BottomPortion.h
 *	is included to close all of the loops.
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */



#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/
/******************************************************************************/

int main (int argc, char ** argv)
{
	#include "DisconnectInsPartsEST_data.h"
	
	Err		theError;
	int32	p[10];
	Item	source;



	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(EST,theError,"OpenAudioFolio",cleanup);

	source	= LoadInstrument("square.dsp", 1, 100);
	CHECKRESULT_SETUP(EST,PARAM_01[0],"LoadInstrument",cleanup2);

	PARAM_01[0]	= LoadInstrument("line_out.dsp", 1, 100);
	CHECKRESULT_SETUP(EST,PARAM_01[0],"LoadInstrument",cleanup2);

	ConnectInstrumentParts(	source, "Output", 0, PARAM_01[0], "Input", 0 );
	
	#define expected 0
	#include "DepthTest_TopPortion.h"

	if( (p[1] > 0) || (p[2] > 0) || (p[3] > 0) )
	{
		BeginTest(EST, "DisconnectInstrumentParts");
		theError = DisconnectInstrumentParts( ARGUMENT_01,ARGUMENT_02,ARGUMENT_03 );
		
		if (theError >= expected)
			LogResult(FAILED, "Expected: Negative return, but got %d from DisconnectInstrumentParts(%d,%s,%d)", 
							theError,ARGUMENT_01,ARGUMENT_02,ARGUMENT_03);
		else
			LogResult(PASSED, "");

	}
	else
	{
		BeginTest(EST, "DisconnectInstrumentParts");
		theError = DisconnectInstrumentParts( ARGUMENT_01,ARGUMENT_02,ARGUMENT_03 );
		
		if (theError < expected)
		{
			LogResult(FAILED, "Expected: Positive return, but got (hex)%x from DisconnectInstrumentParts(%d,%s,%d)", 
							theError,ARGUMENT_01,ARGUMENT_02,ARGUMENT_03);
			PrintfSysErr(theError);
		}
		else
			LogResult(PASSED, "");
	}


	#include "DepthTest_BottomPortion.h"
	
	
	cleanup2:
	UnloadInstrument(PARAM_01[0]);
	UnloadInstrument(source);

	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(EST,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

