/* @(#) StopInstrumentEST.c 96/07/16 1.1 */
/*	
 *	File:				StopInstrumentEST.c
 *	Author:				Eric Matsuno
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the StopInstrument error-checking test.  This program loads an instrument.
 *	The Item number returned by the LoadInstrument call is set to the valid Instrument
 *	Item number in the Instrument argument structure.  A valid and an invalid tag are created and set to
 *	the appropriate members of the tag argument structure.  DepthTest_TopPortion.h is then
 *	included.  This sets up the loops which allows the test program to call StopInstrument for all possible
 *	combinations of the arguments found in the argument structures defined in StopInstrumentEST_data.h. The test
 *	program indicates if the result returned by StopInstrument with that certain configuration of arguments
 *	is correct.  (The first argument in each argument structure is valid.  The first call to StopInstrument
 *	should return no error.  All other calls should return an error.)  After the tests, DepthTest_BottomPortion.h
 *	is included to close all of the loops.
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */

#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/
/******************************************************************************/

int main (int argc, char ** argv)
{
	#include "StopInstrumentEST_data.h"
	
	char *	TagString;
	Err		theError, Result;
	int32	p[10];
	TagArg	BogusTAG[4];



	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(EST,theError,"OpenAudioFolio",cleanup);
	
	PARAM_01[0] = LoadInstrument("square.dsp",0,100);

	BogusTAG[0].ta_Tag = AF_TAG_ADDRESS ;
	BogusTAG[0].ta_Arg = (int32 *) 5;
	BogusTAG[1].ta_Tag = TAG_END;
	
	PARAM_02[0] = NULL;
	PARAM_02[1] = BogusTAG;
	
	#define expected 0
	#include "DepthTest_TopPortion.h"
	
	if( (p[1] > 0) || (p[2] > 0) )
	{
		switch( p[2] )
		{
			case	0 :
							TagString = "NULL";
							break;
			case	1 :
							TagString = "AF_TAG_ADDRESS";
							break;
			default	:
							TagString = "NULL";
							break;
		}
		
		BeginTest(EST, "StopInstrument");
		Result = StopInstrument(ARGUMENT_01,ARGUMENT_02);
			
		if (Result >= expected)
			LogResult(FAILED, "Expected: Negative return, but got %d from StopInstrument(%d,(tag = %s)", 
							Result,ARGUMENT_01,TagString);
		else
			LogResult(PASSED, "");
	}
	else
	{
			
		switch( p[2] )
		{
			case	0 :
							TagString = "NULL";
							break;
			case	1 :
							TagString = "AF_TAG_ADDRESS";
							break;
			default	:
							TagString = "NULL";
							break;
		}
			
		Result	= StartInstrument(ARGUMENT_01,ARGUMENT_02);
		CHECKRESULT(EST, Result,"StartInstrument");
		
		BeginTest(EST, "StopInstrument");
		Result = StopInstrument(ARGUMENT_01,ARGUMENT_02);
			
		if (Result < expected)
		{
			LogResult(FAILED, "Expected: Positive return, but got (hex)%x from StopInstrument(%d,(tag = %s))", 
							Result,ARGUMENT_01,TagString);
			PrintfSysErr(Result);
		}
		else
			LogResult(PASSED, "");
	}

	#include "DepthTest_BottomPortion.h"
	

	cleanup:
	(void)UnloadInstrument(PARAM_01[0]);
	
	theError	= CloseAudioFolio();
	CHECKRESULT(EST,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

