/* @(#) InstrumentResFT.c 96/07/26 1.2 */
/*	
 *	File:				InstrumentResFT.c
 *	Author:				Eric Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the GetInstrumentResourceInfo and DumpInstrumentResourceInfo Test.  This test loads 
 *	every dsp instrument and does five GetInstrumentResourceInfo calls for each instrument.  Each call looks for a specific
 *	set of information.  The first call looks for the number of ticks an instrument uses, the second call
 *	looks for code memory usage, the third call looks for data memory usage, the fourth call returns the
 *	number of FIFOs an instrument uses, and the fifth call returns the number of triggers an instrument uses.
 *
 *	Then the test program calls DumpInstrumentResourceInfo to get the instrument resource information.  The tester
 *	can compare the info given by GetInstrumentResourceInfo and DumpInstrumentResourceInfo to see if they are the same.
 *
 *
 *	Uses the following calls:
 *
 *	OpenAudioFolio	 		CloseAudioFolio			LoadInstrument		GetInstrumentResourceInfo
 *	UnloadInstrument
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */


#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/

int main (int argc, char ** argv)
{
	#include "InstrumentResFT_data.h"
	
	Err						theError;
	int32					p[10];
	Item 					Result;
	InstrumentResourceInfo	info;
	uint32 					infoSize	= sizeof(InstrumentResourceInfo);


	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(FT, theError,"OpenAudioFolio",cleanup);
	
	#define expected 0

	BeginTest(FT, "GetInstrumentResourceInfo()");
	
	for ( p[1] = 0, p[2] = 0; p[1] < 56; p[1]++ )
	{
		/*	Test all instruments with the proper arguments except for the one with bogus name
		*/
		if ( p[1] == 15 )		/*	line_in.dsp	*/
		{
			theError	= EnableAudioInput( 1, NULL );
			
			if ( theError >= 0 )
			{
				Result = LoadInstrument(ARGUMENT_01, 1, 100);
				LogComment("LoadInstrument( %s ) returned %d (0x%x)", ARGUMENT_01, Result, Result);
		
				theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_TICKS);
				LogComment("GetInstrumentResourceInfo returned %d (0x%x)", ARGUMENT_01, theError, theError);
				LogComment("%s:\t\tDSP ticks/Frame: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

				theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_CODE_MEM);
				LogComment("GetInstrumentResourceInfo( %s ) returned %d (0x%x)", ARGUMENT_01, theError, theError);
				LogComment("%s:\t\tDSP Code Memory: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

				theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_DATA_MEM);
				LogComment("GetInstrumentResourceInfo( %s ) returned %d (0x%x)", ARGUMENT_01, theError, theError);
				LogComment("%s:\t\tDSP Data Memory: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

				theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_FIFOS);
				LogComment("GetInstrumentResourceInfo( %s ) returned %d (0x%x)", ARGUMENT_01, theError, theError);
				LogComment("%s:\t\tNumber of FIFOs: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

				theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_TRIGGERS);
				LogComment("GetInstrumentResourceInfo( %s ) returned %d (0x%x)", ARGUMENT_01, theError, theError);
				LogComment("%s:\t\tNumber of Triggers: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

				LogComment("Calling DumpInstrumentResourceInfo()");
				DumpInstrumentResourceInfo( Result, NULL );
				UnloadInstrument(Result);
				
				printf("\n");
			}
			else
				LogComment("FAILED: EnableAudioInput call returned (hex)%x", theError );
		}
		else if ( p[1] != 1 )
		{
			Result = LoadInstrument(ARGUMENT_01, 1, 100);
			LogComment("LoadInstrument returned %d (0x%x)", Result, Result);
		
			theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_TICKS);
			LogComment("GetInstrumentResourceInfo( %s ) returned %d (0x%x)", ARGUMENT_01, theError, theError);
			LogComment("%s:\t\tDSP ticks/Frame: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

			theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_CODE_MEM);
			LogComment("GetInstrumentResourceInfo( %s ) returned %d (0x%x)", ARGUMENT_01, theError, theError);
			LogComment("%s:\t\tDSP Code Memory: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

			theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_DATA_MEM);
			LogComment("GetInstrumentResourceInfo( %s ) returned %d (0x%x)", ARGUMENT_01, theError, theError);
			LogComment("%s:\t\tDSP Data Memory: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

			theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_FIFOS);
			LogComment("GetInstrumentResourceInfo( %s ) returned %d (0x%x)", ARGUMENT_01, theError, theError);
			LogComment("%s:\t\tNumber of FIFOs: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

			theError	= GetInstrumentResourceInfo ( &info, infoSize, Result, AF_RESOURCE_TYPE_TRIGGERS);
			LogComment("GetInstrumentResourceInfo( %s ) returned %d (0x%x)", ARGUMENT_01, theError, theError);
			LogComment("%s:\t\tNumber of Triggers: %d\t\tMaximum Overhead: %d", ARGUMENT_01, info.rinfo_PerInstrument, info.rinfo_MaxOverhead );

			LogComment("Calling DumpInstrumentResourceInfo()");
			DumpInstrumentResourceInfo( Result, NULL );
			UnloadInstrument(Result);
				
			printf("\n");
		}
	}		

	LogResult(InteractiveVerify("Resource Information should match the documented Audio Specifications. Did Test Pass?"), "");

	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(FT,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

