/* @(#) KnobFT.c 96/07/17 1.1 */
/*	
 *	File:				KnobFT.c
 *	Author:				Eric L. Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the Knobs functional test.  This test program checks to see if all the knobs
 *	of all the dsp instruments can be created and set to a certain value.  Then the test program reads
 *	the knob value and compares it to its Set Value.  If the two values are the same, then the test
 *	passes.  The srtructures in Knobs_FT_data.h contain the information necessary to load each
 *	instrument and creat the knobs.
 *
 *	Audio Functions called:
 *
 *	OpenAudioFolio		CloseAudioFolio			CreateKnob			LoadInstrument			
 *	ReadKnob			SetKnobPart				SetKnob				DeleteKnob
 *	UnloadInstrument
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */



#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>
#include <string.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/
/******************************************************************************/

int main (int argc, char ** argv)
{
	#include "KnobFT_data.h"
	
	Err		theError;
	float32	theKnobValue, expKnobValue, tolerance;
	int32	p[10];
	Item 	theInstrument, theKnob[3];



	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	BeginTest(FT, "KnobFT");
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(FT,theError,"OpenAudioFolio",cleanup);
	#define expected 0

	
	for ( p[1] = 0, p[2] = 0, p[3] = 0, p[4] = 0, p[5] = 0, p[6] = 0, p[7] = 0; p[1] < 45; p[1]++, p[4]++, p[5]++, p[6]++, p[7]++ )
	{
		theInstrument = LoadInstrument(ARGUMENT_01,ARGUMENT_02,ARGUMENT_03);
		CHECKRESULT_SETUP(FT,theInstrument,"LoadInstrument",cleanup);
		
		LogComment("%3d. Testing %s instrument: (%s)knob:\t\t(%s)knob:\t\t(%s)knob.", p[1], ARGUMENT_01, ARGUMENT_04, ARGUMENT_05, ARGUMENT_06);		
		/*	Create the knobs for each instrument			*/		
		switch( ARGUMENT_07 )	/*	How many arguments does the instrument have	*/
		{
			case	1 :
						BeginTest(FT, "CreateKnob( %s, %s )", ARGUMENT_01, ARGUMENT_04);
						theKnob[0]	= CreateKnob( theInstrument, ARGUMENT_04, NULL );
						
						if (theKnob[0] < expected)
							LogResult(FAILED, "Expected: Positive return, but got %d from CreateKnob(%d, %s, NULL)", theKnob[0], theInstrument, ARGUMENT_04);
						else
							LogResult(PASSED, "");
						break;

			case	2 :
						BeginTest(FT, "CreateKnob( %s, %s )", ARGUMENT_01, ARGUMENT_04);
						theKnob[0]	= CreateKnob( theInstrument, ARGUMENT_04, NULL );
						
						if (theKnob[0] < expected)
							LogResult(FAILED, "Expected: Positive return, but got %d from CreateKnob(%d, %s, NULL)", theKnob[0], theInstrument, ARGUMENT_04);
						else
							LogResult(PASSED, "");
						
						
						BeginTest(FT, "CreateKnob( %s, %s )", ARGUMENT_01, ARGUMENT_05);
						theKnob[1]	= CreateKnob( theInstrument, ARGUMENT_05, NULL );
						
						if (theKnob[1] < expected)
							LogResult(FAILED, "Expected: Positive return, but got %d from CreateKnob(%d, %s, NULL)", theKnob[1], theInstrument, ARGUMENT_04);
						else
							LogResult(PASSED, "");
						break;
						
			case	3 :
						BeginTest(FT, "CreateKnob( %s, %s )", ARGUMENT_01, ARGUMENT_04);
						theKnob[0]	= CreateKnob( theInstrument, ARGUMENT_04, NULL );
						
						if (theKnob[0] < expected)
							LogResult(FAILED, "Expected: Positive return, but got %d from CreateKnob(%d, %s, NULL)", theKnob[0], theInstrument, ARGUMENT_04);
						else
							LogResult(PASSED, "");
						
						
						BeginTest(FT, "CreateKnob( %s, %s )", ARGUMENT_01, ARGUMENT_05);
						theKnob[1]	= CreateKnob( theInstrument, ARGUMENT_05, NULL );
						
						if (theKnob[1] < expected)
							LogResult(FAILED, "Expected: Positive return, but got %d from CreateKnob(%d, %s, NULL)", theKnob[1], theInstrument, ARGUMENT_05);
						else
							LogResult(PASSED, "");


						BeginTest(FT, "CreateKnob( %s, %s )", ARGUMENT_01, ARGUMENT_06);
						theKnob[2]	= CreateKnob( theInstrument, ARGUMENT_06, NULL );
						
						if (theKnob[2] < expected)
							LogResult(FAILED, "Expected: Positive return, but got %d from CreateKnob(%d, %s, NULL)", theKnob[2], theInstrument, ARGUMENT_06);
						else
							LogResult(PASSED, "");
						
						
						break;
		}

		/*	Set the values of each knob			*/		
		switch( ARGUMENT_07 )	/*	How many arguments does the instrument have	*/
		{
			case	1 :
						BeginTest(FT, "SetKnob/ReadKnob( %s, %s )", ARGUMENT_01, ARGUMENT_04);

						expKnobValue	= .5;
						tolerance		= .01;
						if ( (strcmp(ARGUMENT_04, "SampleRate") == 0) || (strcmp(ARGUMENT_04, "Frequency") == 0) )
						{
							expKnobValue	= 5;
							tolerance		= 1;
							SetKnob( theKnob[0], expKnobValue );
						}
						else
							SetKnob( theKnob[0], expKnobValue );
							
						ReadKnob( theKnob[0], &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnob after SetKnob(%d, %f)", expKnobValue, theKnobValue, theKnob[0], expKnobValue);
							
							
						BeginTest(FT, "SetKnobPart/ReadKnobPart( %s, %s )", ARGUMENT_01, ARGUMENT_04 );
						
						expKnobValue	= .25;
						tolerance		= .01;
						if ( (strcmp(ARGUMENT_04, "SampleRate") == 0) || (strcmp(ARGUMENT_04, "Frequency") == 0) )
						{
							expKnobValue	= 10;
							tolerance		= 1;
							SetKnobPart( theKnob[0], 0, expKnobValue );
						}
						else
							SetKnobPart( theKnob[0], 0, expKnobValue );
						
						ReadKnobPart( theKnob[0], 0, &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnobPart after SetKnobPart(%d, 0, %f)", expKnobValue, theKnobValue, theKnob[0], expKnobValue);
							
						break;
			case	2 :
						BeginTest(FT, "SetKnob/ReadKnob( %s, %s )", ARGUMENT_01, ARGUMENT_04);

						expKnobValue	= .5;
						tolerance		= .01;
						if ( (strcmp(ARGUMENT_04, "SampleRate") == 0) || (strcmp(ARGUMENT_04, "Frequency") == 0) )
						{
							expKnobValue	= 5;
							tolerance		= 1.0;
							SetKnob( theKnob[0], expKnobValue );
						}
						else
							SetKnob( theKnob[0], expKnobValue );
							
						ReadKnob( theKnob[0], &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnob after SetKnob(%d, %f)", expKnobValue, theKnobValue, theKnob[0], expKnobValue);
							
						
						BeginTest(FT, "SetKnob/ReadKnob( %s, %s )", ARGUMENT_01, ARGUMENT_05);
						
						expKnobValue	= .5;
						tolerance		= .01;
						if ( (strcmp(ARGUMENT_05, "SampleRate") == 0) || (strcmp(ARGUMENT_05, "Frequency") == 0) )
						{
							expKnobValue	= 5;
							tolerance		= 1;
							SetKnob( theKnob[1], expKnobValue );
						}
						else
							SetKnob( theKnob[1], expKnobValue );
						
						ReadKnob( theKnob[1], &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnob after SetKnob(%d, %f)", expKnobValue, theKnobValue, theKnob[1], expKnobValue);
					
						
						BeginTest(FT, "SetKnobPart/ReadKnobPart( %s, %s )", ARGUMENT_01, ARGUMENT_04);

						expKnobValue	= .25;
						tolerance		= .01;
						if ( (strcmp(ARGUMENT_04, "SampleRate") == 0) || (strcmp(ARGUMENT_04, "Frequency") == 0) )
						{
							expKnobValue	= 10;
							tolerance		= 1;
							SetKnobPart( theKnob[0], 0, expKnobValue );
						}
						else
							SetKnobPart( theKnob[0], 0, expKnobValue );
							
						ReadKnobPart( theKnob[0], 0, &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnobPart after SetKnobPart(%d, 0, %f)", expKnobValue, theKnobValue, theKnob[0], expKnobValue);
							

						BeginTest(FT, "SetKnobPart/ReadKnobPart( %s, %s )", ARGUMENT_01, ARGUMENT_05);

						expKnobValue	= .25;
						tolerance		= .01;	
						if ( (strcmp(ARGUMENT_05, "SampleRate") == 0) || (strcmp(ARGUMENT_05, "Frequency") == 0) )
						{
							expKnobValue	= 10;
							tolerance		= 1;
							SetKnobPart( theKnob[1], 0, expKnobValue );
						}
						else
							SetKnobPart( theKnob[1], 0, expKnobValue );
						
						ReadKnobPart( theKnob[1], 0, &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnobPart after SetKnobPart(%d, 0, %f)", expKnobValue, theKnobValue, theKnob[1], expKnobValue);
							
						break;
			case	3 :
						BeginTest(FT, "SetKnob/ReadKnob( %s, %s )", ARGUMENT_01, ARGUMENT_04);

						expKnobValue	= .5;
						tolerance		= .01;
						if ( (strcmp(ARGUMENT_04, "SampleRate") == 0) || (strcmp(ARGUMENT_04, "Frequency") == 0) )
						{
							if ( strcmp(ARGUMENT_01, "svfilter.dsp") != 0)
							{
								expKnobValue	= 5;
								tolerance		= 1;
								SetKnob( theKnob[0], expKnobValue );
							}
							else
								SetKnob( theKnob[0], expKnobValue );
						}
						else
							SetKnob( theKnob[0], expKnobValue );
							
						ReadKnob( theKnob[0], &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnob after SetKnob(%d, %f)", expKnobValue, theKnobValue, theKnob[0], expKnobValue);
							
					
						BeginTest(FT, "SetKnob/ReadKnob( %s, %s )", ARGUMENT_01, ARGUMENT_05);

						expKnobValue	= .5;
						tolerance		= .01;
						if ( (strcmp(ARGUMENT_05, "SampleRate") == 0) || (strcmp(ARGUMENT_05, "Frequency") == 0) )
						{
							expKnobValue	= 5;
							tolerance		= 1;
							SetKnob( theKnob[1], expKnobValue );
						}
						else
							SetKnob( theKnob[1], expKnobValue );
							

						ReadKnob( theKnob[1], &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnob after SetKnob(%d, %f)", expKnobValue, theKnobValue, theKnob[1], expKnobValue);
					

						BeginTest(FT, "SetKnob/ReadKnob( %s, %s )", ARGUMENT_01, ARGUMENT_06);

						expKnobValue	= .5;
						tolerance		= .01;
						if ( (strcmp(ARGUMENT_06, "SampleRate") == 0) || (strcmp(ARGUMENT_06, "Frequency") == 0) )
						{
							expKnobValue	= 5;
							tolerance		= 1;
							SetKnob( theKnob[0], expKnobValue );
						}
						else
							SetKnob( theKnob[2], expKnobValue );
						
						ReadKnob( theKnob[2], &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnob after SetKnob(%d, %f)", expKnobValue, theKnobValue, theKnob[2], expKnobValue);
					
						
						BeginTest(FT, "SetKnobPart/ReadKnobPart( %s, %s )", ARGUMENT_01, ARGUMENT_04);
						
						expKnobValue	= .25;
						tolerance		= .01;	
						if ( (strcmp(ARGUMENT_04, "SampleRate") == 0) || (strcmp(ARGUMENT_04, "Frequency") == 0) )
						{
							if ( strcmp(ARGUMENT_01, "svfilter.dsp") != 0)
							{
								expKnobValue	= 10;
								tolerance		= 1;
								SetKnobPart( theKnob[0], 0, expKnobValue );
							}
							else
								SetKnobPart( theKnob[0], 0, expKnobValue );
						}
						else
							SetKnobPart( theKnob[0], 0, expKnobValue );
						
						ReadKnobPart( theKnob[0], 0, &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnobPart after SetKnobPart(%d, 0, %f)", expKnobValue, theKnobValue, theKnob[0], expKnobValue);
							

						BeginTest(FT, "SetKnobPart/ReadKnobPart( %s, %s )", ARGUMENT_01, ARGUMENT_05);

						expKnobValue	= .25;
						tolerance		= .01;		
						if ( (strcmp(ARGUMENT_05, "SampleRate") == 0) || (strcmp(ARGUMENT_05, "Frequency") == 0) )
						{
							expKnobValue	= 10;
							tolerance		= 1;
							SetKnobPart( theKnob[1], 0, expKnobValue );
						}
						else
							SetKnobPart( theKnob[1], 0, expKnobValue );
						

						ReadKnobPart( theKnob[1], 0, &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnobPart after SetKnobPart(%d, 0, %f)", expKnobValue, theKnobValue, theKnob[1], expKnobValue);
							

			
						BeginTest(FT, "SetKnobPart/ReadKnobPart( %s, %s )", ARGUMENT_01, ARGUMENT_06);

						expKnobValue	= .25;
						tolerance		= .01;		
						if ( (strcmp(ARGUMENT_06, "SampleRate") == 0) || (strcmp(ARGUMENT_06, "Frequency") == 0) )
						{
							expKnobValue	= 10;
							tolerance		= 1;
							SetKnobPart( theKnob[0], 0, expKnobValue );
						}
						else
							SetKnobPart( theKnob[2], 0, expKnobValue );
						
						ReadKnobPart( theKnob[2], 0, &theKnobValue ); 
						
						if ( ( theKnobValue - expKnobValue < tolerance) && ( theKnobValue - expKnobValue > -tolerance) )
							LogResult(PASSED, "");
						else
							LogResult(FAILED, "Expected: %f, but got %f from ReadKnobPart after SetKnobPart(%d, 0, %f)", expKnobValue, theKnobValue, theKnob[2], expKnobValue);
							
						break;
		}
		
		/*	Delete the knobs			*/
		switch( ARGUMENT_07 )	/*	How many arguments does the instrument have	*/
		{
			case	1 :
						DeleteKnob( theKnob[0] );
						break;
			case	2 :
						DeleteKnob( theKnob[0] );
						DeleteKnob( theKnob[1] );
						break;
			case	3 :
						DeleteKnob( theKnob[0] );
						DeleteKnob( theKnob[1] );
						DeleteKnob( theKnob[2] );
						break;
		}
						
		UnloadInstrument(theInstrument);
	}		
	
	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(FT, theError,"CloseAudioFolio");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

