/* @(#) AudioResourceFT.c 96/07/26 1.2 */
/*	
 *	File:				AudioResourceFT.c
 *	Author:				Eric Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the GetAudioResourceInfo Test.  This test loads a set of dsp instruments and 
 *	does five GetAudioResourceInfo calls for each instrument.  Each call looks for a specific
 *	set of information.  For each call, GetAudioResourceInfo returns three fields:  the total resource
 *	available, the amount of resource currently available, and the largest contiguous block of resource.
 *
 *	The first call looks for the number of ticks an instrument uses, the second call
 *	looks for code memory usage, the third call looks for data memory usage, the fourth call returns the
 *	number of FIFOs an instrument uses, and the fifth call returns the number of triggers an instrument uses.
 *
 *
 *	Uses the following calls:
 *
 *	OpenAudioFolio	 		CloseAudioFolio			LoadInstrument		GetAudioResourceInfo
 *	UnloadInstrument
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */


#include <audio/audio.h>
#include <audio/parse_aiff.h>
#include <kernel/operror.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/

int main (int argc, char ** argv)
{
	#include "AudioResourceFT_data.h"
	
	Err						theError;
	int32					p[10], i;
	Item 					Result[10];
	AudioResourceInfo		info;
	uint32 					infoSize	= sizeof(AudioResourceInfo);


	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(FT, theError,"OpenAudioFolio",cleanup);
	
	#define expected 0
	
	BeginTest(FT, "GetAudioResourceInfo");

	for ( p[1] = 0; p[1] < 10; p[1]++ )
	{
		/*	Test all instruments with the proper arguments except for the one with bogus name
		*/
		Result[p[1]] = LoadInstrument(ARGUMENT_01, 1, 100);
		LogComment("LoadInstrument( %s ) returned %d", ARGUMENT_01, Result[p[1]]);
		
		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_TICKS);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("TICKS:\tAfter loading %s:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", ARGUMENT_01, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_CODE_MEM);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("CODE_MEM:\tAfter loading %s:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", ARGUMENT_01, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_DATA_MEM);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("DATA_MEM:\tAfter loading %s:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", ARGUMENT_01, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_FIFOS);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("FIFOS:\tAfter loading %s:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", ARGUMENT_01, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_TRIGGERS);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("TRIGGERS:\tAfter loading %s:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", ARGUMENT_01, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

	}
	
	for ( i = 0; i < 10; i++ )
	{
		theError	= UnloadInstrument(Result[i]);
		LogComment("UnloadInstrument returned %d", theError);

		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_TICKS);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("TICKS:\tAfter unloading Instrument #%d:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", i, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_CODE_MEM);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("CODE_MEM:\tAfter unloading Instrument #%d:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", i, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_DATA_MEM);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("DATA_MEM:\tAfter unloading Instrument #%d:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", i, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_FIFOS);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("FIFOS:\tAfter unloading Instrument #%d:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", i, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

		theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_TRIGGERS);
		LogComment("GetAudioResourceInfo returned %d", theError);
		LogComment("TRIGGERS:\tAfter unloading Instrument #%d:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", i, info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

	}
		
	theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_TICKS);
	LogComment("GetAudioResourceInfo returned %d", theError);
	LogComment("TICKS:\tAfter unloading the sample:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

	theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_CODE_MEM);
	LogComment("GetAudioResourceInfo returned %d", theError);
	LogComment("CODE_MEM:\tAfter unloading the sample:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

	theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_DATA_MEM);
	LogComment("GetAudioResourceInfo returned %d", theError);
	LogComment("DATA_MEM:\tAfter unloading the sample:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

	theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_FIFOS);
	LogComment("GetAudioResourceInfo returned %d", theError);
	LogComment("FIFOS:\tAfter unloading the sample:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

	theError	= GetAudioResourceInfo ( &info, infoSize, AF_RESOURCE_TYPE_TRIGGERS);
	LogComment("GetAudioResourceInfo returned %d", theError);
	LogComment("TRIGGERS:\tAfter unloading the sample:\t\tTotal Amount: %d\t\tCurrent Amount Available: %d\t\tLargest Contiguous Block: %d", info.rinfo_Total, info.rinfo_Free, info.rinfo_LargestFreeSpan );

	LogResult(InteractiveVerify("Resource Information should match the documented Audio Specifications. Did Test Pass?"), "");

	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(FT,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

