/* @(#) LongTermTimer_FT.c 96/07/26 1.2 */
/*	
 *	File:				LongTermTimer_FT.c
 *	Author:				Eric L. Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains contains a functional test which verifies that the independent clocks
 *	created by the user keeps accurate time over a long period of time. An independent clock
 *	is created.  The test program uses the global clock to sleep a specified period of time.
 *	When that time is over, the time of the independent clock is read and compared to what the
 *	time of the independent clock should be, which is the time of the independent clock before
 *	the sleep plus the amount of time passed during the sleep.  This test takes about 15 hours.
 *
 *	Audio Functions called:
 *
 *	OpenAudioFolio		CloseAudioFolio			CreateCue			CreateAudioClock			
 *	ReadAudioClock		DeleteAudioClock		DeleteCue			SleepUntilAudioTime				
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */



#include <audio/audio.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>


/**************************************Test Environment Globals****************/
/******************************************************************************/

int main (int argc, char ** argv)
{

	AudioTime	time1, time2, time3, globalTime, delay	= 240*60*60, cumulator;	
	Err			theError;
	int			i, sum	= 0;
	Item		clock1, manyCues[16];


	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(FT,theError,"OpenAudioFolio",cleanup);
	
	for ( i = 0; i < 16; i++ )
	{
		manyCues[i]	= CreateCue( NULL );
		CHECKRESULT_SETUP(FT,manyCues[i],"CreateCue",cleanup);
	}

	/*	Create the clocks	*/
	clock1	= CreateAudioClock(NULL);
	CHECKRESULT_SETUP(FT,clock1,"CreateAudioClock",cleanup);

	/****************************************************************************************************
	 *	Test 1:	Make sure that the independent clock is maintaining the correct time over a long period.
	 *			See if the time resets properly
	 ****************************************************************************************************/
	 
	printf("This test will take 15 hours to run.\n\n");
	for ( i = 0; i < 6; i++ )
	{
		sum	+= i;
		cumulator	= delay*i;
		
		ReadAudioClock( clock1, &time1 );
		globalTime	= GetAudioTime();
		
		SleepUntilTime( manyCues[i],  globalTime + cumulator );

		ReadAudioClock( clock1, &time2 );

		time3	= time1 + cumulator;
		
		BeginTest(FT, "LongTermTimer: %3d hours", sum);
		if ( time3 - time2 < 5 )
			LogResult(PASSED, "" );
		else
			LogResult(FAILED, "After %3d hours, Clock #1 Time is: %8d\t\tClock Time Expected: %8d\t\tGlobal Time: %8d\n", 
							sum, time2, time3, globalTime );
	}

	
	DeleteAudioClock(clock1);
	
	for ( i = 0; i < 16; i++ )
		DeleteCue( manyCues[i] );

	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(FT, theError,"CloseAudioFolio");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}
