/* @(#) Expmod_unsigned_FT.c 96/07/22 1.3 */
/*	
 *	File:				Expmod_unsigned_FT..c
 *	Author:				Eric L. Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the functional test for expmod_unsigned.dsp. expmod_unsigned.dsp is supposed to multiply
 *	an unsigned signal by 2^x (where x is the modulation).  A structure is created which contains the values 
 *	for various input and modulation combinatios.  The output of expmod_unsigned.dsp when given the input
 *	combination is compared against the expected output.
 *
 *	Audio Functions called:
 *
 *		CloseAudioFolio		CreateCue			CreateKnob			LoadInstrument			CreateProbe			
 *		OpenAudioFolio		StartInstrument		StopInstrument		SleepUntilTime			SetKnob
 *		DeleteCue			DeleteProbe			ReadProbe			UnloadInstrument
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */



#include <audio/audio.h>
#include <stdio.h>

#include <TestMacros.h>
#include <TestUtilities.h>
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/
/******************************************************************************/

typedef	struct
		{
			float32	input;
			float32	modulation;
			float32	expectedOutput;
		}  TestValues;


#define	NUM_OF_TESTS		5


int main (int argc, char ** argv)
{
	
	Err		theError;
	float32	theOutputValue;
	int		i;
	Item 	theItem, theTestProbe, theCue, input, modulation;
	TestValues	testList[NUM_OF_TESTS]	= 	{
												1.0,		0,		1.0,
												1.0,		1.0,	2.0,
												-1.0,		1.0,	0,
												1.0,		-1.0,	.5,
												2.0,		1.0,		2.0
											};



	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(FT,theError,"OpenAudioFolio",cleanup);
	
	theCue	= CreateCue( NULL );
	CHECKRESULT_SETUP(FT,theCue,"CreateCue",cleanup);


	/****************************************************************************************************
	 *	Load the expmod_unsigned instrument ant create the two knobs for it.
	 ****************************************************************************************************/
		
	theItem			= LoadInstrument("expmod_unsigned.dsp", 1, 100);
	CHECKRESULT_SETUP(FT,theItem,"LoadInstrument",cleanup);
		
	input			= CreateKnob( theItem, "Input", NULL );
	CHECKRESULT_SETUP(FT,input,"CreateKnob",cleanup);
	
	modulation		= CreateKnob( theItem, "Modulation", NULL );
	CHECKRESULT_SETUP(FT,modulation,"CreateKnob",cleanup);
	
	/*	Create the test probe	*/
	theTestProbe	= CreateProbe( theItem, "Output", NULL );
	CHECKRESULT_SETUP(FT,theTestProbe,"CreateProbe",cleanup);
	

	for ( i = 0; i < NUM_OF_TESTS; i++ )
	{
		StopInstrument( theItem, NULL );

		/*	Test the expmod_unsigned with the testList	*/
		SetKnob( input, testList[i].input );
		SetKnob( modulation, testList[i].modulation );

		StartInstrument( theItem, NULL );
	
		SleepUntilTime( theCue, GetAudioTime() + 4 );
		ReadProbe( theTestProbe, &theOutputValue );
		
		BeginTest(FT, "expmod_unsigned.dsp");
		if ( (theOutputValue - testList[i].expectedOutput < .1) && (theOutputValue - testList[i].expectedOutput > -.1))
			LogResult(PASSED, "" );
		else
			LogResult(FAILED, "Expected %f, but got %f when Input = %f;  modulation = %f.",
							 testList[i].expectedOutput, theOutputValue, testList[i].input, testList[i].modulation );
	}
	
	StopInstrument( theItem, NULL );

	UnloadInstrument( theItem );
	DeleteCue( theCue );
	DeleteProbe( theTestProbe );
	
	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(FT, theError,"CloseAudioFolio");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

