/* @(#) AdvSoundPlayer_FT.c 96/07/29 1.2 */
/*	
 *	File:				AdvSoundPlayer_FT.c
 *	Author:				Eric L. Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the functional tests for the advanced Sound player.  This file contains five tests.
 *
 *	Test 1:	A sound player is created. Play a large sample from memory using the sound player.  Use the control pad
 *			to pause and resumethe soundplayer.  Press Button A to pause player, button B to resume player, and button
 *			X to exit loop.
 *
 *	Test 2:	A small sample is played from memory.  The user should hear one sound.
 *
 *	Test 3:	Three more sounds are added to the soundplayer.  Within the soundplayer, the samples are linked
 *			together.  The soundplayer sequentially plays each sample.  The user should hear four sounds.
 *
 *	Test 4:	The end of the fourth sample is linked to the beginning of the first sample.  A decision function
 *			is created for the marker of the first three samples.  A unique decision function, LoopDecisionFunction,
 *			is created for the end of sample 4. The first three decision functions prints a message and clears
 *			the decision function the first time the decision function is executed.  So the user should only see
 *			the decision function's message just once.  A default default decision function is
 *			created for the sample player.  The default decision function remains active for the duration of the
 *			test.  The message printed by the default decision function should be seen constantly. The loop 
 *			decision function takes an argument, counter, which it uses to determine whether the loop should
 *			be continued.  Counter is decremented before exiting the function.When counter is 0, the soundplayer
 *			Currently the the sounds are played in a loop four times
 *
 *
 *	Test 5:	Play a large sample file directly from disc 
 *
 *	Test 6:	Create markers to point to places within the sound file. Set branches from one marker to another 
 *			within the played sample.  The effect is that the soundplayer skips through sample the second time
 *			the sample is played.
 *
 *	Audio Functions called:
 *
 *		CloseAudioFolio		ConnectInstrumentParts			LoadInstrument					LoadSample
 *		OpenAudioFolio		spAddSample						spCreatePlayer					spStartPlayingVA
 *		spStartReading		spGetPlayerSignalMask			spGetPlayerStatus				spStop
 *		spBranchAtMarker	spSetMarkerDecisionFunction		spSetDefaultDecisionFunction	spPause
 *		spAddSoundFile		spClearDefaultDecisionFunction	spClearMarkerDecisionFunction	spResume
 *		spAddMarker			spService						spDeletePlayer					UnloadInstrument
 *		UnloadSample		WaitSignal
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */

#include <audio/audio.h>
#include <audio/parse_aiff.h>
#include <audio/soundplayer.h>
#include <kernel/operror.h>
#include <misc/event.h>
#include <stdio.h>

#include <TestMacros.h>
#include <tests/3do_scaf.h>


/*	Define prototypes		*/
Err LoopDecisionFunction(	SPAction *resultAction, int32 *remainingCount,
							SPSound *sound, const char *markerName 			);

Err LoopDecisionFunction2(	SPAction *resultAction, int32 *remainingCount,
							SPSound *sound, const char *markerName 			);

Err M1DecisionFunction(		SPAction *resultAction, int32 *remainingCount,
							SPSound *sound, const char *markerName 			);

Err M2DecisionFunction(		SPAction *resultAction, int32 *remainingCount,
							SPSound *sound, const char *markerName 			);

Err M3DecisionFunction(		SPAction *resultAction, int32 *remainingCount,
							SPSound *sound, const char *markerName 			);

Err GlobalDecisionFunction(	SPAction *resultAction, int32 *remainingCount,
							SPSound *sound, const char *markerName 			);


/*	Define Global variables
*/
SPSound		*sound1, *sound2, *sound3, *sound4, *sound5, *sound6;

/**************************************Test Environment Globals****************/
/******************************************************************************/
int main(int argc, char ** argv)
{
	Boolean					hasPlayerPaused;
	ControlPadEventData		cped;
	Err						theError;	
	int32					playersigs, sigs, counter;
	Item					sampleInstrumentItem, lineOutInstrumentItem,
							sampleFileItem1, sampleFileItem2, sampleFileItem3, sampleFileItem4, sampleFileItem5;
	SPPlayer				*player;

	

	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	BeginTest(FT, "AdvSoundPlayer_FT");
	/******************************************************************************/

	/* Initialize the EventBroker. */
	theError = InitEventUtility(1, 0, LC_FocusListener);
	CHECKRESULT_SETUP(FT,theError,"InitEventUtility",cleanup);

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(FT,theError,"OpenAudioFolio",cleanup);
		

	/*	Set up the environment so that we can use sound player.  We load a
		sample playing instrument to play four different sample files.
		We create a line out instrument for audio output.
	*/
	
	/*	Load all of the instruments which will be played in the sequence #3	*/
	sampleInstrumentItem	= LoadInstrument( "sampler_16_f1.dsp", 1, 100 );
	CHECKRESULT_SETUP(FT,sampleInstrumentItem,"LoadInstrument(sampleInstrument)",cleanup);
	
	sampleFileItem1			= LoadSample("Cowbell_44K_16B_M.aiff");
	CHECKRESULT_SETUP(FT,sampleFileItem1,"LoadSample(CowBell)",cleanup);
	sampleFileItem2			= LoadSample("Marimba_44K_16B_M.aiff");
	CHECKRESULT_SETUP(FT,sampleFileItem2,"LoadSample(Marimba)",cleanup);
	sampleFileItem3			= LoadSample("PanFlute_44K_16B_M.aiff");
	CHECKRESULT_SETUP(FT,sampleFileItem3,"LoadSample(PanFlute)",cleanup);
	sampleFileItem4			= LoadSample("Trumpet_44K_16B_M.aiff");
	CHECKRESULT_SETUP(FT,sampleFileItem4,"LoadSample(Trumpet)",cleanup);
	sampleFileItem5			= LoadSample("44K_16B_M.AIFF");
	CHECKRESULT_SETUP(FT,sampleFileItem5,"LoadSample(44K_16B_M.AIFF)",cleanup);

	lineOutInstrumentItem	= LoadInstrument( "line_out.dsp", 1, 100 );
	CHECKRESULT_SETUP(FT,lineOutInstrumentItem,"LoadInstrument(line_out)",cleanup);

	theError	= ConnectInstrumentParts(	sampleInstrumentItem, "Output", 0, 
											lineOutInstrumentItem, "Input", 0 		);
	CHECKRESULT_SETUP(FT,theError,"ConnectInstrumentParts",cleanup);
							

	theError	= ConnectInstrumentParts(	sampleInstrumentItem, "Output", 0, 
											lineOutInstrumentItem, "Input", 1 		);
	CHECKRESULT_SETUP(FT,theError,"ConnectInstrumentParts",cleanup);

	StartInstrument( lineOutInstrumentItem, NULL );
	

	/**********************************************************************************
	 *	Test 1:	Test the sound player by using an event loop	
	 **********************************************************************************/

	LogComment("Test 1.  Create a soundplayer.  Play a large sample in a loop twice while testing with event loop");
	LogComment("         Press Button A to pause player, button B to resume player, and button X to exit loop");

	/*	Create the Sound Player.  For now we'll set it up with 2 buffers and buffer size
		of 65536 (capable holding the entire sound we are testing within one buffer.
		We'll let the function create its own buffers this time. The sound player will use
		sampleInstrumentItem instrument to play samples
	*/
	
	theError	= spCreatePlayer( &player, sampleInstrumentItem, 2, 65536, NULL );
	CHECKRESULT_SETUP(FT,theError,"spCreatePlayer",cleanup);
	
	/*	Specify the SoundFiles one wishes to play		*/
	theError	= spAddSample( &sound5, player, sampleFileItem5 );
	CHECKRESULT_SETUP(FT,theError,"spAddSample(44K_16B_M.AIFF)",cleanup);
	

	theError	= spClearDefaultDecisionFunction(player);
	CHECKRESULT_SETUP(FT,theError,"spClearDefaultDecisionFunction(player)",cleanup);

	/*	Play the sound two times
	*/
	spLoopSound( sound5 );
	counter	= 2;
	
	theError	=  spSetMarkerDecisionFunction(	sound5, SP_MARKER_NAME_END,
												(SPDecisionFunction)LoopDecisionFunction, &counter );
	CHECKRESULT_SETUP(FT,theError,"spSetMarkerDecisionFunction(sound5, SP_MARKER_NAME_END, LoopDecisionFunction, &counter)",cleanup);

	/*	Read the sound from disk or memory				*/
	theError	= spStartReading( sound5, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spStartReading",cleanup);

	/*	Start playing the sound						*/	
	theError	= spStartPlayingVA( player, AF_TAG_AMPLITUDE_FP, ConvertFP_TagData(1.0), TAG_END );
	CHECKRESULT_SETUP(FT,theError,"spStartPlayingVA",cleanup);

	playersigs		= spGetPlayerSignalMask( player );
	hasPlayerPaused	= FALSE;

	do
	{
		GetControlPad (1, FALSE, &cped);   
			
		if ( (cped.cped_ButtonBits & ControlA) == ControlA )
		{
			theError	= spPause( player );
			CHECKRESULT(FT,theError,"spPause");
			hasPlayerPaused	= TRUE;
		}
		else if ( (cped.cped_ButtonBits & ControlB) == ControlB )
		{
			theError	= spResume( player );
			CHECKRESULT(FT,theError,"spResume");
			hasPlayerPaused	= FALSE;
		}
		else
		{
			if ( (hasPlayerPaused == FALSE) && (spGetPlayerStatus(player) & SP_STATUS_F_BUFFER_ACTIVE) )
			{
				sigs	= WaitSignal(playersigs);
				spService(player, sigs);
			}
		}
	} while( ((cped.cped_ButtonBits & ControlX) == 0 ) && (spGetPlayerStatus(player) & SP_STATUS_F_BUFFER_ACTIVE) );
	
	spStop( player );

	BeginTest(FT, "Sound Player");
	LogResult(InteractiveVerify("You should have heard the sample twice. Did Test Pass?"), "");


	/**********************************************************************************
	 *	Test 2:	We play one small sample from memory	
	 **********************************************************************************/

	LogComment("Test 2.  Play one small sample from memory");
	
	/*	Specify the SoundFiles one wishes to play		*/
	theError	= spAddSample( &sound1, player, sampleFileItem1 );
	CHECKRESULT_SETUP(FT,theError,"spAddSample(Cowbell)",cleanup);
	
	
	/*	Read the sound from disk or memory				*/
	theError	= spStartReading( sound1, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spStartReading",cleanup);

	/*	Start playing the sound							*/	
	theError	= spStartPlayingVA( player, AF_TAG_AMPLITUDE_FP, ConvertFP_TagData(1.0), TAG_END );
	CHECKRESULT_SETUP(FT,theError,"spStartPlayingVA",cleanup);

	playersigs	= spGetPlayerSignalMask( player );
	
	while (spGetPlayerStatus(player) & SP_STATUS_F_BUFFER_ACTIVE)
	{
		sigs	= WaitSignal(playersigs);
		spService(player, sigs);
	}
	
	theError	=  spStop( player );
	CHECKRESULT_SETUP(FT,theError,"spStop",cleanup);

	BeginTest(FT, "Sound Player");
	LogResult(InteractiveVerify("You should have heard a small sample. Did Test Pass?"), "");


	/**********************************************************************************
	 *	Test 3:	Add several sample files and play them in a row	
	 **********************************************************************************/

	LogComment("Test 3.  Add four small sample sounds and play them in a row");

	/*	Add three more samples	to the player		*/
	theError	= spAddSample( &sound2, player, sampleFileItem2 );
	CHECKRESULT_SETUP(FT,theError,"spAddSample(Marimba)",cleanup);

	theError	= spAddSample( &sound3, player, sampleFileItem3 );
	CHECKRESULT_SETUP(FT,theError,"spAddSample(PanFlute)",cleanup);

	theError	= spAddSample( &sound4, player, sampleFileItem4 );
	CHECKRESULT_SETUP(FT,theError,"spAddSample(Trumpet)",cleanup);

	/*	Connect the samples together for playback	*/
	theError	=  spBranchAtMarker( sound1, SP_MARKER_NAME_END, sound2, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spBranchAtMarker(sound1, SP_MARKER_NAME_END, sound2, SP_MARKER_NAME_BEGIN)",cleanup);
	theError	=  spBranchAtMarker( sound2, SP_MARKER_NAME_END, sound3, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spBranchAtMarker(sound2, SP_MARKER_NAME_END, sound3, SP_MARKER_NAME_BEGIN)",cleanup);
	theError	=  spBranchAtMarker( sound3, SP_MARKER_NAME_END, sound4, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spBranchAtMarker(sound3, SP_MARKER_NAME_END, sound4, SP_MARKER_NAME_BEGIN)",cleanup);

	/*	Read the sound from disk or memory				*/
	theError	= spStartReading( sound1, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spStartReading",cleanup);

	/*	Start playing the sound						*/	
	theError	= spStartPlayingVA( player, AF_TAG_AMPLITUDE_FP, ConvertFP_TagData(1.0), TAG_END );
	CHECKRESULT_SETUP(FT,theError,"spStartPlayingVA",cleanup);

	playersigs	= spGetPlayerSignalMask( player );
	
	while (spGetPlayerStatus(player) & SP_STATUS_F_BUFFER_ACTIVE)
	{
		sigs	= WaitSignal(playersigs);
		spService(player, sigs);
	}
	
	spStop( player );

	BeginTest(FT, "Sound Player");
	LogResult(InteractiveVerify("You should have heard four samples in a row. Did Test Pass?"), "");
	
		
	/**********************************************************************************
	 *	Test 4:	Loop the sounds a fixed number of times using a decision functions	
	 **********************************************************************************/

	LogComment("Test 4.  Play the four small sample sounds in a loop four times");

	
	/*	Create the loop
	*/
	theError	=  spBranchAtMarker( sound4, SP_MARKER_NAME_END, sound1, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spBranchAtMarker(sound4, SP_MARKER_NAME_END, sound1, SP_MARKER_NAME_BEGIN)",cleanup);

	counter	= 4;


	/*	Set the functions which will be called during each marker
	*/
	theError	=  spSetMarkerDecisionFunction(	sound1, SP_MARKER_NAME_END,
												(SPDecisionFunction)M1DecisionFunction, &counter );
	CHECKRESULT_SETUP(FT,theError,"spSetMarkerDecisionFunction(sound1, SP_MARKER_NAME_END, M1DecisionFunction, &counter)",cleanup);

	theError	=  spSetMarkerDecisionFunction(	sound2, SP_MARKER_NAME_END,
												(SPDecisionFunction)M2DecisionFunction, &counter );
	CHECKRESULT_SETUP(FT,theError,"spSetMarkerDecisionFunction(sound2, SP_MARKER_NAME_END, M2DecisionFunction, &counter)",cleanup);

	theError	=  spSetMarkerDecisionFunction(	sound3, SP_MARKER_NAME_END,
												(SPDecisionFunction)M3DecisionFunction, &counter );
	CHECKRESULT_SETUP(FT,theError,"spSetMarkerDecisionFunction(sound3, SP_MARKER_NAME_END, M3DecisionFunction, &counter)",cleanup);

	theError	=  spSetMarkerDecisionFunction(	sound4, SP_MARKER_NAME_END,
												(SPDecisionFunction)LoopDecisionFunction, &counter );
	CHECKRESULT_SETUP(FT,theError,"spSetMarkerDecisionFunction(sound4, SP_MARKER_NAME_END, LoopDecisionFunction, &counter)",cleanup);

	theError	=  spSetDefaultDecisionFunction(	player,
												(SPDecisionFunction)GlobalDecisionFunction, &counter );
	CHECKRESULT_SETUP(FT,theError,"spSetDefaultDecisionFunction(player, GlobalDecisionFunction, &counter)",cleanup);



	/*	Read the sound from disk or memory				*/
	theError	= spStartReading( sound1, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spStartReading",cleanup);

	/*	Start playing the sound						*/	
	theError	= spStartPlayingVA( player, AF_TAG_AMPLITUDE_FP, ConvertFP_TagData(1.0), TAG_END );
	CHECKRESULT_SETUP(FT,theError,"spStartPlayingVA",cleanup);

	playersigs	= spGetPlayerSignalMask( player );
	
	while (spGetPlayerStatus(player) & SP_STATUS_F_BUFFER_ACTIVE)
	{
		sigs	= WaitSignal(playersigs);
		spService(player, sigs);
	}
	
	spStop( player );

	BeginTest(FT, "Sound Player");
	LogResult(InteractiveVerify("You should have heard four samples being played consecutively in a loop four times. Did Test Pass?"), "");


	/**********************************************************************************
	 *	Test 5:	Test the sound player by playing a sound file directly off the disc	
	 **********************************************************************************/

	LogComment("Test 5.  Play a sound file which is read directly from the disc instead of being loaded into memory");

	/*	Specify the SoundFiles one wishes to play		*/
	theError	= spAddSoundFile( &sound6, player, "LongFile_44K_16B_M.AIFF" );
	CHECKRESULT_SETUP(FT,theError,"spAddSoundFile(LongFile_44K_16B_M.AIFF)",cleanup);
	

	/*	Play the large sample file.
	*/

	/*	Read the sound from disk or memory				*/
	theError	= spStartReading( sound6, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spStartReading",cleanup);

	/*	Start playing the sound						*/	
	theError	= spStartPlayingVA( player, AF_TAG_AMPLITUDE_FP, ConvertFP_TagData(1.0), TAG_END );
	CHECKRESULT_SETUP(FT,theError,"spStartPlayingVA",cleanup);

	playersigs	= spGetPlayerSignalMask( player );
	
	while (spGetPlayerStatus(player) & SP_STATUS_F_BUFFER_ACTIVE)
	{
		sigs	= WaitSignal(playersigs);
		spService(player, sigs);
	}
	
	spStop( player );

	BeginTest(FT, "Sound Player");
	LogResult(InteractiveVerify("You should have heard the sample. Did Test Pass?"), "");


	/**********************************************************************************
	 *	Test 6:	Set markers to points within the sound file and let the sound player
	 *			skip through sections of the sound file
	 **********************************************************************************/

	/*	Set branches so that the sample file skips through the sample
	*/
	theError	=  spAddMarker( sound5, 16000, "marker1" );
	CHECKRESULT_SETUP(FT,theError,"spAddMarker(sound5, 16000, marker1)",cleanup);
	
	theError	=  spAddMarker( sound5, 44100, "marker2" );
	CHECKRESULT_SETUP(FT,theError,"spAddMarker(sound5, 44100, marker2)",cleanup);
	
	
	theError	=  spBranchAtMarker( sound5, "marker1", sound5, "marker2" );
	CHECKRESULT(FT,theError,"spBranchAtMarker(sound5, marker1, sound5, marker2)");


	LogComment("Test 6.  Skip from the beginning to the middle of a sound file.");

	/*	Read the sound from disk or memory				*/
	theError	= spStartReading( sound5, SP_MARKER_NAME_BEGIN );
	CHECKRESULT_SETUP(FT,theError,"spStartReading",cleanup);

	/*	Start playing the sound						*/	
	theError	= spStartPlayingVA( player, AF_TAG_AMPLITUDE_FP, ConvertFP_TagData(1.0), TAG_END );
	CHECKRESULT_SETUP(FT,theError,"spStartPlayingVA",cleanup);

	playersigs	= spGetPlayerSignalMask( player );
	
	while (spGetPlayerStatus(player) & SP_STATUS_F_BUFFER_ACTIVE)
	{
		sigs	= WaitSignal(playersigs);
		spService(player, sigs);
	}
	
	spStop( player );

	BeginTest(FT, "Sound Player");
	LogResult(InteractiveVerify("You should have heard the sample skip ahead while playing. Did Test Pass?"), "");


	/*	Clean up memory
	*/
	spDeletePlayer( player );
	
	UnloadInstrument( sampleInstrumentItem );
	UnloadInstrument( lineOutInstrumentItem );
	
	UnloadSample(sampleFileItem1);
	UnloadSample(sampleFileItem2);
	UnloadSample(sampleFileItem3);
	UnloadSample(sampleFileItem4);
	UnloadSample(sampleFileItem5);
	
	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(FT,theError,"CloseAudioFolio)");
	
	KillEventUtility();

	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

Err LoopDecisionFunction(	SPAction *resultAction, int32 *remainingCount,
							SPSound *sound, const char *markerName 			)
{
	TOUCH(markerName);
	TOUCH(sound);	
	(*remainingCount)--;
	
	if ( *remainingCount <= 0 )
		return spSetStopAction( resultAction );
	else
		return 0;
}

Err M1DecisionFunction(	SPAction *resultAction, int32 *remainingCount,
						SPSound *sound, const char *markerName 			)
{
	Err	theError;
	
	if ( *remainingCount == 4  )
	{
		LogComment("%d. Performing Decision Function for sound1 marker - Clear this marker's decision function.", *remainingCount);
		theError	= spClearMarkerDecisionFunction( sound, markerName );
		CHECKRESULT(FT,theError,"spClearMarkerDecisionFunction(sound1)");
		return ( spSetBranchAction(resultAction, sound2, SP_MARKER_NAME_BEGIN) );
	}
	else
		return 1;
}


Err M2DecisionFunction(	SPAction *resultAction, int32 *remainingCount,
						SPSound *sound, const char *markerName 			)
{
	Err	theError;

	if ( *remainingCount == 4  )
	{
		LogComment("%d. Performing Decision Function for sound2 marker - Clear this marker's decision function.", *remainingCount);
		theError	= spClearMarkerDecisionFunction( sound, markerName );
		CHECKRESULT(FT,theError,"spClearMarkerDecisionFunction(sound2)");
		return ( spSetBranchAction(resultAction, sound3, SP_MARKER_NAME_BEGIN) );
	}
	else
		return 1;
}


Err M3DecisionFunction(	SPAction *resultAction, int32 *remainingCount,
						SPSound *sound, const char *markerName 			)
{
	Err	theError;

	if ( *remainingCount == 4  )
	{
		LogComment("%d. Performing Decision Function for sound3 marker - Clear this marker's decision function.", *remainingCount);
		theError	= spClearMarkerDecisionFunction( sound, markerName );
		CHECKRESULT(FT,theError,"spClearMarkerDecisionFunction(sound3)");
		return ( spSetBranchAction(resultAction, sound4, SP_MARKER_NAME_BEGIN) );
	}
	else
		return 1;
}


Err GlobalDecisionFunction(	SPAction *resultAction, int32 *remainingCount,
							SPSound *sound, const char *markerName 			)
{
	TOUCH(markerName);
	TOUCH(sound);
	TOUCH(resultAction);
	LogComment("%d. Performing Global Decision Function", *remainingCount );
	return (1);
}
