/* @(#) Sound3D_FT2.c 96/07/29 1.2 */
/*	
 *	File:				Sound3D_FT2.c
 *	Author:				Eric L. Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	4/3/96
 *
 *	Version:			M2 2.0
 *
 *	This file tests 3DSound devices created using different flags in the Create3DSoundVA calls.
 *	A 3DSound device is created by calling Create3DSoundVA with one of the available flags. The flags
 *	which are tested are:
 *
 *		S3D_F_DOPPLER;
 *		S3D_F_DISTANCE_AMPLITUDE_SQUARE;
 *		S3D_F_DISTANCE_AMPLITUDE_SONE;
 *		S3D_F_PAN_DELAY;
 *		S3D_F_PAN_AMPLITUDE;
 *		S3D_F_PAN_FILTER;
 *		S3D_F_OUTPUT_HEADPHONES;
 *		S3D_F_SMOOTH_AMPLITUDE;
 *
 *	Two tests are performed on the newly created 3DSound device.  In test 1, the sound is revolved around the user's
 *	head.  In test 2, the sound is moved farther and farther away from the user's position.  The user is verifying
 *	that 3DSound devices work (creates sound) no matter the flag.  Some flags will not make any perceptible difference
 *	in the sound to the user.  This test works best with headphones.
 *
 *	Audio Functions called:
 *
 *		CloseAudioFolio		CreateCue			CreateAttachment		LoadInstrument			LoadSample
 *		OpenAudioFolio		StartInstrument		StopInstrument			UnloadSample
 *		SleepUntilTime		DeleteAttachment	DeleteCue				UnloadInstrument
 *		
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */



#include <audio/audio.h>
#include <audio/parse_aiff.h>
#include <audio/sound3d.h>
#include <stdio.h>
#include <stdlib.h>

#include <TestMacros.h>
#include <tests/3do_scaf.h>



/**************************************Test Environment Globals****************/
/******************************************************************************/

int main (int argc, char ** argv)
{
	
	Err				theError;
	int				i;
	int32			theSignal;
	Item 			theSample, sampleInstrument, theCue, sleepCue, sampleAttachment, lineOut,
					ampKnob, freqKnob, sound3dInstrument;
	PolarPosition4D	startPosition, endPosition;
	Sound3D			*theSound;
	Sound3DParms 	theSoundParms;
	uint32			flags[15];


	InitScaffold(AUDIO, argc, argv);					/* initializes this program */

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(FT, theError,"OpenAudioFolio", cleanup);
	
	theCue	= CreateCue( NULL );
	CHECKRESULT_SETUP(FT, theCue,"CreateCue", cleanup);
	
	sleepCue	= CreateCue( NULL );
	CHECKRESULT_SETUP(FT, sleepCue,"CreateCue", cleanup);
	

	/*	Load the instrument which will play the sample to be written to the delay line,  load the
		sample, and attach them
	*/
	sampleInstrument = LoadInstrument("sampler_16_v1.dsp", 0, 100);
	CHECKRESULT_SETUP(FT, sampleInstrument,"LoadInstrument(sampler_16_v1.dsp)", cleanup);

	ampKnob			= CreateKnob( sampleInstrument, "Amplitude", NULL );
	CHECKRESULT_SETUP(FT, ampKnob,"CreateKnob(sampler_16_v1.dsp)", cleanup);
	
	freqKnob			= CreateKnob( sampleInstrument, "SampleRate", NULL );
	CHECKRESULT_SETUP(FT, freqKnob,"CreateKnob(sampler_16_v1.dsp)", cleanup);
	
	
	theSample = LoadSample("LongFile_44K_16B_M.AIFF");
	CHECKRESULT_SETUP(FT, theSample,"LoadSample(LongFile_44K_16B_M.AIFF)", cleanup);

	sampleAttachment = CreateAttachment( sampleInstrument, theSample, NULL );
	CHECKRESULT_SETUP(FT, sampleAttachment,"CreateAttachment(sampleInstrument, theSample", cleanup);

	MonitorAttachment( sampleAttachment, theCue, CUE_AT_END );

	theSignal	= GetCueSignal( theCue );
	CHECKRESULT_SETUP(FT, theSignal,"GetCueSignal", cleanup);

	/*	Load the line_out Instrument
	*/
	lineOut = LoadInstrument("line_out.dsp", 0, 100);
	CHECKRESULT_SETUP(FT, lineOut,"LoadInstrument(line_out)", cleanup);


flags[0]	= S3D_F_DOPPLER;
flags[1]	= S3D_F_DISTANCE_AMPLITUDE_SQUARE;
flags[2]	= S3D_F_DISTANCE_AMPLITUDE_SONE;
flags[3]	= S3D_F_PAN_DELAY;
flags[4]	= S3D_F_PAN_AMPLITUDE;
flags[5]	= S3D_F_PAN_FILTER;
flags[6]	= S3D_F_OUTPUT_HEADPHONES;
flags[7]	= S3D_F_SMOOTH_AMPLITUDE;

for ( i = 0; i < 9; i++ )
{
	if ( i == 8 )
	{
 		theError	= Create3DSoundVA( &theSound, S3D_TAG_BACK_AMPLITUDE, ConvertFP_TagData(0.0), TAG_END);
		CHECKRESULT_SETUP(FT, theError,"Create3DSound( theSound, NULL )", cleanup);
	}
	else
	{
 		theError	= Create3DSoundVA( &theSound, S3D_TAG_FLAGS, flags[i], TAG_END);
		CHECKRESULT_SETUP(FT, theError,"Create3DSound( theSound, NULL )", cleanup);
	}

	sound3dInstrument = Get3DSoundInstrument( theSound );


	/*	Connect all of the instruments to the mixer
	*/
	theError	= ConnectInstrumentParts(	sampleInstrument, "Output", 0, sound3dInstrument, "Input", 0 		);
	CHECKRESULT_SETUP(FT, theError,"ConnectInstrumentParts(sampleInstrument Output 0, sound3dInstrument Input 0)", cleanup);

	theError	= ConnectInstrumentParts(	sound3dInstrument, "Output", 0, lineOut, "Input", 0 		);
	CHECKRESULT_SETUP(FT, theError,"ConnectInstrumentParts(sound3dInstrument Output 0, lineOut Input 0)", cleanup);

	theError	= ConnectInstrumentParts(	sound3dInstrument, "Output", 1, lineOut, "Input", 1 		);
	CHECKRESULT_SETUP(FT, theError,"ConnectInstrumentParts(sound3dInstrument Output 1, lineOut Input 1)", cleanup);


	/*	Initialize the position of the sound		*/
   startPosition.pp4d_Radius		= 0;	/* in frame ticks */
   startPosition.pp4d_Theta			= 0;	/* in radians */
   startPosition.pp4d_Phi			= 0;	/* in radians */
   startPosition.pp4d_Time			= 0;	/* in frame ticks */
   startPosition.pp4d_or_Theta		= 0;	/* orientation, in radians */
   startPosition.pp4d_or_Phi		= 0;	/* orientation, in radians */


	/*	Initialize the end position of the sound		*/
   endPosition.pp4d_Radius			= 0;							/* in frame ticks */
   endPosition.pp4d_Theta			= 0;							/* in radians */
   endPosition.pp4d_Phi				= 0;							/* in radians */
   endPosition.pp4d_Time			= 0;							/* in frame ticks */
   endPosition.pp4d_or_Theta		= 0;							/* orientation, in radians */
   endPosition.pp4d_or_Phi			= 0;							/* orientation, in radians */


	theError	= StartInstrument( lineOut, NULL );
	CHECKRESULT_SETUP(FT, theError,"StartInstrument(lineOut, NULL)", cleanup);

	theError	= StartInstrument( sampleInstrument, NULL ); 
	CHECKRESULT_SETUP(FT, theError,"StartInstrument(sampleInstrument, NULL)", cleanup);

	theError	= Start3DSound( theSound, &startPosition ); 
	CHECKRESULT_SETUP(FT, theError,"Start3DSound(sampleInstrument, NULL)", cleanup);


	while ( (GetCurrentSignals() & theSignal) != theSignal )
	{
		SleepUntilTime( sleepCue, GetAudioTime() + 720 );

		endPosition.pp4d_Theta			+= .79;							

		LogComment( "Theta is now %f radians", endPosition.pp4d_Theta );
		
		theError	=  Move3DSound( theSound, &startPosition, &endPosition );
		CHECKRESULT_SETUP(FT, theError,"Move3DSound()", cleanup);

		startPosition.pp4d_Theta			= endPosition.pp4d_Theta;
	}	
	startPosition.pp4d_Theta	= endPosition.pp4d_Theta	= 0;
	ClearCurrentSignals( theSignal );
	
	BeginTest(FT, "Sound3D");
	LogResult(InteractiveVerify("Did the sound move according to the Sound3D specifications. Did Test Pass?"), "");



	printf("\nStart playing 3D Sound\n\n" );

	theError	= StartInstrument( lineOut, NULL );
	CHECKRESULT_SETUP(FT, theError,"StartInstrument(lineOut, NULL)", cleanup);

	theError	= StartInstrument( sampleInstrument, NULL ); 
	CHECKRESULT_SETUP(FT, theError,"StartInstrument(sampleInstrument, NULL)", cleanup);

	theError	= Start3DSound( theSound, &startPosition ); 
	CHECKRESULT_SETUP(FT, theError,"Start3DSound(sampleInstrument, NULL)", cleanup);


	while ( (GetCurrentSignals() & theSignal) != theSignal )
	{
		SleepUntilTime( sleepCue, GetAudioTime() + 720 );

		endPosition.pp4d_Radius			+= 100.0;							

		LogComment( "Radius is now %f frames", endPosition.pp4d_Radius );
		
		theError	=  Move3DSound( theSound, &startPosition, &endPosition );
		CHECKRESULT_SETUP(FT, theError,"Move3DSound()", cleanup);

 		theError	=  Get3DSoundParms( theSound, &theSoundParms, sizeof(theSoundParms) );
		CHECKRESULT_SETUP(FT, theError,"Get3DSoundParms()", cleanup);

		SetKnob( ampKnob, theSoundParms.s3dp_DistanceFactor );
		 
		startPosition.pp4d_Radius			= endPosition.pp4d_Radius;
	}	
	startPosition.pp4d_Radius	= endPosition.pp4d_Radius	= 0;
	ClearCurrentSignals( theSignal );
	
	BeginTest(FT, "Sound3D");
	LogResult(InteractiveVerify("Did the sound move according to the Sound3D specifications. Did Test Pass?"), "");

	theError	= Stop3DSound( theSound );
	CHECKRESULT_SETUP(FT, theError,"Stop3DSound()", cleanup);

	StopInstrument( lineOut, NULL );
	StopInstrument( sampleInstrument, NULL );

	Delete3DSound( &theSound );
}	
	
	DeleteCue( theCue );
	DeleteCue( sleepCue );
	DeleteAttachment( sampleAttachment );

	UnloadSample( theSample );
	UnloadInstrument( sampleInstrument );
	UnloadInstrument( lineOut );

cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(FT, theError,"CloseAudioFolio");

	/******************************************************************************/
	TermScaffold();

	return 0;
}
