/**********************************************************************************************
**
**  Copyright (c) 1993-1996, an unpublished work by The 3DO Company.
**  All rights reserved. This material contains confidential
**  information that is the property of The 3DO Company. Any
**  unauthorized duplication, disclosure or use is prohibited.
**
**  CLT integration test
**  Written by:		       Paige Wuestefeld  
**
*************************************************************************************************/

/****************************************************************************************/
/* @(#) cltIT1.c 96/10/29 1.7								*/
/* This program tests 2 of the CLT Vertex commands and many of the GState api calls 	*/
/* If an error occurs in any of the API calls a warning message is printed and the	*/
/* program terminates. The first display is a shaded triangle translating around XY.	*/
/* The triangle wraps around in that when one vertice reaches the edge of the screen	*/
/* it is repositioned.  To exit this display Type A.  The second display is a shaded	*/
/* shaded triangle FAN with W values - this does not yield perspective correction	*/
/* but allows W buffering.  To exit this display Type C.				*/
/*											*/
/* This program also tests many GState API calls, in the absence of any warnings	*/
/* the GState tests have passed. In addition a memory check is done to ensure that 	*/
/* memory which is allocated by GState is returned.					*/
/****************************************************************************************/

#ifdef MACINTOSH
#include <stdio.h>
#include <stdlib.h>
#include <kernel:types.h>
#include <graphics:clt:clt.h>
#include <graphics:clt:gstate.h>
#include <graphics:graphics.h>
#include <graphics:view.h>
#include <kernel:io.h>
#include <kernel:mem.h>
#include <kernel:random.h>
#include <assert.h>
#include <misc:event.h>
#else
#include <stdio.h>
#include <stdlib.h>
#include <kernel/types.h>
#include <graphics/clt/clt.h>
#include <graphics/clt/gstate.h>
#include <graphics/graphics.h>
#include <graphics/view.h>
#include <kernel/io.h>
#include <kernel/mem.h>
#include <kernel/random.h>
#include <assert.h>
#include <misc/event.h>
#endif

/* standard error checking */
#define CKERR(x,y) {Err ick=(x);if(ick<0){printf("Error - %s\n",y); PrintfSysErr(x); exit(1);}}

#define TRISPERLIST 1
#define MAXVERT	75

#define redv	0.0
#define greenv	0.0
#define bluev	1.0

void RndVert(float *x, float *y, float *r, float *g, float *b);

void main(void)
{

gfloat					x[MAXVERT], y[MAXVERT], r[MAXVERT], g[MAXVERT], b[MAXVERT];
gfloat					a, wcorrect1, wcorrect2, r1, g1, b1;
int32					result;
uint32					buttons;
ControlPadEventData			cpad;
Err					err;
int32					buffSignal, gsCount;
int 					cnt;
GState 					*gs;     
Item					viewItem; 
Item 					bitmaps[3]; 
uint32					fbindex = 0;
int					frame=0;
int32					initialmem, finalmem;
MemInfo					meminfo;


 	GetMemInfo(&meminfo, sizeof(MemInfo), MEMTYPE_NORMAL);
	initialmem=meminfo.minfo_TaskAllocatedBytes;
     
	result = InitEventUtility(1,0,LC_ISFOCUSED);
	CKERR( result, "InitEventUtil");

	buffSignal = AllocSignal(0);

	err = OpenGraphicsFolio (); 						
	CKERR( err, "OpenGraphicsFolio");

	gs= GS_Create(); 							
	if (gs==NULL) {printf("Unable to create GState\n"); exit(1);}

	gsCount= GS_GetCount(gs);
	if (gsCount != 0) printf("Error in GS_GetCount\n");
	
        err = GS_AllocBitmaps(bitmaps, 320, 240, BMTYPE_16 , 2, 1); 
	CKERR( err, "AllocBitmaps");

  	err = GS_AllocLists(gs, 2, (32*1024));
	CKERR( err, "AllocLists");
	
	err=GS_SetZBuffer(gs, bitmaps[2]);
	CKERR( err, "GSSetZBuffer");
	
	if (GS_GetZBuffer(gs) != bitmaps[2]) printf("Error in GS_GetZBuffer \n"); 
	
	err=GS_SetVidSignal(gs, buffSignal);
	CKERR( err, "GSSetVidSignal");

	if (GS_GetVidSignal(gs) != buffSignal) printf("Error in GS_GetVidSignal \n"); 	
	
	err=GS_SetDestBuffer(gs, bitmaps[0]); 	
	CKERR( err, "GSSetDestBuffer");

	if (GS_GetDestBuffer(gs) != bitmaps[0]) printf("Error in GS_GetDestBuffer \n"); 
	
	/* create a view item */
	viewItem = CreateItemVA(MKNODEID(NST_GRAPHICS, GFX_VIEW_NODE),
							VIEWTAG_VIEWTYPE, VIEWTYPE_16,
							VIEWTAG_BITMAP, bitmaps[0],
							VIEWTAG_RENDERSIGNAL, buffSignal,
							TAG_END );

	AddViewToViewList( viewItem, 0 );

	err=GS_SetView(gs, viewItem); 	
	CKERR( err, "GSSetView");

	/* check status of a few GState api commands */

	/* these should be equal until we stick something in the command list */
	if ( GS_GetCurListStart(gs) != (gs->gs_ListPtr) ) printf("Error in GS_GetCurListStart\n");

	/* this index should be between 0 and (number of lists allocated-1) */
	if ( GS_GetCmdListIndex(gs) != 0 ) printf("Error in GS_GetCmdListIndex 1 \n");

	printf("\n\nBeginning with VertexRgba test - single triangle - To end Press A on the controller.\n\n");

	/* set necessary registers - turn on perspective, turn off texturing, destination */
	/* and source blending and set discard control register 	*/
	
	CLT_ESCNTL( GS_Ptr(gs), 0, 0, 0);

 	/* use global snippet to turn off texturing */
	CLT_CopySnippetData( GS_Ptr(gs), &CltNoTextureSnippet);

	CLT_DBUSERCONTROL(GS_Ptr(gs), 0, 0, 0, 0, 0, 0, 0, 
					 CLT_Const(DBUSERCONTROL, DESTOUTMASK, ALPHA) |
					 CLT_Const(DBUSERCONTROL, DESTOUTMASK, RED) |
					 CLT_Const(DBUSERCONTROL, DESTOUTMASK, GREEN) |
					 CLT_Const(DBUSERCONTROL, DESTOUTMASK, BLUE) );
					 
	CLT_DBDISCARDCONTROL(GS_Ptr(gs), 0, 0, 1, 1);				 

/********************************************************************/
/* Simple VertexRgba tests			STRIP		    */
/********************************************************************/

	/* get vertice values */
	RndVert(x, y, r, g, b);

	while(1) 
	  {
		frame++;
		/* marks beginning of frame - needed for tear-free double buffering */
		GS_BeginFrame(gs);

		/* clear frame buffer to black start fresh 			*/
		/* WARNING: ClearFrameBuffer changes some register states	*/
		/* which may need to be reset - such as turning off texturing	*/

		CLT_ClearFrameBuffer(gs, 0.0, 0.0, 0.0, 1.0, TRUE, TRUE);
		CLT_Sync(GS_Ptr(gs));

		/* translate triangles in X and Y with wraparound */
	
		for (cnt=0; cnt<(TRISPERLIST*3); cnt++) {
			x[cnt]+=1; if (x[cnt]>310) x[cnt]=1;		
			y[cnt]+=1; if (y[cnt]>230) y[cnt]=1;
			}

		/* triangle has shaded vertices */
		CLT_TRIANGLE(GS_Ptr(gs), 1, RC_STRIP, 0, 0, 1, (3*TRISPERLIST));

		cnt=0;
		while (cnt<(TRISPERLIST*3)) {
       		CLT_VertexRgba(GS_Ptr(gs), x[cnt], y[cnt], r[cnt], g[cnt], b[cnt], 1.0); cnt++;
       		CLT_VertexRgba(GS_Ptr(gs), x[cnt], y[cnt], r[cnt], g[cnt], b[cnt], 1.0); cnt++;
       		CLT_VertexRgba(GS_Ptr(gs), x[cnt], y[cnt], r[cnt], g[cnt], b[cnt], 1.0); cnt++;
			}
		
		CLT_Sync(GS_Ptr(gs));
	 
		/* send command list to TE */
		err=GS_SendList(gs);
		CKERR( err, "SendList");

		err=GS_WaitIO(gs);
		CKERR( err, "GS_WaitIO");

		GS_EndFrame(gs);

		fbindex = 1 - fbindex;
		GS_SetDestBuffer(gs, bitmaps[fbindex]);

		GetControlPad(1, FALSE, &cpad);
		buttons = cpad.cped_ButtonBits;
		if( buttons & ControlA )
		break;

     }	

printf("Finished with shaded vertices \n");

/* check status of a few GState variables */

if (GS_GetCount(gs) != frame) printf("Error in GS_GetCount != framecount\n");

/* odd framecount should yield index of 1 */
if ( frame % 2 != 0 ) {
	if ( GS_GetCmdListIndex(gs) != 1 ) printf("Error in GS_GetCmdListIndex \n");}

if ( GS_GetCmdListIndex(gs) == 1 ) {
		GS_SetList(gs, 0);
		if ( GS_GetCmdListIndex(gs) != 0 ) printf("Error in GS_SetList \n"); 
		}
else {
	GS_SetList(gs, 1);
	if ( GS_GetCmdListIndex(gs) != 1 ) printf("Error in GS_SetList \n"); 
	}

printf("\n\nBeginning with VertexRgbaW test FAN - To end Press C on the controller.\n\n");

/********************************************************************/
/* Simple VertexRgbaW tests			FAN	  	    */
/********************************************************************/

	CLT_ClearFrameBuffer(gs, 0.0, 0.0, 0.0, 1.0, TRUE, TRUE);
	CLT_Sync(GS_Ptr(gs));

	r1=1.0;
	g1=0.0;
	b1=0.0;
	a= 1.0;

	/* use 2 different perspective correction values		*/
	/* this does not actually correct but functions as a w buffer 	*/
	wcorrect1=0.98;				
	wcorrect2=0.02;		

	while(1) 
	  {

		GS_BeginFrame(gs);

		CLT_ClearFrameBuffer(gs, 0.0, 0.0, 0.0, 1.0, TRUE, TRUE);
		CLT_Sync(GS_Ptr(gs));

		/* triangle is a fan with shading and perspective enabled */
       	CLT_TRIANGLE(GS_Ptr(gs), 1, RC_FAN, 1, 0, 1, 7 );
		
       	CLT_VertexRgbaW(GS_Ptr(gs), 160, 170, 1.0, 1.0, 1.0, a, wcorrect1);	/* common first */
       	CLT_VertexRgbaW(GS_Ptr(gs), 120, 170, r1, g1, b1, a, wcorrect1);
       	CLT_VertexRgbaW(GS_Ptr(gs), 120, 135, r1, g1, 1.0, a, wcorrect2);
       	CLT_VertexRgbaW(GS_Ptr(gs), 150, 120, r1, g1, 1.0, a, wcorrect2);
       	CLT_VertexRgbaW(GS_Ptr(gs), 180, 120, r1, g1, 1.0, a, wcorrect2);
       	CLT_VertexRgbaW(GS_Ptr(gs), 210, 135, r1, g1, 1.0, a, wcorrect2);
       	CLT_VertexRgbaW(GS_Ptr(gs), 210, 170, r1, g1, b1, a, wcorrect1);

       	CLT_Sync(GS_Ptr(gs));
	 
	/* send command list to TE */
     	err=GS_SendList(gs);
	CKERR( err, "SendList");

	GS_EndFrame(gs);

	fbindex = 1 - fbindex;
	GS_SetDestBuffer(gs, bitmaps[fbindex]);

	GetControlPad(1, FALSE, &cpad);
	buttons = cpad.cped_ButtonBits;
	if( buttons & ControlC )
		break;

     }	

printf("Finished with fan. \n\n");

/* Free memory associated with views, bitmaps, etc */

	RemoveView(viewItem);

	err=DeleteItem(viewItem);
	CKERR( err, "DeleteItem");

	err=GS_FreeBitmaps(bitmaps, 3);
	CKERR( err, "FreeBitmaps");

	err=GS_FreeLists(gs);
	CKERR( err, "FreeLists");

	err=GS_Delete(gs);
	CKERR( err, "DeleteItem");

	err=KillEventUtility();
	CKERR( err, "KillEventUtil");
	
	err=CloseGraphicsFolio();
	CKERR( err, "CloseGraphicsFolio");

 	GetMemInfo(&meminfo, sizeof(MemInfo), MEMTYPE_NORMAL);
	finalmem=meminfo.minfo_TaskAllocatedBytes;
	if (initialmem != finalmem) printf("Memory not returned %d\n", initialmem-finalmem);

printf("Finished! \n");


 }

void RndVert(float *x, float *y, float *r, float *g, float *b)
{

int cnt;

/* generates initial vertex values and alpha for each triangle 	*/
/* blue with randomly assigned initial vertices					*/

for (cnt=0; cnt<(TRISPERLIST*3); cnt++) {
	x[cnt]=(gfloat)(rand()%320);		/* random x, y values */
	y[cnt]=(gfloat)(rand()%240);
	r[cnt]= redv;				/* r, g, b vlaues as defined */
	g[cnt]= greenv;
	b[cnt]= bluev;
	}

}





