/**********************************************************************************************
**
**  Copyright (c) 1993-1996, an unpublished work by The 3DO Company.
**  All rights reserved. This material contains confidential
**  information that is the property of The 3DO Company. Any
**  unauthorized duplication, disclosure or use is prohibited.
**
**  CLT integration test
**  Written by:		       Paige Wuestefeld  
**
*************************************************************************************************/

/************************************************************************************************/
/* @(#) cltIT2.c 96/10/29 1.6									*/
/* This program displays a textured fan with perspective correction and a simple		*/
/* translucent triangle as a test of source destination blending.  It creates			*/
/* and uses CLT Snippets to set up the proper register values for the triangle			*/
/* and also uses those available global snippets.						*/
/*												*/
/* It uses the utility program texload.c to load in the textures.				*/
/*												*/
/* The translucent triangle travels along Y and is randomly redrawn when it hits 		*/
/* the bitmap boundary. It's coloring is pink and it has varying degrees of transparency	*/
/* at each vertex. The fan has a brick texture which has perspective correction			*/
/*												*/
/* In the absence of any warnings, the GState tests have passed.				*/
/************************************************************************************************/

#ifdef MACINTOSH
#include <stdio.h>
#include <stdlib.h>
#include <kernel:types.h>
#include <graphics:clt:clt.h>
#include <graphics:clt:gstate.h>
#include <graphics:graphics.h>
#include <graphics:view.h>
#include <kernel:io.h>
#include <kernel:mem.h>
#include <kernel:random.h>
#include <assert.h>
#include <misc:event.h>
#else
#include <stdio.h>
#include <stdlib.h>
#include <kernel/types.h>
#include <graphics/clt/clt.h>
#include <graphics/clt/gstate.h>
#include <graphics/graphics.h>
#include <graphics/view.h>
#include <kernel/io.h>
#include <kernel/mem.h>
#include <kernel/random.h>
#include <assert.h>
#include <misc/event.h>
#endif

#include "texload.h"

#define CKERR(x,y) {Err ick=(x);if(ick<0){printf("Error - %s\n",y);PrintfSysErr(x); exit(1);}}

#define NUMSNIP 100
#define MAXVERT	75
#define BMWIDTH 320
#define BMHEIGHT 240
#define numvert 7
#define mySize 90

typedef	struct	mycoords {
	gfloat	x;
	gfloat	y;
	gfloat	u;
	gfloat	v;
	gfloat	w;
} mycoords;

void main(int argc, char **argv)

{

mycoords*				myvtx;

float					xs, ys, dev1, dev2;
int32					result, flag;
uint32					buttons;
ControlPadEventData			cpad;
Err					err;
int32					buffSignal;
int 					cnt;
GState 					*gs;     
Item					viewItem; 
Item 					bitmaps[3]; 
uint32					fbindex = 0;
CltSnippet*				mysnippet;
gfloat					TriX[4], TriY[4];
TextureSnippets 			snips;		/* hold cmd lists to load the texture */

TOUCH(argc);
TOUCH(argv);

myvtx=AllocMem(((numvert+2)*sizeof(mycoords)), MEMTYPE_NORMAL);
if (myvtx==NULL) printf("Memory Allocation Failure\n");
	
result = InitEventUtility(1,0,LC_ISFOCUSED);
CKERR( result, "InitEventUtility");

buffSignal = AllocSignal(0);

err = OpenGraphicsFolio (); 						
CKERR( err, "OpenGraphicsFolio");

gs= GS_Create(); 							
if (gs==NULL) {printf("Unable to create GState\n"); exit(1);}

err = GS_AllocBitmaps(bitmaps, BMWIDTH, BMHEIGHT, BMTYPE_16 , 2, 1); 
CKERR( err, "AllocBitmaps");

err = GS_AllocLists(gs, 2, (32*1024));
CKERR( err, "AllocLists");

err=GS_SetZBuffer(gs, bitmaps[2]);
CKERR( err, "GSSetZBuffer");
	
err=GS_SetVidSignal(gs, buffSignal);
CKERR( err, "GSSetVidSignal");
	
err=GS_SetDestBuffer(gs, bitmaps[0]); 	
CKERR( err, "GSSetDestBuffer");
	
/* create a view item and assoc signal bit for tear-free rendering */
viewItem = CreateItemVA(MKNODEID(NST_GRAPHICS, GFX_VIEW_NODE),
						VIEWTAG_VIEWTYPE, VIEWTYPE_16,
						VIEWTAG_BITMAP, bitmaps[0],
						VIEWTAG_RENDERSIGNAL, buffSignal,
						TAG_END );

AddViewToViewList( viewItem, 0 );

err=GS_SetView(gs, viewItem); 	
CKERR( err, "GSSetView");
	

/********************************************************************/
/* Simple VertexRgbaUvW test	TEXTURED FAN			    */
/********************************************************************/

/* Load texture from disk into TRAM, including its PIP if there is one */
CLT_InitSnippet( &snips.dab );
CLT_InitSnippet( &snips.tab );
snips.lcb = NULL;
snips.pip = NULL;
snips.txdata = NULL;

LoadTexture( &snips, ("brick.utf") );
UseTxLoadCmdLists( gs, &snips );
	
xs=ys=((snips.lcb->XSize * snips.txdata->maxLOD * 2) - 1.0);	/* max X of LOD 0 */

dev1=45.0;
dev2=15.0;

/* initial values for textured FAN (rectangular) with perspective correction 		*/
/* to get perspective correction working with textures you need to send u*w and v*w 	*/
/* also on vertices 3, 4, 5, 6 there is a little "cheat" to position the far points	*/
/* it is not meant to be used in serious endeavor-a real projection should be used 	*/

myvtx->w=0.98;	
myvtx->x=165.0;	
myvtx->y=170.0;	
myvtx->u=xs*.5*myvtx->w; 
myvtx->v=ys*myvtx->w;
myvtx++;

myvtx->w=0.98;	myvtx->x=120.0;	myvtx->y=170.0;	myvtx->u=0.0*myvtx->w;	myvtx->v=ys*myvtx->w;	
myvtx++;

myvtx->w=0.80;	
myvtx->x=120.0+((dev1*myvtx->w)/5.0); 
myvtx->y=120.0; 
myvtx->u=0.0*myvtx->w; 
myvtx->v=0.0*myvtx->w;			
myvtx++;

myvtx->w=0.80;	
myvtx->x=150.0+((dev2*myvtx->w)/5.0); 
myvtx->y=120.0;
myvtx->u=xs*.33*myvtx->w; 
myvtx->v=0.0*myvtx->w;			
myvtx++;

myvtx->w=0.80;	
myvtx->x=180.0-((dev2*myvtx->w)/5.0); 
myvtx->y=120.0;
myvtx->u=xs*.67*myvtx->w; 
myvtx->v=0.0*myvtx->w;			
myvtx++;

myvtx->w=0.80;		
myvtx->x=210.0-((dev1*myvtx->w)/5.0); 	
myvtx->y=120.0; 		
myvtx->u=xs*myvtx->w; 		
myvtx->v=0.0*myvtx->w;			
myvtx++;

myvtx->w=0.98;		
myvtx->x=210.0; 	
myvtx->y=170.0; 		
myvtx->u=xs*myvtx->w; 		
myvtx->v=ys*myvtx->w;			
myvtx++;

flag=0;							/* flag for movement direction */

/* vertices for the simple triangle */
for (cnt=0; cnt<3; cnt++) {					
TriX[cnt] = (gfloat)(rand()%320);	
TriY[cnt] = (gfloat)(rand()%240);
}

/********************************************************************************/
/* create a CLT snippet for my needed register values to create a		*/
/* translucent destination blending case 					*/
/********************************************************************************/

	mysnippet=AllocMem( (NUMSNIP*(sizeof(CltSnippet)) ), MEMTYPE_NORMAL);
		if (mysnippet==NULL) printf("Failed to allocate memory for snippet\n");

	CLT_InitSnippet(mysnippet);
	err=CLT_AllocSnippet(mysnippet, NUMSNIP); /* NUMSNIP words may be put into mysnippet */
	CKERR( err, "AllocSnippet");

	mysnippet->size=4 * 2;	/* four registers set; 2 writes per register - header, value */

	/* enable destination blending and source blending, z buffering but not  */
	/* z buffering output - for transparency to work correctly		*/
	CLT_DBUSERCONTROL(&(mysnippet->data), 1, 0, 0, 0, 1, 1, 0,
			  CLT_Const(DBUSERCONTROL, DESTOUTMASK, ALPHA) |
			  CLT_Const(DBUSERCONTROL, DESTOUTMASK, RED) |
			  CLT_Const(DBUSERCONTROL, DESTOUTMASK, GREEN) |
			  CLT_Const(DBUSERCONTROL, DESTOUTMASK, BLUE) );

	/* A is the triangle - note that even though it is not textured 		*/
	/* the color still goes through the texture mapper which is what we want 	*/
		
	CLT_DBAMULTCNTL(&(mysnippet->data),
			CLT_Const(DBAMULTCNTL, AINPUTSELECT, TEXCOLOR),
			CLT_Const(DBAMULTCNTL, AMULTCOEFSELECT, TEXALPHA),
			CLT_Const(DBAMULTCNTL, AMULTCONSTCONTROL, TEXSSB),
			0);
	
	/* B is the SRC blending buffer		*/
	CLT_DBBMULTCNTL(&(mysnippet->data),
			CLT_Const(DBBMULTCNTL, BINPUTSELECT, SRCCOLOR),
			CLT_Const(DBBMULTCNTL, BMULTCOEFSELECT, TEXALPHACOMPLEMENT),
			CLT_Const(DBBMULTCNTL, BMULTCONSTCONTROL, SRCSSB),
			0);
		
	/* Add the two and clamp on overrun */			
	CLT_DBALUCNTL(&(mysnippet->data), CLT_Const(DBALUCNTL,ALUOPERATION,A_PLUS_BCLAMP), 0);

	mysnippet->data-=mysnippet->size;

	/* turn on perspective correction */
	CLT_ESCNTL( GS_Ptr(gs), 0, 0, 0);

printf("\nDisplaying textured rectangular fan with slight perspective correction\n");
printf("and a moving translucent pink triangle which blends.\n\n");
printf("Press X on the controller to end display.\n\n");

while(1) {

	GS_BeginFrame(gs);

	/* clear frame buffer to blue */
	CLT_ClearFrameBuffer(gs, 0.0, 0.2, 0.5, 1.0, TRUE, TRUE);
	CLT_Sync(GS_Ptr(gs));

/***************************************************************************/
/* Display textured triangle fan					   */
/***************************************************************************/

/* texture mapping registers */

		/* use global snippet to turn on texturing since ClearFrameBuffer turns it off */
		CLT_CopySnippetData( GS_Ptr(gs), &CltEnableTextureSnippet);

		/* clear colorout and alphaout and set from texture */
		CLT_ClearRegister(gs->gs_ListPtr, TXTTABCNTL,
				  FV_TXTTABCNTL_COLOROUT_MASK |
				  FV_TXTTABCNTL_ALPHAOUT_MASK);

		CLT_SetRegister(gs->gs_ListPtr, TXTTABCNTL,
		CLT_SetConst(TXTTABCNTL,COLOROUT,TEXCOLOR) |
		CLT_SetConst(TXTTABCNTL,ALPHAOUT,TEXALPHA));

/* destination blending registers */

		/* disable Destination and SCR blending, enable z buffering and Z buffer output */
		CLT_DBUSERCONTROL(GS_Ptr(gs), 1, 1, 0, 0, 0, 0, 0,
				  CLT_Const(DBUSERCONTROL, DESTOUTMASK, ALPHA) |
				  CLT_Const(DBUSERCONTROL, DESTOUTMASK, RED) |
				  CLT_Const(DBUSERCONTROL, DESTOUTMASK, GREEN) |
				  CLT_Const(DBUSERCONTROL, DESTOUTMASK, BLUE) );

		myvtx=(myvtx-numvert);			/* back to beginning of vertices */

		/* simple logic to move back and forth along X */	
		for (cnt=0; cnt<numvert; cnt++)	{				
			if ( (!flag) && ((myvtx->x+(mySize/2))>BMWIDTH) ) { flag=1;
									break; }
			myvtx++; 
		}		
		myvtx=(myvtx-cnt);

		for (cnt=0; cnt<numvert; cnt++) {						
			if ( (flag) && ((myvtx->x-(mySize/2))<1) ) {	flag=0; 
									break;}
			myvtx++;
		}
		myvtx=(myvtx-cnt);

	CLT_TRIANGLE(GS_Ptr(gs), 1, RC_FAN, 1, 1, 0, numvert );

	for (cnt=0; cnt<numvert; cnt++) {					

	if (flag) myvtx->x=myvtx->x-1.0;
	else myvtx->x=myvtx->x+1.0;

	CLT_VertexUvW(GS_Ptr(gs), myvtx->x, myvtx->y, myvtx->u, myvtx->v, myvtx->w); 
	myvtx++;
	}

       	CLT_Sync(GS_Ptr(gs));

/***************************************************************************/
/* Add simple shaded triangle which is transparent and dest blends         */
/***************************************************************************/

/* texture mapping registers */

	/* use global snippet to turn off texturing */
	CLT_CopySnippetData( GS_Ptr(gs), &CltNoTextureSnippet);
		
	/* sets src base address to current destination buffer, xstride, and offset */
	CLT_SetSrcToCurrentDest(gs);

	/* copy mycnippet to the command list */
	CLT_CopySnippetData( GS_Ptr(gs), mysnippet);

	/* pink triangle with differing levels of transparency at each vertex */
       	CLT_TRIANGLE(GS_Ptr(gs), 1, RC_STRIP, 0, 0, 1, 3);

	CLT_VertexRgba(GS_Ptr(gs), TriX[0], TriY[0], 1.0, 0.0, 1.0, 0.2);
	CLT_VertexRgba(GS_Ptr(gs), TriX[1], TriY[1], 1.0, 0.0, 1.0, 0.6);
	CLT_VertexRgba(GS_Ptr(gs), TriX[2], TriY[2], 1.0, 0.0, 1.0, 0.8);

	/* translate triangle along Y if goes beyond BMWIDTH randomly reposition X and Y */
		for (cnt=0; cnt<3; cnt++) 	{				
			TriY[cnt]+=1.0; 
			if (TriY[cnt]>BMWIDTH) {
			for (cnt=0; cnt<3; cnt++) {					
				TriX[cnt] = (gfloat)(rand()%320);	
				TriY[cnt] = (gfloat)(rand()%240);
			    }							
			}
		 }
			
       	CLT_Sync(GS_Ptr(gs));


		/* send the command list which creates both objects to the TE */

	err=gs->gs_SendList(gs);
	CKERR( err, "SendList");

        GS_EndFrame(gs);

	fbindex = 1 - fbindex;
	GS_SetDestBuffer(gs, bitmaps[fbindex]);

	GetControlPad(1, FALSE, &cpad);
	buttons = cpad.cped_ButtonBits;
	if( buttons & ControlX )
		break;

	}

/* Free memory associated with views, bitmaps, etc */

RemoveView(viewItem);

err=DeleteItem(viewItem);
CKERR( err, "DeleteItem");

err=GS_FreeBitmaps(bitmaps, 3);
CKERR( err, "FreeBitmaps");

err=GS_FreeLists(gs);
CKERR( err, "FreeLists");

err=GS_Delete(gs);
CKERR( err, "DeleteItem");

err=KillEventUtility();
CKERR( err, "KillEventUtil");
	
err=CloseGraphicsFolio();
CKERR( err, "CloseGraphicsFolio");

CLT_FreeSnippet(mysnippet);
FreeMem( myvtx, ( (numvert+1) * sizeof(mycoords) ) );

printf("\nFinished! \n");


 }





