/******************************************************************************
**
**  Copyright (c) 1993-1996, an unpublished work by The 3DO Company.
**  All rights reserved. This material contains confidential
**  information that is the property of The 3DO Company. Any
**  unauthorized duplication, disclosure or use is prohibited.
**  @(#) cltmacrosFT.c 96/10/29 1.3
**  Tests CLT macros.
**  Written by:		       Nick Triantos
**			       Paige Wuestefeld  
******************************************************************************/

#ifdef MACINTOSH
#include <kernel:types.h>
#include <graphics:clt:clt.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tests:3do_scaf.h>
#else
#include <kernel/types.h>
#include <graphics/clt/clt.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tests/3do_scaf.h>
#endif

char*	Clip1(char* x);
char*	Clip2(char* x);
char*	Clip3(char* x);

void	CheckMacros(void);
void	CheckDefines(void);

char	final[50];


main( int argc, char* argv[] )
{

	InitScaffold(GRAPHICS_CLT, argc, argv);

	CheckMacros();
	CheckDefines();

}

/* clip for register addresses */

char*	Clip1(char* x)
{
int		i=0;

strcpy(final," ");

while (x[i] != '=') i++;
strncat(final,x,i);

return (final);

}


/* clip for bit sets */

char*	Clip2(char* x)
{

char*	begin;
int		i=0;

strcpy(final," ");
while (*x != '(') x++;

begin=++x;

while (begin[i] != 'x') i++;
strncat(final,begin,(i-2));

return (final);

}


/* clip for constants */

char*	Clip3(char* x)
{
/* grabs part of string between commas */

char*	begin;
int		i=0;

strcpy(final," ");
while (*x != '(') x++;

begin=++x;

while (begin[i] != ')') i++;
strncat(final,begin,i);

return (final);

}



void CheckMacros(void)
{

/* if any of these fail then the program exits on an assert failure    */
/* if any of these fail there is a MAJOR problem and none of the       */
/* defines to follow will work anyway                                  */

BeginTest(FT, "CLT base macros" );

	assert (PPCBIT(31) == 0x00000001);
	assert (PPCBIT(0) == 0x80000000);
	assert (PPCBIT(8) == 0x00800000);
	assert (PPCMASK(5,31) == 0x00000005);
	assert (PPCMASK(5,30) == 0x0000000A);
	assert (PPCMASK(5,2) == 0xA0000000);
	assert (PPCSHIFT(5) == (31 - 5));
	assert (CLT_Bits(TXTADDRCNTL,INTERFILTER,0x3) == 0x00000180);
	assert (CLT_Mask(TXTADDRCNTL,INTERFILTER) == 0x00000380);
	assert (CLT_Shift(TXTADDRCNTL,INTERFILTER) == 7);
	assert (CLT_Const(TXTADDRCNTL,INTERFILTER,TRILINEAR) == 3);
	assert (CLT_GetBits(TXTADDRCNTL,INTERFILTER,0x87654321) == 0x6);
	assert (CLT_SetConst(TXTADDRCNTL,INTERFILTER,TRILINEAR) == 0x00000180);

LogResult(PASSED, "" );

}

#define Check1(x)			BeginTest(FT, Clip1( (char*)(#x) ) ); (x) ? LogResult(PASSED, "" ) : LogResult(FAILED, "" ); count++;
#define Check2(x)			BeginTest(FT, Clip2( (char*)(#x) ) ); (x) ? LogResult(PASSED, "" ) : LogResult(FAILED, "" ); count++;
#define Check3(x)			BeginTest(FT, Clip3( (char*)(#x) ) ); (x) ? LogResult(PASSED, "" ) : LogResult(FAILED, "" ); count++;

void CheckDefines(void)
{
int count=1;
 
	Check1(RA_DCNTLDATA == 0x0010);

	/* General Controls */

	Check1(RA_DCNTL == 0x0014);
	Check2(CLT_Bits(DCNTL,SYNC,0xFFFFFFFF) == 0x00000020);
	Check2(CLT_Bits(DCNTL,PSE,0xFFFFFFFF) == 0x00000010);
	Check2(CLT_Bits(DCNTL,DINT,0xFFFFFFFF) == 0x00000008);
	Check2(CLT_Bits(DCNTL,JR,0xFFFFFFFF) == 0x00000004);
	Check2(CLT_Bits(DCNTL,JA,0xFFFFFFFF) == 0x00000002);
	Check2(CLT_Bits(DCNTL,TLD,0xFFFFFFFF) == 0x00000001);

	/* Span Walker / Edge Walker Controls */

	Check1(RA_ESCNTL == 0x4000);
	Check2(CLT_Bits(ESCNTL,PERSPECTIVEOFF,0xFFFFFFFF) == 0x00000004);
	Check2(CLT_Bits(ESCNTL,DUSCAN,0xFFFFFFFF) == 0x00000002);
	Check2(CLT_Bits(ESCNTL,DSPOFF,0xFFFFFFFF) == 0x00000001);

	/* Texture Mapper Controls */

	Check1(RA_TXTLDCNTL == 0x6404);
	Check2(CLT_Bits(TXTLDCNTL,COMPRESSED,0xFFFFFFFF) == 0x00000400);
	Check2(CLT_Bits(TXTLDCNTL,LOADMODE,0xFFFFFFFF) == 0x00000300);
	Check3(CLT_SetConst(TXTLDCNTL,LOADMODE,TEXTURE) == 0x00000000);
	Check3(CLT_SetConst(TXTLDCNTL,LOADMODE,MMDMA) == 0x00000100);
	Check3(CLT_SetConst(TXTLDCNTL,LOADMODE,PIP) == 0x00000200);
	Check2(CLT_Bits(TXTLDCNTL,SRCBITOFF,0xFFFFFFFF) == 0x00000007);

	Check1(RA_TXTADDRCNTL == 0x6408);
	Check2(CLT_Bits(TXTADDRCNTL,TEXTUREENABLE,0xFFFFFFFF) == 0x00002000);
	Check2(CLT_Bits(TXTADDRCNTL,MINFILTER,0xFFFFFFFF) == 0x00001C00);
	Check3(CLT_SetConst(TXTADDRCNTL,MINFILTER,POINT) == 0x00000000);
	Check3(CLT_SetConst(TXTADDRCNTL,MINFILTER,BILINEAR) == 0x00000800);
	Check2(CLT_Bits(TXTADDRCNTL,INTERFILTER,0xFFFFFFFF) == 0x00000380);
	Check3(CLT_SetConst(TXTADDRCNTL,INTERFILTER,POINT) == 0x00000000);
	Check3(CLT_SetConst(TXTADDRCNTL,INTERFILTER,LINEAR) == 0x00000080);
	Check3(CLT_SetConst(TXTADDRCNTL,INTERFILTER,BILINEAR) == 0x00000100);
	Check3(CLT_SetConst(TXTADDRCNTL,INTERFILTER,TRILINEAR) == 0x00000180);
	Check2(CLT_Bits(TXTADDRCNTL,MAGFILTER,0xFFFFFFFF) == 0x00000070);
	Check3(CLT_SetConst(TXTADDRCNTL,MAGFILTER,POINT) == 0x00000000);
	Check3(CLT_SetConst(TXTADDRCNTL,MAGFILTER,BILINEAR) == 0x00000020);
	Check2(CLT_Bits(TXTADDRCNTL,LODMAX,0xFFFFFFFF) == 0x0F);

	Check1(RA_TXTPIPCNTL == 0x640C);
	Check2(CLT_Bits(TXTPIPCNTL,PIPSSBSELECT,0xFFFFFFFF) == 0x0001C000);
	Check3(CLT_SetConst(TXTPIPCNTL,PIPSSBSELECT,CONSTANT) == 0x00000000);
	Check3(CLT_SetConst(TXTPIPCNTL,PIPSSBSELECT,TEXTURE) == 0x00004000);
	Check3(CLT_SetConst(TXTPIPCNTL,PIPSSBSELECT,PIP) == 0x00008000);
	Check2(CLT_Bits(TXTPIPCNTL,PIPALPHASELECT,0xFFFFFFFF) == 0x00003800);
	Check3(CLT_SetConst(TXTPIPCNTL,PIPALPHASELECT,CONSTANT) == 0x00000000);
	Check3(CLT_SetConst(TXTPIPCNTL,PIPALPHASELECT,TEXTURE) == 0x00000800);
	Check3(CLT_SetConst(TXTPIPCNTL,PIPALPHASELECT,PIP) == 0x00001000);
	Check2(CLT_Bits(TXTPIPCNTL,PIPCOLORSELECT,0xFFFFFFFF) == 0x00000700);
	Check3(CLT_SetConst(TXTPIPCNTL,PIPCOLORSELECT,CONSTANT) == 0x00000000);
	Check3(CLT_SetConst(TXTPIPCNTL,PIPCOLORSELECT,TEXTURE) == 0x00000100);
	Check3(CLT_SetConst(TXTPIPCNTL,PIPCOLORSELECT,PIP) == 0x00000200);
	Check2(CLT_Bits(TXTPIPCNTL,PIPINDEXOFFSET,0xFFFFFFFF) == 0x000000FF);

	Check1(RA_TXTTABCNTL == 0x6410);
	Check2(CLT_Bits(TXTTABCNTL,FIRSTCOLOR,0xFFFFFFFF) == 0x00000007);
	Check3(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,PRIMALPHA) == 0x00000000);
	Check3(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,PRIMCOLOR) == 0x00000001);
	Check3(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,TEXALPHA) == 0x00000002);
	Check3(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,TEXCOLOR) == 0x00000003);
	Check3(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,CONSTALPHA) == 0x00000004);
	Check3(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,CONSTCOLOR) == 0x00000005);
	Check2(CLT_Bits(TXTTABCNTL,SECONDCOLOR,0xFFFFFFFF) == 0x00000038);
	Check3(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,PRIMALPHA) == 0x00000000);
	Check3(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,PRIMCOLOR) == 0x00000008);
	Check3(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,TEXALPHA) == 0x00000010);
	Check3(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,TEXCOLOR) == 0x00000018);
	Check3(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,CONSTALPHA) == 0x00000020);
	Check3(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,CONSTCOLOR) == 0x00000028);
	Check2(CLT_Bits(TXTTABCNTL,THIRDCOLOR,0xFFFFFFFF) == 0x000001C0);
	Check3(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,PRIMALPHA) == 0x00000000);
	Check3(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,PRIMCOLOR) == 0x00000040);
	Check3(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,TEXALPHA) == 0x00000080);
	Check3(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,TEXCOLOR) == 0x000000C0);
	Check3(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,CONSTALPHA) == 0x00000100);
	Check3(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,CONSTCOLOR) == 0x00000140);
	Check2(CLT_Bits(TXTTABCNTL,COLOROUT,0xFFFFFFFF) == 0x00000600);
	Check3(CLT_SetConst(TXTTABCNTL,COLOROUT,PRIMCOLOR) == 0x00000000);
	Check3(CLT_SetConst(TXTTABCNTL,COLOROUT,TEXCOLOR) == 0x00000200);
	Check3(CLT_SetConst(TXTTABCNTL,COLOROUT,BLEND) == 0x00000400);
	Check2(CLT_Bits(TXTTABCNTL,FIRSTALPHA,0xFFFFFFFF) == 0x00001800);
	Check3(CLT_SetConst(TXTTABCNTL,FIRSTALPHA,PRIMALPHA) == 0x00000000);
	Check3(CLT_SetConst(TXTTABCNTL,FIRSTALPHA,TEXALPHA) == 0x00000800);
	Check3(CLT_SetConst(TXTTABCNTL,FIRSTALPHA,CONSTALPHA) == 0x00001000);
	Check2(CLT_Bits(TXTTABCNTL,SECONDALPHA,0xFFFFFFFF) == 0x00006000);
	Check3(CLT_SetConst(TXTTABCNTL,SECONDALPHA,PRIMALPHA) == 0x00000000);
	Check3(CLT_SetConst(TXTTABCNTL,SECONDALPHA,TEXALPHA) == 0x00002000);
	Check3(CLT_SetConst(TXTTABCNTL,SECONDALPHA,CONSTALPHA) == 0x00004000);
	Check2(CLT_Bits(TXTTABCNTL,ALPHAOUT,0xFFFFFFFF) == 0x00018000);
	Check3(CLT_SetConst(TXTTABCNTL,ALPHAOUT,PRIMALPHA) == 0x00000000);
	Check3(CLT_SetConst(TXTTABCNTL,ALPHAOUT,TEXALPHA) == 0x00008000);
	Check3(CLT_SetConst(TXTTABCNTL,ALPHAOUT,BLEND) == 0x00010000);
	Check2(CLT_Bits(TXTTABCNTL,BLENDOP,0xFFFFFFFF) == 0x00020000);
	Check3(CLT_SetConst(TXTTABCNTL,BLENDOP,LERP) == 0x00000000);
	Check3(CLT_SetConst(TXTTABCNTL,BLENDOP,MULT) == 0x00020000);

	Check1(RA_TXTLODBASE0 == 0x6414);

	Check1(RA_TXTLODBASE1 == 0x6418);

	Check1(RA_TXTLODBASE2 == 0x641C);
	
	Check1(RA_TXTLODBASE3 == 0x6420);

	Check1(RA_TXTLDSRCADDR == 0x6424);

	Check1(RA_TXTCOUNT == 0x6428);
	Check2(CLT_Bits(TXTCOUNT,VALUE,0xFFFFFFFF) == 0x00FFFFFF);

	Check1(RA_TXTUVMAX == 0x642C);
	Check2(CLT_Bits(TXTUVMAX,UMAX,0xFFFFFFFF) == 0x03FF0000);
	Check2(CLT_Bits(TXTUVMAX,VMAX,0xFFFFFFFF) == 0x000003FF);

	Check1(RA_TXTUVMASK == 0x6430);
	Check2(CLT_Bits(TXTUVMASK,UMASK,0xFFFFFFFF) == 0x03FF0000);
	Check2(CLT_Bits(TXTUVMASK,VMASK,0xFFFFFFFF) == 0x000003FF);

	Check1(RA_TXTSRCTYPE01 == 0x6434);
	Check2(CLT_Bits(TXTSRCTYPE01,CDEPTH0,0xFFFFFFFF) == 0x0000000F);
	Check2(CLT_Bits(TXTSRCTYPE01,ADEPTH0,0xFFFFFFFF) == 0x000000F0);
	Check2(CLT_Bits(TXTSRCTYPE01,ISTRANS0,0xFFFFFFFF) == 0x00000100);
	Check2(CLT_Bits(TXTSRCTYPE01,HASSSB0,0xFFFFFFFF) == 0x00000200);
	Check2(CLT_Bits(TXTSRCTYPE01,HASCOLOR0,0xFFFFFFFF) == 0x00000400);
	Check2(CLT_Bits(TXTSRCTYPE01,HASALPHA0,0xFFFFFFFF) == 0x00000800);
	Check2(CLT_Bits(TXTSRCTYPE01,ISLITERAL0,0xFFFFFFFF) == 0x00001000);
	Check2(CLT_Bits(TXTSRCTYPE01,CDEPTH1,0xFFFFFFFF) == 0x000F0000);
	Check2(CLT_Bits(TXTSRCTYPE01,ADEPTH1,0xFFFFFFFF) == 0x00F00000);
	Check2(CLT_Bits(TXTSRCTYPE01,ISTRANS1,0xFFFFFFFF) == 0x01000000);
	Check2(CLT_Bits(TXTSRCTYPE01,HASSSB1,0xFFFFFFFF) == 0x02000000);
	Check2(CLT_Bits(TXTSRCTYPE01,HASCOLOR1,0xFFFFFFFF) == 0x04000000);
	Check2(CLT_Bits(TXTSRCTYPE01,HASALPHA1,0xFFFFFFFF) == 0x08000000);
	Check2(CLT_Bits(TXTSRCTYPE01,ISLITERAL1,0xFFFFFFFF) == 0x10000000);

	Check1(RA_TXTSRCTYPE23 == 0x6438);
	Check2(CLT_Bits(TXTSRCTYPE23,CDEPTH2,0xFFFFFFFF) == 0x0000000F);
	Check2(CLT_Bits(TXTSRCTYPE23,ADEPTH2,0xFFFFFFFF) == 0x000000F0);
	Check2(CLT_Bits(TXTSRCTYPE23,ISTRANS2,0xFFFFFFFF) == 0x00000100);
	Check2(CLT_Bits(TXTSRCTYPE23,HASSSB2,0xFFFFFFFF) == 0x00000200);
	Check2(CLT_Bits(TXTSRCTYPE23,HASCOLOR2,0xFFFFFFFF) == 0x00000400);
	Check2(CLT_Bits(TXTSRCTYPE23,HASALPHA2,0xFFFFFFFF) == 0x00000800);
	Check2(CLT_Bits(TXTSRCTYPE23,ISLITERAL2,0xFFFFFFFF) == 0x00001000);
	Check2(CLT_Bits(TXTSRCTYPE23,CDEPTH3,0xFFFFFFFF) == 0x000F0000);
	Check2(CLT_Bits(TXTSRCTYPE23,ADEPTH3,0xFFFFFFFF) == 0x00F00000);
	Check2(CLT_Bits(TXTSRCTYPE23,ISTRANS3,0xFFFFFFFF) == 0x01000000);
	Check2(CLT_Bits(TXTSRCTYPE23,HASSSB3,0xFFFFFFFF) == 0x02000000);
	Check2(CLT_Bits(TXTSRCTYPE23,HASCOLOR3,0xFFFFFFFF) == 0x04000000);
	Check2(CLT_Bits(TXTSRCTYPE23,HASALPHA3,0xFFFFFFFF) == 0x08000000);
	Check2(CLT_Bits(TXTSRCTYPE23,ISLITERAL3,0xFFFFFFFF) == 0x10000000);

	Check1(RA_TXTEXPTYPE == 0x643C);
	Check2(CLT_Bits(TXTEXPTYPE,CDEPTH,0xFFFFFFFF) == 0x0000000F);
	Check2(CLT_Bits(TXTEXPTYPE,ADEPTH,0xFFFFFFFF) == 0x000000F0);
	Check2(CLT_Bits(TXTEXPTYPE,ISTRANS,0xFFFFFFFF) == 0x00000100);
	Check2(CLT_Bits(TXTEXPTYPE,HASSSB,0xFFFFFFFF) == 0x00000200);
	Check2(CLT_Bits(TXTEXPTYPE,HASCOLOR,0xFFFFFFFF) == 0x00000400);
	Check2(CLT_Bits(TXTEXPTYPE,HASALPHA,0xFFFFFFFF) == 0x00000800);
	Check2(CLT_Bits(TXTEXPTYPE,ISLITERAL,0xFFFFFFFF) == 0x00001000);

	Check1(RA_TXTCONST0 == 0x6440);
	Check2(CLT_Bits(TXTCONST0,SSB,0xFFFFFFFF) == 0x80000000);
	Check2(CLT_Bits(TXTCONST0,ALPHA,0xFFFFFFFF) == 0x7F000000);
	Check2(CLT_Bits(TXTCONST0,RED,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(TXTCONST0,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check2(CLT_Bits(TXTCONST0,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check1(RA_TXTCONST1 == 0x6444);
	Check2(CLT_Bits(TXTCONST1,SSB,0xFFFFFFFF) == 0x80000000);
	Check2(CLT_Bits(TXTCONST1,ALPHA,0xFFFFFFFF) == 0x7F000000);
	Check2(CLT_Bits(TXTCONST1,RED,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(TXTCONST1,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check2(CLT_Bits(TXTCONST1,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check1(RA_TXTCONST2 == 0x6448);
	Check2(CLT_Bits(TXTCONST2,SSB,0xFFFFFFFF) == 0x80000000);
	Check2(CLT_Bits(TXTCONST2,ALPHA,0xFFFFFFFF) == 0x7F000000);
	Check2(CLT_Bits(TXTCONST2,RED,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(TXTCONST2,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check2(CLT_Bits(TXTCONST2,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check1(RA_TXTCONST3 == 0x644C);
	Check2(CLT_Bits(TXTCONST3,SSB,0xFFFFFFFF) == 0x80000000);
	Check2(CLT_Bits(TXTCONST3,ALPHA,0xFFFFFFFF) == 0x7F000000);
	Check2(CLT_Bits(TXTCONST3,RED,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(TXTCONST3,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check2(CLT_Bits(TXTCONST3,BLUE,0xFFFFFFFF) == 0x000000FF);

	/* Destination Blend Controls APW */

	Check1(RA_DBUSERCONTROL == 0x8008);
	Check2(CLT_Bits(DBUSERCONTROL,ZBUFFEN,0xFFFFFFFF) == 0x00000400);
	Check2(CLT_Bits(DBUSERCONTROL,ZOUTEN,0xFFFFFFFF) == 0x00000200);
	Check2(CLT_Bits(DBUSERCONTROL,WINCLIPINEN,0xFFFFFFFF) == 0x00000100);
	Check2(CLT_Bits(DBUSERCONTROL,WINCLIPOUTEN,0xFFFFFFFF) == 0x00000080);
	Check2(CLT_Bits(DBUSERCONTROL,BLENDEN,0xFFFFFFFF) == 0x00000040);
	Check2(CLT_Bits(DBUSERCONTROL,SRCEN,0xFFFFFFFF) == 0x00000020);
	Check2(CLT_Bits(DBUSERCONTROL,DITHEREN,0xFFFFFFFF) == 0x00000010);
	Check2(CLT_Bits(DBUSERCONTROL,DESTOUTMASK,0xFFFFFFFF) == 0x0000000F);
		Check3(CLT_SetConst(DBUSERCONTROL,DESTOUTMASK,ALPHA) == 0x00000008);
		Check3(CLT_SetConst(DBUSERCONTROL,DESTOUTMASK,RED) == 0x00000004);
		Check3(CLT_SetConst(DBUSERCONTROL,DESTOUTMASK,GREEN) == 0x00000002);
		Check3(CLT_SetConst(DBUSERCONTROL,DESTOUTMASK,BLUE) == 0x00000001);


	Check1(RA_DBDISCARDCONTROL == 0x800c);
	Check2(CLT_Bits(DBDISCARDCONTROL,ZCLIPPED,0xFFFFFFFF) == 0x00000008);
	Check2(CLT_Bits(DBDISCARDCONTROL,SSB0,0xFFFFFFFF) == 0x00000004);
	Check2(CLT_Bits(DBDISCARDCONTROL,RGB0,0xFFFFFFFF) == 0x00000002);
	Check2(CLT_Bits(DBDISCARDCONTROL,ALPHA0,0xFFFFFFFF) == 0x00000001);

	Check1(RA_DBSTATUS == 0x8010);

	Check1(RA_DBINTCNTL == 0x8014);
	Check2(CLT_Bits(DBINTCNTL,ALUSTAT,0xFFFFFFFF) == 0x00001FF0);
	Check2(CLT_Bits(DBINTCNTL,ZFUNCSTAT,0xFFFFFFFF) == 0x0000000E);

	Check1(RA_DBXWINCLIP == 0x801c);
	Check2(CLT_Bits(DBXWINCLIP,XWINCLIPMIN,0xFFFFFFFF) == 0x07FF0000);
	Check2(CLT_Bits(DBXWINCLIP,XWINCLIPMAX,0xFFFFFFFF) == 0x000007FF);

	Check1(RA_DBYWINCLIP == 0x8020);
	Check2(CLT_Bits(DBYWINCLIP,YWINCLIPMIN,0xFFFFFFFF) == 0x07FF0000);
	Check2(CLT_Bits(DBYWINCLIP,YWINCLIPMAX,0xFFFFFFFF) == 0x000007FF);

	Check1(RA_DBSRCCNTL == 0x8030);
	Check2(CLT_Bits(DBSRCCNTL,SRCMSBREP,0xFFFFFFFF) == 0x00000002);
	Check2(CLT_Bits(DBSRCCNTL,SRC32BPP,0xFFFFFFFF) == 0x00000001);

	Check1(RA_DBSRCBASEADDR == 0x8034);

	Check1(RA_DBSRCXSTRIDE == 0x8038);
	Check2(CLT_Bits(DBSRCXSTRIDE,XSTRIDE,0xFFFFFFFF) == 0x000007FF);

	Check1(RA_DBSRCOFFSET == 0x803c);
	Check2(CLT_Bits(DBSRCOFFSET,XOFFSET,0xFFFFFFFF) == 0x0FFF0000);
	Check2(CLT_Bits(DBSRCOFFSET,YOFFSET,0xFFFFFFFF) == 0x00000FFF);

	Check1(RA_DBZCNTL == 0x8040);	
	Check2(CLT_Bits(DBZCNTL,ZUPDATEONGREATERZ,0xFFFFFFFF) == 0x00000020);	/* BIT 26 */
	Check2(CLT_Bits(DBZCNTL,PIXOUTONGREATERZ,0xFFFFFFFF) == 0x00000010);		/* BIT 27 */
	Check2(CLT_Bits(DBZCNTL,ZUPDATEONEQUALZ,0xFFFFFFFF) == 0x00000008);		/* BIT 28 */
	Check2(CLT_Bits(DBZCNTL,PIXOUTONEQUALZ,0xFFFFFFFF) == 0x00000004);		/* BIT 29 */
	Check2(CLT_Bits(DBZCNTL,ZUPDATEONSMALLERZ,0xFFFFFFFF) == 0x00000002);	/* BIT 30 */
	Check2(CLT_Bits(DBZCNTL,PIXOUTONSMALLERZ,0xFFFFFFFF) == 0x00000001);		/* BIT 31 */

	Check1(RA_DBZOFFSET == 0x8048);
	Check2(CLT_Bits(DBZOFFSET,ZXOFFSET,0xFFFFFFFF) == 0x0FFF0000);
	Check2(CLT_Bits(DBZOFFSET,ZYOFFSET,0xFFFFFFFF) == 0x00000FFF);

	Check1(RA_DBSSBDSBCNTL == 0x8050);
	Check2(CLT_Bits(DBSSBDSBCNTL,DSBCONST,0xFFFFFFFF) == 0x00000004);
	Check2(CLT_Bits(DBSSBDSBCNTL,DSBSELECT,0xFFFFFFFF) == 0x00000003);
			Check3(CLT_SetConst(DBSSBDSBCNTL,DSBSELECT,OBJSSB) == 0x00000000);
			Check3(CLT_SetConst(DBSSBDSBCNTL,DSBSELECT,CONST) == 0x00000001);
			Check3(CLT_SetConst(DBSSBDSBCNTL,DSBSELECT,SRCSSB) == 0x00000002);

	Check1(RA_DBCONSTIN == 0x8054);
	Check2(CLT_Bits(DBCONSTIN,RED,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(DBCONSTIN,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check2(CLT_Bits(DBCONSTIN,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check1(RA_DBAMULTCNTL == 0x8058);
	Check2(CLT_Bits(DBAMULTCNTL,AINPUTSELECT,0xFFFFFFFF) == 0x000000C0);
			Check3(CLT_SetConst(DBAMULTCNTL,AINPUTSELECT,TEXCOLOR) == 0x00000000);
			Check3(CLT_SetConst(DBAMULTCNTL,AINPUTSELECT,CONSTCOLOR) == 0x00000040);
			Check3(CLT_SetConst(DBAMULTCNTL,AINPUTSELECT,SRCCOLORCOMPLEMENT) == 0x00000080);
			Check3(CLT_SetConst(DBAMULTCNTL,AINPUTSELECT,TEXALPHA) == 0x000000C0);

/* the powers that be have merged 2 fields here - coef select is 3 (noncontiguous) bits: 26, 27, 30 */
/* bit 30 acts as a switch - if 1 then bits 26, 27 are the selected complement */

	Check2(CLT_Bits(DBAMULTCNTL,AMULTCOEFSELECT,0xFFFFFFFF) == 0x00000032);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCOEFSELECT,TEXALPHA) == 0x00000000);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCOEFSELECT,SRCALPHA) == 0x00000010);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCOEFSELECT,CONST) == 0x00000020);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCOEFSELECT,SRCCOLOR) == 0x00000030);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCOEFSELECT,TEXALPHACOMPLEMENT) == 0x00000002);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCOEFSELECT,SRCALPHACOMPLEMENT) == 0x00000012);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCOEFSELECT,CONSTCOMPLEMENT) == 0x00000022);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCOEFSELECT,SRCCOLORCOMPLEMENT) == 0x00000032);

	Check2(CLT_Bits(DBAMULTCNTL,AMULTCONSTCONTROL,0xFFFFFFFF) == 0x0000000C);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCONSTCONTROL,TEXSSB) == 0x00000000);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCONSTCONTROL,SRCSSB) == 0x00000004);

	Check2(CLT_Bits(DBAMULTCNTL,AMULTRJUSTIFY,0xFFFFFFFF) == 0x00000001);

	Check2(RA_DBAMULTCONSTSSB0 == 0x805C);
	Check2(CLT_Bits(DBAMULTCONSTSSB0,RED,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(DBAMULTCONSTSSB0,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check2(CLT_Bits(DBAMULTCONSTSSB0,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check1(RA_DBAMULTCONSTSSB1 == 0x8060);
	Check2(CLT_Bits(DBAMULTCONSTSSB1,RED,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(DBAMULTCONSTSSB1,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check2(CLT_Bits(DBAMULTCONSTSSB1,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check1(RA_DBBMULTCNTL == 0x8064);
	Check2(CLT_Bits(DBBMULTCNTL,BINPUTSELECT,0xFFFFFFFF) == 0x000000C0);
			Check3(CLT_SetConst(DBBMULTCNTL,BINPUTSELECT,SRCCOLOR) == 0x00000000);
			Check3(CLT_SetConst(DBBMULTCNTL,BINPUTSELECT,CONSTCOLOR) == 0x00000040);
			Check3(CLT_SetConst(DBBMULTCNTL,BINPUTSELECT,TEXCOLORCOMPLEMENT) == 0x00000080);
			Check3(CLT_SetConst(DBBMULTCNTL,BINPUTSELECT,SRCALPHA) == 0x000000C0);

/* the powers that be have merged 2 fields here - coef select is 3 (noncontiguous) bits: 26, 27, 30 */
/* bit 30 acts as a switch - if 1 then bits 26, 27 are the selected complement */

	Check2(CLT_Bits(DBBMULTCNTL,BMULTCOEFSELECT,0xFFFFFFFF) == 0x00000032);
			Check3(CLT_SetConst(DBBMULTCNTL,BMULTCOEFSELECT,TEXALPHA) == 0x00000000);
			Check3(CLT_SetConst(DBBMULTCNTL,BMULTCOEFSELECT,SRCALPHA) == 0x00000010);
			Check3(CLT_SetConst(DBBMULTCNTL,BMULTCOEFSELECT,CONST) == 0x00000020);
			Check3(CLT_SetConst(DBBMULTCNTL,BMULTCOEFSELECT,TEXCOLOR) == 0x00000030);
			Check3(CLT_SetConst(DBBMULTCNTL,BMULTCOEFSELECT,TEXALPHACOMPLEMENT) == 0x00000002);
			Check3(CLT_SetConst(DBBMULTCNTL,BMULTCOEFSELECT,SRCALPHACOMPLEMENT) == 0x00000012);
			Check3(CLT_SetConst(DBBMULTCNTL,BMULTCOEFSELECT,CONSTCOMPLEMENT) == 0x00000022);
			Check3(CLT_SetConst(DBBMULTCNTL,BMULTCOEFSELECT,TEXCOLORCOMPLEMENT) == 0x00000032);

	Check2(CLT_Bits(DBAMULTCNTL,AMULTCONSTCONTROL,0xFFFFFFFF) == 0x0000000C);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCONSTCONTROL,TEXSSB) == 0x00000000);
			Check3(CLT_SetConst(DBAMULTCNTL,AMULTCONSTCONTROL,SRCSSB) == 0x00000004);

	Check2(CLT_Bits(DBBMULTCNTL,BMULTRJUSTIFY,0xFFFFFFFF) == 0x00000001);

	Check1(RA_DBBMULTCONSTSSB0 == 0x8068);
	Check2(CLT_Bits(DBBMULTCONSTSSB0,RED,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(DBBMULTCONSTSSB0,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check2(CLT_Bits(DBBMULTCONSTSSB0,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check1(RA_DBBMULTCONSTSSB1 == 0x806c);
	Check2(CLT_Bits(DBBMULTCONSTSSB1,RED,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(DBBMULTCONSTSSB1,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check2(CLT_Bits(DBBMULTCONSTSSB1,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check1(RA_DBALUCNTL == 0x8070);
	Check2(CLT_Bits(DBALUCNTL,ALUOPERATION,0xFFFFFFFF) == 0x000000F8);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,A_PLUS_BCLAMP) == 0x00000000);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,A_PLUS_B) == 0x00000010);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,A_MINUS_BCLAMP) == 0x00000040);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,A_MINUS_B) == 0x00000050);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,B_MINUS_ACLAMP) == 0x00000060);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,B_MINUS_A) == 0x00000070);

			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,OUTPUTZERO) == 0x00000080);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,NEITHER) == 0x00000088);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,NOTA_AND_B) == 0x00000090);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,NOTA) == 0x00000098);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,NOTB_AND_A) == 0x000000A0);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,NOTB) == 0x000000A8);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,XOR) == 0x000000B0);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,NOT_A_AND_B) == 0x000000B8);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,A_AND_B) == 0x000000C0);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,ONEONEQUAL) == 0x000000C8);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,B) == 0x000000D0);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,NOTA_OR_B) == 0x000000D8);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,A) == 0x000000E0);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,NOTB_OR_A) == 0x000000E8);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,A_OR_B) == 0x000000F0);
			Check3(CLT_SetConst(DBALUCNTL,ALUOPERATION,OUTPUTONE) == 0x000000F8);

	Check2(CLT_Bits(DBALUCNTL,FINALDIVIDE,0xFFFFFFFF) == 0x00000007);

	Check1(RA_DBSRCALPHACNTL == 0x8074);
	Check2(CLT_Bits(DBSRCALPHACNTL,ACLAMP,0xFFFFFFFF) == 0x0000003F);

	Check2(CLT_Bits(DBSRCALPHACNTL,ALPHA1,0xFFFFFFFF) == 0x00000030); /* 26, 27 */
	Check2(CLT_Bits(DBSRCALPHACNTL,ALPHAFRAC,0xFFFFFFFF) == 0x0000000C);	/* 28, 29 */
	Check2(CLT_Bits(DBSRCALPHACNTL,ALPHA0,0xFFFFFFFF) == 0x00000003);	/* 30, 31 */

/* unfortunately the constants are only defined for the entire field so we have to use */
/* this longer method                                                                  */

	Check3(CLT_Bits(DBSRCALPHACNTL,ALPHA1,RC_DBSRCALPHACNTL_ACLAMP_LEAVEALONE) == 0x00000000); /* 26, 27 */
	Check3(CLT_Bits(DBSRCALPHACNTL,ALPHA1,RC_DBSRCALPHACNTL_ACLAMP_FORCE1) == 0x00000010); /* 26, 27 */
	Check3(CLT_Bits(DBSRCALPHACNTL,ALPHA1,RC_DBSRCALPHACNTL_ACLAMP_FORCE0) == 0x00000020); /* 26, 27 */

	Check3(CLT_Bits(DBSRCALPHACNTL,ALPHAFRAC,RC_DBSRCALPHACNTL_ACLAMP_LEAVEALONE) == 0x00000000);	/* 28, 29 */
	Check3(CLT_Bits(DBSRCALPHACNTL,ALPHAFRAC,RC_DBSRCALPHACNTL_ACLAMP_FORCE1) == 0x00000004);	/* 28, 29 */
	Check3(CLT_Bits(DBSRCALPHACNTL,ALPHAFRAC,RC_DBSRCALPHACNTL_ACLAMP_FORCE0) == 0x00000008);	/* 28, 29 */

	Check3(CLT_Bits(DBSRCALPHACNTL,ALPHA0,RC_DBSRCALPHACNTL_ACLAMP_LEAVEALONE) == 0x00000000);	/* 30, 31 */
	Check3(CLT_Bits(DBSRCALPHACNTL,ALPHA0,RC_DBSRCALPHACNTL_ACLAMP_FORCE1) == 0x00000001);	/* 30, 31 */
	Check3(CLT_Bits(DBSRCALPHACNTL,ALPHA0,RC_DBSRCALPHACNTL_ACLAMP_FORCE0) == 0x00000002);	/* 30, 31 */


/* the powers that be have merged 2 fields here - now this field is 4 bits long */
	Check1(RA_DBDESTALPHACNTL == 0x8078);
	Check2(CLT_Bits(DBDESTALPHACNTL,DESTCONSTSELECT,0xFFFFFFFF) == 0x0000000F);
			Check3(CLT_SetConst(DBDESTALPHACNTL,DESTCONSTSELECT,TEXALPHA) == 0x00000000);
			Check3(CLT_SetConst(DBDESTALPHACNTL,DESTCONSTSELECT,TEXSSBCONST) == 0x00000001);
			Check3(CLT_SetConst(DBDESTALPHACNTL,DESTCONSTSELECT,SRCSSBCONST) == 0x00000009);
			Check3(CLT_SetConst(DBDESTALPHACNTL,DESTCONSTSELECT,SRCALPHA) == 0x00000002);
			Check3(CLT_SetConst(DBDESTALPHACNTL,DESTCONSTSELECT,RBLEND) == 0x00000003);

	Check1(RA_DBDITHERMATRIXA == 0x8080);
	Check2(CLT_Bits(DBDITHERMATRIXA,X0Y0,0xFFFFFFFF) == 0xF0000000);
	Check2(CLT_Bits(DBDITHERMATRIXA,X1Y0,0xFFFFFFFF) == 0x0F000000);
	Check2(CLT_Bits(DBDITHERMATRIXA,X2Y0,0xFFFFFFFF) == 0x00F00000);
	Check2(CLT_Bits(DBDITHERMATRIXA,X3Y0,0xFFFFFFFF) == 0x000F0000);
	Check2(CLT_Bits(DBDITHERMATRIXA,X0Y1,0xFFFFFFFF) == 0x0000F000);
	Check2(CLT_Bits(DBDITHERMATRIXA,X1Y1,0xFFFFFFFF) == 0x00000F00);
	Check2(CLT_Bits(DBDITHERMATRIXA,X2Y1,0xFFFFFFFF) == 0x000000F0);
	Check2(CLT_Bits(DBDITHERMATRIXA,X3Y1,0xFFFFFFFF) == 0x0000000F);

	Check1(RA_DBDITHERMATRIXB == 0x8084);
	Check2(CLT_Bits(DBDITHERMATRIXB,X0Y2,0xFFFFFFFF) == 0xF0000000);
	Check2(CLT_Bits(DBDITHERMATRIXB,X1Y2,0xFFFFFFFF) == 0x0F000000);
	Check2(CLT_Bits(DBDITHERMATRIXB,X2Y2,0xFFFFFFFF) == 0x00F00000);
	Check2(CLT_Bits(DBDITHERMATRIXB,X3Y2,0xFFFFFFFF) == 0x000F0000);
	Check2(CLT_Bits(DBDITHERMATRIXB,X0Y3,0xFFFFFFFF) == 0x0000F000);
	Check2(CLT_Bits(DBDITHERMATRIXB,X1Y3,0xFFFFFFFF) == 0x00000F00);
	Check2(CLT_Bits(DBDITHERMATRIXB,X2Y3,0xFFFFFFFF) == 0x000000F0);
	Check2(CLT_Bits(DBDITHERMATRIXB,X3Y3,0xFFFFFFFF) == 0x0000000F);

	Check1(RA_DBDESTALPHACONST == 0x807C);
	Check2(CLT_Bits(DBDESTALPHACONST,DESTALPHACONSTSSB0,0xFFFFFFFF) == 0x00FF0000);
	Check2(CLT_Bits(DBDESTALPHACONST,DESTALPHACONSTSSB1,0xFFFFFFFF) == 0x000000FF);

TermScaffold();

printf("Finished! %d macros and defines were checked.\n", count);

}

