/*----------------------------------------------------------------------------------
 *	
 *	FileName:	Test2D_CLT.c
 *	Date:		October 10, 1995
 *	By:			Ali Latifi		x3487
 *
 *  Copyright  1995 by The 3DO Company.  All rights reserved.
 *
 *  Modification History:
 *
 *	By:			Date:		Changed:
 *				
 *--------------------------------------------------------------------------------*/

#include "Test2D_CLT.h"

#define EXTERNAL_RELEASE	1
#undef NUPUPSIM

extern GState*		gGState;
extern Err			gErr;
extern Boolean		gCPadFlag;
extern uint32 		gTexture[];
extern uint32 		gPipData[4];

enum {
		eCltPipControlBlock_Create,
		eCltPipControlBlock_Delete,
		eCltTxData_Create,
		eCltTxData_Delete,

		eCLT_Test_End
};

char* Test2D_CLT_Names[] = {
		"CltPipControlBlock_Create",
		"CltPipControlBlock_Delete",
		"CltTxData_Create",
		"CltTxData_Delete",

		""
};

/* -------------------------------------------------------------------------------- */
/* Initialize test suite */
int32 Test2D_CLT_Initialize(void)
{
	 return 0;
}

/* -------------------------------------------------------------------------------- */
/* Return the number of test's to preform */
int32 Test2D_CLT_Count(void)
{
	return eCLT_Test_End-1;
}

/* -------------------------------------------------------------------------------- */
void Test2D_CLT( TEST2DAPI* theTest2DAPI, int32 testNumber )
{
	SpriteObj			*tempSprite;
	CltTxData			*tempTxDate;
	CltPipControlBlock	*tempPipCtlBlock;
	ControlPadEventData	tempCPad;

	TOUCH(theTest2DAPI);

	BeginTest(FT, Test2D_CLT_Names[testNumber]);

	switch( testNumber ) {

		case eCltPipControlBlock_Create:
			{
				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				tempSprite = Spr_Create(0);
				tempTxDate = CltTxData_Create (gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
				Spr_SetTextureData (tempSprite, tempTxDate);
				tempPipCtlBlock = CltPipControlBlock_Create (gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
				if (tempPipCtlBlock == 0L)
					LogResult(FAILED, "Unable to create pip control block.");
				else
					LogResult(PASSED, "");
				Spr_SetPIP (tempSprite, tempPipCtlBlock);
				Do_Texturing(tempSprite);
				Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
				Spr_Scale (tempSprite, 10., 10.);
				F2_Draw (gGState, tempSprite);
				GS_SendList(gGState);
				GS_WaitIO(gGState);
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = CltPipControlBlock_Delete (tempPipCtlBlock);
				if (gErr < 0)
					printf("Unable to delete the CltPipControlBlock.  Error# %d\n", gErr);
				gErr = CltTxData_Delete (tempTxDate);
				if (gErr < 0)
					printf("Unable to delete the CltTxData.  Error# %d\n", gErr);
				gErr = Spr_Delete(tempSprite);
				if (gErr < 0)
					printf("Unable to delete the sprite.  Error# %d\n", gErr);
			}
			break;
			
		case eCltPipControlBlock_Delete:
			{
				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				tempSprite = Spr_Create(0);
				tempTxDate = CltTxData_Create (gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
				Spr_SetTextureData (tempSprite, tempTxDate);
				tempPipCtlBlock = CltPipControlBlock_Create (gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
				Spr_SetPIP (tempSprite, tempPipCtlBlock);
				Do_Texturing(tempSprite);
				Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
				Spr_Scale (tempSprite, 15., 15.);
				F2_Draw (gGState, tempSprite);
				GS_SendList(gGState);
				GS_WaitIO(gGState);
				gErr = CltPipControlBlock_Delete (tempPipCtlBlock);
				if (gErr < 0)
					LogResult(FAILED, "Unable to delete the CltPipControlBlock.  Error# %d", gErr);
				else
					LogResult(PASSED, "");
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = CltTxData_Delete (tempTxDate);
				if (gErr < 0)
					printf("Unable to delete the CltTxData.  Error# %d\n", gErr);
				gErr = Spr_Delete(tempSprite);
				if (gErr < 0)
					printf("Unable to delete the sprite.  Error# %d\n", gErr);
			}
			break;

		case eCltTxData_Create:
			{
				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				tempSprite = Spr_Create(0);
				tempTxDate = CltTxData_Create (gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
				if (tempTxDate == 0L)
					LogResult(FAILED, "Unable to create CltTxData.");
				else
					LogResult(PASSED, "");
				Spr_SetTextureData (tempSprite, tempTxDate);
				tempPipCtlBlock = CltPipControlBlock_Create (gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
				Spr_SetPIP (tempSprite, tempPipCtlBlock);
				Do_Texturing(tempSprite);
				Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
				Spr_Scale (tempSprite, 20., 20.);
				F2_Draw (gGState, tempSprite);
				GS_SendList(gGState);
				GS_WaitIO(gGState);
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = CltPipControlBlock_Delete (tempPipCtlBlock);
				if (gErr < 0)
					printf("Unable to delete the CltPipControlBlock.  Error# %d\n", gErr);
				gErr = CltTxData_Delete (tempTxDate);
				if (gErr < 0)
					printf("Unable to delete the CltTxData.  Error# %d\n", gErr);
				gErr = Spr_Delete(tempSprite);
				if (gErr < 0)
					printf("Unable to delete the sprite.  Error# %d\n", gErr);
			}
			break;

		case eCltTxData_Delete:
			{
				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				tempSprite = Spr_Create(0);
				tempTxDate = CltTxData_Create (gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
				Spr_SetTextureData (tempSprite, tempTxDate);
				tempPipCtlBlock = CltPipControlBlock_Create (gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
				Spr_SetPIP (tempSprite, tempPipCtlBlock);
				Do_Texturing(tempSprite);
				Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
				Spr_Scale (tempSprite, 25., 25.);
				F2_Draw (gGState, tempSprite);
				GS_SendList(gGState);
				GS_WaitIO(gGState);
				gErr = CltPipControlBlock_Delete (tempPipCtlBlock);
				if (gErr < 0)
					printf("Unable to delete the CltPipControlBlock.  Error# %d\n", gErr);
				gErr = CltTxData_Delete (tempTxDate);
				if (gErr < 0)
					LogResult(FAILED, "Unable to delete the CltTxData.  Error# %d", gErr);
				else
					LogResult(PASSED, "");
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Spr_Delete(tempSprite);
				if (gErr < 0)
					printf("Unable to delete the sprite.  Error# %d\n", gErr);
			}
			break;

		default:
			BeginTest(FT, "<Invalid Test>");
			LogComment("Test called with an invalid testNumber (%d)", testNumber );
	}
}

/* -------------------------------------------------------------------------------- */
