/*----------------------------------------------------------------------------------
 *	
 *	FileName:	Test2D_GRO.c
 *	Date:		October 10, 1995
 *	By:			Ali Latifi		x3487
 *
 *  Copyright  1995 by The 3DO Company.  All rights reserved.
 *
 *  Modification History:
 *
 *	By:			Date:		Changed:
 *				
 *--------------------------------------------------------------------------------*/

#include "Test2D_GRO.h"

#define EXTERNAL_RELEASE	1
#undef NUPUPSIM

extern GState*		gGState;
extern Err			gErr;
extern Boolean		gCPadFlag;
extern uint32 		gTexture[], ggTexture[];
extern uint32 		gPipData[4];

enum {
	eGro_Create,
	eGro_Delete,
	eGro_GetHDelta,
	eGro_GetHeight,
	eGro_GetPosition,
	eGro_GetSpriteArray,
	eGro_GetVDelta,
	eGro_GetWidth,
	eGro_SetHDelta,
	eGro_SetHeight,
	eGro_SetPosition,
	eGro_SetSpriteArray,
	eGro_SetVDelta,
	eGro_SetWidth,

	eGRO_Test_End
};

char* Test2D_GRO_Names[] = {
	"Gro_Create",
	"Gro_Delete",
	"Gro_GetHDelta",
	"Gro_GetHeight",
	"Gro_GetPosition",
	"Gro_GetSpriteArray",
	"Gro_GetVDelta",
	"Gro_GetWidth",
	"Gro_SetHDelta",
	"Gro_SetHeight",
	"Gro_SetPosition",
	"Gro_SetSpriteArray",
	"Gro_SetVDelta",
	"Gro_SetWidth",

	""
};

/* -------------------------------------------------------------------------------- */
/* Initialize test suite */
int32 Test2D_GRO_Initialize(void)
{
	 return 0;
}

/* -------------------------------------------------------------------------------- */
/* Return the number of test's to preform */
int32 Test2D_GRO_Count(void)
{
	return eGRO_Test_End-1;
}

/* -------------------------------------------------------------------------------- */
void Test2D_GRO( TEST2DAPI* theTest2DAPI, int32 testNumber )
{
	uint32				tempValue;
	Point2				tempPoint;
	GridObj 			*tempGrid;
	SpriteObj			*tempSprite, *temp1Sprite, *tempSpArray[9];
	ControlPadEventData	tempCPad;

	BeginTest(FT, Test2D_GRO_Names[testNumber]);

/* ------------------------------------------------------- */
/* ---------- S E T U P   T H E   S P R I T E S ---------- */
/* ------------------------------------------------------- */
	CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);

	tempSprite = Spr_Create(0);
	Spr_CreateTxData (tempSprite, gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
	Spr_CreatePipControlBlock (tempSprite, gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
	Do_Texturing(tempSprite);
	Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
	Spr_Translate (tempSprite, 50, 50);
	Spr_Scale (tempSprite, 5., 5.);
	F2_Draw (gGState, tempSprite);

	temp1Sprite = Spr_Create(0);
	Spr_CreateTxData (temp1Sprite, ggTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
	Spr_CreatePipControlBlock (temp1Sprite, gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
	Do_Texturing(temp1Sprite);
	Spr_ResetCorners (temp1Sprite, SPR_TOPLEFT);
	Spr_Translate (temp1Sprite, 150, 150);
	Spr_Scale (temp1Sprite, 5., 5.);
	F2_Draw (gGState, temp1Sprite);

	GS_SendList(gGState);
	GS_WaitIO(gGState);

/* ------------------------------------------------------- */
	switch( testNumber ) {

		case eGro_Create:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				LogResult(PASSED, "");
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 20.;
				tempPoint.y = 20.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_Delete:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					LogResult(FAILED, "Unable to delete a Grid object.  Error # %d", gErr);
				else
					LogResult(PASSED, "");
			}
			else
				LogResult(UNRESOLVED, "Unable to create a Grid object");
		}
		break;

		case eGro_GetHDelta:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				Gro_GetHDelta (tempGrid, &tempPoint);
				if (tempPoint.x != 70. || tempPoint.y != 0.)
					LogResult(FAILED, "");
				else
					LogResult(PASSED, "");

				printf ("             (x = 70)   X = %f\n", tempPoint.x);
				printf ("             (y = 0)    Y = %f\n", tempPoint.y);
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_GetHeight:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				tempValue = Gro_GetHeight (tempGrid);
				if (tempValue != 3)
					LogResult(FAILED, "");
				else
					LogResult(PASSED, "");
				printf ("             (height = 3)   Height = %d\n", tempValue);
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_GetPosition:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 70.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				Gro_GetPosition (tempGrid, &tempPoint);
				if (tempPoint.x != 70. || tempPoint.y != 80.)
					LogResult(FAILED, "");
				else
					LogResult(PASSED, "");
				printf ("             (x = 70)   X = %f\n", tempPoint.x);
				printf ("             (y = 80)   Y = %f\n", tempPoint.y);
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_GetSpriteArray:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				LogResult(PASSED, "");
				printf ("             Array @ = 0x%lx\n", (int32)Gro_GetSpriteArray(tempGrid));
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_GetVDelta:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				Gro_GetVDelta (tempGrid, &tempPoint);
				if (tempPoint.x != 0. || tempPoint.y != 70.)
					LogResult(FAILED, "");
				else
					LogResult(PASSED, "");
				printf ("             (x = 0)    X = %f\n", tempPoint.x);
				printf ("             (y = 70)   Y = %f\n", tempPoint.y);
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_GetWidth:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				tempValue = Gro_GetWidth (tempGrid);
				if (tempValue != 3)
					LogResult(FAILED, "");
				else
					LogResult(PASSED, "");
				printf ("             (width = 3)   Width = %d\n", tempValue);
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_SetHDelta:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				gErr = Gro_SetHDelta(tempGrid, &tempPoint);
				if (gErr < 0)
					LogResult(FAILED, "Unable to set HDelta.  Error # %d", gErr);
				else
					LogResult(PASSED, "");
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				Gro_GetHDelta (tempGrid, &tempPoint);
				printf ("             (x = 70)   X = %f\n", tempPoint.x);
				printf ("             (y = 0)    Y = %f\n", tempPoint.y);
				if (tempPoint.x != 70. || tempPoint.y != 0.)
					printf("             Warning:  Watch out for the X and Y results.\n");
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_SetHeight:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				gErr = Gro_SetHeight (tempGrid, 3);
				if (gErr < 0)
					LogResult(FAILED, "Unable to set height.  Error # %d", gErr);
				else
					LogResult(PASSED, "");
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				tempValue = Gro_GetHeight (tempGrid);
				printf ("             (height = 3)   Height = %d\n", tempValue);
				if (tempValue != 3)
					printf("             Warning:  Watch out for the height result.\n");
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_SetPosition:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 70.;
				tempPoint.y = 80.;
				gErr = Gro_SetPosition(tempGrid, &tempPoint);
				if (gErr < 0)
					LogResult(FAILED, "Unable to set position.  Error #%d", gErr);
				else
					LogResult(PASSED, "");
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				Gro_GetPosition (tempGrid, &tempPoint);
				printf ("             (x = 70)   X = %f\n", tempPoint.x);
				printf ("             (y = 80)   Y = %f\n", tempPoint.y);
				if (tempPoint.x != 70. || tempPoint.y != 80.)
					printf("             Warning:  Watch out for the X and Y results.\n");
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_SetSpriteArray:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				gErr = Gro_SetSpriteArray (tempGrid, tempSpArray);
				if (gErr < 0)
					LogResult(FAILED, "Unable to set sprite array.  Error #%d", gErr);
				else
					LogResult(PASSED, "");
				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				printf ("             Array @ = 0x%lx\n", (int32)Gro_GetSpriteArray(tempGrid));
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_SetVDelta:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				Gro_SetWidth (tempGrid, 3);
				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				gErr = Gro_SetVDelta(tempGrid, &tempPoint);
				if (gErr < 0)
					LogResult(FAILED, "Unable to set VDelta.  Error# %d", gErr);
				else
					LogResult(PASSED, "");
				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				Gro_GetVDelta (tempGrid, &tempPoint);
				printf ("             (x = 0)    X = %f\n", tempPoint.x);
				printf ("             (y = 70)   Y = %f\n", tempPoint.y);
				if (tempPoint.x != 0. || tempPoint.y != 70.)
					printf("             Warning:  Watch out for the X and Y results.\n");
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		case eGro_SetWidth:
		{
			tempGrid = Gro_Create(0);
			if (tempGrid != 0L)
			{
				gErr = Gro_SetWidth (tempGrid, 3);
				if (gErr < 0)
					LogResult(FAILED, "Unable to set width.  Error# %d", gErr);
				else
					LogResult(PASSED, "");

				Gro_SetHeight (tempGrid, 3);
				tempPoint.x = 80.;
				tempPoint.y = 80.;
				Gro_SetPosition(tempGrid, &tempPoint);
				tempPoint.x = 70.;
				tempPoint.y = 0.;
				Gro_SetHDelta(tempGrid, &tempPoint);
				tempPoint.x = 0.;
				tempPoint.y = 70.;
				Gro_SetVDelta(tempGrid, &tempPoint);

				tempSpArray[0] = tempSpArray[2] = tempSpArray[4] = tempSpArray[6] = tempSpArray[8] = tempSprite;
						tempSpArray[1] = tempSpArray[3] = tempSpArray[5] = tempSpArray[7] = temp1Sprite;
				Gro_SetSpriteArray (tempGrid, tempSpArray);

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				F2_Draw (gGState, tempGrid);
				GS_SendList(gGState);
				GS_WaitIO(gGState);

				tempValue = Gro_GetWidth (tempGrid);
				printf ("             (width = 3)   Width = %d\n", tempValue);
				if (tempValue != 3)
					printf("             Warning:  Watch out for the width result.\n");
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				gErr = Gro_Delete(tempGrid);
				if (gErr < 0)
					printf("Unable to delete a Grid object.  Error # %d", gErr);
			}
			else
				LogResult(FAILED, "Unable to create a Grid object");
		}
		break;

		default:
			BeginTest(FT, "<Invalid Test>");
			LogComment("Test called with an invalid testNumber (%d)", testNumber );
	}
	Spr_Delete(tempSprite);
	Spr_Delete(temp1Sprite);

	TOUCH(theTest2DAPI);

}

/* -------------------------------------------------------------------------------- */
