#ifdef MACINTOSH
#include <kernel:types.h>
#include <graphics:clt:clt.h>
#else
#include <kernel/types.h>
#include <graphics/clt/clt.h>
#endif
#include "mercury.h"
#include "SimpleMatData.h"

#define kFloatsPerVertex       6
#define kSizePerVertex         (kFloatsPerVertex*sizeof(float))

#define donutvtx(x,z,cs,sn,dim) (x+600)*cs/(8.*256.), (x+600)*sn/(8.*256.), z/8, x*cs/(dim*256.), x*sn/(dim*256.), z/dim


float gHitherPlane = 10.0;

/********************************************************************************/
/* Pods:									*/
/*	1.	gCubePrePod		PreLit					*/
/*		gBoxDynPod		DynLit					*/
/*		gTorusDynPod	DynLit						*/
/*										*/
/*	2.	gPlanePrePod	PreLit						*/
/********************************************************************************/
/* Lights:									*/
/*		gNoLightList							*/
/*		gDirLightList							*/
/*		gDirPointFogTransLightList					*/
/*		gPointSpecLightList						*/
/*		gFogOnlyLight							*/
/*		gFogOnlyTransLight						*/
/********************************************************************************/
/* Materials:									*/
/*		gTransMaterial		0.5 0.5 0.5 0.5				*/
/*		gTrans2Material		0.0 0.0 0.0 0.2				*/
/********************************************************************************/
/* pMatrices:									*/
/*		gPodMatrix	Identity					*/
/********************************************************************************/


/* Cube		*/

static float gCubeVertices[] = {
				-150,-150,-25,		1.,0.,0.,	
				-150,-150,25,		0.,1.,0.,	
				-150,-100,-25,		0.,0.,1.,	
				-150,-100,25,		1.,0.,1.,	
				-100,-150,-25,		0.,1.,1.,	
				-100,-150,25,		1.,1.,1.,	
				-100,-100,-25,		0.,1.,1.,	
				-100,-100,25,		0.,1.,1.	
};


/*static float gCubeVertices[] = {
				-150,-150,-25,		1.,1.,1.,	
				-150,-150,25,		0.,0.,1.,	
				-150,-100,-25,		0.,1.,0.,	
				-150,-100,25,		1.,0.,0.,	
				-100,-150,-25,		1.,0.,1.,	
				-100,-150,25,		1.,1.,0.,	
				-100,-100,-25,		0.,1.,1.,	
				-100,-100,25,		0.,0.,0.	
};*/

/* location from first, normals (or color) from second */

static short gCubeSharedVerts[] = {
		0,1,			
		0,2,			
		1,0,			
		1,5,			
		2,0,			
		2,3,			
		3,0,			
		3,5,			
		4,1,			
		4,2,			
		5,4,			
		5,1,			
		6,3,			
		6,2,			
		7,3,			
		7,5				
};


static short gCubeIndices[] = {
		0 + STXT + NEWS + CFAN + PCLK,2,10,12,14,	/* -x */
		8 + NEWS + CFAN + PCLK,16,1,19,	/* -y */
		9 + NEWS + CFAN + PCLK,2,17,21,	/* -z */
		13 + NEWS + CFAN + PCLK,3,20,22,	/* +y */
		6 + NEWS + CFAN + PCLK,7,4,18,	/* +x */
		11 + NEWS + CFAN + PCLK,5,15,23,	/* +z */
		0 + STXT + NEWS + CFAN + PCLK,-1
};

static PodGeometry gCubeGeometry = {
    /* float fxmin, fymin, fzmin */           	-150,-150,-25,
    /* float fxextent, fyextent, fzextent */  	50,50,50,
    /* float *pvertex */                      	gCubeVertices,
    /* short *pshared */		      			gCubeSharedVerts,
    /* uint16 vertexcount */                  	8,
    /* uint16 sharedcount */                  	16,
    /* short *pinxex */                       	gCubeIndices,
    /* float *puv */                          	NULL
};


static Matrix gPodMatrix = {
    1.0, 0.0, 0.0,
    0.0, 1.0, 0.0,
    0.0, 0.0, 1.0,
    0.0, 0.0, 0.0
};

static uint32 gNoLightList[] = {
    0
};

/* change object transparency here */
Material gTransMaterial = {
    /* base color */                          { 0.5, 0.5, 0.5, 0.5},
    /* diffuse color */                       { 0.5, 0.5, 0.5 },
    /* shine */                               0.0,
    /* specular color */                      { 1.0, 1.0, 1.0 }
};

Material gOpaqueRedMaterial = {
    /* base color */                          { 0.1, 0.1, 0.1, 1.0},
    /* diffuse color */                       { 1.0, 0.1, 0.1 },
    /* shine */                               0.05,
    /* specular color */                      { 1.0, 1.0, 1.0 }
};


Pod gCubePrePod = {
    /* uint32 flags */                        0,
    /* struct Pod *pnext */                   &gBoxDynPod,
    /* void (*pcase)(CloseData*) */           M_SetupPreLit,
    /* struct PodTexture *ptexture */         NULL,
    /* struct PodGeometry *pgeometry */       &gCubeGeometry,
    /* Matrix *pmatrix */                     &gPodMatrix,
    /* uint32 *plights */                     gNoLightList,
    /* uint32 *puserdata */                   NULL,
    /* Material *pmaterial */                 &gOpaqueRedMaterial
};


/* Box	*/

static float gBoxVertices[] = {

				100,100,0,	-1,0,0,	/* 0	-x */
				100,100,25,	0,-1,0,	/* 1	-y */
				100,125,0,	0,0,-1,	/* 2	-z */
				100,125,25,	0,1,0,	/* 3	+y */
				125,100,0,	1,0,0,	/* 4	+x */
				125,100,25,	0,0,1,	/* 5	+z */
				125,125,0,	1,0,0,	/* 6	+x */
				125,125,25,	1,0,0	/* 7	+x */
};

/* location from first, normals (or color) from second */

static short gBoxSharedVerts[] = {
		0,1,			/* 8	-y */
		0,2,			/* 9	-z */
		1,0,			/* 10	-x */
		1,5,			/* 11	+z */
		2,0,			/* 12	-x */
		2,3,			/* 13	+y */
		3,0,			/* 14	-x */
		3,5,			/* 15	+z */
		4,1,			/* 16	-y */
		4,2,			/* 17	-z */
		5,4,			/* 18	+x */
		5,1,			/* 19	-y */
		6,3,			/* 20	+y */
		6,2,			/* 21	-z */
		7,3,			/* 22	+y */
		7,5				/* 23	+z */
};


static short gBoxIndices[] = {
		0 + STXT + NEWS + CFAN + PCLK,2,10,12,14,	/* -x */
		8 + NEWS + CFAN + PCLK,16,1,19,	/* -y */
		9 + NEWS + CFAN + PCLK,2,17,21,	/* -z */
		13 + NEWS + CFAN + PCLK,3,20,22,	/* +y */
		6 + NEWS + CFAN + PCLK,7,4,18,	/* +x */
		11 + NEWS + CFAN + PCLK,5,15,23,	/* +z */
		0 + STXT + NEWS + CFAN + PCLK,-1
};

static PodGeometry gBoxGeometry = {
    /* float fxmin, fymin, fzmin */           	100,100,0,
    /* float fxextent, fyextent, fzextent */  	25,25,25,
    /* float *pvertex */                      	gBoxVertices,
    /* short *pshared */		      			gBoxSharedVerts,
    /* uint16 vertexcount */                  	8,
    /* uint16 sharedcount */                  	16,
    /* short *pinxex */                       	gBoxIndices,
    /* float *puv */                          	NULL
};

Material gOpaqueMaterial = {
    /* base color */                          { 0.1, 0.1, 0.1, 1.0 },
    /* diffuse color */                       { 0.5, 0.0, 0.0 },
    /* shine */                               0.05,
    /* specular color */                      { 1.0, 1.0, 1.0 }
};

static LightDir LightDirData = {
	-0.577, -0.577, -0.577,
	1.0, 1.0, 1.0					
	};


static uint32 gDirLightList[] = {
    (uint32) &M_LightDir,
	(uint32) &LightDirData,
	0
};

Pod gBoxDynPod = {
    /* uint32 flags */                        0,
    /* struct Pod *pnext */                   &gTorusDynPod,
    /* void (*pcase)(CloseData*) */           M_SetupDynLitTrans,
    /* struct PodTexture *ptexture */         NULL,
    /* struct PodGeometry *pgeometry */       &gBoxGeometry,
    /* Matrix *pmatrix */                     &gPodMatrix,
    /* uint32 *plights */                     gDirLightList,
    /* uint32 *puserdata */                   NULL,
    /* Material *pmaterial */                 &gTransMaterial
};

/* Torus	*/

static float donutvtx[] =
{
    donutvtx(400,164,256,0,432.),
    donutvtx(164,400,256,0,432.),
    donutvtx(-164,400,256,0,500.),
    donutvtx(-400,164,256,0,800.),
    donutvtx(-400,-164,256,0,800.),
    donutvtx(-164,-400,256,0,500.),
    donutvtx(164,-400,256,0,432.),
    donutvtx(400,-164,256,0,432.),
		
    donutvtx(400,164,181,-181,432.),
    donutvtx(164,400,181,-181,432.),
    donutvtx(-164,400,181,-181,500.),
    donutvtx(-400,164,181,-181,800.),
    donutvtx(-400,-164,181,-181,800.),
    donutvtx(-164,-400,181,-181,500.),
    donutvtx(164,-400,181,-181,432.),
    donutvtx(400,-164,181,-181,432.),
		
    donutvtx(400,164,0,-256,432.),
    donutvtx(164,400,0,-256,432.),
    donutvtx(-164,400,0,-256,500.),
    donutvtx(-400,164,0,-256,800.),
    donutvtx(-400,-164,0,-256,800.),
    donutvtx(-164,-400,0,-256,500.),
    donutvtx(164,-400,0,-256,432.),
    donutvtx(400,-164,0,-256,432.),
		
    donutvtx(400,164,-181,-181,432.),
    donutvtx(164,400,-181,-181,432.),
    donutvtx(-164,400,-181,-181,500.),
    donutvtx(-400,164,-181,-181,800.),
    donutvtx(-400,-164,-181,-181,800.),
    donutvtx(-164,-400,-181,-181,500.),
    donutvtx(164,-400,-181,-181,432.),
    donutvtx(400,-164,-181,-181,432.),

    donutvtx(400,164,-256,0,432.),
    donutvtx(164,400,-256,0,432.),
    donutvtx(-164,400,-256,0,500.),
    donutvtx(-400,164,-256,0,800.),
    donutvtx(-400,-164,-256,0,800.),
    donutvtx(-164,-400,-256,0,500.),
    donutvtx(164,-400,-256,0,432.),
    donutvtx(400,-164,-256,0,432.),
		
    donutvtx(400,164,-181,181,432.),
    donutvtx(164,400,-181,181,432.),
    donutvtx(-164,400,-181,181,500.),
    donutvtx(-400,164,-181,181,800.),
    donutvtx(-400,-164,-181,181,800.),
    donutvtx(-164,-400,-181,181,500.),
    donutvtx(164,-400,-181,181,432.),
    donutvtx(400,-164,-181,181,432.),
		
    donutvtx(400,164,0,256,432.),
    donutvtx(164,400,0,256,432.),
    donutvtx(-164,400,0,256,500.),
    donutvtx(-400,164,0,256,800.),
    donutvtx(-400,-164,0,256,800.),
    donutvtx(-164,-400,0,256,500.),
    donutvtx(164,-400,0,256,432.),
    donutvtx(400,-164,0,256,432.),
		
    donutvtx(400,164,181,181,432.),
    donutvtx(164,400,181,181,432.),
    donutvtx(-164,400,181,181,500.),
    donutvtx(-400,164,181,181,800.),
    donutvtx(-400,-164,181,181,800.),
    donutvtx(-164,-400,181,181,500.),
    donutvtx(164,-400,181,181,432.),
    donutvtx(400,-164,181,181,432.)
};

static short donutindex[] = 
{
    8 + STXT,
    0,				/* texture */
    0,
    9,
    1,
    10 + PCLK,
    2,
    11 + PCLK,
    3,
    12 + PCLK,
    4,
    13 + PCLK,
    5,
    14 + PCLK,
    6,
    15 + PCLK,
    7,
    8 + PCLK,
    0,

    8 + 8 + NEWS + PCLK + CFAN,
    0 + 8,
    9 + 8,
    1 + 8,
    10 + 8 + PCLK,
    2 + 8,
    11 + 8 + PCLK,
    3 + 8,
    12 + 8 + PCLK,
    4 + 8,
    13 + 8 + PCLK,
    5 + 8,
    14 + 8 + PCLK,
    6 + 8,
    15 + 8 + PCLK,
    7 + 8,
    8 + 8 + PCLK,
    0 + 8,

    8 + 16 + NEWS + PCLK + CFAN,
    0 + 16,
    9 + 16,
    1 + 16,
    10 + 16 + PCLK,
    2 + 16,
    11 + 16 + PCLK,
    3 + 16,
    12 + 16 + PCLK,
    4 + 16,
    13 + 16 + PCLK,
    5 + 16,
    14 + 16 + PCLK,
    6 + 16,
    15 + 16 + PCLK,
    7 + 16,
    8 + 16 + PCLK,
    0 + 16,

    8 + 24 + NEWS + PCLK + CFAN,
    0 + 24,
    9 + 24,
    1 + 24,
    10 + 24 + PCLK,
    2 + 24,
    11 + 24 + PCLK,
    3 + 24,
    12 + 24 + PCLK,
    4 + 24,
    13 + 24 + PCLK,
    5 + 24,
    14 + 24 + PCLK,
    6 + 24,
    15 + 24 + PCLK,
    7 + 24,
    8 + 24 + PCLK,
    0 + 24,

    8 + 32 + NEWS + PCLK + CFAN,
    0 + 32,
    9 + 32,
    1 + 32,
    10 + 32 + PCLK,
    2 + 32,
    11 + 32 + PCLK,
    3 + 32,
    12 + 32 + PCLK,
    4 + 32,
    13 + 32 + PCLK,
    5 + 32,
    14 + 32 + PCLK,
    6 + 32,
    15 + 32 + PCLK,
    7 + 32,
    8 + 32 + PCLK,
    0 + 32,

    8 + 40 + NEWS + PCLK + CFAN,
    0 + 40,
    9 + 40,
    1 + 40,
    10 + 40 + PCLK,
    2 + 40,
    11 + 40 + PCLK,
    3 + 40,
    12 + 40 + PCLK,
    4 + 40,
    13 + 40 + PCLK,
    5 + 40,
    14 + 40 + PCLK,
    6 + 40,
    15 + 40 + PCLK,
    7 + 40,
    8 + 40 + PCLK,
    0 + 40,

    8 + 48 + NEWS + PCLK + CFAN,
    0 + 48,
    9 + 48,
    1 + 48,
    10 + 48 + PCLK,
    2 + 48,
    11 + 48 + PCLK,
    3 + 48,
    12 + 48 + PCLK,
    4 + 48,
    13 + 48 + PCLK,
    5 + 48,
    14 + 48 + PCLK,
    6 + 48,
    15 + 48 + PCLK,
    7 + 48,
    8 + 48 + PCLK,
    0 + 48,

    8 - 8 + NEWS + PCLK + CFAN,
    0 + 56,
    9 - 8,
    1 + 56,
    10 - 8 + PCLK,
    2 + 56,
    11 - 8 + PCLK,
    3 + 56,
    12 - 8 + PCLK,
    4 + 56,
    13 - 8 + PCLK,
    5 + 56,
    14 - 8 + PCLK,
    6 + 56,
    15 - 8 + PCLK,
    7 + 56,
    8 - 8 + PCLK,
    0 + 56,

    0 + STXT + NEWS + PCLK + CFAN,
    -1
};

static PodGeometry gTorusGeometry = {
    -1000/8, -1000/8, -400/8,
    2000/8, 2000/8, 800/8,
    donutvtx,
    0,
    sizeof(donutvtx)/(6*sizeof(float)),
    0,
    donutindex,
    NULL
};

LightPoint LightPointData = {
	100.0, 0.0, 100.0,			/* location */
	100000.0 * 32.0 * 1.0,			/* maxdist */
	100000.0,				/* intensity */
	0.5, 0.5, 0.5					
	};

static LightDir LightDirDataSpec = {
	-.577, -.577, -.577,
	0.5, 0.5, 0.5					
	};

uint32 gPointSpecLightList[] = {
    (uint32) &M_LightPoint,
	(uint32) &LightPointData,
    (uint32) &M_LightDirSpec,
	(uint32) &LightDirDataSpec,
	0
};

Material gOpaqueBlueMaterial = {
    /* base color */                          { 0.2, 0.2, 0.2, 1.0 },
    /* diffuse color */                       { 0.0, 0.0, 0.6 },
    /* shine */                               0.05,
    /* specular color */                      { 1.0, 1.0, 1.0 }
};


Pod gTorusDynPod = {
    /* uint32 flags */                        0,
    /* struct Pod *pnext */                   NULL,
    /* void (*pcase)(CloseData*) */           M_SetupDynLit,
    /* struct PodTexture *ptexture */         NULL,
    /* struct PodGeometry *pgeometry */       &gTorusGeometry,
    /* Matrix *pmatrix */                     &gPodMatrix,
    /* uint32 *plights */                     gPointSpecLightList,
    /* uint32 *puserdata */                   NULL,
    /* Material *pmaterial */                 &gOpaqueBlueMaterial
};

/* Plane Separate Pod 	*/

static float gPlaneVertices[] = {
     250.0, -250.0, -100.0, 1.0, 0.0, 0.0, /* 0 */
     250.0,  250.0, -100.0, 0.0, 1.0, 0.0, /* 1 */
    -250.0, -250.0, -100.0, 0.0, 0.0, 1.0, /* 2 */
    -250.0,  250.0, -100.0, 1.0, 0.0, 0.0, /* 3 */
};

static short gPlaneIndices[] = {
		0, 1, 2, 3,
		0 + STXT + NEWS + CFAN + PCLK,-1
};

static PodGeometry gPlaneGeometry = {
    /* float fxmin, fymin, fzmin */           	0,0,0,
    /* float fxextent, fyextent, fzextent */  	500.0,500.0,100.0,
    /* float *pvertex */                      	gPlaneVertices,
    /* short *pshared */		      	NULL,
    /* uint16 vertexcount */                  	4,
    /* uint16 sharedcount */                  	0,
    /* short *pinxex */                       	gPlaneIndices,
    /* float *puv */                          	NULL
};


static Matrix gPodMatrix2 = {
    1.0, 0.0, 0.0,
    0.0, 1.0, 0.0,
    0.0, 0.0, 1.0,
    0.0, 0.0, 0.0
};

Material gTrans2Material = {
    /* base color */                          { 0.0, 0.0, 0.0, 0.4 },
    /* diffuse color */                       { 0.5, 0.5, 0.5 },
    /* shine */                               0.0,
    /* specular color */                      { 1.0, 1.0, 1.0 }
};


Pod gPlanePrePod = {
    /* uint32 flags */                        0,
    /* struct Pod *pnext */                   NULL, 							
    /* void (*pcase)(CloseData*) */           M_SetupPreLitTrans,
    /* struct PodTexture *ptexture */         NULL,
    /* struct PodGeometry *pgeometry */       &gPlaneGeometry,
    /* Matrix *pmatrix */                     &gPodMatrix2,
    /* uint32 *plights */                     gNoLightList,
    /* uint32 *puserdata */                   NULL,
    /* Material *pmaterial */                 &gTrans2Material
};


static LightDir LightDirData2 = {
	-0.577, -0.577, -0.577,
	1.0, 1.0, 1.0					
	};

/*		fog0 = 1.0/(fogfar - fognear) 		*/
/*		fog1 = fognear/(fogfar - fognear)	*/
static LightFog LightFogData = 
{
	1.0/(100.0 - 10.0),	10.0/(100.0 - 10.0)	
};

static LightFog	FogOnly =
{
	1.0/(100.0 - 10.0),	10.0/(100.0 - 10.0)	
};

uint32 gDirPointFogLightList[] = {
    (uint32) &M_LightPoint,
	(uint32) &LightPointData,
    (uint32) &M_LightDir,
	(uint32) &LightDirData2,
	(uint32) &M_LightFog,
	(uint32) &LightFogData,
	0
};

uint32 gDirPointFogTransLightList[] = {
    (uint32) &M_LightPoint,
	(uint32) &LightPointData,
    (uint32) &M_LightDir,
	(uint32) &LightDirData2,
	(uint32) &M_LightFogTrans,
	(uint32) &LightFogData,
	0
};

uint32 gFogOnlyLight[] = {
	(uint32) &M_LightFog,
	(uint32) &FogOnly,
	0
};

uint32 gFogOnlyTransLight[] = {
	(uint32) &M_LightFogTrans,
	(uint32) &FogOnly,
	0
};
