
/*
 * @(#) CompressionTests.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		CompressionTest.c

	Contains:	Test The Compression Lib Written by Martin Taillefer

	Written by:	Eric Hegstrom
				Adapted from code by Martin Taillefer
	

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 10/18/94	ELH		Start revision to use Babs (rewrite !)
		 <1>	 7/27/94	ELH		Version 1.0 CompressionTest.c

	To Do:
*/



#include "globals.h"
#include "CompressUtils.h"


/*****************************************************************************/

/**********************************************************/
void mainInit( void )
{
	Err	err;

	if ((err = OpenCompressionFolio()) < 0)
		LogError(err);

	compWorkPtr = (void *) malloc( GetCompressorWorkBufferSize( NULL ) );
	decompWorkPtr = (void *) malloc( GetDecompressorWorkBufferSize( NULL ) );

	if ((err = CloseCompressionFolio()) < 0)
		LogError(err);
}

/**********************************************************/
int main(int argc,	char **argv)

{
	uint32	indx;
	Err		err;
	int32	wBSize;
		
	InitScaffold( KERNEL, argc, argv);
	mainInit();
	
	/*   ---------      TestOpenCompress     ------------  */
	{
		gTestName = "OpenCompressor";
		BeginTest( BAT, gTestName);
		TestOpenCompress(2);	
		
	}
	
	/*   ---------     TestCloseCompress     ------------  */
	{
		gTestName = "CloseCompressor";
		BeginTest( BAT, gTestName);
		TestCloseCompress(2);	
		
	}
	
	/* OPEN THE COMPRESSION FOLIO FOR THE REST OF THE TESTS */
	if ((err = OpenCompressionFolio()) < 0)
	{
		gTestName = "ERROR ERROR ERROR Initialization Error - TESTS NOT RUN !!";
		BeginTest( BAT, gTestName);
		LogResult(UNRESOLVED, "OpenCompressionFolio Return error # $%x",err );
		LogError(err);
		
		return -1;
	}

	/*   ---------   TestGetCompressorWorkBufferSize  ------------  */
	for (indx = 0; indx < 10; indx++)
	{
		BeginTest( BAT, "GetCompressorWorkBufferSize - iteration #%d", indx);
		if ( (wBSize = GetCompressorWorkBufferSize( NULL )) <= 0 )
			LogResult( FAILED, "GetCompressorWorkBufferSize = %d", wBSize );
		else
			LogResult( PASSED, "GetCompressorWorkBufferSize = %d", wBSize );
		
	}

	/*   ---------   TestCreateCompressorVA  ------------  */
	for (indx = 0; indx < numTCCVA; indx++)
	{
		TestCreateCompressorVA(indx, TRUE);	
	}

	/*   ---------   TestCreateCompressorVA  ------------  */
	for (indx = 0; indx < numTCCVA; indx++)
	{
		TestCreateCompressorVA(indx, FALSE);	
	}

	/*   ---------    TestCreateCompressor   ------------  */
	for (indx = 0; indx < numTCC; indx++)
	{
		TestCreateCompressor(indx, TRUE);	
	}
	
	/*   ---------    TestCreateCompressor   ------------  */
	for (indx = 0; indx < numTCC; indx++)
	{
		TestCreateCompressor(indx, FALSE);	
	}
	
	/*   ---------    TestDeleteCompressor   ------------  */
	for (indx = 0; indx < numTDC; indx++)
	{
		TestDeleteCompressor(indx);	
	}
	
	/*   ---------        TestCompress       ------------  */
	for (indx = 0; indx < numTC; indx++)
	{
		TestCompress(indx);	
	}
	
	/*   ---------       TestDecompress       ------------  */
	for (indx = 0; indx < numTD; indx++)
	{
		TestDecompress(indx);	
	}

	/*   ---------     TestFeedDecompressor   ------------  */
	for (indx = 0; indx < numTFD; indx++)
	{
		TestFeedDecompressor(indx);	
	}
	
	/*   ---------     TestFeedCompressor   ------------  */
	for (indx = 0; indx < numTFC; indx++)
	{
		TestFeedCompressor(indx);	
	}
	
	
	TermScaffold();

	if (( err = CloseCompressionFolio()) < 0)
	{
		LogComment("CloseCompressionFolio Return error # $%x",err );
		LogError( err );
	}
	return 1;
}

