
/*
 * @(#) Decompress.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		Decompress.c

	Contains:	Test The Compression Lib Written by ric Hegstrom

	Written by:	Eric Hegstrom
	

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 10/25/94	ELH		created

	To Do:
*/


#include "globals.h"
#include "CompressUtils.h"

#define	DC_CLEANUP	if (srcData != NULL) free( srcData );			\
					if (destData != NULL) free( destData );			\
					if (decompData != NULL) free( decompData );		
					


/*****************************************************************************/
Err TestDecompress( int32	indx)
{
	#define		compOverhead 4560

	Err			err = -1;
	void		*srcData;
	void		*destData;
	void		*decompData = NULL;
	int32		compSize;
	int32		decompSize;
	
	

	gTestName = TD[indx].name;
	BeginTest( BAT, gTestName);
	
	
	srcData = (void *) malloc( TD[indx].srcSize*sizeof(uint32) );
	if (( srcData == NULL )&&( TD[indx].srcSize*sizeof(uint32)!=0))
	{
		LogResult(UNRESOLVED, "Error Allocating srcData # $%x",err );
		return -1;
	}

	destData = (void *) malloc( (TD[indx].srcSize+compOverhead)*sizeof(uint32) );
	if (( destData == NULL )&&( (TD[indx].srcSize+compOverhead)*sizeof(uint32)!=0))
	{
		LogResult(UNRESOLVED, "Error Allocating destData # $%x",err );
		DC_CLEANUP
		return -1;
	}

	decompData = (void *) malloc( (TD[indx].srcSize+compOverhead)*sizeof(uint32) );
	if (( decompData == NULL )&&( (TD[indx].srcSize+compOverhead)*sizeof(uint32)!=0))
	{
		LogResult(UNRESOLVED, "Error Allocating decompData # $%x",err );
		DC_CLEANUP
		return -1;
	}

	
	FillRandomWord( srcData, TD[indx].srcSize );
	
	/* do the compression */
	
	compSize = Compress(	srcData, TD[indx].srcSize, destData, (TD[indx].srcSize+compOverhead));

	if ( compSize < 0 )
	{
		LogResult(UNRESOLVED, "Compress Error # $%x",compSize );
		LogError(compSize);
		DC_CLEANUP
		return compSize;
	}
	
	/* Do the Decompression */
	
	err = Decompress( destData, compSize, decompData, (TD[indx].srcSize+compOverhead));
	decompSize = err;

	if ( (err >= 0)&&(!TD[indx].expErr) )
	{
		if(	decompSize != TD[indx].srcSize )
			LogResult( FAILED, "decompressed size (%d) DOES NOT match original size (%d)", decompSize, TD[indx].srcSize );
		else if( CompareMem( srcData, decompData, decompSize ) )
			LogResult( PASSED, "decompress data matches original" );
		else
			LogResult( FAILED, "decompress data DOES NOT match original" );
	}
	else if ( (err < 0)&&(TD[indx].expErr) )
	{
		LogResult(PASSED, "Decompress Expected Error # $%x",err );
		/* LogError(err); */
	}
	else if (err < 0)
	{
		LogResult(FAILED, "Decompress Unexpected Error # $%x",err);
		LogError(err);
	}
	else if (err >= 0)
	{
		LogResult(FAILED, "Decompress Unexpexted No-Error ");
	}
	
	
	DC_CLEANUP

	return err;
}

