
/*
 * @(#) FileUtils.c 96/10/21 1.2
 */

#include <file/filefunctions.h>
#include <stdlib.h>
#include <string.h>
#include <kernel/types.h>
#include <kernel/item.h>
#include <kernel/debug.h>
#include <tests/3do_scaf.h>

/* ---------------------------------------- */	
Err	GetFileInfo (FileInfo *fInfo, char *path)
{
	Err		err;
	RawFile	*fPtr;
	
	if ( (err = OpenRawFile( &fPtr, path, FILEOPEN_READ)) < 0)
	{
		LogError( err );
		LogComment( "Error $%x opeing for fileinfo ", err );
		return err;
	}
		
	if ( (err = GetRawFileInfo( fPtr, fInfo, sizeof(FileInfo))) < 0)
	{
		LogError( err );
		LogComment( "Error $%x getting fileinfo ", err );
		CloseRawFile( fPtr );
		return err;
	}
	
	CloseRawFile( fPtr );
	
	return 1;
}
	
/* ---------------------------------------- */			
/* close a file and check for errors */	
int32	MyCloseFile( Item fItem )
{
	int32	err;
	
	if ( (err = CloseFile( fItem )) < 0)
	{
		LogError(err);
		LogComment("Error $%x closing file in MyCloseFile routine", err);
	}
	return err;
}
					
/* ---------------------------------------- */			
/* return true if the file exists */
Boolean	FileExists( char *path )
{
	Item	fItem;
	
	if ( (fItem = OpenFile( path )) >= 0 )	/* the file exists */
	{
		MyCloseFile( fItem );
		return true;
	}
	return false;
}
/* ---------------------------------------- */			
/* create a file and check for errors */	
Err	MyCreateFile( char *path )
{
	Err	err;
	
	if ( (err = CreateFile( path )) < 0)
	{
		LogError(err);
		LogComment("Error $%x creating file in MyCreateFile routine", err);
	}
	return err;
}

/* ---------------------------------------- */			
/* if the file exists delete it and make sure it's gone */
/* returns error number if there is a problem and the file can't */
/* be deleted */
Boolean	MyDeleteFile( char *path )
{
	Err		err;
	
	if ( FileExists( path ) )
	{
		if ( (err = DeleteFile( path )) < 0)
		{
			LogError(err);
			LogComment("Error $%x deleting file in MyDeleteFile routine", err);
			return err;
		}
		if ( FileExists( path ) )
		{
			LogComment("Unable to actually delete file in MyDeleteFile routine");
			return -1;
		}
	}
	return 1;
}

/* ---------------------------------------- */			
/* Creates of Deletes a file based on a Boolean */
int32	MyCreateDeleteFile (	char	*path, 
								Boolean	delete	)	/* if true delete the file */
{
	if (delete)
		return ( MyDeleteFile( path) );
	else
		return ( MyCreateFile( path) );
}

