
/*
 * @(#) ILookupTests.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		ILookupTests.c

	Contains:	Item Lookup call tests, kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):
		
		<3>		6/26/95		ELH		Deleted FindVersionedItem Test (Obsolete)
		 <2>	 7/13/93	CDE		Derived from shell
		 <1>	 6/22/93	CDE		Version 1.0 of TestShell.c

	To Do:
*/


/* HINTS:

	 If you don't know how to do something, try to write the syntax of the routine, and use the word
		UNIMPLEMENTED in the header so you can find it later.
*/

#define	USE_4B1	1

/***** Tool description for Intro, Exit, Help and Session Options displays *****/
#define		kToolDescription		"Item Lookup Tests"

/***** Version string *****/
#define		kVersionStr		"V1.0"

/***** Programmer name for Help and Session Options displays *****/
#define		kProgrammerName		"Charlie Eckhaus"

/***** Call constants *****/
/* HINTS:
		
	 Create one constant for each call, e.g., kCallA corresponds to CallA().
		- Values needn't be sequential but must be  kFullTest (see baseUtils.h)
*/
#define		kCheckItem				1
#define		kFindItem				2
#define		kFindNamedItem			3
#define		kLookupItem				4


/***** Default log filename: *****/
#define		kDefaultLogFile			"ILookupTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
/* HINTS:

	 see usage of gIncrement1 in TestCallA()
	 Change the units to be meaningful for the tool, e.g., for a memory tool, kIncrementUnits = "bytes"
	 Change the default value to the minimum number of units for which the test is meaningful
*/
#define		kIncrement1Units	"units"
#define		kDefaultIncrement1	1


/***** Other test options *****/
#define kDefaultItemName "ILookupTests"
#define kDefaultFType KERNELNODE
#define kDefaultNType TASKNODE
#define kDefaultVers	0
#define kDefaultRev		0

/***** Include files *****/

#include "itemUtils.h"
#include "TestCheckItem.h"
#include "TestFindItem.h"
#include "TestFindNamedItem.h"
#include "TestLookupItem.h"

/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "ILookupTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "ILookupTests.c";	/* contains the name of the program */
char	* gTestName= "ILookupTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;			/* contains what level of testing occurs in this test */
int32	gTestType = 0;			/* interactive or automated */

/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */

/* The Item number corresponding to this task */
Item gThisTask;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* Parameter-checking */
Boolean gInvalidArg;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */
/* HINTS:

	 Add one global for each additional tool option
*/
char	gItemName[64];
uint8	gFType;
uint8	gNType;
uint8	gVers;
uint8	gRev;


#if 0
/***** Utilities *****/

/*
 * Moved to Utils - Stan Shepard
 */
char *printable(char *aName)
{
	if (aName == NULL) return "<NULL>";
	if (aName == " ") return "<space>";
	if (strlen(aName)) return aName;
	return "<empty>";
}

/**** (End Utilities) ****/
#endif

void	ShowHelp(void);
void	ShowHelp()
/* Show Help for this tool */
/* HINTS:

	 Incorporate your additional options, all of the category's call names
*/
{
		printf ( "#################################################################");
		printf ( "%s %s Help", kToolDescription, kVersionStr );
		printf ( "Written by %s for The 3DO Company", kProgrammerName );
		printf (    "#################################################################");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-out << no parameter>>\tDo not use a log file.");
		LogDiagnostic( 80, "     <<filename>>     \tUse specified log file.");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80,"-icmin <<value>>\tIterate calls starting at <<value>>.");
		LogDiagnostic( 80,"-icmax <<value>>\tEnd call iterations at <<value>>.");
		LogDiagnostic( 80,"-itmin <<value>>\tIterate tests starting at <<value>>.");
		LogDiagnostic( 80,"-itmax <<value>>\tEnd test iterations at <<value>>.");
		LogDiagnostic( 80, "");

		printf ( "-inc1 <<value>>\tUse <<value>> as increment."); 
		LogDiagnostic( 80, "");
		
		printf ( "-ftype <<value>>\tUse <<value>> as folio type."); 
		LogDiagnostic( 80, "       k\tKERNELNODE.");
		LogDiagnostic( 80, "       g\tGRAPHICSNODE.");
		LogDiagnostic( 80, "       a\tAUDIONODE.");
		LogDiagnostic( 80, "");
		
		printf ( "-ntype <<value>>\tUse <<value>> as node type."); 
		LogDiagnostic( 80, "       memfree\tMEMFREENODE.");
		LogDiagnostic( 80, "       list\tLISTNODE.");
		LogDiagnostic( 80, "       memhdr\tMEMHDRNODE.");
		LogDiagnostic( 80, "       folio\tFOLIONODE.");
		LogDiagnostic( 80, "       task\tTASKNODE.");
		LogDiagnostic( 80, "       firq\tFIRQNODE.");
		LogDiagnostic( 80, "       sema4\tSEMA4NODE.");
		LogDiagnostic( 80, "       message\tMESSAGENODE.");
		LogDiagnostic( 80, "       msgport\tMSGPORTNODE.");
		LogDiagnostic( 80, "       memlist\tMEMLISTNODE.");
		LogDiagnostic( 80, "       romtag\tROMTAGNODE.");
		LogDiagnostic( 80, "       driver\tDRIVERNODE.");
		LogDiagnostic( 80, "       ioreq\tIOREQNODE.");
		LogDiagnostic( 80, "       device\tDEVICENODE.");
		LogDiagnostic( 80, "       timer\tTIMERNODE.");
		LogDiagnostic( 80, "       errortext\tERRORTEXTNODE.");
		LogDiagnostic( 80, "       template\tAUDIO_TEMPLATE_NODE.");
		LogDiagnostic( 80, "       instrument\tAUDIO_INSTRUMENT_NODE.");
		LogDiagnostic( 80, "       knob\tAUDIO_KNOB_NODE.");
		LogDiagnostic( 80, "       sample\tAUDIO_SAMPLE_NODE.");
		LogDiagnostic( 80, "       cue\tAUDIO_CUE_NODE.");
		LogDiagnostic( 80, "       envelope\tAUDIO_ENVELOPE_NODE.");
		LogDiagnostic( 80, "       attachment\tAUDIO_ATTACHMENT_NODE.");
#if USE_4B1
		LogDiagnostic( 80, "       tuning\tAUDIO_TUNING_NODE.");
#endif
		LogDiagnostic( 80, "       filesystem\tFILESYSTEMNODE.");
		LogDiagnostic( 80, "       file\tFILENODE.");
#if USE_4B1
		LogDiagnostic( 80, "       filealias\tFILEALIASNODE.");
#endif
		LogDiagnostic( 80, "");
		
		printf ( "-inc1 <<value>>\tUse <<value>> as increment."); 
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-error none\tDon't check for errors.");
		LogDiagnostic( 80, "       cont\tWhen an error occurs, report it and continue.");
		LogDiagnostic( 80, "       stop\tWhen an error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-ierror none\tDon't check for incidental errors.");
		LogDiagnostic( 80, "        cont\tWhen an incidental error occurs, report it and continue.");
		LogDiagnostic( 80, "        stop\tWhen an incidental error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-time off\tDon't output timing information.");
		LogDiagnostic( 80, "      on \tOutput timing information.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-func off\tDon't verify functionality.");
		LogDiagnostic( 80,"      on \tVerify functionality.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-fault off\tMake known assumptions.");
		LogDiagnostic( 80,"       on \tSkip known assumptions.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-sel <<no parameter>>\tRun the entire test suite.");
		/* HINTS:

			 Replace the following with your category's call names and descriptions
		*/
		LogDiagnostic( 80,"     CheckItem			\tTest CheckItem.");
		LogDiagnostic( 80,"     FindItem          \tTest FindItem.");
		LogDiagnostic( 80,"     FindNamedItem     \tTest FindNamedItem.");
		LogDiagnostic( 80,"     LookupItem        \tTest LookupItem.");
		LogDiagnostic( 80, "");
		
		printf ( "#################################################################");
}

void ShowOptions(void);
void ShowOptions()
/* Show the option settings for this session */
/* HINTS:

	 Incorporate your additional options and the category's calls
*/
{
		char	aString[64];
		
		printf ( "#################################################################");
		printf ( "%s %s - Session options", kToolDescription, kVersionStr);
		printf ( "Written by %s for The 3DO Company", kProgrammerName );
		printf (   "#################################################################");
		
		if (gUseLogFile) LogDiagnostic( 80, "-out: Use log file: %s.",gLogFile); else LogDiagnostic( 80,"Do not use a log file.");
		
		printf ( "Iterate each test from %i %s",gMinOuterIterations, kIncrement1Units);
		printf ( " to %i %s,", gMaxOuterIterations, kIncrement1Units);
		printf ( " in increments of %i %s.",gIncrement1,kIncrement1Units); 
		printf ( "Within each test, iterate the call from %i to %i.",gMinInnerIterations,gMaxInnerIterations);
		
		printf ("-name: Use %s as item name.", gItemName);

		FolioTypeToStr(gFType, aString);
		printf ("-ftype: Use %s as folio type.", aString);

		NodeTypeToStr(gFType, gNType, aString);
		printf ("-ntype: Use %s as node type.", aString);
		
		printf ("-vers: Use %i as version number.", gVers);

		printf ("-rev: Use %i as revision number.", gRev);

		LogDiagnostic( 80, "-error: ");
		switch (gCheckErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80, "-ierror: ");
		switch (gCheckIncidentalErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for incidental errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an incidental error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first incidental error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80,"-time: ");
		if (gOutputTiming) LogDiagnostic( 80,"Output timing information."); else LogDiagnostic( 80,"Don't output timing information.");

		LogDiagnostic( 80,"-func: ");
		if (gVerify) LogDiagnostic( 80,"Verify functionality."); else LogDiagnostic( 80,"Don't verify functionality.");

		LogDiagnostic( 80,"-fault: ");
		if (gCheckFault) LogDiagnostic( 80,"Skip known assumptions."); else LogDiagnostic( 80,"Make known assumptions.");

		LogDiagnostic( 80,"-sel: ");
		switch (gCallSelector)
			{
			case kFullTest:
				LogDiagnostic( 80,"Run the entire test suite.");
				break;
				
			/* HINTS:
			
				 Customize for your category's calls
			*/
			case kCheckItem:
				LogDiagnostic( 80,"Test CheckItem.");
				break;

			case kFindItem:
				LogDiagnostic( 80,"Test FindItem.");
				break;
				
			case kFindNamedItem:
				LogDiagnostic( 80,"Test FindNamedItem.");
				break;
								
			case kLookupItem:
				LogDiagnostic( 80,"Test LookupItem.");
				break;
				
			} /* end switch */
		
		printf ( "#################################################################");
		
}

bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	TOUCH(theArgCount);
	TOUCH(theArgList);

	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	strcpy(gItemName, kDefaultItemName);
	gFType = kDefaultFType;
	gNType = kDefaultNType;
	gVers = kDefaultVers;
	gRev = kDefaultRev;
	

	gCheckFault = true;
	gVerify = true;
	
	return kRunTest;
}	/* end GetParams */



/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory(void);
void TestCategory()
/* HINTS:

	 Customize for your category's calls
*/
{
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCheckItem ) )
	{
		gTestAreaName = "CheckItem";		/* contains the name of the test    */
		TestCheckItem(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kFindItem ) )
	{
		gTestAreaName = "FindItem";		/* contains the name of the test    */
		TestFindItem(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kFindNamedItem ) )
	{
		gTestAreaName = "FindNamedItem";		/* contains the name of the test    */
		TestFindNamedItem(gMinOuterIterations, gMaxOuterIterations);
	}


	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kLookupItem ) )
	{
		gTestAreaName = "LookupItem";		/* contains the name of the test    */
		TestLookupItem(gMinOuterIterations, gMaxOuterIterations);
	}

}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram( void );

Boolean	InitProgram()
{
#if 0
	int theError;
#endif

	gThisTask = CURRENTTASKITEM;

#if 0
	if ( OpenAudioFolio() )
	{
		LogDiagnostic( 80,"Can't open the audio folio!");
		return FALSE;
	}

	theError = 0;
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
	
#endif	
	return TRUE;
}


int main (int argc, char **argv)
{
	InitScaffold( KERNEL, argc, argv);
	
	gTestName = (char *)malloc( sizeof(char)*1024 );
	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{

			TestCategory(); 
			
		}
	TermScaffold();
	}
	
    return(0);
 }

