
/*
 * @(#) TestCheckItem.c 96/10/24 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"
#include	"itemUtils.h"

/*
	File:		TestCheckItem.c

	Contains:	Test routine for CheckItem().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/13/93	CDE		Derived from shell V1.6

	To Do:
*/


#include "TestCheckItem.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeCheckItemVerifyStr		"Before iteration (%i)/\titem/ $%x,\tftype/ %i,\tntype/ %i,\tname/ %s"
#define		kAfterCheckItemVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyCheckItem ( Boolean beforeCall, long aIndx, void *aResult, Item aItem, uint8 ftype, uint8 ntype, char *aName)
/* Functionality verification for CheckItem */
{
	Boolean succeeded;
	Item actualItem;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeCheckItemVerifyStr, aIndx, aItem, ftype, ntype, printable(aName));
		return true;
	}
	else
	{
		
		LogDiagnostic( 12,kAfterCheckItemVerifyStr, aIndx, aResult);
		
		if ( aResult == NULL ) return false;
		
		actualItem = ( ( (ItemNodeP) aResult )->n_Item );
		succeeded = (actualItem == aItem);
		if (!succeeded)
			LogComment("!! Actual item number: $%x", actualItem);
			
		return ( succeeded );
	}
	
}

#define		CLEANUP

long	CallCheckItem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	void	*aResult = NULL;
	Item	aItem;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	
	TOUCH(currentOuterIteration);

	/***** Preparation for call iterations *****/
	aItem = FindNamedItem( MKNODEID(aFType,aNType), gItemName );
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyCheckItem ( kBeforeCall, minInnerIterations, aResult, aItem, aFType, aNType, gItemName );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = CheckItem( aItem, aFType, aNType );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyCheckItem ( kAfterCall, indx, aResult, aItem, aFType, aNType, gItemName );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long	OneCheckItem (  Boolean expectError, uint8 aFType, uint8 aNType, char *aName )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	void	*aResult = NULL;

	/* Ancillary variables */
	Item	aItem;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aItem = FindNamedItem( MKNODEID(aFType,aNType), aName );
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyCheckItem ( kBeforeCall, gCurrentIndex, aResult, aItem, aFType, aNType, aName );

/*
LogDiagnostic( 80," About to CheckItem( %d, %d, %d )", aItem, aFType, aNType );
*/
	aResult = CheckItem( aItem, aFType, aNType );
/*
LogDiagnostic( 80," CheckItem returns %d", aResult ); 
*/	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyCheckItem ( kAfterCall, gCurrentIndex, aResult, aItem, aFType, aNType, aName );

	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
		
	CLEANUP;
	
	return 0;
}


#if 0
#define	TEST_ONE_CHECKITEM(expVal, fTyp, nTyp, name)\
	sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'", gTestAreaName,fTyp, nTyp, name );	\
	if (expVal == kExpectError) \
		strcat(gTestName, kExpectErrorStr);	\
	BeginTest( BAT, gTestName );	\
	OneCheckItem(expVal, fTyp, nTyp, name); \

#else														
void TEST_ONE_CHECKITEM( Boolean expVal, uint8 fTyp, uint8 nTyp, char *name);
void TEST_ONE_CHECKITEM( Boolean expVal, uint8 fTyp, uint8 nTyp, char *name)
{
	sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'", gTestAreaName, fTyp, nTyp, name );
	if (expVal == kExpectError)
		strcat(gTestName, kExpectErrorStr);
	BeginTest( BAT, gTestName );
	OneCheckItem(expVal, fTyp, nTyp, name);
}
#endif


void ExerciseCheckItem( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "CheckItem";
	STATUS(callName);
	
	/* params: ftype, ntype */
	
	TEST_ONE_CHECKITEM(kExpectSuccess, KERNELNODE, FOLIONODE, "kernel");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, FOLIONODE, "Graphics");
	TEST_ONE_CHECKITEM(kExpectSuccess, KERNELNODE, FOLIONODE, "File");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, FOLIONODE, "audio");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, FOLIONODE, "Operamath");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, SEMA4NODE, "Swi");
	TEST_ONE_CHECKITEM(kExpectSuccess, KERNELNODE, TASKNODE, "Operator");
	TEST_ONE_CHECKITEM(kExpectSuccess, KERNELNODE, TASKNODE, "shell");
	TEST_ONE_CHECKITEM(kExpectSuccess, KERNELNODE, TASKNODE, "EventBroker");
	TEST_ONE_CHECKITEM(kExpectSuccess, KERNELNODE, DRIVERNODE, "timer");
	TEST_ONE_CHECKITEM(kExpectSuccess, KERNELNODE, DEVICENODE, "timer");
	TEST_ONE_CHECKITEM(kExpectSuccess, KERNELNODE, DRIVERNODE, "ram");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, DEVICENODE, "ram");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, DEVICENODE, "CD-ROM");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, DRIVERNODE, "mac");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, DEVICENODE, "mac");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, DRIVERNODE, "SPORT");
	TEST_ONE_CHECKITEM(kExpectError, KERNELNODE, DEVICENODE, "SPORT");

/*
 Invalid type
 low memory
*/
}

void TestCheckItem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseCheckItem();
	else
	{
		callName = "CheckItem";
		STATUS(callName);
		if (gOutputTiming)
			LogComment(kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallCheckItem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogComment(kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

