
/*
 * @(#) TestCloseItem.c 96/10/24 1.4
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestCloseItem.c

	Contains:	Test routine for CloseItem().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/16/93	CDE		Derived from TestOpenItem

	To Do:
*/


#include "TestCloseItem.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr		"Error initializing for %s"
#define		kIncFindNamedErrorStr	"!! Unable to find named item %s/ $%x"
#define		kIncLookupErrorStr		"!! Unable to lookup item $%x"
#define		kIncOpenItemErrorStr	"!! Unable to open item/ result/ $%x, ftype/ %i, ntype/ %i, tags/ $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

#define TAGARG_STR(i)	((char *) tags[i].ta_Arg)
#define TAGARG_UINT8(i)	((uint8) tags[i].ta_Arg)

Boolean		VerifyItemForClose( Item aItem )
{
	ItemNodeP aItemNode;

	/* Disable to investigate further: */
	return true;
		
	aItemNode = (ItemNodeP) LookupItem( aItem );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItemNode == NULL)
		{
			LogComment(kIncLookupErrorStr, aItem);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	
	InspectItemNode( "ItemNode", aItemNode );

	return true;
}

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeCloseItemVerifyStr		"Before iteration (%i)/\titem/ $%x"
#define		kAfterCloseItemVerifyStr		"After iteration (%i)/\tresult/ $%x"
#define		kStrMismatchStr "!! Actual %s/ %s"
#define		kNumMismatchStr "!! Actual %s/ %i"

/***** Test routines *****/
Boolean		VerifyCloseItem ( Boolean beforeCall, long aIndx, Err aResult, Item aItem )
/* Functionality verification for CloseItem */
{
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeCloseItemVerifyStr, aIndx, aItem);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterCloseItemVerifyStr, aIndx, aResult);
		
		VerifyItemForClose( aItem );
		
		return (aResult >= 0);
	}
	
}

#define		CLEANUP

long	CallCloseItem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	char	*aName = gItemName;
	uint8	aVers = gVers;
	uint8	aRev  = gRev;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};
	Item	aItem;
	
	TOUCH(currentOuterIteration);

	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	aItem = FindNamedItem(MKNODEID(aFType,aNType),aName);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncFindNamedErrorStr, aName, aItem);
			LogError( aItem );
			LogResult( UNRESOLVED, "FindNamedItem returned an error");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	aItem = OpenItem( MKNODEID(aFType,aNType), tags );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncOpenItemErrorStr, aItem, aFType, aNType, tags);
			LogError( aItem );
			LogResult( UNRESOLVED, "OpenItem returned an error");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyCloseItem ( kBeforeCall, minInnerIterations, aResult, aItem );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0;
		
		aResult = CloseItem( aItem );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyCloseItem ( kAfterCall, indx, aResult, aItem );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long	OneCloseItem ( Boolean expectError, uint8 aFType, uint8 aNType, char *aName, uint8 aVers, uint8 aRev )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};
	Item	aItem;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	aItem = FindNamedItem(MKNODEID(aFType,aNType),aName);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncFindNamedErrorStr, aName, aItem);
			LogError( aItem );
			LogResult( UNRESOLVED, "FindNamedItem returned an error");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	aItem = OpenItem( MKNODEID(aFType,aNType), tags );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncOpenItemErrorStr, aItem, aFType, aNType, tags);
			LogError( aItem );
			LogResult( UNRESOLVED, "OpenItem returned an error");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyCloseItem ( kBeforeCall, gCurrentIndex, aResult, aItem );

	aResult = CloseItem( aItem );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyCloseItem ( kAfterCall, gCurrentIndex, aResult, aItem );
		
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#if 0
#define	TEST_ONE_CLOSEITEM(expVal, fTyp, nTyp, name, vers, rev)	sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'; vers= %d; rev= %d", gTestAreaName,fTyp, nTyp, name, vers, rev );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneCloseItem(expVal, fTyp, nTyp, name, vers, rev);
														

#else
void TEST_ONE_CLOSEITEM( Boolean expVal, uint8 fTyp, uint8 nTyp, char *name, uint8 vers, uint8 rev);
void TEST_ONE_CLOSEITEM( Boolean expVal, uint8 fTyp, uint8 nTyp, char *name, uint8 vers, uint8 rev)
{
    sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'; vers= %d; rev= %d",
		gTestAreaName, fTyp, nTyp, name, vers, rev );
    if (expVal == kExpectError)
	strcat(gTestName, kExpectErrorStr);
    BeginTest( BAT, gTestName );
    OneCloseItem(expVal, fTyp, nTyp, name, vers, rev);
}
#endif


void ExerciseCloseItem( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "CloseItem";
	STATUS(callName);
	
	/* params: ftype, ntype, name, version, revision */
	
	TEST_ONE_CLOSEITEM(kExpectSuccess, 1, 10, "MSGPORTNODE_Item", gVers, gRev);

/*
 Invalid type
 low memory
*/
}

void TestCloseItem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseCloseItem();
	else
	{
	callName = "CloseItem";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallCloseItem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

