
/*
 * @(#) TestCreateSizedItem.c 96/10/24 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestCreateSizedItem.c

	Contains:	Test routine for CreateSizedItem().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/13/93	CDE		Derived from shell V1.6

	To Do:
*/


#include "TestCreateSizedItem.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kCreateSizedItemVerifyStr		"%s (%i)/ item/ $%x"
#define		kStrMismatchStr "!! Actual %s/ %s"
#define		kNumMismatchStr "!! Actual %s/ %i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncLookupErrorStr	"!! Unable to lookup item $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

#define TAGARG_STR(i)	((char *) tags[i].ta_Arg)
#define TAGARG_UINT8(i)	((uint8) tags[i].ta_Arg)

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeCreateSizedItemVerifyStr		"Before iteration (%i)/\ttype/ $%x,\ttags/ $%x,\tsize/ %i,\tname/ %s,\tVers.Rev/ %i.%i"
#define		kAfterCreateSizedItemVerifyStr		"After iteration (%i)/\tresult/ $%x"
#define		kStrMismatchStr "!! Actual %s/ %s"
#define		kNumMismatchStr "!! Actual %s/ %i"

Boolean		VerifyCreateSizedItem ( Boolean beforeCall, long aIndx, Item aResult, int32 itemType, TagArg *tags, int32 aSize )
/* Functionality verification for CreateSizedItem */
{
	ItemNodeP aItemNode;
	Boolean mismatch;
	Boolean anyMismatch;
	static char *name;
	static long version;
	static long revision;
	
	if (beforeCall == kBeforeCall)
	{		
		name = (char *) (tags[0].ta_Arg);
		version = (long) (tags[1].ta_Arg);
		revision = (long) (tags[2].ta_Arg);
		LogDiagnostic( 12,kBeforeCreateSizedItemVerifyStr, aIndx, itemType, tags, aSize, name, version, revision);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterCreateSizedItemVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return ( false );

		aItemNode = (ItemNodeP) LookupItem( aResult );
		/* Incidental error-checking */
		if (gCheckIncidentalErrors != kNone)
		{
			if (aItemNode == NULL)
			{
				LogComment(kIncLookupErrorStr, aResult);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
		
		anyMismatch = ( mismatch = ( aSize != aItemNode->n_Size ) );
		if ( mismatch ) LogComment(kNumMismatchStr, aItemNode->n_Size);
		
		anyMismatch |= ( mismatch = ( strcmp( name, aItemNode->n_Name) != 0) );
		if (mismatch) LogComment(kStrMismatchStr, aItemNode->n_Name);
		
		anyMismatch |= ( mismatch = ( version != aItemNode->n_Version ) );
		if (mismatch) LogComment(kNumMismatchStr, aItemNode->n_Version);
		
		anyMismatch |= ( mismatch = ( revision != aItemNode->n_Revision ) );
		if ( mismatch ) LogComment(kNumMismatchStr, aItemNode->n_Revision);
		
		return ( !anyMismatch );
	}
	
}

#define		CLEANUP

long	CallCreateSizedItem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	int32	aType;
	char	*aName = gItemName;
	uint8	aVers = gVers;
	uint8	aRev  = gRev;
	int32	aSize = gSize;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};

	TOUCH(currentOuterIteration);

	aType = MKNODEID(aFType,aNType);
	
	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyCreateSizedItem ( kBeforeCall, minInnerIterations, aResult, aType, tags, aSize );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = CreateSizedItem( aType, tags, aSize );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyCreateSizedItem ( kAfterCall, indx, aResult, aType, tags, aSize );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneCreateSizedItem (  Boolean expectError, uint8 aFType, uint8 aNType, char *aName, uint8 aVers, uint8 aRev, int32 aSize )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	int32	aType;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aType = MKNODEID(aFType,aNType);
	
	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyCreateSizedItem ( kBeforeCall, gCurrentIndex, aResult, aType, tags, aSize );

	aResult = CreateSizedItem( aType, tags, aSize );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyCreateSizedItem ( kAfterCall, gCurrentIndex, aResult, aType, tags, aSize );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_CREATESIZEDITEM(expVal, fTyp, nTyp, name, vers, rev, size)	sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'; vers= %d; rev= %d; size= %d", gTestAreaName,fTyp, nTyp, name, vers, rev, size );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneCreateSizedItem(expVal, fTyp, nTyp, name, vers, rev, size); \
														


void ExerciseCreateSizedItem( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "CreateSizedItem";
	STATUS(callName);
	
	/* params: */
	
	TEST_ONE_CREATESIZEDITEM(kExpectError, 1, 10, "MSGPORTNODE_Item", gVers, gRev, 1024);
	TEST_ONE_CREATESIZEDITEM(kExpectError, gFType, gNType, gItemName, gVers, gRev, gSize);
	TEST_ONE_CREATESIZEDITEM(kExpectError, gFType, gNType, gItemName, gVers, gRev, gSize);
/*
 Invalid type
 low memory
*/
}

void TestCreateSizedItem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseCreateSizedItem();
	else
	{
		callName = "CreateSizedItem";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallCreateSizedItem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

