
/*
 * @(#) TestCreateThread.c 96/10/24 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestCreateThread.c

	Contains:	Test routine for CreateThread().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/19/93	CDE		Derived from TestSendSignal

	To Do:
*/


#include "TestCreateThread.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr			"Error initializing for %s"
#define		kIncSpawnThreadErrorStr		"!! Unable to spawn thread/ $%x"
#define		kIncAllocSignalErrorStr		"!! Unable to allocate signal/ $%x"
#define		kIncLookupErrorStr			"!! Unable to lookup item $%x"
#define		kIncDeleteThreadErrorStr	"!! Unable to delete thread $%x/ $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern uint32 gSignalMask;
extern uint32 gThreadSignal;


/***** Test routines *****/


void ThreadForCreateThread(void)
{
	uint32 aSignalMask = gSignalMask;
	int32 aSignal;
	
	aSignal = AllocSignal( aSignalMask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aSignal < 0)
		{
			LogComment(kIncAllocSignalErrorStr, aSignal);
			if (gCheckIncidentalErrors == kStop);
/* CDE: WHAT IS THE RIGHT THING TO DO??? */
		}
	}
	gThreadSignal = (uint32) aSignal;
	while (1)
	{
		WaitSignal(gThreadSignal);			/* Wait until parent acknowledges */
		LogComment("Signal received.");
	}
}


Boolean	CleanupCreateThread( Item aThread )
{
	Err aErr;
	
	aErr = DeleteThread( aThread );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aErr < 0)
		{
			LogComment(kIncDeleteThreadErrorStr, aThread, aErr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	
	return true;
}
	
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeCreateThreadVerifyStr	"Before iteration (%i)/\tname/ %s,\tpri/ %i,\tcode/ $%x,\tstacksize/ $%i"
#define		kAfterCreateThreadVerifyStr		"After iteration (%i)/\tresult/ $%x"
#define		kStrMismatchStr "!! Actual %s/ %s"
#define		kNumMismatchStr "!! Actual %s/ %i"

Boolean		VerifyCreateThread ( Boolean beforeCall, long aIndx, Item aResult, char *name, uint8 pri, void (*code)(), int32 stacksize )
/* Functionality verification for CreateThread */
{
	ItemNodeP aItemNode;
	Boolean mismatch;
	Boolean anyMismatch;
		
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeCreateThreadVerifyStr, aIndx, name, pri, code, stacksize);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterCreateThreadVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return ( false );

		aItemNode = (ItemNodeP) LookupItem( aResult );
		/* Incidental error-checking */
		if (gCheckIncidentalErrors != kNone)
		{
			if (aItemNode == NULL)
			{
				LogComment(kIncLookupErrorStr, aResult);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
		
		anyMismatch = ( mismatch = ( strcmp( name, aItemNode->n_Name) != 0) );
		if (mismatch) LogComment(kStrMismatchStr, aItemNode->n_Name);
		
		anyMismatch |= ( mismatch = ( pri != aItemNode->n_Priority ) );
		if (mismatch) LogComment(kNumMismatchStr, aItemNode->n_Priority);
		
/* UNIMPLEMENTED: WANT TO CHECK CODE, STACKSIZE */

		return ( !anyMismatch );
	}
	
}

#define		CLEANUP		CleanupCreateThread( aResult )


long OneCreateThread (  Boolean expectError, char *aName, uint8 aPriority, int32 aStacksize )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	Item	aResult = (Item) 0;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyCreateThread ( kBeforeCall, gCurrentIndex, aResult, aName, aPriority, ThreadForCreateThread, aStacksize );

/*	aResult = CreateThread(aName, aPriority, ThreadForCreateThread, aStacksize); */
	aResult = CreateThread(ThreadForCreateThread, aName, aPriority, aStacksize, NULL);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyCreateThread ( kAfterCall, gCurrentIndex, aResult, aName, aPriority, ThreadForCreateThread, aStacksize );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}


#if 0
#define	TEST_ONE_CREATETHREAD(expVal, name, pri, sSize)	sprintf( gTestName, "%s | name= '%s'; pri= %d; stack=%d", gTestAreaName,name, pri, sSize );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneCreateThread(expVal, name, pri, sSize);
														

#else
void TEST_ONE_CREATETHREAD( Boolean expVal, char *name, uint8 pri, int32 sSize);
void TEST_ONE_CREATETHREAD( Boolean expVal, char *name, uint8 pri, int32 sSize)
{
    sprintf( gTestName, "%s | name= '%s'; pri= %d; stack=%d", gTestAreaName, name, pri, sSize );
    if (expVal == kExpectError)
	strcat(gTestName, kExpectErrorStr);
    BeginTest( BAT, gTestName );
    OneCreateThread(expVal, name, pri, sSize);
}
#endif


void ExerciseCreateThread( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "CreateThread";
	STATUS(callName);
	
	/* params: name, priority, stacksize */
	
	TEST_ONE_CREATETHREAD(kExpectSuccess, "TestThread", CURRENT_TASK_PRIORITY, 128);
	TEST_ONE_CREATETHREAD(kExpectSuccess, "TestThread", (CURRENT_TASK_PRIORITY + 1), 128);
/*
 Invalid type
 low memory
*/
}

void TestCreateThread( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
	TOUCH(minOuterIterations);
	TOUCH(maxOuterIterations);

	ExerciseCreateThread();
}

