
/*
 * @(#) TestDeleteThread.c 96/10/24 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestDeleteThread.c

	Contains:	Test routine for DeleteThread().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/19/93	CDE		Derived from TestSendSignal

	To Do:
*/


#include "TestDeleteThread.h"
#include "kernel/task.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr			"Error initializing for %s"
#define		kIncAllocSignalErrorStr		"!! Unable to allocate signal/ $%x"
#define		kIncLookupErrorStr			"!! Unable to lookup item $%x"
#define		kIncCreateThreadErrorStr	"!! Unable to create thread/ $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern uint32 gSignalMask;
extern uint32 gThreadSignal;


/***** Test routines *****/


void ThreadForDeleteThread(void)
{
	uint32 aSignalMask = gSignalMask;
	int32 aSignal;
	
	aSignal = AllocSignal( aSignalMask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aSignal < 0)
		{
			LogComment(kIncAllocSignalErrorStr, aSignal);
			if (gCheckIncidentalErrors == kStop);
/* CDE: WHAT IS THE RIGHT THING TO DO??? */
		}
	}
	gThreadSignal = (uint32) aSignal;
	while (1)
	{
		WaitSignal(gThreadSignal);			/* Wait until parent acknowledges */
		LogComment("Signal received.");
	}
}


/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeDeleteThreadVerifyStr	"Before iteration (%i)/\tthread/ $%x"
#define		kAfterDeleteThreadVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyDeleteThread ( Boolean beforeCall, long aIndx, Err aResult, Item aThread )
/* Functionality verification for DeleteThread */
{
	ItemNodeP aItemNode;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeDeleteThreadVerifyStr, aIndx, aThread);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterDeleteThreadVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return false;
			
		/* if it's been deleted we shouldn't be able to find it */
		aItemNode = (ItemNodeP) LookupItem( (Item) aResult );
		
		return (aItemNode == NULL);
	}
	
}

#define		CLEANUP

long CallDeleteThread (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Err		aResult = 0;
	Item	aItem;
	char	*aName = "TestThread";
	uint8	aPriority = CURRENT_TASK_PRIORITY;
	int32	aStacksize = 128;

	TOUCH(currentOuterIteration);

	/* Preparation for call iterations */
/*	aItem = CreateThread(aName, aPriority, ThreadForDeleteThread, aStacksize);	*/

	aItem = CreateThread(ThreadForDeleteThread, aName, aPriority, aStacksize, NULL);
	
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncCreateThreadErrorStr, aItem);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyDeleteThread ( kBeforeCall, minInnerIterations, aResult, aItem );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = DeleteThread( aItem );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyDeleteThread ( kAfterCall, indx, aResult, aItem );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)  return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneDeleteThread (  Boolean expectError, char *aName, uint8 aPriority, int32 aStacksize )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	Err		aResult = 0;

	/* Ancillary variables */
	Item	aItem;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
/*	aItem = CreateThread(aName, aPriority, ThreadForDeleteThread, aStacksize);	*/

	aItem = CreateThread(ThreadForDeleteThread, aName, aPriority, aStacksize, NULL);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncCreateThreadErrorStr, aItem);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyDeleteThread ( kBeforeCall, gCurrentIndex, aResult, aItem );

	aResult = DeleteThread( aItem );

	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyDeleteThread ( kAfterCall, gCurrentIndex, aResult, aItem );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}


#if 0
#define	TEST_ONE_DELETETHREAD(expVal, name, pri, sSize)	sprintf( gTestName, "%s | name= '%s'; pri= %d; stack=%d", gTestAreaName,name, pri, sSize );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneDeleteThread(expVal, name, pri, sSize);
														

#else
void TEST_ONE_DELETETHREAD( Boolean expVal, char *name, uint8 pri, int32 sSize);
void TEST_ONE_DELETETHREAD( Boolean expVal, char *name, uint8 pri, int32 sSize)
{
    sprintf( gTestName, "%s | name= '%s'; pri= %d; stack=%d", gTestAreaName, name, pri, sSize );
    if (expVal == kExpectError)
	strcat(gTestName, kExpectErrorStr);
    BeginTest( BAT, gTestName );
    OneDeleteThread(expVal, name, pri, sSize);
}
#endif


void ExerciseDeleteThread( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "DeleteThread";
	STATUS(callName);
	
	/* params: name, priority, stacksize */
	
	TEST_ONE_DELETETHREAD(kExpectSuccess, "TestThread", CURRENT_TASK_PRIORITY, 128);
	TEST_ONE_DELETETHREAD(kExpectSuccess, "TestThread", (CURRENT_TASK_PRIORITY + 1), 128);
/*
 Invalid type
 low memory
*/
}

void TestDeleteThread( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseDeleteThread();
	else
	{
		callName = "DeleteThread";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallDeleteThread(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

