
/*
 * @(#) itemUtils.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		itemUtils.c

	Contains:	Item call utilities

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <2>	 6/26/95	ELH		Changed GRAPHICSFOLIO to NST_GRAPHICS
		 <1>	 7/15/93	CDE		Derived from ILookupTests.c

	To Do:
*/
#define USE_4B1	1

#include "itemUtils.h"

#define LISTNODE	FIRQLISTNODE


#define 	kUnknownStr	"? (UNKNOWN) %i"
#define		kIncDeleteItemErrorStr		"Error deleting child/ $%x"

uint8 StrToFolioType(char *aString)
/* Convert a folio type specifier string into a folio type */
{
	uint8 aFolioType = 0;

	switch ( aString[0] )
	{
		case 'k':
			aFolioType = NST_KERNEL;
			break;
			
		case 'g':
#if USE_4B1
			aFolioType = NST_GRAPHICS;
#else
			aFolioType = NST_GRAPHICS;
#endif
			break;
			
		case 'f':
			aFolioType = NST_FILESYS;
			break;
			
		case 'a':
			aFolioType = NST_AUDIO;
			break;
/* remove MATHFOLIO for M2 -ELH 7-18-95 */
#if 0
		case 'm':
			aFolioType = MATHFOLIO;
			break;
#endif

		case 't':
			aFolioType = TEST_FTYPE;
			break;
			
		default:       
			gInvalidArg = true;
			break;
	}
				 
	return aFolioType;
}

#define mkstr(s)	#s
#define FolioMatchStrcpy(s,s2)	(aFolioType == s) strcpy(aString, s2)
#define NodeMatchStrcpy(s,s2)	(aNodeType == s) strcpy(aString, s2)
#define StrMatchAssign( aStr, aMatchStr, aVar, aValue )		(strcasecmp(aStr, aMatchStr) == 0) aVar = aValue			
#define StrMatchNodeTypeAssign( aMatchStr, aValue )		StrMatchAssign( aString, aMatchStr, aNodeType, aValue )			


uint8 StrToKernelNodeType(char *aString)
/* Convert a node type specifier string into a kernel node type */
{
	uint8 aNodeType = 0;

	if StrMatchNodeTypeAssign("memfree", MEMFREENODE);
	else if StrMatchNodeTypeAssign("FIRQlist", FIRQLISTNODE);

	else if StrMatchNodeTypeAssign("folio", FOLIONODE);
	else if StrMatchNodeTypeAssign("task", TASKNODE);
	else if StrMatchNodeTypeAssign("firq", FIRQNODE);
	else if StrMatchNodeTypeAssign("sema4", SEMA4NODE);
	else if StrMatchNodeTypeAssign("message", MESSAGENODE);
	else if StrMatchNodeTypeAssign("msgport", MSGPORTNODE);
	else if StrMatchNodeTypeAssign("driver", DRIVERNODE);
	else if StrMatchNodeTypeAssign("ioreq", IOREQNODE);
	else if StrMatchNodeTypeAssign("device", DEVICENODE);
	else if StrMatchNodeTypeAssign("timer", TIMERNODE);
	else if StrMatchNodeTypeAssign("errortext", ERRORTEXTNODE);
	
	return aNodeType;
}


uint8 StrToAudioNodeType(char *aString)
/* Convert a node type specifier string into a audio node type */
{
	uint8 aNodeType = 0;

	if StrMatchNodeTypeAssign("template", AUDIO_TEMPLATE_NODE);
	else if StrMatchNodeTypeAssign("instrument", AUDIO_INSTRUMENT_NODE);
	else if StrMatchNodeTypeAssign("knob", AUDIO_KNOB_NODE);
	else if StrMatchNodeTypeAssign("sample", AUDIO_SAMPLE_NODE);
	else if StrMatchNodeTypeAssign("cue", AUDIO_CUE_NODE);
	else if StrMatchNodeTypeAssign("envelope", AUDIO_ENVELOPE_NODE);
	else if StrMatchNodeTypeAssign("attachment", AUDIO_ATTACHMENT_NODE);
#if USE_4B1
	else if StrMatchNodeTypeAssign("tuning", AUDIO_TUNING_NODE);
#endif

	return aNodeType;
}

uint8 StrToFileFolioNodeType(char *aString)
/* Convert a node type specifier string into a file system node type */
{
	uint8 aNodeType = 0;

	if StrMatchNodeTypeAssign("filesystem", FILESYSTEMNODE);
	else if StrMatchNodeTypeAssign("file", FILENODE);
#if USE_4B1
	else if StrMatchNodeTypeAssign("filealias", FILEALIASNODE);
#endif
	
	return aNodeType;
}

uint8 StrToTestNodeType(char *aString)
/* Convert a test node type specifier string into a node type */
{
	uint8 aNodeType = 0;

	if StrMatchNodeTypeAssign("test", TEST_NTYPE);

	return aNodeType;
}


uint8 StrToNodeType(char *aString)
/* Convert a node type specifier string into a node type */
{
	uint8 aNodeType;

	if ( (aNodeType = StrToKernelNodeType(aString)) != 0 );
	else if ( (aNodeType = StrToAudioNodeType(aString)) != 0 );
	else if ( (aNodeType = StrToFileFolioNodeType(aString)) != 0 );
	else if ( (aNodeType = StrToTestNodeType(aString)) != 0 );
	else gInvalidArg = true;
	
	return aNodeType;
}

void UnknownTypeToStr(uint8 aType, char* aString)
/* Convert an unknown node type to a string */
{
	sprintf(aString, kUnknownStr, aType);
}

void FolioTypeToStr(uint8 aFolioType, char* aString)
/* Convert a folio type into a folio type specifier string */
{
	aString[0] = '\0';
	if FolioMatchStrcpy(NST_KERNEL,"NST_KERNEL");
	else if FolioMatchStrcpy(NST_AUDIO, "NST_AUDIO");
#if USE_4B1
	else if FolioMatchStrcpy(NST_GRAPHICS, "NST_GRAPHICS");
#else
	else if FolioMatchStrcpy(NODE_GRAPHICS, "NST_GRAPHICS");
#endif
	else if FolioMatchStrcpy(NST_FILESYS, "NST_FILESYS");
#if 0
	/* removed for m2 Compatibility ELH 7-18-95 */
	else if FolioMatchStrcpy(MATHFOLIO, "MATHFOLIO");
#endif
	else if FolioMatchStrcpy(TEST_FTYPE, "TEST_FTYPE");
	else UnknownTypeToStr(aFolioType, aString);
}

void KernelNodeTypeToStr(uint8 aNodeType, char* aString)
/* Convert a kernel node type into a string */
{
	aString[0] = '\0';
	if NodeMatchStrcpy(MEMFREENODE, "MEMFREENODE");
	else if NodeMatchStrcpy(LISTNODE, "LISTNODE");
	else if NodeMatchStrcpy(FIRQLISTNODE, "FIRQlist");
	else if NodeMatchStrcpy(FOLIONODE, "FOLIONODE");
	else if NodeMatchStrcpy(TASKNODE, "TASKNODE");
	else if NodeMatchStrcpy(FIRQNODE, "FIRQNODE");
	else if NodeMatchStrcpy(SEMA4NODE, "SEMA4NODE");
	else if NodeMatchStrcpy(MESSAGENODE, "MESSAGENODE");
	else if NodeMatchStrcpy(MSGPORTNODE, "MSGPORTNODE");


	else if NodeMatchStrcpy(DRIVERNODE, "DRIVERNODE");
	else if NodeMatchStrcpy(IOREQNODE, "IOREQNODE");
	else if NodeMatchStrcpy(DEVICENODE, "DEVICENODE");
	else if NodeMatchStrcpy(TIMERNODE, "TIMERNODE");
	else if NodeMatchStrcpy(ERRORTEXTNODE, "ERRORTEXTNODE");
	else UnknownTypeToStr(aNodeType, aString);
}

void GraphicsNodeTypeToStr(uint8 aNodeType, char* aString)
/* Convert a graphics node type into a string */
/* CURRENTLY, THERE ARE NO GRAPHICS NODES */
{
	UnknownTypeToStr(aNodeType, aString);
}

void FileNodeTypeToStr(uint8 aNodeType, char* aString)
/* Convert a file system node type into a string */
{
	aString[0] = '\0';
	if NodeMatchStrcpy(FILESYSTEMNODE, "FILESYSTEMNODE");
	else if NodeMatchStrcpy(FILENODE, "FILENODE");
#if USE_4B1
	else if NodeMatchStrcpy(FILEALIASNODE, "FILEALIASNODE");
#endif
	else UnknownTypeToStr(aNodeType, aString);
}


void AudioNodeTypeToStr(uint8 aNodeType, char* aString)
/* Convert a audio node type into a string */
{
	aString[0] = '\0';
	if NodeMatchStrcpy(AUDIO_TEMPLATE_NODE, "AUDIO_TEMPLATE_NODE");
	else if NodeMatchStrcpy(AUDIO_INSTRUMENT_NODE, "AUDIO_INSTRUMENT_NODE");
	else if NodeMatchStrcpy(AUDIO_KNOB_NODE, "AUDIO_KNOB_NODE");
	else if NodeMatchStrcpy(AUDIO_SAMPLE_NODE, "AUDIO_SAMPLE_NODE");
	else if NodeMatchStrcpy(AUDIO_CUE_NODE, "AUDIO_CUE_NODE");
	else if NodeMatchStrcpy(AUDIO_ENVELOPE_NODE, "AUDIO_ENVELOPE_NODE");
	else if NodeMatchStrcpy(AUDIO_ATTACHMENT_NODE, "AUDIO_ATTACHMENT_NODE");
#if USE_4B1
	else if NodeMatchStrcpy(AUDIO_TUNING_NODE, "AUDIO_TUNING_NODE");
#endif
	else UnknownTypeToStr(aNodeType, aString);
}

void MathNodeTypeToStr(uint8 aNodeType, char* aString)
/* Convert a math node type into a string */
/* CURRENTLY, THERE ARE NO MATH NODES */
{
	UnknownTypeToStr(aNodeType, aString);
}

void TestNodeTypeToStr(uint8 aNodeType, char* aString)
/* Convert a test node type into a string */
/* THIS IS JUST FOR TESTING */
{
	if NodeMatchStrcpy(TEST_NTYPE, "TEST_NTYPE");
	else UnknownTypeToStr(aNodeType, aString);
}



void NodeTypeToStr(uint8 aFolioType, uint8 aNodeType, char* aString)
/* Convert a node type into a string for the specified folio */
{
	if (aFolioType == NST_KERNEL) KernelNodeTypeToStr(aNodeType, aString);
#if USE_4B1
	else if (aFolioType == NST_GRAPHICS) GraphicsNodeTypeToStr(aNodeType, aString);
#else
	else if (aFolioType == NST_GRAPHICS) GraphicsNodeTypeToStr(aNodeType, aString);
#endif
	else if (aFolioType == NST_FILESYS) FileNodeTypeToStr(aNodeType, aString);
	else if (aFolioType == NST_AUDIO) AudioNodeTypeToStr(aNodeType, aString);
#if 0
/* changed for M2 compatibility 7-18-95 */
	else if (aFolioType == MATHFOLIO) MathNodeTypeToStr(aNodeType, aString);
#endif
	else if (aFolioType == TEST_FTYPE) TestNodeTypeToStr(aNodeType, aString);
	
}


/***** Inspect routines from VersionCheckerV1.0 *****/

void InspectNode( char *aName, NodeP aNode )
{
	LogDiagnostic( 12,"%s:", aName);
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"n_Next: $%x", aNode->n_Next);
	LogDiagnostic( 12,"n_Prev: $%x", aNode->n_Prev);
	LogDiagnostic( 12,"n_SubsysType: %i", aNode->n_SubsysType);
	LogDiagnostic( 12,"n_Type: %i", aNode->n_Type);
	LogDiagnostic( 12,"n_Priority: %i", aNode->n_Priority);
	LogDiagnostic( 12,"n_Flags: %i", aNode->n_Flags);
	LogDiagnostic( 12,"n_Size: %i", aNode->n_Size);
	LogDiagnostic( 12,"n_Name: %s", aNode->n_Name);
}

void InspectItemNode( char *aName, ItemNodeP aItemNode )
{
	uint8 aVersion;
	uint8 aRevision;
	
	aVersion= aItemNode->n_Version;
	aRevision= aItemNode->n_Revision;

	InspectNode( aName, (NodeP) aItemNode );
	LogDiagnostic( 12,"ItemNode:");
	LogDiagnostic( 12,"n_Version: %i", aVersion);
	LogDiagnostic( 12,"n_Revision: %i",aRevision);
/*
	LogDiagnostic( 12,"n_FolioFlags: $%x", aItemNode->n_FolioFlags);
*/
	LogDiagnostic( 12,"n_Item: $%x",aItemNode->n_Item);
	LogDiagnostic( 12,"n_Owner: $%x",aItemNode->n_Owner);

}

Boolean KillChild( Item aChildItem )
{
	Err		aErr;
	
	if ( aChildItem >= 0)
	{
		aErr = DeleteItem(aChildItem);
		if (gCheckIncidentalErrors != kNone)
		{
			if (aErr < 0)
			{
				LogComment(kIncDeleteItemErrorStr, aErr);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
	
	}
	
	return true;
}

