
/*
 * @(#) IOTests.c 96/10/21 1.2
 */

/*
	File:		IOTests.c

	Contains:	I/O call tests, Kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/23/93	CDE		Derived from TestShell Version 1.7

	To Do:
*/

#define	USE_4B1		1
#define	USE_1p01	1
#define	USE_BABS	1

/***** Checklist:

	 Change the "File" line above to reflect the name of your tool
	 Change the "Contains" line above to reflect the call category and folio being tested
	 Change the name in the "Written by" line above to your name
	 Add Change History lines above the previous lines:
		- Thereafter, increment the change number, specify the date, your initials, and a brief description
			of the change(s)
		- When you first change this file, replace with "<1> (date) (initials) Derived from shell (version)"
	 Look for bulleted () HINTS below.
*/

/* HINTS:

	 If you don't know how to do something, try to write the syntax of the routine, and use the word
		UNIMPLEMENTED in the header so you can find it later.
*/

/***** Tool description for Intro, Exit, Help and Session Options displays *****/
/* HINTS:
	 Change as appropriate for your tool.
*/
#define		kToolDescription		"I/O Tests"

/***** Version string *****/
/* HINTS:
		
	 Increment the main part when you rewrite the tool
	 Increment the "fractional" part when you make any other changes
*/
#define		kVersionStr		"V1.0"

/***** Programmer credit for Help and Session Options displays *****/
/* HINTS:
	 Replace with or add your name (if you don't, Charlie'll get all the credit).
*/
#define		kProgrammerName		"Charlie Eckhaus"

/***** Call selection constants *****/
/* HINTS:
		
	 Create one constant for each call, e.g., kCallA corresponds to CallA().
		- Values needn't be sequential but must be  kFullTest (see baseUtils.h)
*/
#define		kSendIO				1
#define		kWaitIO				2
#define		kCheckIO			3
#define		kAbortIO			4
#define		kDoIO				5
#if 0
#define		kOpenNamedDevice	6
#endif
#if USE_4B1
#define		kCreateIOReq		7
#define		kDeleteIOReq		8
#endif

/***** Default log filename: *****/
#define		kDefaultLogFile			"IOTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
/* HINTS:

	 see usage of gIncrement1 in TestCallA()
	 Change the units to be meaningful for the tool, e.g., for a memory tool, kIncrementUnits = "bytes"
	 Change the default value to the minimum number of units for which the test is meaningful
*/
#define		kIncrement1Units	"units"
#define		kDefaultIncrement1	1


/***** Other test options *****/
/* These seem like benign values: */
#define		kDefaultDevSpecifier	CMDC_TIMER
#define		kDefaultIOCommand		TIMERCMD_GETTIME_USEC 
#define		kDefaultLength			sizeof(DeviceStatus)
#define		kDefaultIOReqName		"TestIOReq"

/***** Include files *****/
/* HINTS:

	 Add a line for each utilities and call test header used
*/

#include "kernel/devicecmd.h"

#include "BaseUtils.h"
#include "msgUtils.h"
#include "itemUtils.h"
#include "ioUtils.h"
#if 0
#include "testSendIO.h"
#include "testWaitIO.h"
#include "testCheckIO.h"
#include "testAbortIO.h"
#include "testDoIO.h"
#endif
#if 0
#include "testOpenNamedDevice.h"
#endif
#if 0
#if USE_4B1
#include "testCreateIOReq.h"
#include "testDeleteIOReq.h"
#endif
#endif
#include <tests/3do_scaf.h>

/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "IOTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "IOTests.c";	/* contains the name of the program */
char	* gTestName= "IOTests";	/* contains the name of the test    */

/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */


/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* Parameter-checking */
Boolean gInvalidArg;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */
long	gDevSpecifier;
int32	gIOCommand;
int32	gLength;
char	gIOBuffer[256];
char	gIOReqName[64];
int32	gPriority;

/***** Utilities *****/
/* HINTS:

	 Anything you write that looks like it might be a generally useful call belongs here.
	 These will gradually migrate to our utilities libraries.
*/

/**** (End Utilities) ****/


void	ShowHelp(void);
void	ShowHelp()
/* Show Help for this tool */
/* HINTS:

	 Incorporate your additional options, all of the category's call names
*/
{
		printf ( "\n#################################################################\n");
		printf ( "%s %s Help\n", kToolDescription, kVersionStr );
		printf ( "Written by %s for The 3DO Company\n", kProgrammerName );
		printf (    "#################################################################\n");
		printf( "\n");
		
		printf( "-out << no parameter>>\tDo not use a log file.\n");
		printf( "     <<filename>>     \tUse specified log file.\n");
		printf( "\n");
		
		printf("-icmin <<value>>\tIterate calls starting at <<value>>.\n");
		printf("-icmax <<value>>\tEnd call iterations at <<value>>.\n");
		printf("-itmin <<value>>\tIterate tests starting at <<value>>.\n");
		printf("-itmax <<value>>\tEnd test iterations at <<value>>.\n");
		printf( "\n");

		printf ( "-inc1 <<value>>\tUse <<value>> as increment.\n"); 
		printf( "\n");
		
		printf( "-dev   \tDevice to test.\n");
		printf( "       \tCDROM\n");
		printf( "       \tSPORT\n");
		printf( "       \tMac\n");
		printf( "       \tTimer\n");
		printf( "\n");

		printf( "-cmd <<value>>\tUse <<value>> as command in I/O request.\n");
		printf( "\n");

		printf( "-len <<value>>\tUse <<value>> as buffer length in I/O request.\n");
		printf( "\n");
		
		printf( "-error none\tDon't check for errors.\n");
		printf( "       cont\tWhen an error occurs, report it and continue.\n");
		printf( "       stop\tWhen an error occurs, report it and stop the tool.\n");
		printf( "\n");

		printf( "-ierror none\tDon't check for incidental errors.\n");
		printf( "        cont\tWhen an incidental error occurs, report it and continue.\n");
		printf( "        stop\tWhen an incidental error occurs, report it and stop the tool.\n");
		printf( "\n");

		printf( "-time off\tDon't output timing information.\n");
		printf( "      on \tOutput timing information.\n");
		printf( "\n");

		printf("-func off\tDon't verify functionality.\n");
		printf("      on \tVerify functionality.\n");
		printf( "\n");

		printf("-fault off\tMake known assumptions.\n");
		printf("       on \tSkip known assumptions.\n");
		printf( "\n");

		printf("-sel <<no parameter>>\tRun the entire test suite.\n");
		/* HINTS:
		
			 Customize for your category's calls
		*/
		printf("     SendIO            \tTest SendIO.\n");
		printf("     WaitIO            \tTest WaitIO.\n");
		printf("     CheckIO           \tTest CheckIO.\n");
		printf("     AbortIO           \tTest AbortIO.\n");
		printf("     DoIO              \tTest DoIO.\n");
#if USE_4B1
		printf("     CreateIOReq       \tTest CreateIOReq.\n");
		printf("     DeleteIOReq       \tTest DeleteIOReq.\n");
#endif
		printf( "\n");
		
		printf ( "#################################################################\n");
}

void ShowOptions(void);
void ShowOptions()
/* Show the option settings for this session */
/* HINTS:

	 Incorporate your additional options and the category's calls
*/
{
		printf ( "\n#################################################################\n");
		printf ( "%s %s - Session options\n", kToolDescription, kVersionStr);
		printf ( "Written by %s for The 3DO Company\n", kProgrammerName );
		printf (   "#################################################################\n");
		
		if (gUseLogFile) printf( "-out: Use log file: %s.\n",gLogFile); else printf("Do not use a log file.\n");
		
		printf ( "Iterate each test from %i %s",gMinOuterIterations, kIncrement1Units);
		printf ( " to %i %s,", gMaxOuterIterations, kIncrement1Units);
		printf ( " in increments of %i %s.\n",gIncrement1,kIncrement1Units); 
		printf ( "Within each test, iterate the call from %i to %i.\n",gMinInnerIterations,gMaxInnerIterations);
		
		printf("-dev: ");
		switch (gDevSpecifier)
			{
			case kCDROM:
				printf("CD-ROM\n");
				break;
				
			case kTimer:
				printf("Timer\n");
				break;
				
			case kMac:
				printf("Mac\n");
				break;
				
			case kSPORT:
				printf("SPORT\n");
				break;
				
			} /* end switch */
		
		printf("-cmd: %i\n", gIOCommand);
		printf("-len: %i\n", gLength);
		
		printf( "-error: ");
		switch (gCheckErrors)
			{
			case kNone:
				printf("Don't check for errors.\n");
				break;
				
			case kCont:
				printf("When an error occurs, report it and continue.\n");
				break;
				
			case kStop:
				printf("Stop the tool on the first error to occur.\n");
				break;
			
			} /* end switch */

		printf( "-ierror: ");
		switch (gCheckIncidentalErrors)
			{
			case kNone:
				printf("Don't check for incidental errors.\n");
				break;
				
			case kCont:
				printf("When an incidental error occurs, report it and continue.\n");
				break;
				
			case kStop:
				printf("Stop the tool on the first incidental error to occur.\n");
				break;
			
			} /* end switch */

		printf("-time: ");
		if (gOutputTiming) printf("Output timing information.\n"); else printf("Don't output timing information.\n");

		printf("-func: ");
		if (gVerify) printf("Verify functionality.\n"); else printf("Don't verify functionality.\n");

		printf("-fault: ");
		if (gCheckFault) printf("Skip known assumptions.\n"); else printf("Make known assumptions.\n");

		printf("-sel: ");
		switch (gCallSelector)
			{
			case kFullTest:
				printf("Run the entire test suite.\n");
				break;
				
			/* HINTS:
			
				 Customize for your category's calls
			*/
			case kSendIO:
				printf("Test SendIO.\n");
				break;
					
			case kWaitIO:
				printf("Test WaitIO.\n");
				break;
					
			case kCheckIO:
				printf("Test CheckIO.\n");
				break;
					
			case kAbortIO:
				printf("Test AbortIO.\n");
				break;
					
			case kDoIO:
				printf("Test DoIO.\n");
				break;
					
#if 0
			case kOpenNamedDevice:
				printf("Test OpenNamedDevice.\n");
				break;
#endif

#if USE_4B1
			case kCreateIOReq:
				printf("Test CreateIOReq.\n");
				break;

			case kDeleteIOReq:
				printf("Test DeleteIOReq.\n");
				break;
#endif

			} /* end switch */
		
		printf ( "#################################################################\n");
		
}

bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	
	int			paramIndx = 0;
	Boolean		done = false;
	char		*tempStr;
	long		aLong;
	
	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	/* HINTS:
	
		 Initialize the globals for each additional tool option
	*/
	gDevSpecifier = kDefaultDevSpecifier;
	gIOCommand = kDefaultIOCommand;
	gLength = kDefaultLength;
	strcpy(gIOReqName,kDefaultIOReqName);
	gPriority = CURRENT_TASK_PRIORITY;
	
	gCheckFault = true;
	gVerify = true;
	
	return kRunTest;
}	/* end GetParams */



/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory(void);
void TestCategory()
/* HINTS:

	 Customize for your category's calls
*/
{
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kSendIO ) )
	{
		gTestAreaName = "SendIO";		/* contains the name of the test    */
		TestSendIO(gMinOuterIterations, gMaxOuterIterations);
	}
	
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kWaitIO ) )
	{
		gTestAreaName = "WaitIO";		/* contains the name of the test    */
		TestWaitIO(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCheckIO ) )
	{
		gTestAreaName = "CheckIO";		/* contains the name of the test    */
		TestCheckIO(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kAbortIO ) )
	{
		gTestAreaName = "AbortIO";		/* contains the name of the test    */
		TestAbortIO(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kDoIO ) )
	{
		gTestAreaName = "DoIO";		/* contains the name of the test    */
		TestDoIO(gMinOuterIterations, gMaxOuterIterations);
	}

#if 0
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kOpenNamedDevice ) )
	{
		gTestAreaName = "OpenNamedDevice";		/* contains the name of the test    */
		TestOpenNamedDevice(gMinOuterIterations, gMaxOuterIterations);
	}
#endif

#if USE_4B1
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCreateIOReq ) )
	{
		gTestAreaName = "CreateIOReq";		/* contains the name of the test    */
		TestCreateIOReq(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kDeleteIOReq ) )
	{
		gTestAreaName = "DeleteIOReq";		/* contains the name of the test    */
		TestDeleteIOReq(gMinOuterIterations, gMaxOuterIterations);
	}
#endif

}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram(void);
Boolean	InitProgram()
{
	int32 theError;
	

#if 0
	if ( OpenAudioFolio() )
	{
		printf("Can't open the audio folio!");
		return FALSE;
	}

	theError = SleepSeconds( 2 );
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
	
	return TRUE;
#endif	
}


int main (int argc, char **argv)
{
	InitScaffold (KERNEL, argc, argv );
	
	
	gTestName = (char *)malloc( sizeof(char)*1024 );
	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{
			
			TestCategory(); 
		}


	}
	TermScaffold();
 }






