
/*
 * @(#) TestAbortIO.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestAbortIO.c

	Contains:	Test routines for AbortIO().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/23/93	CDE		Derived from TestAbortIO

	To Do:
*/

#define	USE_4B1		1
#define	USE_1p01	1
#define	USE_BABS	1


#include "TestAbortIO.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeAbortIOVerifyStr	"Before iteration (%i)/\tioReq/ $%x"
#define		kAfterAbortIOVerifyStr	"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyAbortIO ( Boolean beforeCall, long aIndx, Err aResult, Item ioRequest )
/* Functionality verification for AbortIO */
{
	Boolean succeeded;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeAbortIOVerifyStr, aIndx, ioRequest);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterAbortIOVerifyStr, aIndx, aResult);

		succeeded = (aResult >= 0);
		if (succeeded)
			InspectIOReq(ioRequest);
			
		return succeeded;
	}
	
}

long	CallAbortIO (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Err		aResult;
	Item	aIOReq = 0;
	IOInfo  aIOInfo;
	char	*aName = "TestIOReq";
	int32	aPriority = CURRENT_TASK_PRIORITY;
	long	aDevSpecifier = gDevSpecifier;
	Item	aDevice = 0;
	int32	aIOCommand = gIOCommand;
	int32	aLength = gLength;
	
	memset(&aIOInfo,0,sizeof(IOInfo));
	aIOInfo.ioi_Command = aIOCommand;
	aIOInfo.ioi_Send.iob_Buffer = gIOBuffer;
	aIOInfo.ioi_Send.iob_Len = aLength;
	aIOInfo.ioi_Recv.iob_Buffer = gIOBuffer;
	aIOInfo.ioi_Recv.iob_Len = aLength;
	
	/* Preparation for call iterations */
	if ( !SetupIO( aName, aPriority, aDevSpecifier, 0, &aDevice, &aIOReq, &aIOInfo ) ) return kBreakTime;

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyAbortIO ( kBeforeCall, minInnerIterations, aResult, aIOReq );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
/* DOCs say it returns an Err; Headers define it as a void function */
/* Headers agree with DOCs now so lets check the error  - elh 12/22/94 */
#ifdef USE_BABS
		aResult = AbortIO( aIOReq );
#else
		AbortIO( aIOReq );
		aResult = 0;		/* Fake the return value */
#endif
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyAbortIO ( kAfterCall, indx, aResult, aIOReq );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					KillIOReq(aIOReq);
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	KillIOReq(aIOReq);

	return accumulator;
}

/* Individual call for parameter-testing */
long	OneAbortIO (Boolean expectError, char *aName, int32 aPriority, Item aDevSpecifier, int32 aIOCommand, int32 aLength )
{
	Boolean theError = FALSE;
	
	/* Return value */
	Err aResult;

	/* Ancillary variables */
	Item	aIOReq = 0;
	IOInfo  aIOInfo;
	Item	aDevice = 0;
	
  	gCurrentIndex++;
	
	memset(&aIOInfo,0,sizeof(IOInfo));
	aIOInfo.ioi_Command = aIOCommand;
	aIOInfo.ioi_Send.iob_Buffer = gIOBuffer;
	aIOInfo.ioi_Send.iob_Len = aLength;
	aIOInfo.ioi_Recv.iob_Buffer = gIOBuffer;
	aIOInfo.ioi_Recv.iob_Len = aLength;
	
	/* Preparation for call */
	if ( !SetupIO( aName, aPriority, aDevSpecifier, 0, &aDevice, &aIOReq, &aIOInfo ) ) return kBreakTime;

	/* Functionality verification (pre-) */
	if (gVerify) VerifyAbortIO( kBeforeCall, gCurrentIndex, aResult, aIOReq);
	
/* DOCs say it returns an Err; Headers define it as a void function */
/*	aResult = AbortIO( aIOReq ); */
	AbortIO( aIOReq );
	aResult = 0;		/* Fake the return value */
		
	/* Functionality verification (post-) */
	if (gVerify) theError = !VerifyAbortIO( kAfterCall, gCurrentIndex, aResult, aIOReq);
		
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogComment(kCallErrorStr, gCurrentIndex);
			else
			{
				LogComment(kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				KillIOReq(aIOReq);
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogComment(kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	KillIOReq(aIOReq);

	return 0;
}

#define	TEST_ONE_ABORTIO(expVal, name, pri, dev, com, len)	sprintf( gTestName, "%s - %s | pri= %d; dev= %d; com= %d; len= %d", gTestAreaName, name, pri, dev, com, len);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneAbortIO(expVal, name, pri, dev, com, len); \
														


void ExerciseAbortIO( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "AbortIO";
	STATUS(callName);
	
	/* params: name, priority, devSpecifier, IOCommand, length */
	TEST_ONE_ABORTIO(kExpectSuccess, "", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_ABORTIO(kExpectSuccess, " ", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_ABORTIO(kExpectSuccess, "W", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_ABORTIO(kExpectSuccess, "w", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_ABORTIO(kExpectSuccess, "5", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_ABORTIO(kExpectSuccess, "This is quite an unreasonably lengthy name", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);

	TEST_ONE_ABORTIO(kExpectSuccess, "TestIOReq", 0, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_ABORTIO(kExpectSuccess, "TestIOReq", TASK_MIN_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_ABORTIO(kExpectSuccess, "TestIOReq", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_ABORTIO(kExpectSuccess, "TestIOReq", TASK_MAX_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_ABORTIO(kExpectSuccess, "TestIOReq", 255, gDevSpecifier, gIOCommand, gLength);

	TEST_ONE_ABORTIO(kExpectSuccess, "TestIOReq", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, (gLength - 1));
	TEST_ONE_ABORTIO(kExpectSuccess, "TestIOReq", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, (gLength + 1));
/*
 Name: empty, short, lengthy, special characters
 Pri: 0, minimum, current task, maximum, 255
 DevSpecifier: kCDROM, kTimer, kMac, kSPORT, <= 0, non-device
 I/O command: CMD_WRITE, CMD_READ, CMD_STATUS
 Length: buffer length
 IOReq population: none, 1, many
 low memory
*/
}

void TestAbortIO( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseAbortIO();
	else
	{
		callName = "AbortIO";
		/* HINTS:
	
			 Replace with your call name.
		*/
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
			/* HINTS:
		
				 Possibly customize for your call.
			*/
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallAbortIO(indx, gMinInnerIterations, gMaxInnerIterations);
				/* HINTS:
			
					 Change to reflect your call name.
				*/
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
			/* HINTS:
		
				 Possibly customize for your call.
			*/
	}
}


