
/*
 * @(#) ThreadUtils.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		ThreadUtils.c

	Contains:	Thread test utilities

	Written by:	Charlie Eckhaus (based on SpawnThread.c by Jay Moreland,lynn ackler,philippe cassereau)

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 6/28/93	CDE		Derived from SpawnThread

	To Do:
*/

#define	USE_4B1		1
#define	USE_1p01	1
#define	USE_BABS	1

#include "threadUtils.h"

/* symbolic tag indices for the TagArgs array */
static enum TASKTAGIX
{
	TID_PRI,
	TID_MAXQ,
	TID_PC,
	TID_NAME,
	TID_STACKSIZE,
	TID_SP
};
  
Item SpawnThread(ThreadProc proc, int stacksize, int pri, char *name)
{
	Item	aThread;
	char	*theStack;
	TagArg	tags[] =
	{
		TAG_ITEM_PRI,				(void *)0,
#if USE_1p01
		CREATETASK_TAG_MAXQ,		(void *) 5000,
#else
		CREATETASK_TAG_MAXQ,		(void *) 2,
#endif
		CREATETASK_TAG_PC,			(void *)0,
		TAG_ITEM_NAME,				0,
		CREATETASK_TAG_STACKSIZE,	(void *)0,
		CREATETASK_TAG_SP,			(void *)0,
		TAG_END,					0
	};
	
	/* need to make our own stack */
	theStack = (char *) AllocMem(stacksize, MEMTYPE_ANY);
	if (theStack == NULL)
		return (Item) 0;

  														/* Set thread's: */
	tags[TID_PRI].ta_Arg = (void *) pri;				/* priority */
	tags[TID_PC].ta_Arg = (void *) proc;				/* initial pc */
	tags[TID_NAME].ta_Arg = (void *) name;				/* name (optional) */
	tags[TID_STACKSIZE].ta_Arg = (void *) stacksize;	/* stack size */
	tags[TID_SP].ta_Arg = theStack + stacksize;			/* stack pointer (stacks grow down) */

	/* Create the item for the task */
	aThread = CreateItem( MKNODEID(KERNELNODE, TASKNODE), tags) ;
	if (aThread < 0)
		FreeMem(theStack, stacksize);

	return aThread;
}  

