
/*
 * @(#) ioUtils.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		ioUtils.c

	Contains:	I/O test utilities 

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <2>	 8/19/93	CDE		Added io_MsgItem,ioSigItem to InspectIOReq
		 <1>	 7/21/93	CDE		Derived from msgUtils

	To Do:
*/
#define	USE_4B1		1
#define	USE_1p01	1
#define	USE_BABS	1

#include "ioUtils.h"

Item	gMessagePort = (Item) 0;

void InspectIOReq( Item aIOReq )
{
	IOReq *aIOReqP;
	
	aIOReqP = (IOReq *) LookupItem( aIOReq );
	if (aIOReqP == NULL)
		LogComment(kIncLookupItemErrorStr, aIOReq);
	else
	{
		LogDiagnostic( 12,"io_Actual: $%x", aIOReqP->io_Actual);
		LogDiagnostic( 12,",\tio_Error: $%x", aIOReqP->io_Error);
/*
		LogDiagnostic( 12,"io_Actual: $%x", aIOReqP->io_Actual);
		LogDiagnostic( 12,"io_Error: $%x", aIOReqP->io_Error);
		LogDiagnostic( 12,"io_MsgItem: $%x", aIOReqP->io_MsgItem);
		LogDiagnostic( 12,"io_SigItem: $%x", aIOReqP->io_SigItem);
*/
	}
}

void InspectIOBuf( IOBuf *aIOBuf )
{
	LogDiagnostic( 12,"iob_Buffer: $%x", aIOBuf->iob_Buffer);
	LogDiagnostic( 12,"iob_Len: %i", aIOBuf->iob_Len);
}

void InspectIOInfo( IOInfo *aIOInfo )
{
	LogDiagnostic( 12,"ioi_Command: %i", aIOInfo->ioi_Command);
	LogDiagnostic( 12,"ioi_Flags: %i", aIOInfo->ioi_Flags);
#if 0
	LogDiagnostic( 12,"ioi_Unit: %i", aIOInfo->ioi_Unit);

	LogDiagnostic( 12,"ioi_Flags2: $%x", aIOInfo->ioi_Flags2);
	LogDiagnostic( 12,"ioi_Pad: %i", aIOInfo->ioi_Pad);
#endif
	LogDiagnostic( 12,"ioi_CmdOptions: $%x", aIOInfo->ioi_CmdOptions);
#if 0
	LogDiagnostic( 12,"ioi_User: $%x", aIOInfo->ioi_User);
#endif
	LogDiagnostic( 12,"ioi_Offset: %i", aIOInfo->ioi_Offset);
	LogDiagnostic( 12,"ioi_Send:");
	InspectIOBuf(&aIOInfo->ioi_Send);
	LogDiagnostic( 12,"ioi_Recv:");
	InspectIOBuf(&aIOInfo->ioi_Recv);
}

Item CreateTestDevice ( long devSpecifier )
/* Create a device for testing */	
{
	char *aName;
	Item aDevice;
	
	switch (devSpecifier)
	{
		case kSPORT:
#if USE_1p01
			aName = "SPORT";
#else
			return SPORTDevice;
#endif
			break;

		case kCDROM:
			aName = "CD-ROM";
			break;
			
		case kMac:
			aName = "mac";
			break;
			
		case kTimer:
			aName = "timer";
			break;
			
		default:
			return 0;
	}
	
	aDevice = OpenItem ( FindNamedItem( MKNODEID(KERNELNODE,DEVICENODE), aName ), 0 );

	return aDevice;
}

Item CreateTestIOReq ( char *aName, int32 pri, Item aDevice, Item messagePort )
/* Create an I/O request for testing */	
{
	Item	aItem;
	TagArg	targs[5];
	
	targs[0].ta_Tag = CREATEIOREQ_TAG_DEVICE;
	targs[0].ta_Arg = (void *) aDevice;
	targs[1].ta_Tag = TAG_ITEM_NAME;
	targs[1].ta_Arg = (void *) aName;
	targs[2].ta_Tag = TAG_ITEM_PRI;
	targs[2].ta_Arg = (void *) pri;
	
/* NEED TO TEST THIS WITH AND WITHOUT THE TAG, WITH zero AND non-zero ta_Arg */
	targs[3].ta_Tag = CREATEIOREQ_TAG_REPLYPORT;
	targs[3].ta_Arg = (void *) messagePort;
	
	targs[4].ta_Tag = TAG_END;
	targs[4].ta_Arg = (void *) 0;
	
	aItem = CreateItem(MKNODEID(KERNELNODE,IOREQNODE), targs);
	return aItem;
}

Boolean SetupIO( char *aName, int32 pri, long devSpecifier, Item messagePort, Item *device, Item *ioReq, IOInfo *ioInfo )
{
	Item	aIOReq;
	Err		aResult;
	Item	aDevice;
	IOReq	*aIOReqP;
	
	aDevice = CreateTestDevice ( devSpecifier );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aDevice <= 0)
		{
			LogComment(kIncOpenDeviceErrorStr, devSpecifier);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	*device = aDevice;

	/*
		If no messagePort specified,
			if the global messagePort doesn't exist yet, create it
			Use the global messagePort
		Otherwise, use the specified messagePort
	*/
	/* WE PROBABLY NEED TO TEST CREATEMESSAGEPORT WITH 0, SO
		EVENTUALLY THIS NEEDS TO BE CHANGED. */
	if (messagePort == 0)
	{
		if ( gMessagePort == 0 )
		{
			messagePort = CreateMessagePort( "TestMessagePort" );
			/* Incidental error-checking */
			if (gCheckIncidentalErrors != kNone)
			{
				if (messagePort <= 0)
				{
					LogComment(kIncCreateMsgPortErrorStr, messagePort);
					if (gCheckIncidentalErrors == kStop) return false;
				}
			}
			gMessagePort = messagePort;
		}
		else messagePort = gMessagePort;
	}
		
	aIOReq = CreateTestIOReq( aName, pri, aDevice, messagePort );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aIOReq <= 0)
		{
			LogComment(kIncCreateIOReqErrorStr, aIOReq);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	*ioReq = aIOReq;
	
	aResult = SendIO(aIOReq, ioInfo);
	/* wack the error field into aResult if needed */
	aIOReqP = (IOReq *) LookupItem( aIOReq );
	if (aIOReqP != NULL)
		if (aIOReqP->io_Error < 0 ) aResult = aIOReqP->io_Error;
	
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aResult < 0)
		{
			LogError( aResult );
			LogComment(kIncSendIOErrorStr, aResult);
			LogResult( UNRESOLVED, "SendIO returned an error");
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	
	return true;
}
	

Boolean KillIOReq( Item aIOReq )
{
	Err		aErr;
	
	if ( aIOReq >= 0)
	{
		aErr = DeleteIOReq(aIOReq);
		if (gCheckIncidentalErrors != kNone)
		{
			if (aErr < 0)
			{
				LogComment(kIncIOReqErrorStr, aErr);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
	
	}
	
	return true;
}



