
/*
 * @(#) TestDeleteMsg.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestDeleteMsg.c

	Contains:	Test routine for DeleteMsg().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/28/93	CDE		Derived from TestDeleteIOReq

	To Do:
*/


#include "TestDeleteMsg.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kDeleteMsgVerifyStr		"%s (%i)/ \tresult/ $%x,\titem/ $%x"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncLookupErrorStr	"Error looking up item $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

#define		kBeforeDeleteMsgVerifyStr	"Before iteration (%i)/\titem/ $%x"
#define		kAfterDeleteMsgVerifyStr	"After iteration (%i)/\tresult/ $%x"

/***** Test routines *****/
Boolean		VerifyDeleteMsg ( Boolean beforeCall, long aIndx, Err aResult, Item aItem )
/* Functionality verification for DeleteMsg */
{
	ItemNodeP aItemNode;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeDeleteMsgVerifyStr, aIndx, aItem);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterDeleteMsgVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return false;

		/* The item is deleted if we can't find it */
		aItemNode = (ItemNodeP) LookupItem( aItem );
		if ( aItemNode != NULL )
			LogComment("!! Item still exists");
			
		return ( aItemNode == NULL );
	}
	
}

#define		CLEANUP		DeleteMsgPort(aReplyPort)

long CallDeleteMsg (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Err		aResult = 0;
	char *aPortName = gPortName;
	char *aMsgName = gMsgName;
	uint8 aPriority = gPriority;
	Item	aReplyPort = (Item) 0;
	Item	aMsg = (Item) 0;
	
	/***** Functionality verification (pre-call) *****/
	aReplyPort = CreateMessagePort( aPortName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aReplyPort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	
	aMsg = CreateMsg( aMsgName, aPriority, aReplyPort );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMsg <= 0)
		{
			LogComment(kIncCreateMsgErrorStr, aMsg, aReplyPort);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	
	if (gVerify) VerifyDeleteMsg ( kBeforeCall, minInnerIterations, aResult, aMsg );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = DeleteMsg( aMsg  );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyDeleteMsg ( kAfterCall, indx, aResult, aMsg );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
					{
						CLEANUP;
						return kBreakTime;
					}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
		
	CLEANUP;
	
	return accumulator;
}

long OneDeleteMsg ( Boolean expectError, Item aReplyPort, char *aMsgName, uint8 aPriority )
/* Individual call for parameter-testing */
{
 	Boolean theError= FALSE;
	
	/* Return value */
	Err		aResult = 0;

	/* Ancillary variables */
	Item	aMsg = (Item) 0;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aMsg = CreateMsg( aMsgName, aPriority, aReplyPort );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMsg <= 0)
		{
			LogComment(kIncCreateMsgErrorStr, aMsg, aReplyPort);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyDeleteMsg ( kBeforeCall, gCurrentIndex, aResult, aMsg );

	aResult = DeleteMsg( aMsg  );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyDeleteMsg ( kAfterCall, gCurrentIndex, aResult, aMsg );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}


#define	TEST_ONE_DELETEMSG(expVal, port, name, pri)	sprintf( gTestName, "%s | name= '%s'; port= 0x%x; pri= %d", gTestAreaName, name, port, pri);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneDeleteMsg(expVal, port, name, pri); \
														

#define EXERCISE_NAME(priority,expect)	gCurrentIndex = 0;\
										LogDiagnostic( 12,"-- For priority %i, varying name --", priority);\
										TEST_ONE_DELETEMSG(expect, aReplyPort, "", priority);\
										TEST_ONE_DELETEMSG(expect, aReplyPort, " ", priority);\
										TEST_ONE_DELETEMSG(expect, aReplyPort, "W", priority);\
										TEST_ONE_DELETEMSG(expect, aReplyPort, "w", priority);\
										TEST_ONE_DELETEMSG(expect, aReplyPort, "5", priority);\
										TEST_ONE_DELETEMSG(expect, aReplyPort, "This is quite an unreasonably lengthy name", priority)

Boolean ExerciseDeleteMsg( void )
{
	char	*callName;
	Item	aReplyPort = (Item) 0;

	gCurrentIndex = 0;
	
	callName = "DeleteMsg";
	STATUS(callName);
	
	aReplyPort = CreateMessagePort( gPortName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aReplyPort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	
	/* params: portname, msgname, priority */
	EXERCISE_NAME(0, kExpectSuccess);
	EXERCISE_NAME(TASK_MIN_PRIORITY, kExpectSuccess);
	EXERCISE_NAME(CURRENT_TASK_PRIORITY, kExpectSuccess);
	EXERCISE_NAME(TASK_MAX_PRIORITY, kExpectSuccess);
	EXERCISE_NAME(255, kExpectSuccess);
	
	CLEANUP;
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestDeleteMsg( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseDeleteMsg();
	else
	{
		callName = "DeleteMsg";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallDeleteMsg(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


