
/*
 * @(#) TestDeleteMsgPort.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestDeleteMsgPort.c

	Contains:	Test routine for DeleteMsgPort().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/28/93	CDE		Derived from TestDeleteMsg

	To Do:
*/


#include "TestDeleteMsgPort.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncLookupErrorStr	"Error looking up item $%x"
#define		kIncAllocSignalErrorStr	"Error allocating signal"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeDeleteMsgPortVerifyStr		"Before iteration (%i)/\titem/ $%x"
#define		kAfterDeleteMsgPortVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyDeleteMsgPort ( Boolean beforeCall, long aIndx, Err aResult, Item aItem )
/* Functionality verification for DeleteMsgPort */
{
	ItemNodeP aItemNode;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeDeleteMsgPortVerifyStr, aIndx, aItem);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterDeleteMsgPortVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return false;

		/* The item is deleted if we can't find it */
		aItemNode = (ItemNodeP) LookupItem( aItem );
		if (aItemNode != NULL)
			LogComment("!! Item still exists");
			
		return ( aItemNode == NULL );
	}
	
}

#define		CLEANUP		FreeSignal(aSignal)

long	CallDeleteMsgPort (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Err		aResult = 0;
	char	*aPortName = gPortName;
	uint8	aPriority = gPriority;
	int32	aAllocSignalResult;
	uint32	aSignal;
	Item	aMsgPort = (Item) 0;
	
	/***** Functionality verification (pre-call) *****/
	aAllocSignalResult = AllocSignal( 0 );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aAllocSignalResult <= 0)
		{
			LogComment(kIncAllocSignalErrorStr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	aSignal = (uint32) aAllocSignalResult;

	aMsgPort = CreateMsgPort( aPortName, aPriority, aSignal );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMsgPort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	
	if (gVerify) VerifyDeleteMsgPort ( kBeforeCall, minInnerIterations, aResult, aMsgPort );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = DeleteMsgPort( aMsgPort  );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyDeleteMsgPort ( kAfterCall, indx, aResult, aMsgPort );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
					{
						CLEANUP;
						return kBreakTime;
					}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
		
	CLEANUP;
	
	return accumulator;
}

long OneDeleteMsgPort ( Boolean expectError, char *aPortName, uint8 aPriority )
/* Inner test */
{
	Boolean theError= FALSE;
	
	/* Return value */
	Err		aResult = 0;
	
	/* Ancillary variables */
	int32	aAllocSignalResult;
	uint32	aSignal;
	Item	aMsgPort = (Item) 0;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aAllocSignalResult = AllocSignal( 0 );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aAllocSignalResult <= 0)
		{
			LogComment(kIncAllocSignalErrorStr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	aSignal = (uint32) aAllocSignalResult;

	aMsgPort = CreateMsgPort( aPortName, aPriority, aSignal );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMsgPort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyDeleteMsgPort ( kBeforeCall, gCurrentIndex, aResult, aMsgPort );

	aResult = DeleteMsgPort( aMsgPort  );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyDeleteMsgPort ( kAfterCall, gCurrentIndex, aResult, aMsgPort );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_DELETEMSGPORT(expVal, name, pri)	sprintf( gTestName, "%s | name= '%s'; pri= %d", gTestAreaName, name, pri);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneDeleteMsgPort(expVal, name, pri); \
														


#define EXERCISE_NAME(priority,expect)	gCurrentIndex = 0;\
										LogDiagnostic( 12,"-- For priority %i, varying name --", priority);\
										TEST_ONE_DELETEMSGPORT(expect, "", priority);\
										TEST_ONE_DELETEMSGPORT(expect, " ", priority);\
										TEST_ONE_DELETEMSGPORT(expect, "W", priority);\
										TEST_ONE_DELETEMSGPORT(expect, "w", priority);\
										TEST_ONE_DELETEMSGPORT(expect, "5", priority);\
										TEST_ONE_DELETEMSGPORT(expect, "This is quite an unreasonably lengthy name", priority)

void ExerciseDeleteMsgPort( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "DeleteMsgPort";
	STATUS(callName);
	
	/* params: portname, priority */
	EXERCISE_NAME(0, kExpectSuccess);
	EXERCISE_NAME(TASK_MIN_PRIORITY, kExpectSuccess);
	EXERCISE_NAME(CURRENT_TASK_PRIORITY, kExpectSuccess);
	EXERCISE_NAME(TASK_MAX_PRIORITY, kExpectSuccess);
	EXERCISE_NAME(255, kExpectSuccess);
/*
	...
 Invalid type
 low memory
*/
}

void TestDeleteMsgPort( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseDeleteMsgPort();
	else
	{
		callName = "DeleteMsgPort";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallDeleteMsgPort(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


