
/*
 * @(#) TestReplyMsg.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestReplyMsg.c

	Contains:	Test routines for ReplyMsg().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/21/93	CDE		Derived from TestShell.c V1.6

	To Do:
*/


#include "TestReplyMsg.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncGetMsgErrorStr	"Error getting message/ sent/ $%x, received/ $%x, messagePort/ $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern char gPortName[];

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeReplyMsgVerifyStr	"Before iteration (%i)/\tmessage/ $%x,\treturnValue/ $%x,\tdataPtr/ $%x,\tdataSize/ $%x"
#define		kAfterReplyMsgVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyReplyMsg ( Boolean beforeCall, long aIndx, Err aResult, Item aMessage, int32 aReturnValue, void *aDataPtr, int32 aDataSize )
/* Functionality verification for ReplyMsg */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeReplyMsgVerifyStr, aIndx, aMessage, aReturnValue, aDataPtr, aDataSize);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterReplyMsgVerifyStr, aIndx, aResult);
		return ( aResult == 0 );
	}
	
}

#define		CLEANUP		DeleteMsgPort(aReplyPort);\
						DeleteMsgPort(aMessagePort);\
						DeleteMsg(receivedMessage)

long	CallReplyMsg (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Err aResult = 0;
	Item aMessagePort = 0;
	char *aName = gPortName;
	Item sentMessage = 0;
	Item receivedMessage = 0;
	int32 aReturnValue = 0;
	void *aDataPtr = NULL;
	int32 aDataSize = 0;
	
	/* Preparation for call iterations */
	if ( !SetupMessage( &aMessagePort, &sentMessage, aName, aDataPtr, aDataSize ) ) return kBreakTime;	

	receivedMessage = GetMsg(aMessagePort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (receivedMessage != sentMessage)
		{
			LogComment(kIncGetMsgErrorStr, sentMessage, receivedMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyReplyMsg ( kBeforeCall, minInnerIterations, aResult, receivedMessage, aReturnValue, aDataPtr, aDataSize );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = ReplyMsg( receivedMessage, aReturnValue, aDataPtr, aDataSize );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyReplyMsg ( kAfterCall, indx, aResult, receivedMessage, aReturnValue, aDataPtr, aDataSize );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneReplyMsg ( Boolean expectError, Item aMessagePort, Item aMessage, void *aDataPtr, int32 aDataSize )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	Err aResult = 0;

	/* Ancillary variables */
	int32 aReturnValue = 0;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyReplyMsg ( kBeforeCall, gCurrentIndex, aResult, aMessage, aReturnValue, aDataPtr, aDataSize );

	aResult = ReplyMsg( aMessage, aReturnValue, aDataPtr, aDataSize );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyReplyMsg ( kAfterCall, gCurrentIndex, aResult, aMessage, aReturnValue, aDataPtr, aDataSize );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

Boolean ExerciseReplyMsg( void )
{
	char	*callName;
	Item	aMessagePort = (Item) 0;
	Item	aReplyPort = (Item) 0;
	Item	aMessage = (Item) 0;
	char	message[64] = "This is the test message";
	int32	aResult;
	Item	receivedMessage = (Item) 0;
	void	*aDataPtr = (void *) message;
	int32	aDataSize = 64;
	
	gCurrentIndex = 0;
	
	callName = "ReplyMsg";
	STATUS(callName);
	
	aReplyPort = CreateMessagePort( "ReplyPort" );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aReplyPort <= 0)
		{
			LogComment(kIncCreateReplyPortErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				return FALSE;
			}
		}
	}

	aMessagePort = CreateMessagePort( gPortName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessagePort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				DeleteMsgPort(aReplyPort);
				return FALSE;
			}
		}
	}

	aMessage = CreateMessage(aReplyPort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessage <= 0)
		{
			LogComment(kIncCreateMsgErrorStr, aMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop)
			{
				DeleteMsgPort(aReplyPort);
				DeleteMsgPort(aMessagePort);
				return FALSE;
			}
		}
	}
	
/*	Executing this reply screws up the following send! */
/*	(using either aMessagePort or aReplyport as the argument to the above CreateMessage call) */
/*	pipeData("-- Before sending message --"); */
/*	OneReplyMsg(kExpectError, aMessagePort, aMessage, aDataPtr, aDataSize); */
	
	aResult = SendMsg(aMessagePort, aMessage, aDataPtr, aDataSize);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aResult != 0)
		{
			LogComment(kIncSendMsgErrorStr, aResult, aMessagePort, aMessage, aDataPtr, aDataSize);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return FALSE;
			}
		}
	}

#define	TEST_ONE_REPLYMSG(expVal, port, msg, val1, val2)	sprintf( gTestName, "%s | port= 0x%x; msg= 0x%x; ptr= 0x%x; size= 0x%x", gTestAreaName, port, msg, val1, val2);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneReplyMsg(expVal, port, msg, val1, val2); \
														

	
	LogDiagnostic( 12,"-- Before getting message --");
	TEST_ONE_REPLYMSG(kExpectError, aMessagePort, aMessage, aDataPtr, aDataSize);	/* Before getting the message */

	receivedMessage = GetMsg(aMessagePort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (receivedMessage != aMessage)
		{
			LogComment(kIncGetMsgErrorStr, aMessage, receivedMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return FALSE;
			}
		}
	}
	
	
	LogDiagnostic( 12,"-- After getting message --");
	TEST_ONE_REPLYMSG(kExpectSuccess, aMessagePort, receivedMessage, aDataPtr, aDataSize);
	
	CLEANUP;
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestReplyMsg( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseReplyMsg();
	else
	{
		callName = "ReplyMsg";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallReplyMsg(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


