
/*
 * @(#) TestReplySmallMsg.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestReplySmallMsg.c

	Contains:	Test routines for ReplySmallMsg().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/22/93	CDE		Derived from TestShell.c V1.6

	To Do:
*/


#include "TestReplySmallMsg.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kVerifyStr		"%s (%i)"
#define		kReplySmallMsgVerifyStr	"%s (%i)/\tresult/ $%x,\tmessage/ $%x,\treturnValue/ $%x,\tval1/ $%x,\tval2/ $%x"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncSendSmallMsgErrorStr	"Error sending small message, result/ $%x, messagePort/ $%x, message/ $%x, val1/ $%x, val2/ $%x"
#define		kIncGetMsgErrorStr	"Error getting message/ sent/ $%x, received/ $%x, messagePort/ $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern char gPortName[];

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeReplySmallMsgVerifyStr	"Before iteration (%i)/\tmessage/ $%x,\treturnValue/ $%x,\tval1/ $%x,\tval2/ $%x"
#define		kAfterReplySmallMsgVerifyStr	"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyReplySmallMsg ( Boolean beforeCall, long aIndx, Err aResult, Item aMessage, int32 aReturnValue, uint32 aVal1, uint32 aVal2 )
/* Functionality verification for ReplySmallMsg */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeReplySmallMsgVerifyStr, aIndx, aMessage, aReturnValue, aVal1, aVal2);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterReplySmallMsgVerifyStr, aIndx, aResult);
		return ( aResult == 0 );
	}
	
}

#define		CLEANUP		DeleteMsgPort(aMessagePort);\
						DeleteMsg(receivedMessage)

long	CallReplySmallMsg (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Err aResult = 0;
	Item aMessagePort = 0;
	char *aName = gPortName;
	Item sentMessage = 0;
	Item receivedMessage = 0;
	int32 aReturnValue = 0;
	uint32 aVal1 = 0;
	uint32 aVal2 = 0;
	
	/* Preparation for call iterations */
	aMessagePort = CreateMessagePort( aName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessagePort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	sentMessage = CreateMessage(aMessagePort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (sentMessage <= 0)
		{
			LogComment(kIncCreateMsgErrorStr, sentMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	aResult = SendSmallMsg(aMessagePort, sentMessage, aVal1, aVal2);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aResult < 0)
		{
			LogComment(kIncSendSmallMsgErrorStr, aResult, aMessagePort, sentMessage, aVal1, aVal2);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	receivedMessage = GetMsg(aMessagePort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (receivedMessage != sentMessage)
		{
			LogComment(kIncGetMsgErrorStr, sentMessage, receivedMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyReplySmallMsg ( kBeforeCall, minInnerIterations, aResult, receivedMessage, aReturnValue, aVal1, aVal2 );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = ReplySmallMsg( receivedMessage, aReturnValue, aVal1, aVal2 );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyReplySmallMsg ( kAfterCall, indx, aResult, receivedMessage, aReturnValue, aVal1, aVal2 );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	CLEANUP;
	
	return accumulator;
}

long OneReplySmallMsg ( Boolean expectError, Item aMessagePort, Item receivedMessage, uint32 aVal1, uint32 aVal2)
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	Err aResult = 0;
	
	/* Ancillary variables */
	int32 aReturnValue = 0;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyReplySmallMsg ( kBeforeCall, gCurrentIndex, aResult, receivedMessage, aReturnValue, aVal1, aVal2 );

	aResult = ReplySmallMsg( receivedMessage, aReturnValue, aVal1, aVal2 );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyReplySmallMsg ( kAfterCall, gCurrentIndex, aResult, receivedMessage, aReturnValue, aVal1, aVal2 );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

Boolean ExerciseReplySmallMsg( void )
{
	char	*callName;
	Item	aMessagePort = 0;
	Item	aMessage = 0;
	char	message[64] = "This is the test message";
	int32	aResult;
	Item receivedMessage = 0;
	uint32 aVal1 = (uint32) message;
	uint32 aVal2 = 64;
	
	gCurrentIndex = 0;
	
	callName = "ReplySmallMsg";
	STATUS(callName);
	
	aMessagePort = CreateMessagePort( gPortName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessagePort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}

	aMessage = CreateMessage(aMessagePort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessage <= 0)
		{
			LogComment(kIncCreateMsgErrorStr, aMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop)
			{
				DeleteMsgPort(aMessagePort);
				return FALSE;
			}
		}
	}
	
/*	pipeData("-- Before sending the message --"); */
/*	OneReplySmallMsg(kExpectError, aMessagePort, aMessage, aVal1, aVal2); */
	
	aResult = SendSmallMsg(aMessagePort, aMessage, aVal1, aVal2);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aResult != 0)
		{
			LogComment(kIncSendMsgErrorStr, aResult, aMessagePort, aMessage, aVal1, aVal2);
			if (gCheckIncidentalErrors == kStop)
			{
				DeleteMsgPort(aMessagePort);
				DeleteMsg(aMessage);
				return FALSE;
			}
		}
	}

#define	TEST_ONE_REPLYSMALLMSG(expVal, port, msg, val1, val2)	sprintf( gTestName, "%s | port= 0x%x; msg= 0x%x; val1= 0x%x; val2= 0x%x", gTestAreaName, port, msg, val1, val2);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneReplySmallMsg(expVal, port, msg, val1, val2); \
														

	
	LogDiagnostic( 12,"-- Before getting message --");
	TEST_ONE_REPLYSMALLMSG(kExpectError, aMessagePort, aMessage, aVal1, aVal2);

	receivedMessage = GetMsg(aMessagePort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (receivedMessage != aMessage)
		{
			LogComment(kIncGetMsgErrorStr, aMessage, receivedMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return FALSE;
			}
		}
	}
	
	
	LogDiagnostic( 12,"-- After getting message --");
	TEST_ONE_REPLYSMALLMSG(kExpectSuccess, aMessagePort, receivedMessage, aVal1, aVal2);
	
	CLEANUP;
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestReplySmallMsg( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseReplySmallMsg();
	else
	{
		callName = "ReplySmallMsg";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallReplySmallMsg(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


