
/*
 * @(#) MiscMemTests.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		MiscMemTests.c

	Contains:	Miscellaneous memory call tests, Kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <3>	 6/27/95	ELH		Removed GetMemType and GetBankBits test code (removed from M2)
		 <3>	 8/19/93	CDE		Commented out ReceiverTask code (deferred for later study)
		 							Improved -char message in ShowOptions.
		 <2>	 6/28/93	CDE		Derived from shell
		 <1>	 6/22/93	CDE		Version 1.0 of TestShell.c

	To Do:
*/


/***** Version string *****/
#define		kVersionStr		"V1.0"


/***** Call selection constants *****/
#define		kControlMem		1
#define		kGetPageSize	2
#define		kmemcpy			3
#define		kmemset			4


/***** Default log filename: *****/
#define		kDefaultLogFile			"MiscMemTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
#define		kIncrement1Units	"bytes"
#define		kDefaultIncrement1	1


/***** Other test options *****/
#define		kDefaultMemType		MEMTYPE_ANY
#define		kDefaultCommand		MEMC_OKWRITE
#define		kDefaultCharValue	(int) 'x'

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output formats: */
/* (Now declared with call verification functions) */

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr		"Error initializing for %s"

/* Other incidental error reporting formats: */
#define		kIncAllocMemErrorStr	"Before iteration (%i)/\tsize/ %i,\ttype/ $%x, Unable to allocate memory"
#define		kIncAllocSignalErrorStr	"Before iteration (%i)/ Unable to allocate signal, error/ %i"
#define		kIncSpawnErrorStr		"Before iteration (%i)/ Unable to spawn test thread, Item/ %i"
#define		kIncFreeSignalErrorStr	"Unable to free signal, error/ %i"
#define		kIncDeleteItemErrorStr	"Unable to delete item, error/ %i"

/* Other: */
#define		kTestThreadStackSize	128
#define		kTestThreadName			"TestThread"

/*#define USE_STRCASECMP */

/***** Include files *****/

#include "threadUtils.h"

#include "extraUtils.h"


/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "AllocTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "MiscMemTests.c";	/* contains the name of the program */
char	* gTestName= "MiscMemTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;			/* contains what level of testing occurs in this test */
int32	gTestType = 0;			/* interactive or automated */

/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */

/* The Item number corresponding to this task */
Item gThisTask;

/* Invalid command line argument */
Boolean		gInvalidArg;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */
/* HINTS:

	 Add one global for each additional tool option
*/
static uint32 gMemType;
static int32 gCommand;
int gCharValue;

/* Thread stuff (from Examples:TaskDemo.c) */
static ulong gFromParent, gFromThread;

/* Globals shared by tool and thread */
static void *gTestMemory = NULL;
static int32 gTestSize = 0;

#if 0
/***** Utilities *****/
/* Convert a memory type specifier string into a memory type */
uint32 StrToMemType(char *aString)
{
	char *p;
	uint32 aMemType = 0;

	gInvalidArg = false;
	
	for ( p = aString ; *p ; p++ )
	{
		switch(*p)
		{
			case 'a':
				aMemType |= MEMTYPE_ANY;
				break;
				
			case 'v':
				aMemType |= MEMTYPE_VRAM;
				break;
				
			case 'd':
				aMemType |= MEMTYPE_DMA;
				break;
				
			case 'c':
				aMemType |= MEMTYPE_CEL;
				break;
				
			default:
				gInvalidArg = true;
				break;
		}
				 
	}
   
	return aMemType;
}
#endif

/* Convert a memory type to a display string */
void MemTypeToStr(uint32 aMemType, char* aString)
{
	if (aMemType == MEMTYPE_NORMAL) strcpy(aString,"MEMTYPE_NORMAL");
#if 0
	if (aMemType == MEMTYPE_ANY) strcpy(aString,"MEMTYPE_ANY");
	else
	{
		aString[0] = '\0';
		if (aMemType & MEMTYPE_VRAM) strcat(aString, "MEMTYPE_VRAM|");
		else if (aMemType & MEMTYPE_DMA) strcat(aString, "MEMTYPE_DMA|");
		else if (aMemType & MEMTYPE_CEL) strcat(aString, "MEMTYPE_CEL|");
		aString[strlen(aString) - 1] = '\0';
	}
#endif
}



int isAll(char *s, int c, size_t n)
/* If the first n characters of s are c, return 0,
	else return the number of characters tested */
{
	size_t aIndex;
	
	for ( aIndex = 0; aIndex < n ; aIndex++ )
	{
		if ( s[ aIndex ] != (char) c ) return (int) (aIndex + 1);
	}
	return 0;
}

/**** (End Utilities) ****/


void	ShowHelp()
/* Show Help for this tool */
{
		kprintf ( "#################################################################");
		kprintf ( "Miscellaneous memory tests %s Help", kVersionStr );
		kprintf ( "Written by Charlie Eckhaus for The 3DO Company" );
		kprintf (    "#################################################################");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-out << no parameter>>\tDo not use a log file.");
		LogDiagnostic( 80, "     <<filename>>     \tUse specified log file.");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80,"-icmin <<value>>\tIterate calls starting at <<value>>.");
		LogDiagnostic( 80,"-icmax <<value>>\tEnd call iterations at <<value>>.");
		LogDiagnostic( 80,"-itmin <<value>>\tIterate tests starting at <<value>>.");
		LogDiagnostic( 80,"-itmax <<value>>\tEnd test iterations at <<value>>.");
		LogDiagnostic( 80, "");

		kprintf ( "-inc1 <<value>>\tUse <<value>> as increment."); 
		LogDiagnostic( 80, "");
		
		kprintf ( "-type <<value>>\tUse <<value>> as requested memory type."); 
		kprintf ( "               \t(combine by concatenation of the following letters).");
		kprintf ( "               \ta\tANY");
		kprintf ( "               \tv\tVRAM");
		kprintf ( "               \tc\tCEL");
		kprintf ( "               \td\tDMA");
		LogDiagnostic( 80, "");
		
		kprintf ( "-char <<value>>\tUse <<value>> as memset character value."); 
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-error none\tDon't check for errors.");
		LogDiagnostic( 80, "       cont\tWhen an error occurs, report it and continue.");
		LogDiagnostic( 80, "       none\tWhen an error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-ierror none\tDon't check for incidental errors.");
		LogDiagnostic( 80, "        cont\tWhen an incidental error occurs, report it and continue.");
		LogDiagnostic( 80, "        none\tWhen an incidental error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-time off\tDon't output timing information.");
		LogDiagnostic( 80, "      on \tOutput timing information.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-func off\tDon't verify functionality.");
		LogDiagnostic( 80,"      on \tVerify functionality.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-fault off\tMake known assumptions.");
		LogDiagnostic( 80,"       on \tSkip known assumptions.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-sel <<no parameter>>\tRun the entire test suite.");
		LogDiagnostic( 80,"     ControlMem           \tTest ControlMem.");
		LogDiagnostic( 80,"     GetPageSize          \tTest GetPageSize.");
		LogDiagnostic( 80,"     memcpy               \tTest memcpy.");
		LogDiagnostic( 80,"     memset               \tTest memset.");
		LogDiagnostic( 80, "");
		
		kprintf ( "#################################################################");
}

void ShowOptions()
/* Show the option settings for this session */
{
		char aString[64];

		kprintf ( "#################################################################");
		kprintf ( "Miscellaneous memory tests %s - Session options", kVersionStr);
		kprintf ( "Written by Charlie Eckhaus for The 3DO Company" );
		kprintf (   "#################################################################");
		
		if (gUseLogFile) LogDiagnostic( 80, "-out: Use log file: %s.",gLogFile); else LogDiagnostic( 80,"Do not use a log file.");
		
		kprintf ( "Iterate each test from %i %s",gMinOuterIterations, kIncrement1Units);
		kprintf ( " to %i %s,", gMaxOuterIterations, kIncrement1Units);
		kprintf ( " in increments of %i %s.",gIncrement1,kIncrement1Units); 
		kprintf ( "Within each test, iterate the call from %i to %i.",gMinInnerIterations,gMaxInnerIterations);
		
		MemTypeToStr(gMemType, aString);
		LogDiagnostic( 80,"-type: %s",aString);
		
		LogDiagnostic( 80,"-char: Use %i as char value in memset test.", gCharValue);
		
		LogDiagnostic( 80, "-error: ");
		switch (gCheckErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80, "-ierror: ");
		switch (gCheckIncidentalErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for incidental errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an incidental error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first incidental error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80,"-time: ");
		if (gOutputTiming) LogDiagnostic( 80,"Output timing information."); else LogDiagnostic( 80,"Don't output timing information.");

		LogDiagnostic( 80,"-func: ");
		if (gVerify) LogDiagnostic( 80,"Verify functionality."); else LogDiagnostic( 80,"Don't verify functionality.");

		LogDiagnostic( 80,"-fault: ");
		if (gCheckFault) LogDiagnostic( 80,"Skip known assumptions."); else LogDiagnostic( 80,"Make known assumptions.");

		LogDiagnostic( 80,"-sel: ");
		switch (gCallSelector)
			{
			case kFullTest:
				LogDiagnostic( 80,"Run the entire test suite.");
				break;
				
			case kControlMem:
				LogDiagnostic( 80,"Test ControlMem.");
				break;
				
			case kGetPageSize:
				LogDiagnostic( 80,"Test GetPageSize.");
				break;
				
			case kmemcpy:
				LogDiagnostic( 80,"Test memcpy.");
				break;
				
			case kmemset:
				LogDiagnostic( 80,"Test memset.");
				break;
				
			} /* end switch */
		
		kprintf ( "#################################################################");
}

bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	
	int paramIndx;
	
	paramIndx = 0;
	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	gMemType = kDefaultMemType;
	gCommand = kDefaultCommand;
	gCharValue = kDefaultCharValue;

	gCheckFault = true;
	gVerify = true;

	return kRunTest;
}	/* end GetParams */



/******************************
  *
  * Test routines for ControlMem
  *
  ******************************/

/*
	THIS MAY REQUIRE A FULL-BLOWN TASK (NOT A THREAD) (UNIMPLEMENTED)
*/

void TestThread( void )
/* Test thread for ControlMem verification */
{
	Item myParent = THREAD_PARENT;
/*	see below:

	long indx;
*/
	void *aVoidPtr;
	
	/* Allocate a signal to wait for (use first available bit) */
	gFromParent = AllocSignal( 0 );
	
	SendSignal( myParent, gFromThread );		/* I'm ready */
	
	/* use the same iterations as CallControlMem() */
/*
	Would like to use:

	for ( indx = gMinInnerIterations;indx <= gMaxInnerIterations; indx++ )
		 THIS IS CHEATING! (should use CallControlMem's actual parameter values)

 	but get a data error if the task ends!
*/
	while (true)
	{
		WaitSignal( gFromParent );				/* Wait until parent is ready */
		
		/* Do something to the allocated memory */
		aVoidPtr = memset(gTestMemory, 'x', gTestSize);
		
		SendSignal( myParent, gFromThread );	/* I'm done */
	}
	
	/* Don't need to free signal because OS cleans up (?) */
	/* FreeSignal( gFromParent ); */
}


#define		kBeforeControlMemVerifyStr	"Before iteration (%i)/\tptr/ $%x,\tsize/ %i,\t/command/ %i,\ttargetTask/ $%x"
#define		kAfterControlMemVerifyStr	"After iteration (%i)/\tresult/ $%x"

Boolean VerifyControlMem ( Boolean beforeCall, long aIndx, int32 aResult, void *aVoidPtr, int32 aSize, int32 aCommand, Item aTask )
/* Functionality verification for ControlMem */
{
	Boolean succeeded;
	int32	newSize;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeControlMemVerifyStr, aIndx, aVoidPtr, aSize, aCommand, aTask);
		return true;
	}
	else
	{
		
		LogDiagnostic( 12,kAfterControlMemVerifyStr, aIndx, aResult);

		succeeded = (aResult == 0);
		if ( succeeded )
		{
			/* Signal TestThread to change the memory */
			gTestSize = aSize;
			SendSignal( aTask, gFromParent );		/* Change memory */
			WaitSignal( gFromThread );				/* Wait until done */
			
			/* See if TestThread behaved properly */
			newSize = strlen( (char *) gTestMemory);
			succeeded = ( newSize == aSize );
			if (!succeeded)
				LogComment("?? Task (thread) couldn't change memory");
		}

		return ( succeeded );
	}
	
}

long	CallControlMem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError;
	
  	/* Variables for call parameters and return value */
	void 	*aVoidPtr;
	uint32	aMemType = gMemType;
	int32	aSize = gIncrement1;
	int32	aCommand = gCommand;
	Item	aThreadItem;
	int32	aResult;
	Err		aErr;

/*	Item	aChildItem; */
	
	
	/***** Preparation for call iterations *****/
	theError = FALSE;
	
	/* Allocate and clear some memory */
	aVoidPtr = AllocMem( aSize, aMemType );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aVoidPtr == NULL )
		{
			LogDiagnostic( 12,kIncAllocMemErrorStr, minInnerIterations, aSize, aMemType);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	gTestMemory = aVoidPtr;
	aVoidPtr = memset(gTestMemory, '\0', aSize);

	/* Allocate a signal for communicating with the thread */
	aResult = AllocSignal( 0 );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aResult <= 0)
		{
			LogDiagnostic( 12,kIncAllocSignalErrorStr, minInnerIterations, aResult);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeMem( aVoidPtr, aSize);
				return kBreakTime;
			}
		}
	}
	gFromThread = (ulong) aResult;

	/* Spawn the thread and wait until it's ready */
	aThreadItem = SpawnThread( TestThread, kTestThreadStackSize, CURRENT_TASK_PRIORITY, kTestThreadName);	
	if (gCheckIncidentalErrors != kNone)
	{
		if (aThreadItem <= 0)
		{
			LogDiagnostic( 12,kIncSpawnErrorStr, minInnerIterations, aThreadItem);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeMem( aVoidPtr, aSize);
				FreeSignal(gFromThread);
				return kBreakTime;
			}
		}
	}
	WaitSignal(gFromThread);
	
/*	aChildItem = LoadProgram( "/remote/receivertask" ); */
/*	if ( aChildItem < 0) LogDiagnostic( 80,"Failed to LoadProgram, error %i", aChildItem); */

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyControlMem ( kBeforeCall, minInnerIterations, aResult, aVoidPtr, aSize, aCommand, aThreadItem );

	/***** Call iterations *****/
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = ControlMem( aVoidPtr, aSize, aCommand, aThreadItem);
/*		aResult = ControlMem( aVoidPtr, aSize, aCommand, aChildItem); */
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyControlMem ( kAfterCall, indx, aResult, aVoidPtr, aSize, aCommand, aThreadItem );
/*		if (gVerify) theError = !VerifyControlMem ( kAfterCall, indx, aResult, aVoidPtr, aSize, aCommand, aChildItem ); */
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogDiagnostic( 12,kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	/* Cleanup for call iterations */
	aResult = FreeSignal(gFromThread);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aResult < 0)
		{
			LogDiagnostic( 12,kIncFreeSignalErrorStr, aResult);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeMem( aVoidPtr, aSize);
				return kBreakTime;
			}
		}
	}
	
	aErr = DeleteItem(aThreadItem);
/*	aErr = DeleteItem(aChildItem); */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aErr < 0)
		{
			LogDiagnostic( 12,kIncDeleteItemErrorStr, aErr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeMem( aVoidPtr, aSize);
				return kBreakTime;
			}
		}
	}
	
	
	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneControlMem ( Boolean expectError, int32 aSize, int32 aCommand, uint32 aMemType)
/* Individual call for parameter-testing */
{
 	Boolean theError = FALSE;
	
	/* Return value */
	int32	aResult;

	/* Ancillary variables */
	void 	*aVoidPtr;
	Item	aThreadItem;
	Err		aErr;
	long	myReturn = 0;

/*	Item	aChildItem; */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	/* Allocate and clear some memory */
	aVoidPtr = AllocMem( aSize, aMemType );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aVoidPtr == NULL )
		{
			LogDiagnostic( 12,kIncAllocMemErrorStr, gCurrentIndex, aSize, aMemType);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	gTestMemory = aVoidPtr;
	aVoidPtr = memset(gTestMemory, '\0', aSize);

	/* Allocate a signal for communicating with the thread */
	aResult = AllocSignal( 0 );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aResult <= 0)
		{
			LogDiagnostic( 12,kIncAllocSignalErrorStr, gCurrentIndex, aResult);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeMem( aVoidPtr, aSize);
				return kBreakTime;
			}
		}
	}
	gFromThread = (ulong) aResult;

	/* Spawn the thread and wait until it's ready */
	aThreadItem = SpawnThread( TestThread, kTestThreadStackSize, CURRENT_TASK_PRIORITY, kTestThreadName);	
	if (gCheckIncidentalErrors != kNone)
	{
		if (aThreadItem <= 0)
		{
			LogDiagnostic( 12,kIncSpawnErrorStr, gCurrentIndex, aThreadItem);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeMem( aVoidPtr, aSize);
				FreeSignal(gFromThread);
				return kBreakTime;
			}
		}
	}
	WaitSignal(gFromThread);
	
/*	aChildItem = LoadProgram( "/remote/receivertask" ); */
/*	if ( aChildItem < 0) LogDiagnostic( 80,"Failed to LoadProgram, error %i", aChildItem); */

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyControlMem ( kBeforeCall, gCurrentIndex, aResult, aVoidPtr, aSize, aCommand, aThreadItem );

	aResult = ControlMem( aVoidPtr, aSize, aCommand, aThreadItem);
/*	aResult = ControlMem( aVoidPtr, aSize, aCommand, aChildItem); */
		
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyControlMem ( kAfterCall, gCurrentIndex, aResult, aVoidPtr, aSize, aCommand, aThreadItem );
/*	if (gVerify) theError = !VerifyControlMem ( kAfterCall, gCurrentIndex, aResult, aVoidPtr, aSize, aCommand, aChildItem ); */
		
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				myReturn = kBreakTime;
				goto errorExit;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
errorExit:
/* Cleanup for call iterations */
	aResult = FreeSignal(gFromThread);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aResult < 0)
		{
			LogDiagnostic( 12,kIncFreeSignalErrorStr, aResult);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeMem( aVoidPtr, aSize);
				return kBreakTime;
			}
		}
	}
	
	aErr = DeleteItem(aThreadItem);
/*	aErr = DeleteItem(aChildItem); */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aErr < 0)
		{
			LogDiagnostic( 12,kIncDeleteItemErrorStr, aErr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeMem( aVoidPtr, aSize);
				return kBreakTime;
			}
		}
	}
	
	return myReturn;
}

#define	TEST_ONE_CONTROL_MEM(OneProc,expVal, size1, cmd, type)		sprintf( gTestName, "%s - %s | cmd = %i size = %d", gTestAreaName, aString, cmd, size1);	\
																	if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
																	BeginTest( BAT, gTestName );	\
																	OneProc( expVal, size1, cmd, type ); \
																	
														
#define EXERCISE_SIZEControlMem(cmd, type)		gCurrentIndex = 0;\
										MemTypeToStr(type,aString);\
										LogDiagnostic( 12, "-- For type: %s, command: %i: --", aString, cmd );\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectError, 0, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectSuccess, 1, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectSuccess, 2, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectSuccess, 3, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectSuccess, 4, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectSuccess, 8, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectSuccess, 16, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectSuccess, 32, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectSuccess, 1024, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectSuccess, 32768, cmd, type);\
										TEST_ONE_CONTROL_MEM(OneControlMem, kExpectError, (GetPageSize(type) * 64), cmd, type)
										
#define EXERCISE_COMMAND(cmd)	EXERCISE_SIZEControlMem(cmd,MEMTYPE_NORMAL);

#if 0
#define EXERCISE_COMMAND(cmd)	EXERCISE_SIZEControlMem(cmd,MEMTYPE_ANY);\
								EXERCISE_SIZEControlMem(cmd,MEMTYPE_CEL);\
								EXERCISE_SIZEControlMem(cmd,MEMTYPE_DMA);\
								EXERCISE_SIZEControlMem(cmd,MEMTYPE_VRAM);
#endif

void ExerciseControlMem( void )
{
	char	*callName;
	char	aString[64];

	gCurrentIndex = 0;
	
	callName = "ControlMem";
	STATUS(callName);
	
	/* params: size, command, type */
/* THE FOLLOWING LINE CAUSES AN ABORT; INVESTIGATE; UNIMPLEMENTED */
/*	EXERCISE_COMMAND(MEMC_NOWRITE); */
	EXERCISE_COMMAND(MEMC_OKWRITE);
	EXERCISE_COMMAND(MEMC_GIVE);
/*	EXERCISE_COMMAND(MEMC_SC_GIVE);	*/
/*
	...
 Invalid type
 low memory
*/
}

void TestControlMem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseControlMem();
	else
	{
		callName = "ControlMem";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallControlMem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}



/******************************
  *
  * Test routines for GetPageSize
  *
  ******************************/

#define		kBeforeGetPageSizeVerifyStr	"Before iteration (%i)/\ttype/ $%x"
#define		kAfterGetPageSizeVerifyStr	"After iteration (%i)/\tresult/ %i"

Boolean		VerifyGetPageSize ( Boolean beforeCall, long aIndx, int32 aResult, uint32 aType )
/* Functionality verification for GetPageSize */
{
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeGetPageSizeVerifyStr, aIndx, aType);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterGetPageSizeVerifyStr, aIndx, aResult);
		return true;
	}
	
}

long CallGetPageSize (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError;
	
  	/* Variables for call parameters and return value */
	uint32	aMemType = gMemType;
	int32 aResult = 0;
	
	theError = FALSE;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyGetPageSize ( kBeforeCall, minInnerIterations, aResult, aMemType );

	/***** Call iterations *****/
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = GetPageSize( aMemType );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyGetPageSize ( kAfterCall, indx, aResult, aMemType );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogDiagnostic( 12,kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneGetPageSize ( Boolean expectError, uint32 aMemType )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	int32 aResult = 0;
	
	/* Ancillary variables */
	/* (none) */
	
	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyGetPageSize ( kBeforeCall, gCurrentIndex, aResult, aMemType );

	aResult = GetPageSize( aMemType );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyGetPageSize ( kAfterCall, gCurrentIndex, aResult, aMemType );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{	
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

void ExerciseGetPageSize( void )
{
	char	*callName;
	char aString[64];

	gCurrentIndex = 0;
	
	callName = "GetPageSize";
	STATUS(callName);
	
#define	TEST_ONE_GET_PAGE_SIZE(expVal, type)	MemTypeToStr(type,aString);\
												if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
												sprintf( gTestName, "%s - %s", gTestAreaName, aString);	\
												BeginTest( BAT, gTestName );	\
												OneGetPageSize( expVal, type); \
												
	/* params: type */
	TEST_ONE_GET_PAGE_SIZE(kExpectSuccess, MEMTYPE_ANY);
	
#if 0
	TEST_ONE_GET_PAGE_SIZE(kExpectSuccess, MEMTYPE_NORMAL);
	TEST_ONE_GET_PAGE_SIZE(kExpectSuccess, MEMTYPE_CEL);
	TEST_ONE_GET_PAGE_SIZE(kExpectSuccess, MEMTYPE_DMA);
	TEST_ONE_GET_PAGE_SIZE(kExpectSuccess, MEMTYPE_VRAM);
#endif
/*
	...
 Invalid type
 low memory
*/
}

void TestGetPageSize( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseGetPageSize();
	else
	{
		callName = "GetPageSize";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallGetPageSize(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for memcpy
  *
  ******************************/

#define		kBeforememcpyVerifyStr		"Before iteration (%i)/\ts1/ %s, s2/ ($%x, %s), n/ %i"
#define		kAftermemcpyVerifyStr		"After iteration (%i)/\tresult/ $%x,\ts2/ %s"

Boolean		Verifymemcpy ( Boolean beforeCall, long aIndx, void *aResult, char *aSrcPtr, char *aDstPtr, size_t aSize )
/* Functionality verification for GetPageSize */
{
	 Boolean cmpResult;
	 
	 if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforememcpyVerifyStr, aIndx, aSrcPtr, aDstPtr, aDstPtr, aSize);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAftermemcpyVerifyStr, aIndx, aResult, aDstPtr);

		cmpResult = strncmp(aSrcPtr, aDstPtr, aSize);

		return ( (cmpResult == 0) && ( aResult == aDstPtr) );
	}
	
}

long Callmemcpy (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError;
	
	/* Variables for call parameters and return value */
	void *aResult;
	void *aSrcPtr, *aDstPtr;
	size_t aSize = ( size_t ) gIncrement1;
	size_t aSrcLen;

	theError = FALSE;
	
	aSrcPtr = "The quick brown fox jumps over the lazy dog";
	aDstPtr = (void *) malloc( 45 );
	strcpy( aDstPtr, "!This is the destination string for memcpy!");
	
	/* for now: */
	aSrcLen = strlen( (char *) aSrcPtr);
	if ( aSize > aSrcLen ) aSize = aSrcLen;
	
	/* Functionality verification (pre-call) */
	if (gVerify) Verifymemcpy ( kBeforeCall, minInnerIterations, aResult, (char *) aSrcPtr, (char *) aDstPtr, aSize );

	/***** Call iterations *****/
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = memcpy( aDstPtr, aSrcPtr, aSize );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !Verifymemcpy ( kAfterCall, indx, aResult, (char *) aSrcPtr, (char *) aDstPtr, aSize );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogDiagnostic( 12,kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}
	free( aDstPtr );

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long Onememcpy ( Boolean expectError, size_t aSize )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	void *aResult;

	/* Ancillary variables */
	void *aSrcPtr, *aDstPtr;
	size_t aSrcLen;
	size_t aDstLen;

  	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	aSrcPtr = "The quick brown fox jumps over the lazy dog";
	aDstPtr = (void *) malloc( 45 );
	strcpy( aDstPtr, "!This is the destination string for memcpy!");
	
	/* for now: */
	aSrcLen = strlen( (char *) aSrcPtr);
	if ( aSize > aSrcLen ) aSize = aSrcLen;
	aSrcLen = strlen( (char *) aDstPtr);
	if ( aSize > aDstLen ) aSize = aDstLen;
	
	/* Functionality verification (pre-call) */
	if (gVerify) Verifymemcpy ( kBeforeCall, gCurrentIndex, aResult, (char *) aSrcPtr, (char *) aDstPtr, aSize );

	aResult = memcpy( aDstPtr, aSrcPtr, aSize );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !Verifymemcpy ( kAfterCall, gCurrentIndex, aResult, (char *) aSrcPtr, (char *) aDstPtr, aSize );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	free( aDstPtr );

	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

void Exercisememcpy( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "memcpy";
	STATUS(callName);
	
#define	TEST_ONE_MEMCPY(expVal, size)		sprintf( gTestName, "%s | size = %d", gTestAreaName, size);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											Onememcpy( expVal, size); \
											
	/* params: size */
	TEST_ONE_MEMCPY(kExpectSuccess, ( size_t ) 0);
	TEST_ONE_MEMCPY(kExpectSuccess, ( size_t ) 1);
	TEST_ONE_MEMCPY(kExpectSuccess, ( size_t ) 2);
	TEST_ONE_MEMCPY(kExpectSuccess, ( size_t ) 3);
	TEST_ONE_MEMCPY(kExpectSuccess, ( size_t ) 4);
	TEST_ONE_MEMCPY(kExpectSuccess, ( size_t ) 8);
	TEST_ONE_MEMCPY(kExpectSuccess, ( size_t ) 16);
	TEST_ONE_MEMCPY(kExpectSuccess, ( size_t ) 32);
	TEST_ONE_MEMCPY(kExpectSuccess, ( size_t ) 1024); /* should be truncated to source/dest. length */
										
/*
	...
 Invalid type
 low memory
*/
}

void Testmemcpy( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		Exercisememcpy();
	else
	{
		callName = "memcpy";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = Callmemcpy(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for memset
  *
  ******************************/

#define		kBeforememsetVerifyStr		"Before iteration (%i)/\ts/ ($%x, %s), c/ %i, n/ %i"
#define		kAftermemsetVerifyStr		"After iteration (%i)/\tresult/ $%x,\ts/ %s"

Boolean		Verifymemset ( Boolean beforeCall, long aIndx, void *aResult, char *aDstPtr, int aInt, size_t aSize )
/* Functionality verification for GetPageSize */
{
	 int cmpResult;
	 
	 if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforememsetVerifyStr, aIndx, aDstPtr, aDstPtr, aInt, aSize);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAftermemsetVerifyStr, aIndx, aResult, aDstPtr);
		
		cmpResult = isAll(aDstPtr, aInt, aSize);
		
		return ( ( cmpResult == 0 ) && ( aResult == aDstPtr) );
	}
	
}

long Callmemset (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError;
	
	/* Variables for call parameters and return value */
	void *aResult;
	int aInt = gCharValue;
	void *aDstPtr;
	size_t aSize = ( size_t ) gIncrement1;
	size_t aSrcLen;

	theError = FALSE;
	
	aDstPtr = (void *) malloc( 45 );
	strcpy( aDstPtr, "The quick brown fox jumps over the lazy dog");
	
	/* for now: */
	aSrcLen = strlen( (char *) aDstPtr);
	if ( aSize > aSrcLen ) aSize = aSrcLen;
	
	/* Functionality verification (pre-call) */
	if (gVerify) Verifymemset ( kBeforeCall, minInnerIterations, aResult, (char *) aDstPtr, aInt, aSize );

	/***** Call iterations *****/
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = memset( aDstPtr, aInt, aSize );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !Verifymemset ( kAfterCall, indx, aResult, (char *) aDstPtr, aInt, aSize );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogDiagnostic( 12,kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	free(aDstPtr);
	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long Onememset ( Boolean expectError, int aInt, size_t aSize)
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	void *aResult;

	/* Ancillary variables */
	void *aDstPtr;
	size_t aSrcLen;

  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aDstPtr = (void *) malloc( 45 );
	strcpy( aDstPtr, "The quick brown fox jumps over the lazy dog");
	
	/* for now: */
	aSrcLen = strlen( (char *) aDstPtr);
	if ( aSize > aSrcLen ) aSize = aSrcLen;
	
	/* Functionality verification (pre-call) */
	if (gVerify) Verifymemset ( kBeforeCall, gCurrentIndex, aResult, (char *) aDstPtr, aInt, aSize );

	aResult = memset( aDstPtr, aInt, aSize );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !Verifymemset ( kAfterCall, gCurrentIndex, aResult, (char *) aDstPtr, aInt, aSize );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	free( aDstPtr );
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

#define	TEST_ONE_MEMSET(expVal, ch, size)		sprintf( gTestName, "%s - byte value = 0x'%x' | size = %d", gTestAreaName, ch, size);	\
												if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
												BeginTest( BAT, gTestName );	\
												Onememset( expVal, ch, size ); \
												
												
#define EXERCISE_SIZEmemset(c)	gCurrentIndex = 0;\
								TEST_ONE_MEMSET(kExpectSuccess, c, ( size_t ) 0);\
								TEST_ONE_MEMSET(kExpectSuccess, c, ( size_t ) 1);\
								TEST_ONE_MEMSET(kExpectSuccess, c, ( size_t ) 2);\
								TEST_ONE_MEMSET(kExpectSuccess, c, ( size_t ) 3);\
								TEST_ONE_MEMSET(kExpectSuccess, c, ( size_t ) 4);\
								TEST_ONE_MEMSET(kExpectSuccess, c, ( size_t ) 8);\
								TEST_ONE_MEMSET(kExpectSuccess, c, ( size_t ) 16);\
								TEST_ONE_MEMSET(kExpectSuccess, c, ( size_t ) 32);\
								TEST_ONE_MEMSET(kExpectSuccess, c, ( size_t ) 1024)

#define space ' '

void Exercisememset( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "memset";
	STATUS(callName);
	
	/* params: charValue, size */
	EXERCISE_SIZEmemset(0);
	EXERCISE_SIZEmemset(space);
	EXERCISE_SIZEmemset('4');
	EXERCISE_SIZEmemset('W');
	EXERCISE_SIZEmemset('w');
	EXERCISE_SIZEmemset(255);
	EXERCISE_SIZEmemset(256);
	EXERCISE_SIZEmemset(255+'W'); /* see if we get 'W' here */
/*
	...
 Invalid type
 low memory
*/
}

void Testmemset( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		Exercisememset();
	else
	{
		callName = "memset";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = Callmemset(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory()
{
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kControlMem ) )
	{
		gTestAreaName = "ControlMem";		/* contains the name of the test    */
		TestControlMem(gMinOuterIterations, gMaxOuterIterations);
	}
	
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kGetPageSize ) )
	{
		gTestAreaName = "GetPageSize";		/* contains the name of the test    */
		TestGetPageSize(gMinOuterIterations, gMaxOuterIterations);
	}
	
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kmemcpy ) )
	{
		gTestAreaName = "kmemcpy";		/* contains the name of the test    */
		Testmemcpy(gMinOuterIterations, gMaxOuterIterations);
	}
	
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kmemset ) )
	{
		gTestAreaName = "kmemset";		/* contains the name of the test    */
		Testmemset(gMinOuterIterations, gMaxOuterIterations);
	}
}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram()
{
	int32 theError;
	
	gThisTask = CURRENTTASKITEM;

#if 0
	if ( OpenAudioFolio() )
	{
		LogDiagnostic( 80,"Can't open the audio folio!");
		return FALSE;
	}
#endif	
	theError = 0;
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
	
	return TRUE;
}


int main (int argc, char **argv)
{
	InitScaffold( KERNEL, argc, argv);
	
	gTestName = (char *)malloc( sizeof(char)*1024 );
	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{
			
			TestCategory(); 
		}

	}
		TermScaffold();	
 }






