
/*
 * @(#) SignalTests.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		SignalTests.c

	Contains:	Signal call tests, Kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/19/93	CDE		Derived from shell V1.6

	To Do:
*/

#define		USE_4B1		1

/***** Tool description for Intro, Exit, Help and Session Options displays *****/
#define		kToolDescription		"Signal tests"

/***** Version string *****/
#define		kVersionStr		"V1.0"

/***** Programmer credit for Help and Session Options displays *****/
#define		kProgrammerName		"Charlie Eckhaus"

/***** Call selection constants *****/
/* HINTS:
		
	 Create one constant for each call, e.g., kCallA corresponds to CallA().
		- Values needn't be sequential but must be  kFullTest (see baseUtils.h)
*/
#define		kAllocSignal		1
#define		kSendSignal			2
#define		kFreeSignal			3
#define		kWaitSignal			4
#if USE_4B1
#define		kGetCurrentSignals	5
#endif

/***** Default log filename: *****/
#define		kDefaultLogFile			"SignalTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
#define		kIncrement1Units	"units"
#define		kDefaultIncrement1	1


/***** Other test options *****/
#define		kDefaultSignalMask	0

/***** Include files *****/

#include "TestAllocSignal.h"
#include "TestSendSignal.h"
#include "TestFreeSignal.h"
#include "TestWaitSignal.h"

#if USE_4B1
#include "TestGetCurrentSignals.h"
#endif

/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "SignalTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "SignalTests.c";	/* contains the name of the program */
char	* gTestName= "SignalTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;			/* contains what level of testing occurs in this test */
int32	gTestType = 0;			/* interactive or automated */


/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */

/* The Item number corresponding to this task */
Item gThisTask;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* Parameter-checking */
Boolean gInvalidArg;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */
uint32	gSignalMask;
uint32 gThreadSignal;

/***** Utilities *****/
/* HINTS:

	 Anything you write that looks like it might be a generally useful call belongs here.
	 These will gradually migrate to our utilities libraries.
*/

#define		kIncAllocSignalErrorStr	"!! Unable to allocate signal/ $%x"
Boolean AllocateAllSignals( int32 *allocatedSignals )
/* allocate all available signals.
	return the corresponding signalMask in allocatedSignals
*/
{
	int32 aSignal;
	int32 allSignals = 0;
	
	do	
	{
		aSignal = AllocSignal( 0 );
		if (aSignal > 0)
			allSignals |= aSignal;
		else if (gCheckIncidentalErrors != kNone)
		{
			if (aSignal < 0)
			{
				LogComment(kIncAllocSignalErrorStr, aSignal);
				if (gCheckIncidentalErrors == kStop)
					return FALSE;
			}
		}
	} while (aSignal > 0);
	
	*allocatedSignals = allSignals;
	
	return TRUE;
}
	
/**** (End Utilities) ****/


void	ShowHelp(void);
void	ShowHelp()
/* Show Help for this tool */
{
		printf ( "#################################################################");
		printf ( "%s %s Help", kToolDescription, kVersionStr );
		printf ( "Written by %s for The 3DO Company", kProgrammerName );
		printf (    "#################################################################");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-out << no parameter>>\tDo not use a log file.");
		LogDiagnostic( 80, "     <<filename>>     \tUse specified log file.");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80,"-icmin <<value>>\tIterate calls starting at <<value>>.");
		LogDiagnostic( 80,"-icmax <<value>>\tEnd call iterations at <<value>>.");
		LogDiagnostic( 80,"-itmin <<value>>\tIterate tests starting at <<value>>.");
		LogDiagnostic( 80,"-itmax <<value>>\tEnd test iterations at <<value>>.");
		LogDiagnostic( 80, "");

		printf ( "-inc1 <<value>>\tUse <<value>> as increment."); 
		LogDiagnostic( 80, "");
		
		printf ( "-mask <<value>>\tUse (hex) <<value>> as signal mask."); 
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-error none\tDon't check for errors.");
		LogDiagnostic( 80, "       cont\tWhen an error occurs, report it and continue.");
		LogDiagnostic( 80, "       stop\tWhen an error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-ierror none\tDon't check for incidental errors.");
		LogDiagnostic( 80, "        cont\tWhen an incidental error occurs, report it and continue.");
		LogDiagnostic( 80, "        stop\tWhen an incidental error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-time off\tDon't output timing information.");
		LogDiagnostic( 80, "      on \tOutput timing information.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-func off\tDon't verify functionality.");
		LogDiagnostic( 80,"      on \tVerify functionality.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-fault off\tMake known assumptions.");
		LogDiagnostic( 80,"       on \tSkip known assumptions.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-sel <<no parameter>>\tRun the entire test suite.");
		LogDiagnostic( 80,"     AllocSignal         \tTest AllocSignal.");
		LogDiagnostic( 80,"     SendSignal          \tTest SendSignal.");
		LogDiagnostic( 80,"     FreeSignal          \tTest FreeSignal.");
		LogDiagnostic( 80,"     WaitSignal          \tTest WaitSignal.");
#if USE_4B1
		LogDiagnostic( 80,"     GetCurrentSignals   \tTest GetCurrentSignals.");
#endif
		LogDiagnostic( 80, "");
		
		printf ( "#################################################################");
}

void ShowOptions(void);
void ShowOptions()
/* Show the option settings for this session */
{
		printf ( "#################################################################");
		printf ( "%s %s - Session options", kToolDescription, kVersionStr);
		printf ( "Written by %s for The 3DO Company", kProgrammerName );
		printf (   "#################################################################");
		
		if (gUseLogFile) LogDiagnostic( 80, "-out: Use log file: %s.",gLogFile); else LogDiagnostic( 80,"Do not use a log file.");
		
		printf ( "Iterate each test from %i %s",gMinOuterIterations, kIncrement1Units);
		printf ( " to %i %s,", gMaxOuterIterations, kIncrement1Units);
		printf ( " in increments of %i %s.",gIncrement1,kIncrement1Units); 
		printf ( "Within each test, iterate the call from %i to %i.",gMinInnerIterations,gMaxInnerIterations);
		
		LogDiagnostic( 80, "-mask: $%x", gSignalMask);
		
		LogDiagnostic( 80, "-error: ");
		switch (gCheckErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80, "-ierror: ");
		switch (gCheckIncidentalErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for incidental errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an incidental error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first incidental error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80,"-time: ");
		if (gOutputTiming) LogDiagnostic( 80,"Output timing information."); else LogDiagnostic( 80,"Don't output timing information.");

		LogDiagnostic( 80,"-func: ");
		if (gVerify) LogDiagnostic( 80,"Verify functionality."); else LogDiagnostic( 80,"Don't verify functionality.");

		LogDiagnostic( 80,"-fault: ");
		if (gCheckFault) LogDiagnostic( 80,"Skip known assumptions."); else LogDiagnostic( 80,"Make known assumptions.");

		LogDiagnostic( 80,"-sel: ");
		switch (gCallSelector)
			{
			case kFullTest:
				LogDiagnostic( 80,"Run the entire test suite.");
				break;
				
			case kAllocSignal:
				LogDiagnostic( 80,"Test AllocSignal.");
				break;
				
			case kSendSignal:
				LogDiagnostic( 80,"Test SendSignal.");
				break;
				
			case kFreeSignal:
				LogDiagnostic( 80,"Test FreeSignal.");
				break;
				
			case kWaitSignal:
				LogDiagnostic( 80,"Test WaitSignal.");
				break;
				
#if USE_4B1
			case kGetCurrentSignals:
				LogDiagnostic( 80,"Test GetCurrentSignals.");
				break;
#endif				
			} /* end switch */
		
		printf ( "#################################################################");
		
}

bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	TOUCH(theArgCount);
	TOUCH(theArgList);
	
	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	gSignalMask = kDefaultSignalMask;
	
	gVerify = true;
	gCheckFault = true;
	

	return kRunTest;
}	/* end GetParams */



/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory(void);
void TestCategory()
{
	if ( ( gCallSelector == kAllocSignal ) || ( gCallSelector == kAllocSignal ) )
	{
		gTestAreaName = "Ffs";		/* contains the name of the test    */
		TestAllocSignal(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kSendSignal ) )
	{
		gTestAreaName = "SendSignal";		/* contains the name of the test    */
		TestSendSignal(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kFreeSignal ) )
	{
		gTestAreaName = "FreeSignal";		/* contains the name of the test    */
		TestFreeSignal(gMinOuterIterations, gMaxOuterIterations);
	}

#if USE_4B1
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kGetCurrentSignals ) )
	{
		gTestAreaName = "GetCurrentSignals";		/* contains the name of the test    */
		TestGetCurrentSignals(gMinOuterIterations, gMaxOuterIterations);
	}
#endif

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kWaitSignal ) )
	{
		gTestAreaName = "WaitSignal";		/* contains the name of the test    */
		TestWaitSignal(gMinOuterIterations, gMaxOuterIterations);
	}

}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram(void);
Boolean	InitProgram()
{
#if 0
	int32 theError;
#endif
	
	gThisTask = CURRENTTASKITEM;

#if 0
	if ( OpenAudioFolio() )
	{
		LogDiagnostic( 80,"Can't open the audio folio!");
		return FALSE;
	}
#endif	
#if 0
	theError = 0;
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
#endif
	
	return TRUE;
}


int main (int argc, char **argv)
{
	InitScaffold( KERNEL, argc, argv);
	
	gTestName = (char *)malloc( sizeof(char)*1024 );
	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{

			TestCategory(); 
			
		}
	TermScaffold();
	}
	
	return(0);
 }

