
/*
 * @(#) TestAllocSignal.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestAllocSignal.c

	Contains:	Test routine for AllocSignal().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/10/93	CDE		Derived from shell

	To Do:
*/


/***** Checklist:

	 Look for bulleted () HINTS below.
*/

/* HINTS:

	 Change to reflect your call name.
*/
#include "TestAllocSignal.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeAllocSignalVerifyStr		"Before iteration (%i)/\tmask/ $%x"
#define		kAfterAllocSignalVerifyStr		"After iteration (%i)/\tresult/ $%x"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
Boolean		VerifyAllocSignal ( Boolean beforeCall, long aIndx, int32 aResult, uint32 aSignalMask )
/* Functionality verification for AllocSignal */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeAllocSignalVerifyStr, aIndx, aSignalMask);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterAllocSignalVerifyStr, aIndx, aResult);
		return (aResult > 0);
	}
	
}

#define		CLEANUP

long	CallAllocSignal (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	uint32 aSignalMask = gSignalMask;
	int32 aResult = 0;
	
	TOUCH(currentOuterIteration);

	/* Functionality verification (pre-call) */
	if (gVerify) theError = !VerifyAllocSignal ( kBeforeCall, minInnerIterations, aResult, aSignalMask );

	/* Preparation for call iterations */
	
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (theError)
		{
			LogComment(kIncidentalErrorStr, "AllocSignal");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = AllocSignal( aSignalMask );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyAllocSignal ( kAfterCall, indx, aResult, aSignalMask );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneAllocSignal ( Boolean expectError, uint32 aSignalMask )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	int32 aResult = 0;
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	/* (none) */

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyAllocSignal ( kBeforeCall, gCurrentIndex, aResult, aSignalMask );

	aResult = AllocSignal( aSignalMask );

	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyAllocSignal ( kAfterCall, gCurrentIndex, aResult, aSignalMask );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_ALLOCSIGNAL(expVal, sigMask)		sprintf( gTestName, "%s | task= 0x%x; SignalMask = 0x%x", gTestAreaName, sigMask);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														aAllocResult = OneAllocSignal(expVal,sigMask);
														


void ExerciseAllocSignal( void )
{
	char	*callName;
	int32	aFreeResult;
	long	aAllocResult = 1;
	long	saveCheckErrors;
	uint32	allSignals;

	gCurrentIndex = 0;
	
	callName = "AllocSignal";
	STATUS(callName);
	
	/* should fail to allocate system signals */
	LogDiagnostic( 12,"-- System signals --");
#if 0
	TEST_ONE_ALLOCSIGNAL(kExpectError, SIGF_MEMLOW);
	TEST_ONE_ALLOCSIGNAL(kExpectError, SIGF_MEMGONE);
#endif
	TEST_ONE_ALLOCSIGNAL(kExpectError, SIGF_ABORT);
	TEST_ONE_ALLOCSIGNAL(kExpectError, SIGF_IODONE);
	TEST_ONE_ALLOCSIGNAL(kExpectError, SIGF_DEADTASK);

	saveCheckErrors = gCheckErrors;
	gCheckErrors = kStop;
	LogDiagnostic( 12,"-- All available signals --");
	while (aAllocResult)
	{
		TEST_ONE_ALLOCSIGNAL(kExpectSuccess, 0);
	}
/*	while (TEST_ONE_ALLOCSIGNAL(kExpectSuccess, 0) == 0);	// allocate all available signals */
	gCheckErrors = saveCheckErrors;
		
	/* Cleanup */
#if 0
	allSignals = 0xFFFFFFFF;	/* fails */
#endif
	allSignals = 0x3fffff00;	/* in 1p1, should free everything */
								/* could use ClearFlag(GetCurrentSignals(),(system mask)) if GetCurrentSignals worked */
	aFreeResult = FreeSignal(allSignals);
	if (aFreeResult)
		LogComment("Attempt to free all signals (0x%x) failed: 0x%x", allSignals, aFreeResult);
		
/*
	...
 low memory
*/
}

void TestAllocSignal( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseAllocSignal();
	else
	{
		callName = "AllocSignal";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallAllocSignal(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

