
/*
 * @(#) TestSendSignal.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestSendSignal.c

	Contains:	Test routine for SendSignal().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <2>	 8/19/93	CDE		Augmented message in ThreadForSendSignal
		 <1>	 7/19/93	CDE		Derived from shell version 1.6

	To Do:
*/


#include "TestSendSignal.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeSendSignalVerifyStr		"Before iteration (%i)/\ttask/ $%x,\tsignal/ $%x"
#define		kAfterSendSignalVerifyStr		"After iteration (%i)/\tresult/ $%x"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncSpawnThreadErrorStr	"!! Unable to spawn thread/ $%x"
#define		kIncAllocSignalErrorStr	"!! Unable to allocate signal/ $%x"
#define		kIncDeleteItemErrorStr	"!! Unable to delete thread $%x, result/ $%x"
#define		kIncFreeSignalErrorStr	"!! Unable to free signal $%x, result/ $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern uint32 gSignalMask;

/***** Test routines *****/

extern uint32 gThreadSignal;

void ThreadForSendSignal(void)
{
	uint32 aSignalMask = gSignalMask;
	int32 aSignal;
	
	aSignal = AllocSignal( aSignalMask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aSignal < 0)
		{
			LogComment(kIncAllocSignalErrorStr, aSignal);
			if (gCheckIncidentalErrors == kStop);
/* CDE: WHAT IS THE RIGHT THING TO DO??? */
		}
	}
	gThreadSignal = (uint32) aSignal;
	while (1)
	{
		WaitSignal(gThreadSignal);			/* Wait until parent acknowledges */
		LogDiagnostic( 80,"::Signal received by thread.");
	}
}


Boolean FreeThreadForSendSignal( Item aTask )
{
	Err aErr;

	aErr = DeleteItem( aTask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aErr < 0)
		{
			LogComment(kIncDeleteItemErrorStr, aTask, aErr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	return true;
}
	
Boolean	CleanupSendSignal( Item aTask, int32 aSignal )
{
	int32 aInt32;
	
	if (!FreeThreadForSendSignal( aTask )) return false;
	
	if ( aSignal >= 0)
	{
		aInt32 = FreeSignal( gThreadSignal );
		if (gCheckIncidentalErrors != kNone)
		{
			if (aInt32 < 0)
			{
				LogComment(kIncFreeSignalErrorStr, gThreadSignal, aInt32);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
	}
	
	return true;
}
	
Boolean		VerifySendSignal ( Boolean beforeCall, long aIndx, Err aResult, Item task, int32 signalMask )
/* Functionality verification for SendSignal */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeSendSignalVerifyStr, aIndx, task, signalMask);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterSendSignalVerifyStr, aIndx, aResult);
		return (aResult == 0);
	}
	
}

#define		CLEANUP		CleanupSendSignal( aTask, aSignal )

long CallSendSignal( long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError = FALSE;
	
	Err 	aErr = 0;

  	/* Variables for call parameters and return value */
	uint8	aPriority = CURRENT_TASK_PRIORITY;
	Item	aTask;
	int32	aSignal;
	
	TOUCH(currentOuterIteration);

	/* Preparation for call iterations */
	aTask = SpawnThread(ThreadForSendSignal, 128, aPriority, "TestThread");
	
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aTask < 0)
		{
			LogComment(kIncSpawnThreadErrorStr, aTask);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	aSignal = (int32) gThreadSignal;
	
	/* Functionality verification (pre-call) */
	if (gVerify) theError = !VerifySendSignal ( kBeforeCall, minInnerIterations, aErr, aTask, aSignal );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aErr = SendSignal( aTask, gThreadSignal );
		LogError( aErr );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifySendSignal ( kAfterCall, indx, aErr, aTask, aSignal );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	if ( !CLEANUP ) return kBreakTime;
	
	return accumulator;
}

long OneSendSignal( Boolean expectError, Item aTask, int32 aSignal )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	Err 	aErr = 0;
	
	/* Ancillary variables */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	/* Functionality verification (pre-call) */
	if (gVerify) theError = !VerifySendSignal ( kBeforeCall, gCurrentIndex, aErr, aTask, aSignal );

	aErr = SendSignal( aTask, aSignal );
	LogError( aErr );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifySendSignal ( kAfterCall, gCurrentIndex, aErr, aTask, aSignal );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

#define	TEST_ONE_SENDSIGNAL(expVal, task, sigMask)	sprintf( gTestName, "%s | task= 0x%x; Signal = 0x%x", gTestAreaName, task, sigMask);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneSendSignal(expVal, task, sigMask);
														


Boolean ExerciseSendSignal( void )
{
	char	*callName;
	Item	aTask;
	int32	aSignal;
	uint32	aSignalBit;
	long	saveCheckErrors;

	gCurrentIndex = 0;
	
	callName = "SendSignal";
	STATUS(callName);
	
	aTask = SpawnThread(ThreadForSendSignal, 128, CURRENT_TASK_PRIORITY, "TestThread");
	aSignal = gThreadSignal;
	
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aTask < 0)
		{
			LogComment(kIncSpawnThreadErrorStr, aTask);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}

	/* params: priority */
	LogDiagnostic( 12,"-- Sending system signals --");
	TEST_ONE_SENDSIGNAL(kExpectSuccess, aTask, gThreadSignal);
#if 0
	TEST_ONE_SENDSIGNAL(kExpectError, aTask, SIGF_MEMLOW);
	TEST_ONE_SENDSIGNAL(kExpectError, aTask, SIGF_MEMGONE);
#endif
	TEST_ONE_SENDSIGNAL(kExpectError, aTask, SIGF_ABORT);
	TEST_ONE_SENDSIGNAL(kExpectError, aTask, SIGF_IODONE);
	TEST_ONE_SENDSIGNAL(kExpectError, aTask, SIGF_DEADTASK);

	saveCheckErrors = gCheckErrors;
	gCheckErrors = kStop;

	LogDiagnostic( 12,"-- Sending all signals (0x80000000 .. 0x00000080) --");
	aSignalBit = 0x80000000;
	TEST_ONE_SENDSIGNAL(kExpectError, aTask, aSignalBit);
	aSignalBit >>= 1;
	while (aSignalBit > 0x00000080)
	{
		TEST_ONE_SENDSIGNAL(kExpectSuccess, aTask, aSignalBit);
		aSignalBit >>= 1;
	}

	gCheckErrors = saveCheckErrors;
	
	CLEANUP;
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestSendSignal( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseSendSignal();
	else
	{
		callName = "SendSignal";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallSendSignal(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

