
/*
 * @(#) TestWaitSignal.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestWaitSignal.c

	Contains:	Test routine for WaitSignal().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <2>	 8/19/93	CDE		Added OpenAudioFolio() to ThreadForSendSignal,
		 							Changed stacksize to a parameter, value from 128 to 512
		 <1>	 7/19/93	CDE		Derived from TestSendSignal

	To Do:
*/

#define		USE_1p01		1

#include "TestWaitSignal.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeWaitSignalVerifyStr		"Before iteration (%i)/\tsignal/ $%x"
#define		kAfterWaitSignalVerifyStr		"After iteration (%i)/\tresult/ $%x"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncSpawnThreadErrorStr	"!! Unable to spawn thread/ $%x"
#define		kIncAllocSignalErrorStr	"!! Unable to allocate signal/ $%x"
#define		kIncDeleteItemErrorStr	"!! Unable to delete thread $%x, result/ $%x"
#define		kIncFreeSignalErrorStr	"!! Unable to free signal $%x, result/ $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern uint32 gSignalMask;

extern Boolean AllocateAllSignals( int32 *allocatedSignals );

uint32 gSentByThread;		/* signal to be sent from the thread (and waited for by parent) */
uint32 gSentByParent;		/* signal to be sent by the parent (and waited for by thread) */

/***** Test routines *****/

void ThreadForWaitSignal(void)
/* Test thread:
	Send signal and sleep for a while.
	MAYBE GetControlPad WOULD BE BETTER?
*/
{
	Item	myParent = THREAD_PARENT;
	int32	aSignal;
	
#if 0
	/* Threads must open the folios for themselves */
	if ( OpenAudioFolio() )
	{
		LogDiagnostic( 80,"Can't open the audio folio!");
	}
#endif

	aSignal = AllocSignal( 0 );
	if (aSignal < 0)
		LogComment(kIncAllocSignalErrorStr, 0);

	gSentByParent = (uint32) aSignal;
	
	while (true)
	{
		LogDiagnostic( 80,"Sending signal");
		SendSignal(myParent, gSentByThread);
		WaitSignal(gSentByParent);
	}
}


Boolean FreeThreadForWaitSignal( Item aTask )
{
	Err aErr;

	aErr = DeleteItem( aTask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aErr < 0)
		{
			LogComment(kIncDeleteItemErrorStr, aTask, aErr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	return true;
}
	
Boolean	CleanupWaitSignal( Item aTask, int32 sentByThread )
/* Clean up for TestWaitSignal:
	Delete the thread task.
	If the original value returned by AllocSignal was not an error,
		Free the thread signal.
*/
{
	int32 aInt32;
	
	if (!FreeThreadForWaitSignal( aTask )) return false;
	
	if ( sentByThread >= 0)
	{
		aInt32 = FreeSignal( gSentByThread );
		if (gCheckIncidentalErrors != kNone)
		{
			if (aInt32 < 0)
			{
				LogComment(kIncFreeSignalErrorStr, sentByThread, aInt32);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
	}
	
	return true;
}
	
Boolean		VerifyWaitSignal ( Boolean beforeCall, long aIndx, uint32 aResult, uint32 signal )
/* Functionality verification for WaitSignal */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeWaitSignalVerifyStr, aIndx, signal);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterWaitSignalVerifyStr, aIndx, aResult);
		return true;
	}
	
}

#define		CLEANUP		CleanupWaitSignal( aTask, aSignal )

long CallWaitSignal( long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	int		aStackSize = 512;
	uint8	aPriority = CURRENT_TASK_PRIORITY;
	uint32 	aResult = 0;
	Item	aTask;
	int32	aSignal;
	
	uint32 aSignalMask = gSignalMask;
	
	TOUCH(currentOuterIteration);

	aSignal = AllocSignal( aSignalMask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aSignal < 0)
		{
			LogComment(kIncAllocSignalErrorStr, aSignalMask);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	gSentByThread = (uint32) aSignal;

	/* Preparation for call iterations */
	aTask = SpawnThread(ThreadForWaitSignal, aStackSize, aPriority, "TestThread");
	
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aTask < 0)
		{
			LogComment(kIncSpawnThreadErrorStr, aTask);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	SendSignal(aTask, gSentByParent);	/* Tell the thread to go ahead and send the signal */

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyWaitSignal ( kBeforeCall, minInnerIterations, aResult, gSentByThread );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = WaitSignal(gSentByThread);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyWaitSignal ( kAfterCall, indx, aResult, gSentByThread );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	if ( !CLEANUP ) return kBreakTime;
	
	return accumulator;
}

long OneWaitSignal( Boolean expectError, Item aTask, uint32 aSignalMask )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	uint32 	aResult = 0;

	/* Ancillary variables */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	SendSignal(aTask, gSentByParent);	/* Tell the thread to go ahead and send the signal */

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyWaitSignal ( kBeforeCall, gCurrentIndex, aResult, aSignalMask );

	aResult = WaitSignal(aSignalMask);

	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyWaitSignal ( kAfterCall, gCurrentIndex, aResult, aSignalMask );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

#define	TEST_ONE_WAITSIGNAL(expVal, task, sigMask)	sprintf( gTestName, "%s | task= 0x%x; Signal Mask= 0x%x", gTestAreaName, task, sigMask);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneWaitSignal(expVal, task, sigMask); \
														


Boolean ExerciseWaitSignal( void )
{
	char	*callName;
	int32	aSignal;
	int		aStackSize = 512;
	uint8	aPriority = CURRENT_TASK_PRIORITY;
	Item	aTask;
#if 0
	long	saveCheckErrors;
#endif
	int32	allocatedSignals = 0;
	uint32	aSignalBit;
	Boolean myReturnValue = FALSE;
	int32	aInt32;

	gCurrentIndex = 0;
	
	callName = "WaitSignal";
	STATUS(callName);
	
	/* Preparation for call iterations */
	aTask = SpawnThread(ThreadForWaitSignal, aStackSize, aPriority, "TestThread");
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aTask < 0)
		{
			LogComment(kIncSpawnThreadErrorStr, aTask);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}

	/* Thread should be waiting for gSentByParent, which is sent by call preparation in OneWaitSignal */
	
	aSignal = AllocSignal( 0 );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aSignal < 0)
		{
			LogComment(kIncAllocSignalErrorStr, 0);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	gSentByThread = (uint32) aSignal;		/* this is the signal we'll respond to */
	
	/* params: task, signalmask */
	
	LogDiagnostic( 12,"-- No bits set: --");
/*	TEST_ONE_WAITSIGNAL(kExpectError, aTask, 0);	// would wait forever! */
	LogDiagnostic( 12,"-- All bits set: --");
	TEST_ONE_WAITSIGNAL(kExpectError, aTask, 0xFFFFFFFF);	/* if this doesn't generate an error, comment it out */

/* Find a way to generate these:
	LogDiagnostic( 12,"-- System signals: --");
#if 0
	TEST_ONE_WAITSIGNAL(kExpectSuccess, aTask, SIGF_MEMLOW);
	TEST_ONE_WAITSIGNAL(kExpectSuccess, aTask, SIGF_MEMGONE);
#endif
	TEST_ONE_WAITSIGNAL(kExpectSuccess, aTask, SIGF_ABORT);
	TEST_ONE_WAITSIGNAL(kExpectSuccess, aTask, SIGF_IODONE);
	TEST_ONE_WAITSIGNAL(kExpectSuccess, aTask, SIGF_DEADTASK);
*/

#if 0
	saveCheckErrors = gCheckErrors;
#endif
	gCheckErrors = kStop;

	LogDiagnostic( 12,"-- Allocating all available signals: --");
	AllocateAllSignals(&allocatedSignals);
/* Should hang if the thread never sends any of them:
	TEST_ONE_WAITSIGNAL(kExpectSuccess, aTask, allocatedSignals);
*/

	LogDiagnostic( 12,"-- All available signals (collectively, after freeing): --");
	aInt32 = FreeSignal( allocatedSignals );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aInt32 < 0)
		{
			LogComment(kIncFreeSignalErrorStr, allocatedSignals, aInt32);
			if (gCheckIncidentalErrors == kStop) goto errorExit;
		}
	}
	TEST_ONE_WAITSIGNAL(kExpectError, aTask, allocatedSignals);	/* could (should?) get error because they're not allocated */
	
	LogDiagnostic( 12,"-- All available signals (individually): --");
	LogDiagnostic( 12,"(CURRENTLY DISABLED)");
	AllocateAllSignals(&allocatedSignals);
	aSignalBit = 0x80000000;
	while (aSignalBit > 0)
	{
/* Should hang if the thread never sends it:
		if (aSignalBit & allocatedSignals)
			TEST_ONE_WAITSIGNAL(kExpectSuccess, aSignalBit);
*/
		aSignalBit >>= 1;
	}
	aInt32 = FreeSignal( allocatedSignals );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aInt32 < 0)
		{
			LogComment(kIncFreeSignalErrorStr, allocatedSignals, aInt32);
			if (gCheckIncidentalErrors == kStop) goto errorExit;
		}
	}
	
	myReturnValue = TRUE;
	
errorExit:
	CLEANUP;
	
	return myReturnValue;
/*
	...
 Invalid type
 low memory
*/
}

void TestWaitSignal( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseWaitSignal();
	else
	{
		callName = "WaitSignal";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallWaitSignal(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

