
/*
 * @(#) ListUtils.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
 *	ListUtils.c
 *
 *	Written by Nick Triantos
 *	15-Dec-1994
 *
 *	 1993 by The 3DO Company. All rights reserved.
 *	This material constitutes confidential and proprietary
 *	information of the 3DO Company and shall not be used by
 *	any Person or for any purpose except as expressly
 *	authorized in writing by the 3DO Company.
 *
 *	In general, the following tests work as follows:
 *		For each function call, set up the function name, then do the following tests:
 *		BAT:		Ensure functionality.  Set up some data, call the
 *					function, ensure that the result is OK
 *		EST:		Ensure that the correct errors are returned when it's
 *					appropriate.
 *		HIGH_RISK:	Send invalid data, NULL ptrs, etc.  In general, for kernel
 *					tests like these, HIGH_RISK will probably fail, since
 *					rock-solid error-checking would come at the cost of speed.
 *
 */


/* ---------- Includes	*/

#include <kernel/debug.h>
#include <stdio.h>
#include <kernel/mem.h>
#include <string.h>
#include <tests/3do_scaf.h>
#include <kernel/nodes.h>
#include <kernel/list.h>


/* ---------- Constants	*/

#define kProgName					"List Utilities"
#define kTestName					"ListUtilsTests"
#define kMaxTestNameSize			30
#define kInvalidPtr					0x654321

/*
 * Un comment the following line to perform the kMyDebug option without compile warnings
 */
/*#define	KMYDEBUG	1 */
#define	kMyDebug					FALSE

/* ---------- Types		*/

typedef struct NickNode {
	Node		node ;
	char		name[25] ;
} NickNode ;


/* ---------- Prototypes	*/

void			TestDumpNode					(void) ;

void			TestFindNodeFromHead			(void) ;
void			TestFindNodeFromHead_BAT		(void) ;
void			TestFindNodeFromHead_EST		(void) ;
void			TestFindNodeFromHead_RISK		(void) ;

void			TestFindNodeFromTail			(void) ;
void			TestFindNodeFromTail_BAT		(void) ;
void			TestFindNodeFromTail_EST		(void) ;
void			TestFindNodeFromTail_RISK		(void) ;

void			TestGetNodeCount				(void) ;
void			TestGetNodeCount_BAT			(void) ;
void			TestGetNodeCount_EST			(void) ;
void			TestGetNodeCount_RISK			(void) ;

void			TestGetNodePosFromHead			(void) ;
void			TestGetNodePosFromHead_BAT		(void) ;
void			TestGetNodePosFromHead_EST		(void) ;
void			TestGetNodePosFromHead_RISK		(void) ;

void			TestGetNodePosFromTail			(void) ;
void			TestGetNodePosFromTail_BAT		(void) ;
void			TestGetNodePosFromTail_EST		(void) ;
void			TestGetNodePosFromTail_RISK		(void) ;

void			TestInsertNodeAfter				(void) ;
void			TestInsertNodeAfter_BAT			(void) ;
void			TestInsertNodeAfter_EST			(void) ;
void			TestInsertNodeAfter_RISK		(void) ;

void			TestInsertNodeBefore_BAT		(void) ;
void			TestInsertNodeBefore_EST		(void) ;
void			TestInsertNodeBefore_RISK		(void) ;
void			TestInsertNodeBefore			(void) ;

void			TestScanList					(void) ;
void			TestScanList_BAT				(void) ;
void			TestScanList_EST				(void) ;
void			TestScanList_RISK				(void) ;

void			TestScanListB					(void) ;
void			TestScanListB_BAT				(void) ;
void			TestScanListB_EST				(void) ;
void			TestScanListB_RISK				(void) ;

void			ClearList						(List* theList) ;


/* ---------- Globals	*/

#define kNameCount		11
char*				gNames[] = {
						"Billy", "Bob", "Nick", "Roger", "Sue", "Ann",
						"Michelle", "Mike", "Germaine", "Laura", "Lisa"
					} ;
char*				gProgName ;
char*				gTestName ;
int32				gFlag ;
int32				gTestLevel ;
int32				gTestType ;
List				gNickList ;

Boolean				gDoBAT = TRUE, gDoEST = TRUE, gDoHIGH_RISK = FALSE ;


/* ---------- Code	*/

int main(int argc, char* argv[])
{
	gProgName = (char*)AllocMem(sizeof(kProgName), MEMTYPE_ANY) ;
	gTestName = (char*)AllocMem(kMaxTestNameSize, MEMTYPE_ANY) ;
	strcpy(gProgName, kProgName) ;
	strcpy(gTestName, kTestName) ;
	gTestType = 0 ;
	
	InitScaffold( KERNEL, argc, argv) ;
	
#ifdef KMYDEBUG
	if (kMyDebug) LogDiagnostic( 80,"db 1");
#endif
	TestDumpNode() ;
#ifdef KMYDEBUG
	if (kMyDebug) LogDiagnostic( 80,"db 2");
#endif
	TestFindNodeFromHead() ;
#ifdef KMYDEBUG
	if (kMyDebug) LogDiagnostic( 80,"db 3");
#endif
	TestFindNodeFromTail() ;
	TestGetNodeCount() ;
	TestGetNodePosFromHead() ;
	TestGetNodePosFromTail() ;
	TestInsertNodeAfter() ;
	TestInsertNodeBefore() ;
	TestScanList() ;
	TestScanListB() ;

	TermScaffold() ;
	
	FreeMem(gProgName, sizeof(kProgName)) ;
	FreeMem(gTestName, kMaxTestNameSize) ;

	return(0);
}


/* ---- DumpNode()	*/
void TestDumpNode()
{
	NickNode		oneNode ;
	
	strcpy(gTestName, "TestDumpNode") ;
	if (gDoBAT) {
		gTestLevel = BAT ;

		BeginTest( BAT, gTestName) ;
		
#ifdef KMYDEBUG
		if (kMyDebug) LogDiagnostic( 80,"db 1a");
#endif
		InitList(&gNickList, "Nick's List") ;
/*		strcpy((char*)&oneNode.name, (char*)&gNames[0]) ;
*/
		AddHead(&gNickList, (Node*)&oneNode) ;
		DumpNode((Node*)&oneNode, "Banner for the DumpNode() call") ;
#ifdef KMYDEBUG
		if (kMyDebug) LogDiagnostic( 80,"db 1b");
#endif
		LogResult(UNRESOLVED, "Output needs to be inspected.") ;
#ifdef KMYDEBUG
		if (kMyDebug) LogDiagnostic( 80,"db 1c");
#endif
		
#ifdef KMYDEBUG
		if (kMyDebug) LogDiagnostic( 80,"db 1d");
#endif
	}
#ifdef KMYDEBUG
	if (kMyDebug) LogDiagnostic( 80,"db 1e");
#endif
}


/* ---- FindNodeFromHead()	*/

void TestFindNodeFromHead()
{
	strcpy(gTestName, "FindNodeFromHead()") ;

	if (gDoBAT)				TestFindNodeFromHead_BAT() ;
	if (gDoEST)				TestFindNodeFromHead_EST() ;
	if (gDoHIGH_RISK)		TestFindNodeFromHead_RISK() ;
}


void TestFindNodeFromHead_BAT()
{
	NickNode*		oneNode ;
	NickNode*		resultNode ;
	int				i ;
	Boolean			allWorked = TRUE ;

	InitList(&gNickList, "Nick's List") ;
	gTestLevel = BAT ;
	BeginTest( BAT, gTestName) ;
	for (i=0 ; i<kNameCount ; i++) {
		oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode == NULL) {
			LogDiagnostic( 12,"Couldn't allocate memory for a NickNode!") ;
			LogResult(UNRESOLVED,"Mem. error in AllocMem") ;
			goto TFNFH_BAT_cleanup ;
		}
		strcpy((char*)oneNode->name, (char*)gNames[i]) ;
		AddTail(&gNickList, (Node*)oneNode) ;
	}
	oneNode = NULL ;		/*  So we don't delete it during cleanup.	*/
	for (i=0 ; i<kNameCount ; i++) {
		resultNode = (NickNode*)FindNodeFromHead(&gNickList, i) ;
		if (strcmp(resultNode->name, gNames[i]) != 0) {
			allWorked = FALSE ;
		}
	}
	if (allWorked)
		LogResult(PASSED, "") ;
	else
		LogResult(FAILED, "One or more names didn't come back OK.") ;
TFNFH_BAT_cleanup:
	if (oneNode != NULL)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
	
	if (gFlag == FAILED)
		return ;
}


void TestFindNodeFromHead_EST()
{
	gTestLevel = EST ;
	BeginTest( BAT, gTestName) ;
	LogResult(PASSED, "No error states to check.") ;
	
}


void TestFindNodeFromHead_RISK()
{
	NickNode*		resultNode ;

	gTestLevel = HIGH_RISK ;
	BeginTest( BAT, gTestName) ;
	LogDiagnostic( 12,"Testing a NULL list ptr with FindNodeFromHead( ) ...") ;
	resultNode = (NickNode*)FindNodeFromHead(NULL, 0) ;
	if ((int32)resultNode > 0) {
		LogDiagnostic( 12,"FindNodeFromHead(NULL,...) should return NULL or error!") ;
		LogResult(FAILED, "NULL List returned a valid ptr!") ;
		goto TFNFH_RISK_cleanup ;
	}
	LogDiagnostic( 12,"Testing an invalid list ptr with FindNodeFromHead( ) ...") ;
	resultNode = (NickNode*)FindNodeFromHead((List*)kInvalidPtr, 0) ;
	if ((int32)resultNode > 0) {
		LogDiagnostic( 12,"FindNodeFromHead(0x654321,...) should return NULL or error!") ;
		LogResult(FAILED, "Invalid List returned a valid ptr!") ;
		goto TFNFH_RISK_cleanup ;
	}
	LogResult(PASSED,"") ;
  TFNFH_RISK_cleanup: 
  ;
	
}


/* ---- FindNodeFromTail()	*/

void TestFindNodeFromTail()
{
	strcpy(gTestName, "FindNodeFromTail()") ;

	if (gDoBAT)				TestFindNodeFromTail_BAT() ;
	if (gDoEST)				TestFindNodeFromTail_EST() ;
	if (gDoHIGH_RISK)		TestFindNodeFromTail_RISK() ;
}


void TestFindNodeFromTail_BAT()
{
	NickNode*		oneNode = NULL ;
	NickNode*		resultNode ;
	int				i ;
	Boolean			allWorked = TRUE ;

	InitList(&gNickList, "Nick's List") ;
	gTestLevel = BAT ;
	BeginTest( BAT, gTestName) ;
	for (i=0 ; i<kNameCount ; i++) {
		oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode == NULL) {
			LogDiagnostic( 12,"Couldn't allocate memory for a NickNode!") ;
			LogResult(UNRESOLVED,"Mem. error in AllocMem") ;
			goto TFNFT_BAT_cleanup ;
		}
		strcpy((char*)oneNode->name, (char*)gNames[i]) ;
		AddHead(&gNickList, (Node*)oneNode) ;
	}
	oneNode = NULL ;		/* So we don't delete it during cleanup.	*/
	for (i=0 ; i<kNameCount ; i++) {
		resultNode = (NickNode*)FindNodeFromTail(&gNickList, i) ;
		if (strcmp(resultNode->name, gNames[i]) != 0) {
			allWorked = FALSE ;
		}
	}
	if (allWorked)
		LogResult(PASSED, "") ;
	else
		LogResult(FAILED, "One or more names didn't come back OK.") ;
TFNFT_BAT_cleanup:
	if (oneNode != NULL)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
	
	if (gFlag == FAILED)
		return ;
}


void TestFindNodeFromTail_EST()
{
	gTestLevel = EST ;
	BeginTest( BAT, gTestName) ;
	LogResult(PASSED, "No error states to check.") ;
	
}


void TestFindNodeFromTail_RISK()
{
	NickNode*		resultNode ;

	gTestLevel = HIGH_RISK ;
	BeginTest( BAT, gTestName) ;
	LogDiagnostic( 12,"Testing a NULL list ptr with FindNodeFromTail( ) ...") ;
	resultNode = (NickNode*)FindNodeFromTail(NULL, 0) ;
	if ((int32)resultNode > 0) {
		LogDiagnostic( 12,"FindNodeFromTail(NULL,...) should return NULL or error!") ;
		LogResult(FAILED, "NULL List returned a valid ptr!") ;
		goto TFNFT_HR_cleanup ;
	}
	LogDiagnostic( 12,"Testing an invalid list ptr with FindNodeFromTail( ) ...") ;
	resultNode = (NickNode*)FindNodeFromTail((List*)kInvalidPtr, 0) ;
	if ((int32)resultNode > 0) {
		LogDiagnostic( 12,"FindNodeFromTail(0x654321,...) should return NULL or error!") ;
		LogResult(FAILED, "Invalid List returned a valid ptr!") ;
		goto TFNFT_HR_cleanup ;
	}
	LogResult(PASSED,"") ;
TFNFT_HR_cleanup:
;
}


/* ---- GetNodeCount()	*/

#define kInitialNodeCount		3
#define kIncrementedNodeCount	7
#define kDeletedNodeCount		2

void TestGetNodeCount()
{
	strcpy(gTestName, "GetNodeCount()") ;

	if (gDoBAT)				TestGetNodeCount_BAT() ;
	if (gDoEST)				TestGetNodeCount_EST() ;
	if (gDoHIGH_RISK)		TestGetNodeCount_RISK() ;
}


void TestGetNodeCount_BAT()
{
	NickNode*		oneNode ;
	int				i ;
	Boolean			allWorked ;
	
	InitList(&gNickList, "Counting list") ;
	gTestLevel = BAT ;
	BeginTest( BAT, gTestName) ;
	allWorked = TRUE ;
	for (i=0 ; i<kInitialNodeCount ; i++) {
		oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode == NULL) {
			LogDiagnostic( 12,"Couldn't allocate memory for a NickNode!") ;
			LogResult(UNRESOLVED,"Mem. error in AllocMem") ;
			goto TGNC_BAT_cleanup ;
		}
		AddHead(&gNickList, (Node*)oneNode) ;
	}
	if (GetNodeCount(&gNickList) != kInitialNodeCount) {
		allWorked = FALSE ;
		goto TGNC_BAT_cleanup ;
	}
	for (i=0 ; i<(kIncrementedNodeCount-kInitialNodeCount) ; i++) {
		oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode == NULL) {
			LogDiagnostic( 12,"Couldn't allocate memory for a NickNode!") ;
			LogResult(UNRESOLVED,"Mem. error in AllocMem") ;
			goto TGNC_BAT_cleanup ;
		}
		AddHead(&gNickList, (Node*)oneNode) ;
	}
	if (GetNodeCount(&gNickList) != kIncrementedNodeCount) {
		allWorked = FALSE ;
		goto TGNC_BAT_cleanup ;
	}
	for (i=0 ; i<(kIncrementedNodeCount-kDeletedNodeCount) ; i++) {
		oneNode = (NickNode*)RemHead(&gNickList) ;
		if (oneNode == NULL)
			LogDiagnostic( 12,"List Empty!") ;
	}
	if (GetNodeCount(&gNickList) != kDeletedNodeCount) {
		allWorked = FALSE ;
		goto TGNC_BAT_cleanup ;
	}
TGNC_BAT_cleanup:
	if (allWorked)
		LogResult(PASSED, "") ;
	else
		LogResult(FAILED, "One or more calls to GetNodeCount() returned a wrong answer!") ;
	ClearList(&gNickList) ;
	
}


void TestGetNodeCount_EST()
{
	gTestLevel = EST ;
	BeginTest( BAT, gTestName) ;
	LogResult(PASSED, "No error states to check.") ;
	
}


void TestGetNodeCount_RISK()
{
	int				i ;
	
	gTestLevel = HIGH_RISK ;
	BeginTest( BAT, gTestName) ;
	LogDiagnostic( 12,"        Testing GetNodeCount(NULL) ;") ;
	i = (int)GetNodeCount(NULL) ;
	if (i>0) {
		LogDiagnostic( 12,"        GetNodeCount(NULL) should return 0 or an error!") ;
		LogResult(FAILED, "GetNodeCount(NULL) doesn't return an error!") ;
		goto TGNC_HR_cleanup ;
	}
	LogDiagnostic( 12,"        Testing GetNodeCount(0x654321) ;") ;
	i = (int)GetNodeCount((List*)kInvalidPtr) ;
	if (i>0) {
		LogDiagnostic( 12,"        GetNodeCount(0x654321) should return 0 or an error!") ;
		LogResult(FAILED, "GetNodeCount(0x654321) doesn't return an error!") ;
		goto TGNC_HR_cleanup ;
	}
	LogResult(PASSED, "") ;
TGNC_HR_cleanup:
;	
}


/* ---- GetNodePosFromHead()	*/

void TestGetNodePosFromHead()
{
	strcpy(gTestName, "GetNodePosFromHead()") ;

	if (gDoBAT)				TestGetNodePosFromHead_BAT() ;
	if (gDoEST)				TestGetNodePosFromHead_EST() ;
	if (gDoHIGH_RISK)		TestGetNodePosFromHead_RISK() ;
}


void TestGetNodePosFromHead_BAT()
{
	int			i ;
	NickNode*	oneNode ;
	Boolean		allWorked ;
	
	InitList(&gNickList, "TGNPFH List") ;
	gTestLevel = BAT ;
	BeginTest( BAT, gTestName) ;
	for (i=0 ; i<kNameCount ; i++) {
		oneNode = (NickNode*) AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode==NULL) {
			LogResult(UNRESOLVED, "AllocMem failed for a NickNode") ;
			goto TGNPFH_BAT_cleanup ;
		}
		strcpy((char*)oneNode->name, (char*)gNames[i]) ;
		AddTail(&gNickList, (Node*)oneNode) ;
	}
	oneNode = (NickNode*)FirstNode(&gNickList) ;
	allWorked = TRUE ;
	for (i=0 ; i<kNameCount ; i++) {
		if ( !IsNode(&gNickList, (Node*)oneNode) ) {
			LogResult(UNRESOLVED, "One of the nodes was invalid!") ;
			oneNode = NULL ;
			goto TGNPFH_BAT_cleanup ;
		}
		if ( GetNodePosFromHead(&gNickList, (Node*)oneNode) != i ) {
			allWorked = FALSE ;
		}
		oneNode = (NickNode*)NextNode((Node*)oneNode) ;
	}
	oneNode = NULL ;
	if (allWorked)
		LogResult(PASSED, "") ;
	else
		LogResult(FAILED, "One of the positions seems to be wrong!") ;
TGNPFH_BAT_cleanup:
	
	if (oneNode)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
}


void TestGetNodePosFromHead_EST()
{
	int			i ;
	NickNode*	oneNode ;
	
	InitList(&gNickList, "TGNPFH List") ;
	gTestLevel = EST ;
	BeginTest( BAT, gTestName) ;
	oneNode = (NickNode*) AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	/* Check if oneNode comes out OK when there are no nodes in the list */
	if (oneNode==NULL) {
		LogResult(UNRESOLVED, "AllocMem failed for a NickNode") ;
		goto TGNPFH_EST_cleanup ;
	}
	if ( GetNodePosFromHead(&gNickList, (Node*)oneNode) != -1 ) {
		LogResult(FAILED,"A node was found in an empty list!") ;
		goto TGNPFH_EST_cleanup ;
	}
	FreeMem(oneNode, sizeof(NickNode)) ;
	/*  Now fill the list, then check a node not in the list*/
	for (i=0 ; i<kNameCount ; i++) {
		oneNode = (NickNode*) AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode==NULL) {
			LogResult(UNRESOLVED, "AllocMem failed for a NickNode") ;
			goto TGNPFH_EST_cleanup ;
		}
		strcpy((char*)oneNode->name, (char*)gNames[i]) ;
		AddTail(&gNickList, (Node*)oneNode) ;
	}
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	if (oneNode==NULL) {
		LogResult(UNRESOLVED, "AllocMem failed for a NickNode") ;
		goto TGNPFH_EST_cleanup ;
	}
	if ( GetNodePosFromHead(&gNickList, (Node*)oneNode) != -1 ) {
		LogResult(FAILED,"A node was found in a full list!") ;
		goto TGNPFH_EST_cleanup ;
	}
	LogResult(PASSED, "") ;
TGNPFH_EST_cleanup:
	
	if (oneNode)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
}


void TestGetNodePosFromHead_RISK()
{
	InitList(&gNickList, "TGNPFH List") ;
	gTestLevel = HIGH_RISK ;
	BeginTest( BAT, gTestName) ;
	LogResult(UNRESOLVED, "not done yet...") ;
	
}


/* ---- GetNodePosFromTail()	*/

void TestGetNodePosFromTail()
{
	strcpy(gTestName, "GetNodePosFromTail()") ;
	
	if (gDoBAT)				TestGetNodePosFromTail_BAT() ;
	if (gDoEST)				TestGetNodePosFromTail_EST() ;
	if (gDoHIGH_RISK)		TestGetNodePosFromTail_RISK() ;
}


void TestGetNodePosFromTail_BAT()
{
	int			i ;
	NickNode*	oneNode ;
	Boolean		allWorked ;
	
	InitList(&gNickList, "TGNPFT List") ;
	gTestLevel = BAT ;
	BeginTest( BAT, gTestName) ;
	for (i=0 ; i<kNameCount ; i++) {
		oneNode = (NickNode*) AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode==NULL) {
			LogResult(UNRESOLVED, "AllocMem failed for a NickNode") ;
			goto TGNPFT_BAT_cleanup ;
		}
		strcpy((char*)oneNode->name, (char*)gNames[i]) ;
		AddTail(&gNickList, (Node*)oneNode) ;
	}
	oneNode = (NickNode*)FirstNode(&gNickList) ;
	allWorked = TRUE ;
	for (i=0 ; i<kNameCount ; i++) {
		if ( !IsNode(&gNickList, (Node*)oneNode) ) {
			LogResult(UNRESOLVED, "One of the nodes was invalid!") ;
			oneNode = NULL ;
			goto TGNPFT_BAT_cleanup ;
		}
		if ( (int)GetNodePosFromTail(&gNickList, (Node*)oneNode) != (kNameCount-i-1) ) {
			allWorked = FALSE ;
		}
		oneNode = (NickNode*)NextNode((Node*)oneNode) ;
	}
	oneNode = NULL ;
	if (allWorked)
		LogResult(PASSED, "") ;
	else
		LogResult(FAILED, "One of the positions seems to be wrong!") ;
TGNPFT_BAT_cleanup:
	
	if (oneNode)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
}


void TestGetNodePosFromTail_EST()
{
	int			i ;
	NickNode*	oneNode ;
	
	InitList(&gNickList, "TGNPFT List") ;
	gTestLevel = EST ;
	BeginTest( BAT, gTestName) ;
	oneNode = (NickNode*) AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	/* Check if oneNode comes out OK when there are no nodes in the list	*/
	if (oneNode==NULL) {
		LogResult(UNRESOLVED, "AllocMem failed for a NickNode") ;
		goto TGNPFT_EST_cleanup ;
	}
	if ( GetNodePosFromTail(&gNickList, (Node*)oneNode) != -1 ) {
		LogResult(FAILED,"A node was found in an empty list!") ;
		goto TGNPFT_EST_cleanup ;
	}
	FreeMem(oneNode, sizeof(NickNode)) ;
	/* Now fill the list, then check a node not in the list	*/
	for (i=0 ; i<kNameCount ; i++) {
		oneNode = (NickNode*) AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode==NULL) {
			LogResult(UNRESOLVED, "AllocMem failed for a NickNode") ;
			goto TGNPFT_EST_cleanup ;
		}
		strcpy((char*)oneNode->name, (char*)gNames[i]) ;
		AddTail(&gNickList, (Node*)oneNode) ;
	}
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	if (oneNode==NULL) {
		LogResult(UNRESOLVED, "AllocMem failed for a NickNode") ;
		goto TGNPFT_EST_cleanup ;
	}
	if ( GetNodePosFromTail(&gNickList, (Node*)oneNode) != -1 ) {
		LogResult(FAILED,"A node was found in a full list!") ;
		goto TGNPFT_EST_cleanup ;
	}
	LogResult(PASSED, "") ;
TGNPFT_EST_cleanup:
	
	if (oneNode)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
}


void TestGetNodePosFromTail_RISK()
{
	gTestLevel = HIGH_RISK ;
	BeginTest( BAT, gTestName) ;
	LogResult(UNRESOLVED, "not done yet...") ;
	
}


/* ---- InsertNodeAfter()	*/

void TestInsertNodeAfter()
{
	strcpy(gTestName, "InsertNodeAfter()") ;
	
	if (gDoBAT)				TestInsertNodeAfter_BAT() ;
	if (gDoEST)				TestInsertNodeAfter_EST() ;
	if (gDoHIGH_RISK)		TestInsertNodeAfter_RISK() ;
}


void TestInsertNodeAfter_BAT()
{
	NickNode*		oneNode ;
	NickNode*		foundNode ;
	Boolean			allWorked ;
	
	gTestLevel = BAT ;
	BeginTest( BAT, gTestName) ;
	InitList(&gNickList, "TINA_BAT List") ;
	/* Build a list of 3 nodes */
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	if (oneNode == NULL) {
		LogResult(UNRESOLVED, "AllocMem couldn't make a NickNode!") ;
		goto TINA_BAT_cleanup ;
	}
	strcpy(oneNode->name, gNames[0]) ;
	AddTail(&gNickList, (Node*)oneNode) ;
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	if (oneNode == NULL) {
		LogResult(UNRESOLVED, "AllocMem couldn't make a NickNode!") ;
		goto TINA_BAT_cleanup ;
	}
	strcpy(oneNode->name, gNames[1]) ;
	AddTail(&gNickList, (Node*)oneNode) ;
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	if (oneNode == NULL) {
		LogResult(UNRESOLVED, "AllocMem couldn't make a NickNode!") ;
		goto TINA_BAT_cleanup ;
	}
	strcpy(oneNode->name, gNames[2]) ;
	AddTail(&gNickList, (Node*)oneNode) ;
	/* Now, find the 2nd of the 3 nodes */
	foundNode = (NickNode*) FindNodeFromHead(&gNickList, 1) ;
	if (strcmp(foundNode->name, gNames[1]) != 0) {
		LogResult(UNRESOLVED, "AddTail() or FindNodeFromHead() seems to have failed!") ;
		goto TINA_BAT_cleanup ;
	}
	/* Make the new node to insert */
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	strcpy(oneNode->name, gNames[3]) ;
	InsertNodeAfter((Node*) foundNode, (Node*) oneNode) ;
	/* Now, check the list to see if all worked.  The order of the names should be 0,1,3,2 */
	allWorked = TRUE ;
	oneNode = (NickNode*)FirstNode(&gNickList) ;
	if (strcmp(oneNode->name, gNames[0]) != 0)		allWorked = FALSE ;
	oneNode = (NickNode*) NextNode((Node*) oneNode) ;
	if (strcmp(oneNode->name, gNames[1]) != 0)		allWorked = FALSE ;
	oneNode = (NickNode*) NextNode((Node*) oneNode) ;
	if (strcmp(oneNode->name, gNames[3]) != 0)		allWorked = FALSE ;
	oneNode = (NickNode*) NextNode((Node*) oneNode) ;
	if (strcmp(oneNode->name, gNames[2]) != 0)		allWorked = FALSE ;
	if (allWorked)
		LogResult(PASSED, "") ;
	else
		LogResult(FAILED, "The node names didn't match the expected order!") ;
	oneNode = NULL ;
TINA_BAT_cleanup:
	
	if (oneNode != NULL)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
}


void TestInsertNodeAfter_EST()
{
	gTestLevel = EST ;
	BeginTest( BAT, gTestName) ;
	LogResult(PASSED, "No error states to check.") ;
	
}


void TestInsertNodeAfter_RISK()
{
	gTestLevel = HIGH_RISK ;
	BeginTest( BAT, gTestName) ;
	LogResult(UNRESOLVED, "not done yet...") ;
	
}


/* ---- InsertNodeBefore()  */
 
void TestInsertNodeBefore()
{
	strcpy(gTestName, "InsertNodeBefore()") ;
	
	if (gDoBAT)				TestInsertNodeBefore_BAT() ;
	if (gDoEST)				TestInsertNodeBefore_EST() ;
	if (gDoHIGH_RISK)		TestInsertNodeBefore_RISK() ;
}


void TestInsertNodeBefore_BAT()
{
	NickNode*		oneNode ;
	NickNode*		foundNode ;
	Boolean			allWorked ;
	
	gTestLevel = BAT ;
	BeginTest( BAT, gTestName) ;
	InitList(&gNickList, "TINB_BAT List") ;
	/* Build a list of 3 nodes */
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	if (oneNode == NULL) {
		LogResult(UNRESOLVED, "AllocMem couldn't make a NickNode!") ;
		goto TINB_BAT_cleanup ;
	}
	strcpy(oneNode->name, gNames[0]) ;
	AddTail(&gNickList, (Node*)oneNode) ;
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	if (oneNode == NULL) {
		LogResult(UNRESOLVED, "AllocMem couldn't make a NickNode!") ;
		goto TINB_BAT_cleanup ;
	}
	strcpy(oneNode->name, gNames[1]) ;
	AddTail(&gNickList, (Node*)oneNode) ;
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	if (oneNode == NULL) {
		LogResult(UNRESOLVED, "AllocMem couldn't make a NickNode!") ;
		goto TINB_BAT_cleanup ;
	}
	strcpy(oneNode->name, gNames[2]) ;
	AddTail(&gNickList, (Node*)oneNode) ;
	/* Now, find the 2nd of the 3 nodes */
	foundNode = (NickNode*) FindNodeFromHead(&gNickList, 1) ;
	if (strcmp(foundNode->name, gNames[1]) != 0) {
		LogResult(UNRESOLVED, "AddTail() or FindNodeFromHead() seems to have failed!") ;
		goto TINB_BAT_cleanup ;
	}
	/* Make the new node to insert */
	oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
	strcpy(oneNode->name, gNames[3]) ;
	InsertNodeBefore((Node*) foundNode, (Node*) oneNode) ;
	/* Now, check the list to see if all worked.  The order of the names should be 0,3,1,2 */
	allWorked = TRUE ;
	oneNode = (NickNode*)FirstNode(&gNickList) ;
	if (strcmp(oneNode->name, gNames[0]) != 0)		allWorked = FALSE ;
	oneNode = (NickNode*) NextNode((Node*) oneNode) ;
	if (strcmp(oneNode->name, gNames[3]) != 0)		allWorked = FALSE ;
	oneNode = (NickNode*) NextNode((Node*) oneNode) ;
	if (strcmp(oneNode->name, gNames[1]) != 0)		allWorked = FALSE ;
	oneNode = (NickNode*) NextNode((Node*) oneNode) ;
	if (strcmp(oneNode->name, gNames[2]) != 0)		allWorked = FALSE ;
	if (allWorked)
		LogResult(PASSED, "") ;
	else
		LogResult(FAILED, "The node names didn't match the expected order!") ;
	oneNode = NULL ;
TINB_BAT_cleanup:
	
	if (oneNode != NULL)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
}


void TestInsertNodeBefore_EST()
{
	gTestLevel = EST ;
	BeginTest( BAT, gTestName) ;
	LogResult(PASSED, "No error states to check.") ;
	
}


void TestInsertNodeBefore_RISK()
{
	gTestLevel = HIGH_RISK ;
	BeginTest( BAT, gTestName) ;
	LogResult(UNRESOLVED, "not done yet...") ;
	
}


/* ---- ScanList()	*/

void TestScanList()
{
	strcpy(gTestName, "TestScanList") ;
	
	if (gDoBAT)				TestScanList_BAT() ;
	if (gDoEST)				TestScanList_EST() ;
	if (gDoHIGH_RISK)		TestScanList_RISK() ;
}


void TestScanList_BAT()
{
	int			i ;
	NickNode*	oneNode ;
	Boolean		allWorked = TRUE ;
	
	gTestLevel = BAT ;
	BeginTest( BAT, gTestName) ;
	InitList(&gNickList, "ScanList List") ;
	for (i=0 ; i<kNameCount ; i++) {
		oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode == NULL) {
			LogResult(UNRESOLVED, "AllocMem couldn't make a NickNode!") ;
			goto TSL_BAT_cleanup ;
		}
		strcpy(oneNode->name, gNames[i]) ;
		AddTail(&gNickList, (Node*)oneNode) ;
	}
	oneNode = NULL ;
	i = 0 ;
	ScanList(&gNickList, oneNode, NickNode) {
		if (strcmp(oneNode->name, gNames[i]) != 0) {
			allWorked = FALSE ;
			break ;
		}
		i++ ;
	}
	oneNode = NULL ;
	if (allWorked)
		LogResult(PASSED, "") ;
	else
		LogResult(FAILED, "The node names didn't match the expected order!") ;
TSL_BAT_cleanup:
	if (oneNode != NULL)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
	
}


void TestScanList_EST()
{
	gTestLevel = EST ;
	BeginTest( BAT, gTestName) ;
	LogResult(PASSED, "No error states to check.") ;
	
}


void TestScanList_RISK()
{
	gTestLevel = HIGH_RISK ;
	BeginTest( BAT, gTestName) ;
	LogResult(UNRESOLVED, "not done yet...") ;
	
}


/* ---- ScanListB	*/

void TestScanListB()
{
	strcpy(gTestName, "TestScanListB") ;
	
	if (gDoBAT)				TestScanListB_BAT() ;
	if (gDoEST)				TestScanListB_EST() ;
	if (gDoHIGH_RISK)		TestScanListB_RISK() ;
}


void TestScanListB_BAT()
{
	int			i ;
	NickNode*	oneNode ;
	Boolean		allWorked = TRUE ;
	
	gTestLevel = BAT ;
	BeginTest( BAT, gTestName) ;
	InitList(&gNickList, "ScanList List") ;
	for (i=0 ; i<kNameCount ; i++) {
		oneNode = (NickNode*)AllocMem(sizeof(NickNode), MEMTYPE_NORMAL) ;
		if (oneNode == NULL) {
			LogResult(UNRESOLVED, "AllocMem couldn't make a NickNode!") ;
			goto TSLB_BAT_cleanup ;
		}
		strcpy(oneNode->name, gNames[i]) ;
		AddTail(&gNickList, (Node*)oneNode) ;
	}
	oneNode = NULL ;
	i = kNameCount-1 ;
	ScanListB(&gNickList, oneNode, NickNode) {
		if (strcmp(oneNode->name, gNames[i]) != 0) {
			allWorked = FALSE ;
			break ;
		}
		i-- ;
	}
	oneNode = NULL ;
	if (allWorked)
		LogResult(PASSED, "") ;
	else
		LogResult(FAILED, "The node names didn't match the expected order!") ;
TSLB_BAT_cleanup:
	if (oneNode != NULL)
		FreeMem(oneNode, sizeof(NickNode)) ;
	ClearList(&gNickList) ;
	
}


void TestScanListB_EST()
{
	gTestLevel = EST ;
	BeginTest( BAT, gTestName) ;
	LogResult(PASSED, "No error states to check.") ;
	
}


void TestScanListB_RISK()
{
	gTestLevel = HIGH_RISK ;
	BeginTest( BAT, gTestName) ;
	LogResult(UNRESOLVED, "not done yet...") ;
	
}


/* ---- Misc. Utilities	*/

void ClearList(List* theList)
{
	while (GetNodeCount(theList) != 0)
		RemHead(theList) ;
}


