/* @(#) scaf_MemRation.c 96/07/11 1.1 */
#include <kernel/debug.h>
#include <kernel/mem.h>
#include <kernel/types.h>
#include <stdlib.h>
#include <string.h>

#define	ErrReport		kprintf
#define	ErrNumReport	PrintfSysErr
#define	Report			kprintf

static Boolean	gScafMemRationInited = FALSE;

/* ------------------------------------------------------------------------------- */
/* returns true if the character is a digit -- i.e  ch is an element of [0..9] */
Boolean IsDigit (char ch)
{
	if ( (ch >= '0') && (ch <= '9') )
		return TRUE;
		
	return FALSE;
}

/* ------------------------------------------------------------------------------- */
/* returns the integer value of the numeric string starting at indx in strin st */
/* advance st to the first non disit character after the number */

uint32	GetNumStr(	char *st,	uint32 *indx )
{
	char	numStr[64] = '\0';
	uint32	n = 0;
	
	while( IsDigit( st[*indx] ) )		/* put the next digits in a string */
		numStr[n++] = st[(*indx)++];
		
	numStr[n] = '\0';					/* terminate the string */	
	
	return ( atoi( numStr ) );			/* convert it to an integer and return it */
}
		
	

/* ------------------------------------------------------------------------------- */
/*	startup memory rationing based on the options in the option string */

void	StartMemRation( char *optionStr )
{
	Err		err;
	uint32	len;
	TagArg	tag[20];
	uint32	n = 0;
	uint32	indx = 0;
	
	Report( "### Start Memory Rationing -- option string = \"%s\"\n", optionStr );
	
	tag[indx].ta_Tag = RATIONMEMDEBUG_TAG_ACTIVE;		/* enable memory rationing */
	tag[indx++].ta_Arg = (void *)TRUE;

	len = strlen( optionStr );
	n = 0;
	while (n < len)
	{
		switch ( optionStr[n] )	{							/* set up the option flags based on the option string */
		
			case 'L':		tag[indx].ta_Tag = RATIONMEMDEBUG_TAG_MINSIZE;
							n++;
							tag[indx++].ta_Arg = (void *)GetNumStr( optionStr, &n );
							break;
							
			case 'M':		tag[indx].ta_Tag = RATIONMEMDEBUG_TAG_MAXSIZE;
							n++;
							tag[indx++].ta_Arg = (void *)GetNumStr( optionStr, &n );
							break;
							
			case 'C':		tag[indx].ta_Tag = RATIONMEMDEBUG_TAG_COUNTDOWN;
							n++;
							tag[indx++].ta_Arg = (void *)GetNumStr( optionStr, &n );
							break;
							
			case 'I':		tag[indx].ta_Tag = RATIONMEMDEBUG_TAG_INTERVAL;
							n++;
							tag[indx++].ta_Arg = (void *)GetNumStr( optionStr, &n );
							break;
							
			case 'R':		tag[indx].ta_Tag = RATIONMEMDEBUG_TAG_RANDOM;
							tag[indx++].ta_Arg = (void *)TRUE;
							n++;
							break;
							
			case 'V':		tag[indx].ta_Tag = RATIONMEMDEBUG_TAG_VERBOSE;
							tag[indx++].ta_Arg = (void *)TRUE;
							n++;
							break;
							
			case 'B':		tag[indx].ta_Tag = RATIONMEMDEBUG_TAG_BREAKPOINT_ON_RATIONING;
							tag[indx++].ta_Arg = (void *)TRUE;
							n++;
							break;
							
			case 'S':		tag[indx].ta_Tag = RATIONMEMDEBUG_TAG_SUPER;
							tag[indx++].ta_Arg = (void *)TRUE;
							n++;
							break;
							
								
		}	
	}
		
	tag[indx].ta_Tag =	TAG_END;						/* don't forget to terminate the tag array */
	tag[indx].ta_Arg = 	0;


	if ( (err = RationMemDebug( tag )) < 0 )			/* setup MemDebug options */
	{
		ErrNumReport( err );
		ErrReport( "Error $%x calling RationMemDebug(\"%s\")\n", optionStr);
		return;
	}
	gScafMemRationInited = TRUE;
}


	
	
/* ------------------------------------------------------------------------------- */
/*	terminates memory rationing */

void	TermMemRation( void )
{
	TagArg	tag[2];
	Err		err;
	
	if ( !gScafMemRationInited )	return;			/* don't do if we didn't initialize the memRation stuff */
	gScafMemRationInited = FALSE;
	Report( "### Terminate Memory Rationing\n" );

	tag[0].ta_Tag = RATIONMEMDEBUG_TAG_ACTIVE;		/* disable memory rationing */
	tag[0].ta_Arg = (void *)FALSE;

	tag[1].ta_Tag = TAG_END;						/* terminate tag arg array */
	tag[1].ta_Arg = (void *)0;

	if ( (err = RationMemDebug( tag )) < 0 )		/* Terminate MemDebug options */
	{
		ErrNumReport( err );
		ErrReport( "Error $%x calling RationMemDebug in TermMemRation\n");
		return;
	}

}
