/*
	File:		tests.c

	Contains:	contains tests for DSAPISuite

	Written by:	amasri

	Copyright:	 1996 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <3>	  5/2/96	PUT YOUR INITIALS HERE		add more test cases

	To Do:

 @(#) tests.c 96/07/11 1.2

*/


#include <string.h>				/* memset */
#include <kernel/debug.h>
#include <kernel/mem.h>			/* for AllocMemTrack, FreeMemTrack   */
#include <audio/audio.h>
#include <graphics/view.h>		/* VIEWTAG_BITMAP */

#include <streaming/datastreamlib.h>
#include <streaming/dsstreamdefs.h>
#include <streaming/satemplatedefs.h>
#include <streaming/sacontrolmsgs.h>

#include "testvideostream.h"
#include <tests/3do_scaf.h>
#include "joypad.h"
#include "tests.h"

/* Internal routines */

bool FillMemPool(void* argValue, MemPoolEntryPtr poolEntry);

/*global variables*/

extern char* sTestName;

bool FillMemPool(void* argValue, MemPoolEntryPtr poolEntry){
	TOUCH(argValue);
	poolEntry->data[0] = (uint32)(poolEntry->next);
	return TRUE;
}


/*
 * DoMemPoolTest: create a mempool with the call,
 * CreateMemPool with options. Call EnumerateMemPoolEntries, 
 * AllocPoolMem, ReturnPoolMem, and DeletePoolMem.
 */

Err DoMemPoolTest(void){
	Err				status = kDSNoErr;
	MemPoolPtr 		pool;
	int32			iNumEntries = 20;
	int32			iSizeEntry = 8;
	uint32			fOptions = (MEMTYPE_DMA | MEMTYPE_FILL);
	bool			fForEachFunc;
	MemPoolEntry*	entryPtr;
	
	sTestName = "CreateMemPoolWithOptions";
	BeginTest(FT, sTestName);
	pool = CreateMemPoolWithOptions( iNumEntries, iSizeEntry, fOptions);
	if (pool == NULL){
		LogResult( FAILED, "%s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
	else {
		LogResult( PASSED, "%s passes", sTestName);
	}
	sTestName = "EnumerateMemPoolEntries";
	BeginTest(FT, sTestName);
	fForEachFunc = EnumerateMemPoolEntries(pool,(ForEachPoolMemberFuncPtr)(&FillMemPool), NULL);
	if (!fForEachFunc){
		LogResult( FAILED, "%s fails", sTestName );
		goto FAILED;
	}
	else {
		LogResult( PASSED, "%s passes", sTestName);
	}
	sTestName = "AllocPoolMem";
	BeginTest(FT, sTestName);
	entryPtr = AllocPoolMem(pool);
	if (entryPtr==NULL){
		LogResult( FAILED, "%s fails with error %d", sTestName, (int)status);
		status = (Err)entryPtr;
		TOUCH(status);
		goto FAILED;
		goto FAILED;
	}
	else {
		LogResult( PASSED, "%s passes", sTestName);
	}
	sTestName = "ReturnPoolMem";
	BeginTest(FT, sTestName);
	ReturnPoolMem(pool,entryPtr);
	fForEachFunc = ForEachFreePoolMember(pool, (ForEachFreePoolMemberFuncPtr)&FillMemPool, NULL);
	if (!fForEachFunc){
		LogResult( FAILED, "%s fails", sTestName);
		status = kDSNoErr;
		goto FAILED;
	}
	else {
		LogResult( PASSED, "%s passes", sTestName);
	}
	sTestName = "DeleteMemPool";
	BeginTest(FT, sTestName);
	DeleteMemPool(pool);
	LogResult( PASSED, "%s executes", sTestName);
	status = kDSNoErr;
FAILED:
	return status;
}

/*
 * DoClockTest: Set the presentation clock, then get it, then
 * restore the original setting.
 */
Err DoClockTest(PlayerPtr ctx){
 
	uint32		iBranch = 12;
	uint32 		iTime = 5000;
	DSClock		oldClock, testClock;
	Err			status = kDSNoErr;
	bool		bResult;
 
	sTestName = "DSGetPresentationClock";
	BeginTest(FT, sTestName);
	DSGetPresentationClock(ctx->streamCBPtr, &oldClock);
	LogResult( PASSED, "%s executes without error", sTestName);
	sTestName = "DSSetPresentationClock";
	BeginTest(FT, sTestName);
	DSSetPresentationClock(ctx->streamCBPtr, iBranch, iTime);
	LogResult( PASSED, "%s executes without error", sTestName);
	DSGetPresentationClock(ctx->streamCBPtr, &testClock);
	sTestName = "DSClockLT";
	BeginTest(FT, sTestName);
	bResult = DSClockLT( oldClock.branchNumber, oldClock.streamTime, testClock.branchNumber, testClock.streamTime );
	if (bResult == FALSE){
		LogResult( FAILED, "%s fails", sTestName );
		goto FAILED;
	}
	else {
		LogResult( PASSED, "%s passes", sTestName);
	}
	DSSetPresentationClock(ctx->streamCBPtr, oldClock.branchNumber, oldClock.streamTime);
	DSGetPresentationClock(ctx->streamCBPtr, &testClock);
	sTestName = "DSClockLE";
	BeginTest(FT, sTestName);
	bResult = DSClockLE( oldClock.branchNumber, oldClock.streamTime, testClock.branchNumber, testClock.streamTime );
	if (bResult == FALSE){
		LogResult( FAILED, "%s fails", sTestName );
		goto FAILED;
	}
	else {
		LogResult( PASSED, "%s passes", sTestName);
	}
FAILED:
	return status;
}

int32 DoEZFlixCurrTimeTest(EZFlixRecPtr channel){
	Err status = kDSNoErr;
	int32 time;
	sTestName = "EZFlixCurrTime";
	BeginTest(FT, sTestName);
	time = EZFlixCurrTime (channel);
	if (time <= 0){
		LogResult( FAILED, "%s returns time value %d", sTestName, (int)time);
		goto FAILED;
	}
	else {
		LogResult( PASSED, "%s passes", sTestName);
	}
FAILED:
	return status;
}

int32 DoEZFlixDurationTest(EZFlixRecPtr channel){
	int32 duration;
	Err status = kDSNoErr;
	sTestName = "EZFlixDuration";
	BeginTest(FT, sTestName);
	duration = EZFlixDuration(channel);
	if (duration <= 0){
		LogResult( FAILED, "%s returns duration value %d", sTestName, (int)duration);
		goto FAILED;
	}
	else {
		LogResult( PASSED, "%s passes", sTestName);
	}
FAILED:
	return status;
}

Err DoChannelMaskTest(PlayerPtr ctx){
	/*Now test whether the channel is actually enabled*/
	int32 	status;
	uint32	channel;

	sTestName = "DSGetChannel";
	BeginTest(FT, sTestName);
	status = DSGetChannel(ctx->messageItem, NULL, ctx->streamCBPtr,
		SNDS_CHUNK_TYPE, 1, &channel);
	if ( status < 0 ) {
		LogResult( FAILED, "%s returns channel value %d", sTestName, (int)status);
		goto FAILED;
	}
	else {
		LogResult( PASSED, "%s passes", sTestName);
	}
FAILED:
	return 0;
}


 

 	
	
	
	
