/*
	File:		testvideostream.h

	Contains:	datastreaming test application, based on VideoPlayer

	Written by:	amasri

	Copyright:	 1996 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		<3+>	 3/27/96	am		remove controlsubscriber.h
		 <3>	 3/26/96	am		add 3do_scaf.h

	To Do:

 @(#) testvideostream.h 96/07/11 1.2

*/


#ifndef	_TESTSTREAM_H_
#define	_TESTSTREAM_H_

#include <kernel/types.h>
#include <graphics/graphics.h>
#include <graphics/bitmap.h>

#include <streaming/datastream.h>
#include <streaming/saudiosubscriber.h>
#include <streaming/mpegvideosubscriber.h>
#include <streaming/threadhelper.h>
#include <streaming/mpegbufferqueues.h>
#include <streaming/ezflixsubscriber.h>
#include <streaming/preparestream.h>
#include <streaming/dserror.h>
#include <streaming/subscribertraceutils.h>

#include <tests/3do_scaf.h>

enum { cntFrameBuffers = 2 };

enum { kTraceOpDoConnect = 4001,
	   kTraceOpDoDisconnect = 4002,
	   kTraceOpGetData = 4003,
	   kTraceOpGoMarker = 4004 };

typedef struct ScreenInfo {
	Item		bitmapItem[cntFrameBuffers];		/* one Bitmap Item per display buffer */
	Bitmap		*bitmap[cntFrameBuffers];			/* bitmap[i]->bm_Buffer points to the first pixel in the buffer */
	void		*allocatedBuffer[cntFrameBuffers];	/* for deallocation; may include padding on each end */
	Item		viewItem;							/* the View Item to display */
	uint32		renderSignal;						/* this signal tells us when a bitmap has reached the display */
	} ScreenInfo;

/* kUserFunctionPeriod defines how often the user function gets called to check for such things
 * as control pad input. This is in units of audio ticks (approx 240 Hz). 60 Hz polling is nice. */
#define kUserFunctionPeriod (240/60)
typedef struct Player	Player, *PlayerPtr;

#define ReleaseNumber "M2_3.0_95.05.11"

/*typedef int32 (*PlayVideoUserFn)(void *ctx, void *userContext);*/
typedef int32 (*PlayVideoUserFn)(PlayerPtr ctx);

/**************************************/
/* Internal player context descriptor */
/**************************************/
typedef struct Player {
	PlayVideoUserFn			userFn;					/* user callback function pointer */
	void					*userContext;			/* for use by user callback function */
	Item					userCue;				/* Cue for periodic wake ups to check user function */
	uint32					userCueSignal;


	DSHeaderChunk			hdr;					/* Copy of stream header from stream file */

	DSDataBufPtr			bufferList;				/* ptr to linked list of buffers used by streamer */
	Item					acqMsgPort;				/* data acquisition request msg port */
	DSStreamCBPtr			streamCBPtr;			/* ptr to stream thread's context */

	ScreenInfo				*screenInfoPtr;			/* for drawing to the screen */

	Item					messagePort;			/* port for receiving end of stream message */
	uint32					messagePortSignal;

	Item					messageItem;			/* msg item for sending streamer requests */
	Item					endOfStreamMessageItem; /* msg item that is replied to as end of stream */

	Item					frameCompletionPort;	/* The port for recieving frame completion messages */
	uint32					frameCompletionPortSignal;

	Item					lastFrameMsg;			/* a reference to the last message we recieved.  This is
													 * used to reply to the message once a new frame has been
													 * put up on the screen */

	Item					mpegVideoSubMsgPort;	/* MPEGVideo subscriber request message port */
	Item					synchCue;				/* Audio Cue Item for synchronization timing */
	uint32					synchCueSignal;
	MPEGBufferPtr			lastMPEGBufferPtr;		/* pointer to the last mpeg buffer we recieved */	
	int32					bitDepth;				/* Bitdepth of playback */
	EZFlixContextPtr		contextPtr;				/* EZFlix subscriber context ptr */
	EZFlixRecPtr			channelPtr;				/* EZFlix subscriber channel ptr */

	bool					streamHasAudio;			/* Audio present in the stream? */
	bool					fLoop;
} Player, *PlayerPtr;
	
	enum VideoType { MPEG, EZFX };

/*****************************/
/* Public routine prototypes */
/*****************************/

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


Err		TestVideoStream(ScreenInfo *screenInfoPtr, uint32 bitDepth,
			char *streamFileName, PlayVideoUserFn userFn, void *userContext, bool floop);
void	DismantlePlayer(PlayerPtr ctx);
void	Usage( void );
Item 	DNewDataAcq(char *fileName, int32 deltaPriority);
Err 	DDisposeDataAcq(Item dataAcqMsgPort);
Err		myUserFunction(PlayerPtr ctx);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif	/* _TESTSTREAM_H_ */
