/* @(#) testvideostream.c 96/07/24 1.1 */
/*
	File:		testvideostream.c

	Contains:	test dispatcher for dsbranchsuite.c

	Written by:	allan masri

	Copyright:	 1996 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <3>	 4/24/96	am		remove fLoop, automate branching tests

	To Do:
*/

/* Copyright (C) 1995, an unpublished work by The 3DO Company.*/
/* All rights reserved. This material contains confidential*/
/* information that is the property of The 3DO Company. Any*/
/* unauthorized duplication, disclosure or use is prohibited.*/
/* */
/* @(#) playvideostream.c 95/10/02 1.25 */
/* Distributed with Portfolio V28.0*/

#include <string.h>				/* memset */
#include <kernel/debug.h>
#include <kernel/mem.h>			/* for AllocMemTrack, FreeMemTrack   */
#include <audio/audio.h>
#include <graphics/view.h>		/* VIEWTAG_BITMAP */

#include <streaming/datastreamlib.h>
#include <streaming/datastream.h>
#include <streaming/dsstreamdefs.h>
#include <streaming/satemplatedefs.h>
#include <streaming/sacontrolmsgs.h>

#include "testvideostream.h"
#include "joypad.h"
#include "tests.h"
#include <tests/3do_scaf.h>



/************************************/
/* Constants and structures         */
/************************************/
#undef kDefaultNumBuffers					/* override the setting in preparestream.h */
#define kDefaultNumBuffers		6			/* suggested number of stream buffers to use */


/************************************/
/* Local utility routine prototypes */
/************************************/
static int32	InitVideoPlayerFromStreamHeader(PlayerPtr ctx, char* streamFileName, uint32 bitDepth);
static int32	HandleMPEGFrames(PlayerPtr ctx);
Err DisplayEZFX(PlayerPtr ctx);
Err DisplayMPEG(PlayerPtr ctx);


static uint32 gVideoType;
extern char* sTestName;

/* test flags to prevent tests being performed more than once */
bool fDrawEZflixToBuffer 	= FALSE;
bool fSendFreeEZFlixSignal 	= FALSE;
bool fABS_MIDGOP_SETUP 		= FALSE;
bool fABS_MIDGOP_TEST 		= FALSE;
bool fFORW_TIME_SETUP 		= FALSE;
bool fFORW_TIME_TEST 		= FALSE;
bool fBACK_TIME_SETUP		= FALSE;
bool fBACK_TIME_TEST		= FALSE;
bool fFORWARD_SETUP 		= FALSE;
bool fFORWARD_TEST 			= FALSE;
bool fBACKWARD_SETUP 		= FALSE;
bool fBACKWARD_TEST 		= FALSE;
bool fABSOLUTE_SETUP 		= FALSE;
bool fABSOLUTE_TEST 		= FALSE;
bool fABS_TIME_SETUP 		= FALSE;
bool fABS_TIME_TEST			= FALSE;


/*********************************************************************************************
 * This function is called each time through the player's main loop and can be used to
 * implement any user interface actions necessary. Returning a zero causes playback to
 * continue. Any other value causes playback to cease and immediately return from
 * the TestVideoStream() function.
 *********************************************************************************************/
/*********************************************************************************************
 * This function is called each time through the player's main loop and can be used to
 * implement any user interface actions necessary. Returning a zero causes playback to
 * continue. Any other value causes playback to cease and immediately return from
 * the PlayVideoStream() function.
 *********************************************************************************************/
int32 myUserFunction(PlayerPtr ctx)
	{
	int32			status = kDSNoErr;
	JoyPadStatePtr	jp;

	/* Our only additional context is a pointer to the joypad state. */
	jp = (JoyPadStatePtr) ctx->userContext;

	/* Read control pad #1. */
	GetJoyPad(jp, 1);


	if ( jp->xBtn )
		/* "X" aka Stop button ==> Return 1 to stop the stream */
		return 1;

	else if ( jp->upArrow )
		{
		/* Jump forward to the next marker in the stream. */
		status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, 1,
							GOMARKER_FORWARD);
		if ( status == kDSBranchNotDefined )
			{
			PERR(("Can't skip forward: The stream needs a marker table.\n"));
			status = kDSNoErr;
			}
		else if ( status == kDSRangeErr )
			{
			/* Skip forward range error. Beyond end? If we're looping, jump to
			 * the start of the next loop. */
			if ( ctx->fLoop )
				status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, 0,
					GOMARKER_ABSOLUTE);
			else
				{
				PERR(("Skip forward range error. Beyond end?\n"));
				status = kDSNoErr;
				}
			}
		FAIL_NEG("DSGoMarker fwd", status);
		}

	else if ( jp->downArrow )
		{
		/* Jump backward to the previous marker in the stream. */
		status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, 1,
							GOMARKER_BACKWARD);
		if ( status == kDSBranchNotDefined )
			{
			PERR(("Can't skip backward: The stream needs a marker table.\n"));
			status = kDSNoErr;
			}
		else if ( status == kDSRangeErr )
			/* Skip backward range error. Beyond start? Jump back to start. */
			status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, 0,
				GOMARKER_ABSOLUTE);
		FAIL_NEG("DSGoMarker bwd", status);
		}

	else if ( jp->leftArrow )
		{
		/* Jump to the beginning */
		/* For streamer testing: unshifted-leftArrow ==> ASAP flush branch (the
		 * normal way to branch in response to a user button push),
		 * shifted arrow => butt-joint branch. */
		status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, 0,
			( jp->rightShift | jp->leftShift ) ?
				GOMARKER_ABSOLUTE | GOMARKER_NO_FLUSH_FLAG : GOMARKER_ABSOLUTE);
		FAIL_NEG("DSGoMarker 0", status);
		}

	else if ( jp->startBtn )
		{
		/* Pause / Unpause the stream */
		if ( DSIsRunning(ctx->streamCBPtr) )
			{
			/* Note: Stop without SOPT_FLUSH so we can later resume. */
			status = DSStopStream(ctx->messageItem, NULL, ctx->streamCBPtr, SOPT_NOFLUSH);
			FAIL_NEG("DSStopStream", status);
			}
		else
			{
			status = DSStartStream(ctx->messageItem, NULL, ctx->streamCBPtr, SOPT_NOFLUSH);
			FAIL_NEG("DSStartStream", status);
			}
		}
	
	else if (( jp->aBtn ) && (gVideoType == MPEG)) {
		/* Execute branching tests. Execute a different test each time the a button
		 * is pressed. We use the flags to assure that each test is only performed
		 * once. Control drops through the if statements.
		 */
		if (!fABSOLUTE_SETUP & !fABSOLUTE_TEST){
			status = MPEG_GOMARKER_ABSOLUTE_SETUP(ctx);
			fABSOLUTE_SETUP = TRUE;
			goto FAILED;
		}
		if (!fBACKWARD_SETUP & !fBACKWARD_TEST){
			status = MPEG_GOMARKER_BACKWARD_SETUP(ctx);
			fBACKWARD_SETUP = TRUE;
			goto FAILED;
		}
		if (!fFORWARD_SETUP & !fFORWARD_TEST){
			status = MPEG_GOMARKER_FORWARD_SETUP(ctx);
			fFORWARD_SETUP = TRUE;
			goto FAILED;
		}
		if (!fABS_TIME_SETUP & !fABS_TIME_TEST){
			status = MPEG_GOMARKER_ABS_TIME_SETUP(ctx);
			fABS_TIME_SETUP = TRUE;
			goto FAILED;
		}
		if (!fBACK_TIME_SETUP & !fBACK_TIME_TEST){
			status = MPEG_GOMARKER_BACK_TIME_SETUP(ctx);
			fBACK_TIME_SETUP = TRUE;
			goto FAILED;
		}
		if (!fFORW_TIME_SETUP & !fFORW_TIME_TEST){
			status = MPEG_GOMARKER_FORW_TIME_SETUP(ctx);
			fFORW_TIME_SETUP = TRUE;
			goto FAILED;
		}
		if (!fABS_MIDGOP_SETUP & !fABS_MIDGOP_TEST){
			status = MPEG_GOMARKER_ABS_MIDGOP_SETUP(ctx);
			fABS_MIDGOP_SETUP = TRUE;
			goto FAILED;
		}
	}
	else if ( ( jp->bBtn ) && ( gVideoType == EZFX ) ){
		/* Execute branching tests. Execute a different test each time the a button
		 * is pressed. We use the flags to assure that each test is only performed
		 * once. Control drops through the if statements.
		 */
		if (!fABSOLUTE_SETUP & !fABSOLUTE_TEST){
			status = EZFX_GOMARKER_ABSOLUTE_SETUP(ctx);
			fABSOLUTE_SETUP = TRUE;
			goto FAILED;
		}
		if (!fBACKWARD_SETUP & !fBACKWARD_TEST){
			status = EZFX_GOMARKER_BACKWARD_SETUP(ctx);
			fBACKWARD_SETUP = TRUE;
			goto FAILED;
		}
		if (!fFORWARD_SETUP & !fFORWARD_TEST){
			status = EZFX_GOMARKER_FORWARD_SETUP(ctx);
			fFORWARD_SETUP = TRUE;
			goto FAILED;
		}
		if (!fABS_TIME_SETUP & !fABS_TIME_TEST){
			status = EZFX_GOMARKER_ABS_TIME_SETUP(ctx);
			fABS_TIME_SETUP = TRUE;
			goto FAILED;
		}
		if (!fBACK_TIME_SETUP & !fBACK_TIME_TEST){
			status = EZFX_GOMARKER_BACK_TIME_SETUP(ctx);
			fBACK_TIME_SETUP = TRUE;
			goto FAILED;
		}
		if (!fFORW_TIME_SETUP & !fFORW_TIME_TEST){
			status = EZFX_GOMARKER_FORW_TIME_SETUP(ctx);
			fFORW_TIME_SETUP = TRUE;
			goto FAILED;
		}
	}


FAILED:
	return status;
	}


Err DisplayMPEG(PlayerPtr ctx) {
	/* NOTE: A better scheme here would be to also wait for a control port event
	 * signal (rather than an audio Cue to poll the control port) and for a
	 * time-to-display-frame signal (rather than a subroutine that sleeps
	 * until time to display and sleeps until the display has begun). */
	int32			status = 0;
	int32			playerResult = 0;
	uint32			signalMask;
	uint32			currentSignal;
	uint32			frameCount = 0;
	DSRequestMsg	EOSMessage;
	Item			msgItem;
	bool			keepRunning = TRUE;
	
	/* Set up the mask for signals we're expecting */
	signalMask =	ctx->frameCompletionPortSignal |		/* Frame Completions */
					ctx->messagePortSignal |				/* End of stream replys */
					ctx->userCueSignal;						/* Periodic user events */
	while ( keepRunning ){
		currentSignal = WaitSignal(signalMask);
		
		/* If a user function is defined, then call it.
		 * If it returns a non-zero value, then exit. */
		if ( currentSignal & ctx->userCueSignal ){
			if ( ctx->userFn != NULL ){
				playerResult = (*ctx->userFn)(ctx);
				if ( playerResult )
					keepRunning = FALSE;
				else
					SignalAtTime(ctx->userCue,
						GetAudioTime() + kUserFunctionPeriod);
			}
		}
	
		if ( currentSignal & ctx->frameCompletionPortSignal ){
			status = HandleMPEGFrames(ctx);
			FAIL_NEG("HandleMPEGFrames", status);
		}
				
		/* Check for end-of-stream signal. */
		if ( currentSignal & ctx->messagePortSignal ){
			while ( (status = msgItem = GetMsg(ctx->messagePort)) != 0 ){
				FAIL_NEG("PlayVideoStream GetMsg", status);
				if ( msgItem == ctx->endOfStreamMessageItem ) {
					status = MESSAGE(msgItem)->msg_Result;
					if ( status == kDSSTOPChunk ){
						status = DSWaitEndOfStream(ctx->endOfStreamMessageItem,
							&EOSMessage, ctx->streamCBPtr);
						FAIL_NEG("DSWaitEndOfStream", status);

						APRNT(("Reached a stream STOP chunk. "
							"Press >/|| to resume or STOP to exit.\n"));
					}

					else if ( status < 0 ){
						ERROR_RESULT_STATUS("EOS reply", status); goto FAILED; 
					}

					else if ( ctx->fLoop ){
						status = DSWaitEndOfStream(ctx->endOfStreamMessageItem,
							&EOSMessage, ctx->streamCBPtr);
						FAIL_NEG("DSWaitEndOfStream", status);

						status = DSGoMarker(ctx->messageItem, NULL,
							ctx->streamCBPtr, 0,
							GOMARKER_ABSOLUTE | GOMARKER_NO_FLUSH_FLAG);
						FAIL_NEG("DSGoMarker 0", status);
					}

					else
						keepRunning = FALSE;
				}

				else
					PERR(("PlayVideoStream: dequeued an unexpected message from "
						"ctx->messagePort!\n"));
			}	/* while ( GetMsg() returns a Message Item or an error code ) */
		}
	frameCount++;
	}
	/* Cleanup; deallocate [NB: will be done elsewhere] */
	/*DismantlePlayer(ctx);*/

FAILED:
	return ( status < 0 ) ? status : playerResult;
}


Err DisplayEZFX(PlayerPtr ctx){
	int32			status;
	int32			playerResult = 0;
	Bitmap*			bitmap;
	uint32			time;
	uint32			frameCount;
	int32			drawingScreen;

	/* Make sure the first screen is displayed, and we're drawing into
	 * the second screen.
	 */


	status = ModifyGraphicsItemVA(ctx->screenInfoPtr->viewItem,
							VIEWTAG_BITMAP,
								ctx->screenInfoPtr->bitmapItem[0],
							TAG_END);

	if (status < 0) 
		goto FAILED;
	drawingScreen = 1;
	
	/* Get a pointer to the bitmap we will draw into */
	bitmap = ctx->screenInfoPtr->bitmap[ drawingScreen ];
	time = GetAudioTime();
	frameCount = 0;
	while ( TRUE )
	{
		/* If a user function is defined, then call it. If the user function
		 * returns a non-zero value, then exit. */
		if ( ctx->userFn )
		{
			playerResult = (*ctx->userFn)(ctx);
			if ( playerResult )
				break;
		}

		/* Exit if we received the end-of-stream msg reply.
		 * status < 0 is an error. status > 0 is our EOS-reply msg Item. */
		status = GetMsg(ctx->messagePort);
		FAIL_NEG("PlayEZFlixStream GetMsg", status);
		if ( status > 0 )
		{
			time = GetAudioTime() - time;
			TOUCH(time);	/* avert compiler warning of variable set but unused */
			break;
		}

		/* In case there's no EZFlix in this stream, skip
		 * trying to draw something that doesn't exist.
		 */
		if ( ctx->contextPtr != 0 ){
			/*
			 * Draw next EZFlix frame directly into the screen buffer.
			 */
			DrawEZFlixToBuffer( ctx->contextPtr, ctx->channelPtr, bitmap );	
			FAIL_NEG("DrawEZFlixToBuffer", status);
							
			/* Display the screen we just drew, and switch to drawing into
			 * the next screen.
			 */
			status = ModifyGraphicsItemVA(ctx->screenInfoPtr->viewItem,
						VIEWTAG_BITMAP,
							ctx->screenInfoPtr->bitmapItem[drawingScreen],
						TAG_END);
			FAIL_NEG("ModifyGraphicsItemVA", status);

			if (++drawingScreen >= cntFrameBuffers)
				drawingScreen = 0;
			bitmap = ctx->screenInfoPtr->bitmap[ drawingScreen ];

			if (fABSOLUTE_SETUP & !fABSOLUTE_TEST){
				status = EZFX_GOMARKER_ABSOLUTE_TEST(ctx);
				fABSOLUTE_TEST = TRUE;
			}
			if (fABS_TIME_SETUP & !fABS_TIME_TEST){
				status = EZFX_GOMARKER_ABS_TIME_TEST(ctx);
				fABS_TIME_TEST = TRUE;
			}
			if (fBACKWARD_SETUP & !fBACKWARD_TEST){
				status = EZFX_GOMARKER_BACKWARD_TEST(ctx);
				fBACKWARD_TEST = TRUE;
			}
			if (fFORWARD_SETUP & !fFORWARD_TEST){
				status = EZFX_GOMARKER_FORWARD_TEST(ctx);
				fFORWARD_TEST = TRUE;
			}
			if (fBACK_TIME_SETUP & !fBACK_TIME_TEST){
				status = EZFX_GOMARKER_BACK_TIME_TEST(ctx);
				fBACK_TIME_TEST = TRUE;
			}
			if (fFORW_TIME_SETUP & !fFORW_TIME_TEST){
				status = EZFX_GOMARKER_FORW_TIME_TEST(ctx);
				fFORW_TIME_TEST = TRUE;
			}
			/* TBD
			 * Flash-write black into the next screen.  The size of frames coming
			 * from the stream can change at any time, so we want to be sure we always
			 * draw over a black background to avoid garbage on the edges when a smaller
			 * frame follows a larger frame.
			 */

			++frameCount;
		}
	}
	FAILED:

	/* Cleanup; deallocate [NB: will be done elsewhere] */
	/* DismantlePlayer(ctx); */
	return ( status < 0 ) ? status : playerResult;
}


/*******************************************************************************************
 * Test stream initialization and report results via dsLog
 *******************************************************************************************/
Err	TestVideoStream(ScreenInfo *screenInfoPtr, uint32 bitDepth,
		char* streamFileName, PlayVideoUserFn userFn, void *userContext)
{
	int32			status;
	int32			playerResult = 0;
	DSRequestMsg	EOSMessage;
	PlayerPtr		ctx;
	Player			playerContext;
		
	memset(&playerContext, 0, sizeof(playerContext));
	ctx = &playerContext;

	/* Remember the screen context pointer that we will use for drawing. */
	ctx->screenInfoPtr = screenInfoPtr;

	/* Open the specified file, locate the stream header, and initialize
	 * all resources necessary for performing stream playback. */
	status = InitVideoPlayerFromStreamHeader(&playerContext, streamFileName, bitDepth);
	FAIL_NEG("InitVideoPlayerFromStreamHeader fails\n",status);	
	
	/* Remember the user function pointer, if any */
	ctx->userFn = userFn;
	ctx->userContext = userContext;

	/* If we were supplied with a user function, we have to initiate a
	 * wake up signal to make sure it gets called regularly */
	if (userFn){
		status = CreateItem(MKNODEID(AUDIONODE, AUDIO_CUE_NODE), NULL);
		ctx->userCue = status;
		ctx->userCueSignal = GetCueSignal(ctx->userCue);
	}

	/* Register for end of stream notification. Do it before DSStartStream so
	 * we won't miss it in a short or empty stream. */
	status = DSWaitEndOfStream(ctx->endOfStreamMessageItem, &EOSMessage, ctx->streamCBPtr);
	FAIL_NEG("DSWaitEndOfStream",status);
	/* Preroll the stream */
	status = DSPreRollStream(ctx->messageItem, NULL, ctx->streamCBPtr, 2);
	FAIL_NEG("DSPreRollStream",status);
	
	/* Start the stream running */
	status = DSStartStream(ctx->messageItem, NULL, ctx->streamCBPtr, 0);
	FAIL_NEG("DSStartStream",status);

	/* Prime the user cue signal if present */
	if ( ctx->userCue )
		SignalAtTime(ctx->userCue, GetAudioTime() + kUserFunctionPeriod);
	
	if (gVideoType == MPEG)
		DisplayMPEG(ctx);
	else if (gVideoType == EZFX)
		DisplayEZFX(ctx);

	/* stop the stream */
	
	status = DSStopStream(ctx->messageItem, NULL, ctx->streamCBPtr, SOPT_FLUSH);
	FAIL_NEG("DSStopStream",status);
	
	/* Don't flush if EZFlix data was not present */
	if (gVideoType == EZFX){
		if ( ctx->contextPtr != 0 ){
			/* Flush anything held by the EZFlix subscriber */
			FlushEZFlixChannel( ctx->contextPtr, ctx->channelPtr );
		}
	}
	/* Get rid of stuff we allocated */
	DismantlePlayer( ctx );


FAILED:

	return ( status < 0 ) ? status : playerResult;
}


/*********************************************************************************************
 * Since video type (EZFlix or MPEG) is determinged from header,
 * we can no longer use a default stream header.
 *********************************************************************************************/
static int32 UseDefaultStreamHeader(DSHeaderChunkPtr headerPtr){
	TOUCH(headerPtr);
	return kDSHeaderNotFound;
}

/*******************************************************************************************
 * Routine to get the stream header info, and perform all necessary allocations
 * and initializations necessary for stream playback.
 *******************************************************************************************/
static Err	InitVideoPlayerFromStreamHeader(PlayerPtr ctx, char* streamFileName,
		uint32 bitDepth)
	{
	int32				status;
	long				subscriberIndex;
	long				channelNum;
	int32				i;
	SAudioCtlBlock		ctlBlock;	/* Control blocks for communicating with subscribers */
	MPEGVideoCtlBlock	mpegCtl;
	DSHeaderSubsPtr		subsPtr;
	Boolean				fStreamHasAudio = false;	/* assume no audio subscriber */
	
	/* Get the stream header loaded */
	status = FindAndLoadStreamHeader(&ctx->hdr, streamFileName);

	/* If there was no stream header then use a default */
	/* NB: UseDefaultStreamHeader will not work with this tool */
	if ( status == kDSHeaderNotFound ){
		status = UseDefaultStreamHeader(&ctx->hdr);
		FAIL_NEG( "UseDefaultStreamHeader", status);
	}
	FAIL_NEG( "FindAndLoadStreamHeader", status);
	
	/* Make sure this playback code is compatible with the version of the
	 * data in the stream file. */
	if ( ctx->hdr.headerVersion != DS_STREAM_VERSION )
		return kDSVersionErr;

	/* Allocate the stream buffers and build the linked list of
	 * buffers that are input to the streamer. */
	ctx->bufferList = CreateBufferList(ctx->hdr.streamBuffers, ctx->hdr.streamBlockSize);
	FAIL_NIL("CreateBufferList", ctx->bufferList);
	/* We need to create a message port and message items
	 * to communicate with the data streamer. */
	ctx->messagePort = NewMsgPort(&ctx->messagePortSignal);
	FAIL_NEG( "NewMsgPort", ctx->messagePort);

	ctx->messageItem = CreateMsgItem(ctx->messagePort);
	FAIL_NEG( "CreateMsgItem", ctx->messageItem);
	ctx->endOfStreamMessageItem = CreateMsgItem(ctx->messagePort);
	FAIL_NEG( "CreateMsgItem", ctx->endOfStreamMessageItem);
	status = CreateItem(MKNODEID(AUDIONODE, AUDIO_CUE_NODE), NULL);
	ctx->synchCue = status;


	/* Open a data acquisition thread on the specified file */
	status = ctx->acqMsgPort = NewDataAcq(streamFileName, ctx->hdr.dataAcqDeltaPri);
	FAIL_NEG( "NewDataAcq", status);

	status = NewDataStream(&ctx->streamCBPtr, 		/* output: stream control block ptr */
					ctx->bufferList, 				/* pointer to buffer list */
					ctx->hdr.streamBlockSize, 		/* size of each buffer */
					ctx->hdr.streamerDeltaPri,		/* streamer thread relative priority */
					ctx->hdr.numSubsMsgs);			/* number of subscriber messages */

	FAIL_NEG( "NewDataAcq", status);

	/* Connect the stream to its data supplier */
	status = DSConnect(ctx->messageItem, NULL, ctx->streamCBPtr, 
					ctx->acqMsgPort);
	FAIL_NEG( "DSConnect", status);

	/* Loop through the subscriber descriptor table and initialize all
	 * subscribers specified in the table. */
	for ( subscriberIndex = 0;
			subsPtr = ctx->hdr.subscriberList + subscriberIndex,
			subsPtr->subscriberType != 0;
			subscriberIndex++ )
		{
		switch ( subsPtr->subscriberType )
			{
			case MPVD_CHUNK_TYPE:
				/* Remember the target bit depth of the playback we're starting */
				ctx->bitDepth = bitDepth;
				
				/* This is an MPEG stream, so ignore other video types */
				gVideoType = MPEG;
				
				/* Also allocate a message port for receiving frame completion signals */
				ctx->frameCompletionPort = NewMsgPort(&ctx->frameCompletionPortSignal);
				FAIL_NEG( "NewMsgPort", ctx->frameCompletionPort);

				/* Instantiate an MPEG video subscriber and sign it up for
				 * a DataStreamer subscription (now in one easy step!) */
				status = NewMPEGVideoSubscriber(ctx->streamCBPtr,
							subsPtr->deltaPriority, ctx->messageItem, bitDepth);
				FAIL_NEG( "NewMPEGVideoSubscriber", status);
				
				
				/* The MPEG Subscriber requires a message port to send information
				 * about decompressed frames to.  This enables the application to 
				 * take care of all display activities. */
				mpegCtl.SetClientPort.clientPort = ctx->frameCompletionPort;
				status = DSControl(ctx->messageItem,
							NULL,
							ctx->streamCBPtr,
							MPVD_CHUNK_TYPE,
							kMPEGVideoCtlOpSetClientPort,
							&mpegCtl);

				FAIL_NEG( "DSControl: FrameCompletionPort", status);
				ctx->mpegVideoSubMsgPort = status;
			
				/* Now send the MPEGVideoSubscriber a pointer to the frame buffer to
				 * decompress into.  Note that we're only sending a chunk of memory, so we
				 * could theoretically provide a pointer to any kind of DMA-able memory. */

				mpegCtl.SetFrameBuffers.frameBuffers.numFrameBuffers = cntFrameBuffers;
				mpegCtl.SetFrameBuffers.frameBuffers.frameBufferSize = (bitDepth == 16) ?
					NTSC_FRAME_WIDTH * NTSC_FRAME_HEIGHT * 2 :
					NTSC_FRAME_WIDTH * NTSC_FRAME_HEIGHT * 4;
					
				mpegCtl.SetFrameBuffers.frameBuffers.bitDepth = bitDepth;

				for ( i = 0; i < cntFrameBuffers; i++ )
					mpegCtl.SetFrameBuffers.frameBuffers.frameBuffer[i] =
						ctx->screenInfoPtr->bitmap[i]->bm_Buffer;
		
				status = DSControl(ctx->messageItem,
							NULL,
							ctx->streamCBPtr,
							MPVD_CHUNK_TYPE,
							kMPEGVideoCtlOpSetFrameBuffers,
							&mpegCtl);
				FAIL_NEG( "DSControl: SetFrameBuffers", status);
				ctx->mpegVideoSubMsgPort = status;
				break;

			case EZFLIX_CHUNK_TYPE:
				/* This is an EZFlix stream, so ignore other video types */
				gVideoType = EZFX;
				status = NewEZFlixSubscriber( ctx->streamCBPtr, subsPtr->deltaPriority, 
												ctx->messageItem, 1, &ctx->contextPtr);
				FAIL_NEG( "NewEZFlixSubscriber", status);
				status = InitEZFlixStreamState(ctx->streamCBPtr,/* Need this for DSGetClock in sub */
							ctx->contextPtr,					/* The subscriber's context */
							&ctx->channelPtr,					/* The channel's context */
							0,									/* The channel number */
							true );								/* true = flush on synch msg from DS */
				FAIL_NEG( "InitEZFlixStreamState", status);
				break;
			
			case SNDS_CHUNK_TYPE:
				status = NewSAudioSubscriber(ctx->streamCBPtr, subsPtr->deltaPriority,
							ctx->messageItem);
				FAIL_NEG( "NewSAudioSubscriber", status);
				fStreamHasAudio = true;
				break;

			default:
				PERR(("InitVideoPlayerFromStreamHeader - unknown subscriber type '%.4s'\n",
					(char*)&subsPtr->subscriberType));
				status = kDSInvalidTypeErr;
				goto FAILED;
			}
		}

	/* If the stream has audio, then do some additional initializations. */
	if ( fStreamHasAudio )
	{
		/* Preload audio instrument templates, if any are specified */
		if ( ctx->hdr.preloadInstList != 0 ) {	ctlBlock.loadTemplates.tagListPtr = ctx->hdr.preloadInstList;
			status = DSControl(ctx->messageItem, NULL, ctx->streamCBPtr, SNDS_CHUNK_TYPE,
								 kSAudioCtlOpLoadTemplates, &ctlBlock);
			FAIL_NEG( "DSControl:LoadTemplates", status);
		}
		/* Enable a second audio channel so we can set up a test */
		ctx->hdr.enableAudioChan = 0x0003;
	
		/* Enable any audio channels whose enable bit is set.
		 * NOTE: Channel zero is enabled by default, so we don't check it. */
		for ( channelNum = 1; channelNum < 32; channelNum++ )
		{
			/* If the bit corresponding to the channel number is set,
			 * then tell the audio subscriber to enable that channel. */
			if ( ctx->hdr.enableAudioChan & (1L << channelNum) )
			{	status = DSSetChannel(ctx->messageItem, NULL, ctx->streamCBPtr, 
					SNDS_CHUNK_TYPE, channelNum, CHAN_ENABLED, CHAN_ENABLED);
				FAIL_NEG( "DSSetChannel", status);
			}
		}
	
		/* Set the audio clock to use the selected channel */	
		ctlBlock.clock.channelNumber = ctx->hdr.audioClockChan;
		status = DSControl(ctx->messageItem, NULL, ctx->streamCBPtr, SNDS_CHUNK_TYPE,
								 kSAudioCtlOpSetClockChan, &ctlBlock);
		FAIL_NEG("DSControl:SetClockChan", status);
	}
	return kDSNoErr;

FAILED:
	DismantlePlayer(ctx);
	return status;
}


/*******************************************************************************************
 * Routine to free all resources associated with a Player structure. Assumes that all
 * relevant fields are set to ZERO when the struct is initialized.
 *
 *	NOTE:	THE ORDER OF THE FOLLOWING DISPOSALS IS IMPORTANT. DO NOT CHANGE UNLESS
 *			YOU KNOW WHAT YOU ARE DOING.
 *
 *******************************************************************************************/
void	DismantlePlayer(PlayerPtr ctx)
{
	int32				status;

	/* If we have a frame hanging out, return it to the subscriber before
	 * we turn off the subscribers. */
	 if (gVideoType == MPEG){
		if ( ctx->lastFrameMsg )
			ReplyMsg(ctx->lastFrameMsg, 0, ctx->lastMPEGBufferPtr, sizeof(MPEGBuffer));
	}
	status = DisposeDataStream(ctx->messageItem, ctx->streamCBPtr);
	FAIL_NEG("DisposeDataStream", status);
	status = DisposeDataAcq(ctx->acqMsgPort);
	FAIL_NEG("DisposeDataAcq", status);
	if (gVideoType == EZFX){
		status = DestroyEZFlixStreamState( ctx->contextPtr, ctx->channelPtr);
		FAIL_NEG("DestroyEZFlixStreamState", status);
	}
	FreeMemTrack(ctx->bufferList);
	status = DeleteMsg(ctx->messageItem);
	FAIL_NEG("HandleMPEGFrames DeleteMsg(ctx->messageItem)", status);
	status = DeleteMsg(ctx->endOfStreamMessageItem);
	FAIL_NEG("HandleMPEGFrames DeleteMsg(ctx->endOfStreamMessageItem)", status);
	status = DeleteMsgPort(ctx->messagePort);
	FAIL_NEG("HandleMPEGFrames DeleteMsgPort(ctx->messagePort)", status);
	status = DeleteItem(ctx->synchCue);
	FAIL_NEG("HandleMPEGFrames DeleteItem(ctx->synchCue)", status);
	if (gVideoType == MPEG) {
		status = DeleteMsgPort(ctx->frameCompletionPort);
		FAIL_NEG("HandleMPEGFrames DeleteMsgPort(ctx->frameCompletionPort)", status);
	}

FAILED: 
	;
}

/*******************************************************************************************
 * Handle an MPEG frame completion message: schedule its display time and return the previous
 * frame buffer to the streamer for refilling.
 *
 * [TBD] This is currently implemented by waiting here, in this subroutine. This delays the
 *   app's response to other signals like control port signals. The player's main loop should
 *   instead wait on these two additional signals (time to display a buffer, time to return a
 *   buffer for refilling) and do the work then.
 * [TBD] When we're stopped--at least at EOS--don't return buffers for refilling.
 *******************************************************************************************/
int32		HandleMPEGFrames(PlayerPtr ctx)
	{
	int32					status = kDSNoErr;
	Item					theMsg;
	Message					*msgPtr;
	ScreenInfo				*const screenInfo = ctx->screenInfoPtr;
	DSClock					theClock;
	uint32					thePTS;
	
	/* Get any pending frames off the frame completion port and display them */
	while ( (theMsg = GetMsg(ctx->frameCompletionPort)) != 0 )
		{
		if ( theMsg > 0 )
			{
			msgPtr = (Message *)LookupItem(theMsg);
				/* [TBD] Should probably error check item lookup */
			
			if ( msgPtr != NULL )
				{
				const MPEGBufferPtr		theBuffer = (MPEGBufferPtr)msgPtr->msg_DataPtr;
				const int32				screenIndex = (int32)theBuffer->userData;
				/* wait until the frame is ready to be displayed, then throw it
				 * up on the screen */
				SleepUntilTime(ctx->synchCue, theBuffer->displayTime);

				status = ModifyGraphicsItemVA(screenInfo->viewItem,
							VIEWTAG_BITMAP, screenInfo->bitmapItem[screenIndex],
							TAG_END);
				FAIL_NEG("HandleMPEGFrames ModifyGraphicsItemVA", status);

				
				/* wait until the previous buffer is no longer being displayed */
				WaitSignal(screenInfo->renderSignal);

				
				/* Now that we're done with it, return the last frame to the
				 * subscriber to be recycled. */
				if (ctx->lastFrameMsg != 0)
					{
					status = ReplyMsg(ctx->lastFrameMsg, 0, ctx->lastMPEGBufferPtr,
								sizeof(MPEGBuffer));
					FAIL_NEG("HandleMPEGFrames ReplyMsg", status);
					}

				/* Remember this frame so that we can reply to it when the next
				 * frame is ready to display. */
				ctx->lastFrameMsg		= theMsg;
				ctx->lastMPEGBufferPtr	= theBuffer;
				/* Check the branches started above to make sure
				 * they worked.
				 */
				if (fABSOLUTE_SETUP & !fABSOLUTE_TEST){
					status = MPEG_GOMARKER_ABSOLUTE_TEST(ctx);
					fABSOLUTE_TEST = TRUE;
				}
				if (fABS_TIME_SETUP & !fABS_TIME_TEST){
					status = MPEG_GOMARKER_ABS_TIME_TEST(ctx);
					fABS_TIME_TEST = TRUE;
				}
				if (fBACKWARD_SETUP & !fBACKWARD_TEST){
					status = MPEG_GOMARKER_BACKWARD_TEST(ctx);
					fBACKWARD_TEST = TRUE;
				}
				if (fFORWARD_SETUP & !fFORWARD_TEST){
					status = MPEG_GOMARKER_FORWARD_TEST(ctx);
					fFORWARD_TEST = TRUE;
				}				
				if (fBACK_TIME_SETUP & !fBACK_TIME_TEST){
					status = MPEG_GOMARKER_BACK_TIME_TEST(ctx);
					fBACK_TIME_TEST = TRUE;
				}
				if (fFORW_TIME_SETUP & !fFORW_TIME_TEST){
					status = MPEG_GOMARKER_FORW_TIME_TEST(ctx);
					fFORW_TIME_TEST = TRUE;
				}
				if (fABS_MIDGOP_SETUP & !fABS_MIDGOP_TEST){
					status = MPEG_GOMARKER_ABS_MIDGOP_TEST(ctx);
					fABS_MIDGOP_TEST = TRUE;
				}
				DSGetPresentationClock(ctx->streamCBPtr, &theClock);
				thePTS = theClock.streamTime;				 
				 
				
				 /* MPEG_GOMARKER_ABSOLUTE test starts at 472 and branches to 1072 */
				if (thePTS > 471) {
					if (!fABSOLUTE_SETUP & !fABSOLUTE_TEST){
						MPEG_GOMARKER_ABSOLUTE_SETUP(ctx);
						fABSOLUTE_SETUP = TRUE;
					}
				}
				/* MPEG_GOMARKER_BACKWARD starts at 1312 and branches to 592 */
				if (thePTS > 1311) {
					if (!fBACKWARD_SETUP & !fBACKWARD_TEST & fABSOLUTE_TEST){
						MPEG_GOMARKER_BACKWARD_SETUP(ctx);
						fBACKWARD_SETUP = TRUE;
					}
				}
				/* MPEG_GOMARKER_FORWARD starts at 832 and branches to 1551 */
				if (thePTS > 831) {
					if (!fFORWARD_SETUP & !fFORWARD_TEST & fBACKWARD_TEST){
						MPEG_GOMARKER_FORWARD_SETUP(ctx);
						fFORWARD_SETUP = TRUE;
					}
				}
				/* MPEG_GOMARKER_ABS_TIME starts at 1911 and branches to 1072 */
				if (thePTS > 1910) {
					if (!fABS_TIME_SETUP & !fABS_TIME_TEST & fFORWARD_TEST){
						MPEG_GOMARKER_ABS_TIME_SETUP(ctx);
						fABS_TIME_SETUP = TRUE;
					}
				}
				/* MPEG_GOMARKER_BACK_TIME starts at 1431 and branches to 712 */
				if (thePTS > 1430) {
					if (!fBACK_TIME_SETUP & !fBACK_TIME_TEST & fABS_TIME_TEST){
						MPEG_GOMARKER_BACK_TIME_SETUP(ctx);
						fBACK_TIME_SETUP = TRUE;
					}
				}
				/* MPEG_GOMARKER_FORW_TIME starts at 1791 and branches to 2511 */
				if (thePTS > 1790) {
					if (!fFORW_TIME_SETUP & !fFORW_TIME_TEST & fBACK_TIME_TEST){
						status = MPEG_GOMARKER_FORW_TIME_SETUP(ctx);
						fFORW_TIME_SETUP = TRUE;
					}
				}
				/* MPEG_GOMARKER_ABS_MIDGOP starts at 2871 and branches to 1096 */
				if (thePTS > 2870) {
					if (!fABS_MIDGOP_SETUP & !fABS_MIDGOP_TEST & fFORW_TIME_TEST){
						status = MPEG_GOMARKER_ABS_MIDGOP_SETUP(ctx);
						fABS_MIDGOP_SETUP = TRUE;
					}
				}

			}
		}
	}

FAILED:
	return status;
}
